Package: TSSS
Version: 1.3.4-7
Title: Time Series Analysis with State Space Model
Authors@R: c(person("Yoshinori", "Kawasaki", role = c("aut", "cre"),
             email = "kawasaki@ism.ac.jp"),
             person("Junji", " Nakano", role="aut"),
             person("Masami", "Saga", role = "aut"),
             person("The Institute of Statistical Mathematics", role = c("cph", "fnd")))
Author: Yoshinori Kawasaki [aut, cre],
  Junji Nakano [aut],
  Masami Saga [aut],
  The Institute of Statistical Mathematics [cph, fnd]
Maintainer: Yoshinori Kawasaki <kawasaki@ism.ac.jp>
Copyright: file COPYRIGHTS
Depends: R (>= 4.0.0)
Suggests: utils
Imports: graphics
Description: Functions for statistical analysis, modeling and simulation of time
 series with state space model, based on the methodology in Kitagawa
 (2020, ISBN: 978-0-367-18733-0).
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2026-04-07 07:42:31 UTC; msaga17deko
Repository: CRAN
Date/Publication: 2026-04-10 07:50:10 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 22:57:07 UTC; unix
Archs: TSSS.so.dSYM

Package: TSSS
Version: 1.3.4-7
Title: Time Series Analysis with State Space Model
Authors@R: c(person("Yoshinori", "Kawasaki", role = c("aut", "cre"),
             email = "kawasaki@ism.ac.jp"),
             person("Junji", " Nakano", role="aut"),
             person("Masami", "Saga", role = "aut"),
             person("The Institute of Statistical Mathematics", role = c("cph", "fnd")))
Author: Yoshinori Kawasaki [aut, cre],
  Junji Nakano [aut],
  Masami Saga [aut],
  The Institute of Statistical Mathematics [cph, fnd]
Maintainer: Yoshinori Kawasaki <kawasaki@ism.ac.jp>
Copyright: file COPYRIGHTS
Depends: R (>= 4.0.0)
Suggests: utils
Imports: graphics
Description: Functions for statistical analysis, modeling and simulation of time
 series with state space model, based on the methodology in Kitagawa
 (2020, ISBN: 978-0-367-18733-0).
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2026-04-07 07:42:31 UTC; msaga17deko
Repository: CRAN
Date/Publication: 2026-04-10 07:50:10 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 04:53:26 UTC; unix
Archs: TSSS.so.dSYM

useDynLib(TSSS, .registration = TRUE, .fixes = "C_")

import(graphics)
importFrom("stats", "quantile", "ts", "tsp", "start", "na.omit")
importFrom("grDevices", "dev.new")
importFrom("utils", "capture.output")

export(unicor, crscor, period, fftper, pdfunc, klinfo, boxcox, lsqr, armachar,
 marspc, arfit, marfit, marlsq, lsar, lsar.chgpt, tsmooth, armafit, armafit2,
 polreg, trend, season, tvvar, tvar, tvspc, ngsmth, simssm, ngsim, pfilter,
 pfilterNL)

# for TSSS.OMP
export(print.tvvar, plot.tvvar, plot.pfilter)

S3method(print, armafit)
S3method(print, armafit2)
S3method(print, boxcox)
S3method(print, klinfo)
S3method(print, lsar)
S3method(print, lsqr)
S3method(print, maryule)
S3method(print, marlsq)
S3method(print, ngsmth)
S3method(print, season)
S3method(print, smooth)
S3method(print, trend)
S3method(print, tvvar)

S3method(plot, arfit)
S3method(plot, arma)
S3method(plot, boxcox)
S3method(plot, chgpt)
S3method(plot, crscor)
S3method(plot, lsar)
S3method(plot, lsqr)
S3method(plot, marspc)
S3method(plot, ngsmth)
S3method(plot, pdfunc)
S3method(plot, pfilter)
S3method(plot, polreg)
S3method(plot, season)
S3method(plot, simulate)
S3method(plot, smooth)
S3method(plot, spg)
S3method(plot, trend)
S3method(plot, tvspc) 
S3method(plot, tvvar)
S3method(plot, unicor)

*     particle_smoother.f       for R function  JUl.12,2019
*     Monte-S.f   8/13/2002     Small memory version   
*     monte.f     12/06/2001    
C     @MONTE.JCGS.FIG32:                    JUN.  ,1995
C     @NFILTER.SIM4:    FORWARD SMOOTHING   NOV.20,1992
C     @NFILTER.SIM3:    SMOOTHING           NOV.19,1992
C     @NFILTER.SIM2:    MONTE CARLO METHOD  NOV.02,1992
C
      SUBROUTINE PFILTER(Y,N,M,MODEL,LAG,INID,SIG2,TAU2,ALPHA,BIGTAU2,
     *                   SIG2I,XMIN,XMAX,IX,T,FF)
C
cc      PARAMETER( NMAX=500,IDEV=1,MM=1000000, LAG=20 )
cc      IMPLICIT REAL*8(A-H,O-Z)
cc	  real*4  PS, PST
cc      DIMENSION  Y(NMAX), T(NMAX,8), PS(MM,0:LAG), PST(MM,0:LAG)
cc      CHARACTER   DATE*8, TIME2*10
      INTEGER N, M, MODEL, LAG, INID, IX
      DOUBLE PRECISION Y(N), SIG2, TAU2, ALPHA, BIGTAU2, SIG2I, XMIN,
     *                 XMAX, T(N,8), FF
c local
      INTEGER L
ccc      DOUBLE PRECISION PS(M,0:LAG), PST(M,0:LAG)
      DOUBLE PRECISION, allocatable :: PS(:,:), PST(:,:)
      allocate(PS(M,0:LAG))
      allocate(PST(M,0:LAG))
cc      COMMON  /INIT/ IX
cc      common  /COMIST/  IST

cc      IX = 1992110113
cc      IST = 0
C
cc      MODEL = 1
*           M is the number of particles. 1000 - 100000
*           LAG:  Fixed lag smoother LAG=0 => filter
*           IX:  initial seed for random number generator
cc      M = 1000000
cc      M = 100000
      L = 1
cc	  INID = 0
*
cc      IX = 2019071117
C     IX = 2002081119
*      IX = 1997102221
cc      IX0= IX
*
*     sig2  observation noise variance
*     tau2: system noise variance
cc      SIG2 = 1.045D0
cc      TAU2 = 3.5300D-5
cc      IF( MODEL.EQ.0 )  THEN
cc        SIG2 = 1.043D0
cc        TAU2 = 1.2200D-2
cc        SIG2 = 1.048D0
cc        TAU2 = 1.4000D-2
cc      END IF
cc      IF( MODEL.EQ.2 )  THEN
cc        SIG2 = 1.03D0
cc        TAU2 = 0.00013d0
cc      END IF
*
*      READTS reads in data: change this subroutine
*
cc      CALL  READTS( IDEV,Y,N )
cc      WRITE(6,600)  MODEL, M, L, IX, INID
cc  600 FORMAT( 1H ,'MODEL =',I2,3X,'M =',I6,3X,'L =',I3,3X,'IX =',I12,
cc     *            3X,'INID =',I3 )
C
cc      CALL  DATE_AND_TIME( DATE,TIME2 )     
cc      WRITE(6,650)  DATE, TIME2      
cc  650 FORMAT( 'Date',2X,A8,3X,'Time:',2X,A10 )
cc      CALL  FILTER( Y,N,M,L,T,PS,PST,MODEL,SIG2,TAU2,LAG,NMAX,FF,MM,
cc     *              INID )
      CALL  CFILTER( Y,N,M,L,T,PS,PST,MODEL,SIG2,TAU2,LAG,ALPHA,BIGTAU2,
     *               SIG2I,XMIN,XMAX,FF,INID,IX )
cc      CALL  DATE_AND_TIME( DATE,TIME2 )     
cc      WRITE(6,650)  DATE, TIME2      
C
cc      open( 6,file='results.dat' )
cc      CALL  PTDATA( Y,T,N,M,L,MODEL,SIG2,TAU2,IX0,LAG,NMAX,FF )
cc	close( 6 )
C
cc      STOP
      deallocate(PS, PST)
      return
      E N D
cc      SUBROUTINE  FILTER( Y,N,M,L,T,PS,PST,MODEL,SIG2,TAU2,LAG,NMAX,FF,
cc     *                    MM, INID )
      SUBROUTINE  CFILTER( Y,N,M,L,T,PS,PST,MODEL,SIG2,TAU2,LAG,ALPHA,
     *                     BIGTAU2,SIG2I,XMIN,XMAX,FF,INID,IX)
cc      IMPLICIT REAL*8(A-H,O-Z)
cc	  real*4  PS, PST
cc      DIMENSION  Y(N), F(1000000), P(1000000), S(1000000)
cc      DIMENSION  PS(MM,0:LAG), T(NMAX,8), PST(MM,0:LAG)
cc      DIMENSION  TT(7)
      INTEGER N, M, L, MODEL, LAG, INID, IX
      DOUBLE PRECISION Y(N), T(N,8), PS(M,0:LAG), PST(M,0:LAG), SIG2,
     *                 TAU2, ALPHA, BIGTAU2, SIG2I, XMIN, XMAX, FF
c local
      INTEGER I, II, IJ, I0, J, JJ
      DOUBLE PRECISION PI, TAU, F(M), P(M), S(M), TT(7), RNOR, RR, SUM, 
ccx     1           RUNIFT
     1                 GAUS2, random, BIGTAU, GB, ymin, ymax
cc      COMMON  /INIT/ IX
      DATA  PI /3.1415926535D0/
c
c----- Initialize Mersenne Twister with given seed value
      call init(IX)
c-----
*
      TAU = DSQRT( TAU2 )
      BIGTAU = DSQRT( BIGTAU2 )
      GB = DSQRT( SIG2I )
      ymin = MINVAL(Y)
      ymax = MAXVAL(Y)
      FF = 0.0D0

*       Specify initial distribution : you can change these
      DO 10 I=1,M
      IF( INID.EQ.3 )  F(I) = 0.0D0
cc      IF( INID.EQ.2 )  F(I) = RCAU( 1.0D0 )
      IF( INID.EQ.2 )  F(I) = GB * DTAN( random()*PI )
cc      IF( INID.EQ.1 )  F(I) = -4.0D0 + 8*RUNIFT(IX)
      IF( INID.EQ.1 )  F(I) = ymin + (ymax - ymin)*random()
cc   10 IF( INID.EQ.0 )  F(I) = RNOR( 1.0D0 )
      IF( INID.EQ.0 )  F(I) = RNOR( GB )
   10 CONTINUE
*
*    Monte Carlo filtering   N is data length
*
      DO 100 II=1,N
*
*     Prediction
*
      I0 = min0( ii,lag )
cc
      DO 20 I=1,M
      IF( MODEL.EQ.0 ) P(I) = F(I) + RNOR( 1.0D0 )*TAU
cc      IF( MODEL.EQ.1 ) P(I) = F(I) + RCAU( 1.0D0 )*TAU
ccx      IF( MODEL.EQ.1 ) P(I) = F(I) + DTAN(RUNIFT(IX)*PI )*TAU
      IF( MODEL.EQ.1 ) P(I) = F(I) + DTAN( random()*PI )*TAU
      IF( MODEL.EQ.2 ) then
ccx      RR = RUNIFT(IX)
      RR = random()
cc      IF( RR.le.0.991d0 )  P(I) = F(I) + RNOR( 1.0D0 )*TAU
cc      IF( RR.GT.0.991d0 )  P(I) = F(I) + RNOR( 1.0d0 )*2.0d0
      IF( RR.le.ALPHA )  P(I) = F(I) + RNOR( 1.0D0 )*TAU
      IF( RR.GT.ALPHA )  P(I) = F(I) + RNOR( 1.0d0 )*BIGTAU
      end if
   20 CONTINUE
C
*     Filtering
*
*   GAUS2:  Gaussian density, change this if you want other distribution
      SUM = 0.0D0
      DO 30 I=1,M*L
      S(I) = GAUS2( Y(II)-P(I),SIG2 )
cc   30 SUM = SUM + S(I)
      SUM = SUM + S(I)
   30 CONTINUE
*
*     FF: log-likelihood
*
      FF = FF + DLOG( SUM/M )
cc   35 CONTINUE
*
*     Re-sampling
*
      S(1) = S(1)/SUM
      DO 40 I=2,M
cc   40 S(I) = S(I-1) + S(I)/SUM
      S(I) = S(I-1) + S(I)/SUM
   40 CONTINUE
C  
      JJ = 1
      DO 50 I=1,M
ccx      RR = (I-1+RUNIFT(IX))/M
      RR = (I-1+random())/M
      DO 60 J=JJ,M*L
      IF( S(J).GE.RR )  GO TO 55
   60 CONTINUE
cc   55 IF( J.GE.M )  J = M
   55 CONTINUE
      IF( J.GE.M )  J = M
      F(I) = P(J)
      JJ = J
      I0 = min0( ii,lag )
      DO 56 IJ=1,I0
cc   56 PST(I,IJ) = PS(J,IJ-1)
      PST(I,IJ) = PS(J,IJ-1)
   56 CONTINUE
   50 CONTINUE
*
cc      DO 65 IJ=1,I0
      DO 66 IJ=1,I0
      DO 65 I=1,M
cc   65 PS(I,IJ) = PST(I,IJ)
      PS(I,IJ) = PST(I,IJ)
   65 CONTINUE
   66 CONTINUE
C
      DO 70 I=1,M
cc   70 PS(I,0) = F(I)
      PS(I,0) = F(I)
   70 CONTINUE
cc   90 CONTINUE

      IF( II.LE.LAG )  GO TO 100
      do 320 I=1,8
cc  320 T(II-LAG,I) = 0.0d0
      T(II-LAG,I) = 0.0d0
  320 CONTINUE

cc      CALL  DENSTY( PS(1,LAG),M,TT )
      CALL  DENSTY1( PS(1,LAG),M,TT,XMIN,XMAX )

      DO 310 I=1,7
cc  310 T(II-LAG,I) = TT(I)
      T(II-LAG,I) = TT(I)
  310 CONTINUE

  100 CONTINUE
*
*     Compute Posterior Distributions
*     T(II,J), J=4: MEdian, other: 1,2,3 sigma interval
*              J=8  MEAN
*
cc      WRITE(6,660)  TAU2, SIG2, FF
cc  660 FORMAT( 1H ,F15.8,F10.4,F13.4 )
      do 200 II=0,LAG-1
cc      CALL  DENSTY( PS(1,II),M,TT )
      CALL  DENSTY1( PS(1,II),M,TT,XMIN,XMAX )
      DO 210 I=1,7
cc  210 T(N-II,I) = TT(I)
      T(N-II,I) = TT(I)
  210 CONTINUE
*      DO 110 I=1,M
*  110 SUM = SUM + PS(I,II)
*      T(II,8) = SUM/M
  200 CONTINUE
C
      RETURN
      E N D

      double precision FUNCTION  RNOR( GB )
cc      IMPLICIT REAL*8(A-H,O-Z)
      DOUBLE PRECISION GB
c local
      INTEGER I
ccx      DOUBLE PRECISION X, Y, S, RUNIFT
      DOUBLE PRECISION X, Y, S, random
cc      COMMON  /INIT/ IX
      DATA   I /0/
      data   y/0.0d0/
      data   s/0.0d0/
ccx      IF( IX.EQ.0 )  IX = 1991041911
      IF( I.GT.0 )  GO TO 30
ccx   10 X = 2.0*RUNIFT(IX) - 1.0
ccx      Y = 2.0*RUNIFT(IX) - 1.0
   10 X = 2.0*random() - 1.0
      Y = 2.0*random() - 1.0
      S = X**2 + Y**2
      IF( S.GE.1.0 )  GO TO 10
      S = DSQRT( -2.0*DLOG(S)/S )
      RNOR = X*S
      I = 1
      GO TO 40
   30 RNOR = Y*S
      I = 0
   40 CONTINUE
      RNOR = RNOR*GB
C      WRITE(6,*)  RNOR
      RETURN
      E N D
      DOUBLE PRECISION FUNCTION  GAUS2( X,SIG2 )
C
cc      IMPLICIT  REAL*8(A-H,O-Z)
      DOUBLE PRECISION X, SIG2, C1
      DATA  C1  /2.506628275D0/
C
      GAUS2 = DEXP( -X**2/(2*SIG2) )/(C1*DSQRT(SIG2))
      RETURN
      E N D
      
      DOUBLE PRECISION FUNCTION  CAUC2( X,SIG2 )
C
cc      IMPLICIT  REAL*8(A-H,O-Z)
      DOUBLE PRECISION X, SIG2, PI
      DATA  PI  /3.141592653D0/
C
      CAUC2 = DSQRT(SIG2)/PI/(X**2 + SIG2)
      RETURN
      E N D
      
      SUBROUTINE  SORT( Y,N )
cc      IMPLICIT REAL*8 (A-H,O-Z)
cc      DIMENSION  Y(N)
      INTEGER N
      DOUBLE PRECISION Y(N)
c local
      INTEGER I, J, JJ, L
      DOUBLE PRECISION YY
C
      IF( Y(2).LT.Y(1) )  THEN
         YY   = Y(1)
         Y(1) = Y(2)
         Y(2) = YY
      END IF
      DO 100  I=3,N
      JJ = I
      DO 20  J=I-1,1,-1
         IF( Y(I).LT.Y(J) )  JJ = J
   20 CONTINUE
      IF( JJ.LT.I )  THEN
            YY = Y(I)
            DO 10 L=I-1,JJ,-1
cc   10          Y(L+1) = Y(L)
               Y(L+1) = Y(L)
   10       CONTINUE
            Y(JJ) = YY
      END IF
  100 CONTINUE
      RETURN
      E N D
      
cc      SUBROUTINE  DENSTY( P,M,T )
cc      IMPLICIT REAL*8(A-H,O-Z)
cc	  real*4  P
cc      REAL*4  PROB
cc      DIMENSION  P(M), Q(1000), QQ(0:1000), PROB(7), T(7)
      SUBROUTINE  DENSTY1( P,M,T,XMIN,XMAX )

      INTEGER M
      DOUBLE PRECISION P(M), T(7), XMIN, XMAX
c local
cxxx      PARAMETER(K = 1000)
      INTEGER, PARAMETER :: K = 1000
      INTEGER I, J
      DOUBLE PRECISION Q(K), QQ(0:K), PROB(7), PP, DX
cc      DATA  PROB /0.0013, 0.0227, 0.1587, 0.5000, 0.8413, 0.9773,
cc     *            0.9987/
      DATA  PROB /0.0013D0, 0.0227D0, 0.1587D0, 0.5D0, 0.8413D0,
     *             0.9773D0, 0.9987D0/
C
cc      XMIN = -4.0D0
cc      XMAX =  4.0D0
cc      K = 1000
      DX = (XMAX-XMIN)/K
C
cc      DO 10 I=1,K
cc   10 Q(I) = 0.0D0
      Q(1:k) = 0.0D0
      DO 20 I=1,M
cc      J = (P(I) - XMIN)/DX
      J = INT((P(I) - XMIN)/DX)
      IF( J.Le.0 )  J = 1
      IF( J.GT.K )  J = K
cc   20 Q(J) = Q(J) + 1
      Q(J) = Q(J) + 1
   20 CONTINUE
      DO 30 I=1,K
cc   30 Q(I) = Q(I)/M
      Q(I) = Q(I)/M
   30 CONTINUE
C
      QQ(0) = 0.0D0
      DO 40 I=1,K
cc   40 QQ(I) = QQ(I-1) + Q(I)
      QQ(I) = QQ(I-1) + Q(I)
   40 CONTINUE
C     WRITE(6,*)  (P(I),I=1,K)
C     WRITE(6,*)  (QQ(I),I=1,K)
C
cc      DO 60 J=1,7
      DO 61 J=1,7
      PP = PROB(J)
      DO 50 I=1,K
      IF(QQ(I-1).LE.PP .AND. QQ(I).GT.PP)  GO TO 60
   50 CONTINUE
   60 T(J) = XMIN + (I-1)*DX + DX*(PP - QQ(I-1))/(QQ(I) - QQ(I-1))
   61 CONTINUE
C
      RETURN
      E N D

C     PROGRAM 12.1  SEASON
      SUBROUTINE SEASON( Y0,N,M1,M2,M3,M4,PERIOD,JYEAR,MONTH,
     & TAU2,NS,NFE,NPE,AR,LOGT,IOPT,OUTMIN,OUTMAX,NMAX,MJ,
     & FF,OVAR,AIC,XSS,VSS,DEFF,IER1,IER2 ) 
C
C  ...  SEASONAL ADJUSTMENT BY STATE SPACE MODELING  ...
C
C     Inputs:
C        M1:      Trend Order (M1=0,1,2,3)
C        M2:      Seasonal Order (M1=0,1,2)
C        M3:      AR Order
C        M4:      Trading day effect (M4=0 or 6)
C        PERIOD:  Number of seasons in one period
C                 (=12 for monthly data, =4 for quarterly data)
C        IPARAM:  =0    Use defalt initail values
C                 =1    Read intial values
C        OUTMIN:  Lower limit of the observations
C        OUTMAX:  Upper limit of the observations
C        JYEAR:   Starting year of the data (e.g. 1967)
C        MONTH:   Starting month of the data (e.g. 1)
C        TAU2(I): System noise variances   (I=1,4)
C        AR(I):   AR coefficients (I=1,M3)
C        NS:      Start position of filtering
C        NFE:     End point of filtering
C        NPE:     End point of prediction
C        IOPT:    = 1     To get MLE
C     Parameters:
C        NMAX:    Adjustable dimension of Y
C        MJ:      Adjustable dimension of XF, VF, etc.
C        MAXM:    Adjustable dimension of A, B, C
C        NC:      Number of components
C     @TEST.FILTER2O    NOV.29,1990, SEP.02,1992
C
cc      PARAMETER( NMAX=160,MJ=20,MAXM=22,NC=10)
cxxx      PARAMETER( MAXM=22,NC=9 )
      INTEGER, PARAMETER :: MAXM=22, NC=9
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      INTEGER*4  PERIOD
cc      DIMENSION  M(10), TAU2(10), TAU20(10)
cc      DIMENSION  XMEAN(10), XVAR(MAXM,10), AA(20), AR(10), PAR(10)
cc      DIMENSION  A(MAXM,10), B(MAXM,10), C(MAXM,10), Q(10,10)
cxx      DIMENSION  M(NC), TAU2(4)
cxx      DIMENSION  XMEAN(NC), XVAR(MAXM,NC), AA(M3+3), AR(M3), PAR(M3)
cxx      DIMENSION  A(MAXM,NC), B(MAXM,NC), C(MAXM,NC), Q(NC,NC)
cxx      DIMENSION  XPS(MJ,NMAX), XFS(MJ,NMAX), XSS(MJ,NMAX)
cxx      DIMENSION  VPS(MJ,MJ,NMAX), VFS(MJ,MJ,NMAX), VSS(MJ,MJ,NMAX)
cc      DIMENSION  XF(MJ), VF(MJ,MJ), MTYPE(10)
cxx      DIMENSION  XF(MJ), VF(MJ,MJ), MTYPE(NC)
cc      COMMON  /C92827/  M1, M2, M3, M4, PERIOD
cc      COMMON  /C92825/  OUTMIN, OUTMAX
cc      COMMON  /C92826/  Y(NMAX), REG(NMAX,7)
cxx      DIMENSION  Y(N), REG(NMAX,7)
cc      COMMON  /C92907/  ALIMIT
cc      COMMON  /CMMODL/  M, XMEAN, XVAR, N, NS, NFE, NPE
cc      COMMON  / CCC /  ISW, IPR, ISMT, IDIF
cxx      DIMENSION  Y0(N), DEFF(NMAX)
C
      INTEGER N, M1, M2, M3, M4, PERIOD, JYEAR, MONTH, NS, NFE, NPE,
     1        LOGT, IOPT, NMAX, MJ, IER1, IER2
      DOUBLE PRECISION Y0(N), TAU2(4), AR(M3), OUTMIN, OUTMAX, FF, OVAR,
     1                 AIC, XSS(MJ,NMAX), VSS(MJ,MJ,NMAX), DEFF(NMAX)
c local
      INTEGER I, ISW, IDIF, J, L, MM, NP, M(NC), MTYPE(NC), ID
      DOUBLE PRECISION XMEAN(NC), XVAR(MAXM,NC), AA(M3+3), PAR(M3),
     1                 A(MAXM,NC), B(MAXM,NC), C(MAXM,NC), Q(NC,NC),
     2                 XPS(MJ,NMAX), XFS(MJ,NMAX), VPS(MJ,MJ,NMAX),
     3                 VFS(MJ,MJ,NMAX), XF(MJ), VF(MJ,MJ), Y(N),
     4                 REG(NMAX,7), SIG2, ALIMIT
C
      EXTERNAL  FFSEAS
C
      XSS = 0.0D0
      VSS = 0.0D0
      DEFF = 0.0D0
C
      XMEAN = 0.0D0
      XVAR = 0.0D0
      AA = 0.0D0
      PAR = 0.0D0
      A = 0.0D0
      B = 0.0D0
      C = 0.0D0
      Q = 0.0D0
      XPS = 0.0D0
      XFS = 0.0D0
      VPS = 0.0D0
      VFS = 0.0D0
      XF = 0.0D0
      VF = 0.0D0
C
C  ...  Read Time Series  ...
C
cc      CALL  READTS( 1,Y,N )
      DO 100 I = 1,N
cxx  100 Y(I) = Y0(I)
      Y(I) = Y0(I)
  100 CONTINUE
C
C  ...  Read Model Orders  ...
C
cc      READ( 5,* )  M1, M2, M3, M4, PERIOD
C
C  ...  Set Defalt Parameters  ...
C
cc      IPR  = 7
      SIG2 = 1.0D0
      ALIMIT = 0.9D0
cc      CALL  SPARAM( M1,M2,M3,M4,N,TAU2,AR,OUTMIN,OUTMAX,NS,NFE,NPE,
cc     *              LOGT,IOPT )
C
cc      READ( 5,* )  IPARAM
cc      IF( IPARAM.EQ.1 )  THEN
cc         READ( 5,* )  (TAU2(I),I=1,3)
cc         READ( 5,* )  NS, NFE, NPE
cc         READ( 5,* )  LOGT, IOPT
cc         READ( 5,* )  OUTMIN, OUTMAX
cc         IF(M3.GT.0)  READ( 5,* )  (AR(I),I=1,M3)
cc      END IF
cc      WRITE(6,*) M1,M2,M3,M4,PERIOD
cc      WRITE(6,*) (TAU2(I),I=1,3)
cc      WRITE(6,*) (AR(I),I=1,M3)
C
cc      IF( M4.GT.0 )  READ( 5,* )  JYEAR, MONTH
C
C  ...  Log Transformation  ...
C
      IF( LOGT.EQ.1 )  THEN
         IER1 = -1
         DO 5 I=1,N
cc    5    Y(I) = DLOG10( Y(I) )
            IF( Y(I).LE.0 ) RETURN
cxx    5       Y(I) = DLOG10( Y(I) )
            Y(I) = DLOG10( Y(I) )
    5    CONTINUE
c 2023/3/4
      ELSE IF ( LOGT.EQ.2 ) THEN
         IER1 = -1
         DO 6 I=1,N
            IF( Y(I).LE.0 ) RETURN
            Y(I) = DLOG( Y(I) )
    6    CONTINUE
      END IF
      IER1 = 0
C
C  ...  Trading day factor  ...
C
cc      IF( M4.GT.0 )  CALL  TRADE( JYEAR,MONTH,N,NMAX,REG )
      IF( M4.GT.0 )  THEN
         CALL  TRADE( JYEAR,MONTH,NMAX,NMAX,REG )
cc      DO 10 I=1,N
cxx      DO 10 I=1,NMAX
      DO 11 I=1,NMAX
      DO 10 J=1,6
cxx   10 REG(I,J) = REG(I,J) - REG(I,7)
      REG(I,J) = REG(I,J) - REG(I,7)
   10 CONTINUE
   11 CONTINUE
      END IF
C
      CALL  PARCOR( AR,M3,PAR )
      NP = ID(M1) + ID(M2) + ID(M3)
      DO 20 I=1,NP
cxx   20 AA(I) = DLOG( TAU2(I)/(1.0D0-TAU2(I)) )
      AA(I) = DLOG( TAU2(I)/(1.0D0-TAU2(I)) )
   20 CONTINUE
c----------    2013/07/02
      ier2 = 0
      DO 30 I=1,M3
         if( ALIMIT .le. PAR(I) ) ier2 = -1
cxx   30 AA(NP+I) = DLOG( (ALIMIT+PAR(I))/(ALIMIT-PAR(I)) )
      AA(NP+I) = DLOG( (ALIMIT+PAR(I))/(ALIMIT-PAR(I)) )
   30 CONTINUE
      if( ier2 .eq. -1 ) return
c----------
cc      DO 40 I=1,NC
cc   40 TAU20(I) = TAU2(I)
C
C  ...  Maximum Likelihood Method  ...
C
cc      IF( IOPT.EQ.1 )  CALL  DAVIDN( FFSEAS,AA,NP+M3,2 )
      IF( IOPT.EQ.1 )  THEN
         CALL  DAVIDN1( FFSEAS,AA,NP+M3,2,
     *   Y,N,M1,M2,M3,M4,PERIOD,REG,OUTMIN,OUTMAX,ALIMIT,ISW,IDIF,
     *   M,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,IER1,IER2 )
         if( ier2.ne.0 ) return
      END IF
C
      DO 50 I=1,NP
cxx   50 TAU2(I) = DEXP( AA(I) )/(1.0D0 + DEXP( AA(I) ) )
      TAU2(I) = DEXP( AA(I) )/(1.0D0 + DEXP( AA(I) ) )
   50 CONTINUE
      DO 60 I=1,M3
cxx   60 PAR(I)  = ALIMIT*(DEXP(AA(NP+I))-1.0D0)/(DEXP(AA(NP+I))+1.0D0)
      PAR(I)  = ALIMIT*(DEXP(AA(NP+I))-1.0D0)/(DEXP(AA(NP+I))+1.0D0)
   60 CONTINUE
      CALL  ARCOEF( PAR,M3,AR )
C
C  ...  The Maxmum Likelihood Model  ...
C
cc      CALL  SETABC( M,L,NC,MTYPE,TAU2,MAXM,MM,AR,Y,N,A,B,C,Q,XMEAN,
cc     *              XVAR )
      CALL  SETABC1( M1,M2,M3,M4,PERIOD,M,L,NC,MTYPE,TAU2,MAXM,MM,AR,
     *         Y,N,A,B,C,Q,XMEAN,XVAR,ier2 )
      if( ier2.ne.0 ) return
      CALL  ISTAT1( L,M,MJ,MAXM,A,XMEAN,XVAR,XF,VF )
C
C  ...  The Kalman Filter  ... 
C
cc      CALL  FILTR1( Y,XF,VF,A,B,C,Q,SIG2,REG,MTYPE,NMAX,M,MAXM,NC,L,
cc     *      NS,NFE,NPE,MJ,N, OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,OVAR )
      CALL  FILTR1( Y,XF,VF,A,B,C,Q,SIG2,REG,MTYPE,NMAX,M,MAXM,NC,L,
     *     NS,NFE,NPE,MJ,N, OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,OVAR,ier2 )
      if( ier2.ne.0 ) return
C
C  ...  Fixed Interval Smoothing  ...
C
cc      CALL  SMOTH1( A,M,MAXM,L,NS,NFE,NPE,MJ,
      CALL  SMOTH1( A,M,MAXM,L,NS,NFE,NPE,NMAX,MJ,
     *              VFS,VPS,VSS,XFS,XPS,XSS )
      AIC = -2*FF + 2*(NP+M3+MM)
C
C  ...  Print out and draw estimates  ...
C
cc      CALL  PRSEAS( M1,M2,M3,M4,PERIOD,TAU2,TAU20,OVAR,FF,AIC,AR,
cc     *              XSS,N,REG,NMAX,MJ )
cxx      CALL  PRSEAS( M1,M2,M3,M4,PERIOD,XSS,DEFF,N,REG,NMAX,MJ )
      CALL  PRSEAS( M1,M2,M3,M4,PERIOD,XSS,DEFF,REG,NMAX,MJ )
C      CALL  PTSEAS( Y,XSS,VSS,N,NPE,MJ,M1,M2,M3,M4,
C     *              PERIOD,TAU2,OVAR,FF,AIC,REG,NMAX )
C
cc      STOP
      RETURN
      E N D
cc      SUBROUTINE  PRSEAS( M1,M2,M3,M4,LPER,TAU2,TAU20,OVAR,FF,AIC,
cc     *                    AR,XSS,N,REG,NMAX,MJ )
cxx      SUBROUTINE  PRSEAS( M1,M2,M3,M4,LPER,XSS,DEFF,N,REG,NMAX,MJ )
      SUBROUTINE  PRSEAS( M1,M2,M3,M4,LPER,XSS,DEFF,REG,NMAX,MJ )
C
C  ...  Print out Estimates  ...
C
C     Inputs:
C        M1-M4:  Model orders
C        LPER:   Period
C        TAU2:   Variance of the system noise
C        TAU20:  Variance of the system noise (initial estimat)
C        OVAR:   Variance of the observational noise
C        FF:     Log-Likelihood of the mode
C        AIC:    AIC of the model
C        AR:     AR coefficients
C        XSS:    Smoothed state vector
C        N:      Data length
C        REG:    Number of trading days
C        NMAX:   Adjustable dimension of REG
C        MJ:     Adjustable dimension of XSS
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      CHARACTER  TITLE*72
cc      DIMENSION  TAU2(*), TAU20(*), AR(*)
cxx      DIMENSION  XSS(MJ,NMAX), REG(NMAX,7)
cc      COMMON     /CMDATA/  TITLE
cxx      DIMENSION  DEFF(NMAX)
C
      INTEGER M1, M2, M3, M4, LPER, NMAX, MJ 
      DOUBLE PRECISION XSS(MJ,NMAX), DEFF(NMAX), REG(NMAX,7)
c local
      INTEGER I, J, M12, M123, NP, ID
      DOUBLE PRECISION SUM
C
      NP = ID(M1) + ID(M2) + ID(M3)
      M12  = M1 + M2*(LPER-1)
      M123 = M12 + M3
cc      WRITE(6,600)
cc      WRITE(6,610) TITLE
cc      WRITE(6,620) M1, M2, M3, M4, LPER
cc      WRITE(6,630)
cc      WRITE(6,640)
cc      WRITE(6,650) (TAU20(I),I=1,NP)
cc      WRITE(6,660)
cc      WRITE(6,640)
cc      WRITE(6,650) (TAU2(I),I=1,NP)
cc      IF( M3.GT.0 )  THEN
cc      WRITE(6,690)
cc      WRITE(6,700) (AR(I),I=1,M3)
cc      END IF
cc      WRITE(6,680) OVAR, FF, AIC
cc      IF( M1.GT.0 )  THEN
cc        WRITE(6,720)
cc        WRITE(6,710)  (XSS(1,I),I=1,N)
cc      END IF
cc      IF( M2.GT.0 )  THEN
cc        WRITE(6,730)
cc        WRITE(6,710)  (XSS(M1+1,I),I=1,N)
cc      END IF
cc      IF( M3.GT.0 )  THEN
cc        WRITE(6,740)
cc        WRITE(6,710)  (XSS(M12+1,I),I=1,N)
cc      END IF
      IF( M4.GT.0 )  THEN
cc        DO 210 I=1,N
        DO 210 I=1,NMAX
        SUM = 0.0D0
        DO 200 J=1,6
cxx  200   SUM = SUM + XSS(M123+J,I)*REG(I,J)
        SUM = SUM + XSS(M123+J,I)*REG(I,J)
  200   CONTINUE
cc  210   REG(I,7) = SUM
cxx  210 DEFF(I) = SUM
      DEFF(I) = SUM
  210 CONTINUE
cc        WRITE(6,750)
cc        WRITE(6,710)  (REG(I,7),I=1,N)
      END IF
      RETURN
cxx  600 FORMAT( 1H1,'PROGRAM 10.1:  SEASONAL ADJUSTMENT' )
cxx  610 FORMAT( 1H ,A72 )
cxx  620 FORMAT( 1H ,'M1 =',I2,3X,'M2 =',I2,3X,'M3 =',I2,3X,'M4 =',I2,
cxx     *        'PERIOD =',I3 )
cxx  630 FORMAT( 1H ,5X,'***  INITIAL ESTIMATE  ***' )
cc  640 FORMAT( 1H ,'TAU2(I),I=1,NC:' )
cxx  640 FORMAT( 1H ,'TAU2(I),I=1,NP:' )
cxx  650 FORMAT( 1H ,5D13.6 )
cxx  660 FORMAT( 1H ,5X,'***  FINAL ESTIMATE  ***' )
cxx  680 FORMAT( 1H ,'SIG2 =',D13.6,3X,'LOG-LIKELIHOOD =',F13.4,3X,
cxx     *            'AIC =',F13.4 )
cxx  690 FORMAT( 1H ,'AR COEFFICIENTS' )
cxx  700 FORMAT( 1H ,5F10.6 )
cxx  710 FORMAT( 1H ,6F12.5 )
cxx  720 FORMAT( 1H ,'TREND COMPONENT' )
cxx  730 FORMAT( 1H ,'SEASONAL COMPONENT' )
cxx  740 FORMAT( 1H ,'AR COMPONENT' )
cxx  750 FORMAT( 1H ,'TRADING DAY EFFECT' )
      E N D
      SUBROUTINE  FILTR1( Y,XF,VF,A,B,C,Q,SIG2,REG,MTYPE,NMAX,M,MMAX,
     *                    NCM,NC,NS,N,NPE,MJ,NDIM,OUTMIN,OUTMAX,
     *                    VFS,VPS,XFS,XPS,FF,OVAR,IER )
C
C  ...  Kalman filter  ...
C
C     Inputs:
C        Y:      time series
C        N:      data length
C        NS:     Start position of filtering
C        NE:     End position of prediction
C        XF:     Initial state vector
C        VF:     Initial covariance matrix
C        A:      Parameters of the matrix F
C        B:      Parameters of the matrix G
C        C:      Parameters of the matrix H
C        Q:      Covariance matrix of the system noise
C        M:      Dimension of the state vector
C        MMAX:   Upper limit of the dimension of each component
C        NC:     Number of components
C        MJ:     Adjustable dimension of XF, VF
C        NDIM:   Adjustable dimension of XFS, XPS, VFS, VPS
C                = 0   XF, XP, VF, VP are not stored
C                > 0   They are stored for smoothing
C        OUTMIN: Lower limit for detecting outliers
C        OUTMAX: Upper limit for detecting outliers
C        SIG2:   Variance of the observational noise
C     Outputs:
C        VFS:    Covariance matrices of the filter
C        VPS:    Covariance matrices of the predictor
C        XFS:    Mean vectors of the filter
C        XPS:    Mean vectors of the predictor
C        FF:     Log likelihood
C        OVAR:   Estimated variance
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(NMAX), REG(NMAX,*)
cxx      DIMENSION  Y(NDIM), REG(NMAX,7)
cxx      DIMENSION  A(MMAX,NCM), B(MMAX,NCM), C(MMAX,NCM), M(NCM)
cc      DIMENSION  XF(MJ), VF(MJ,MJ), XP(40), VP(40,40)
cc      DIMENSION  XFS(MJ,N), XPS(MJ,N), Q(NCM,NCM)
cc      DIMENSION  VFS(MJ,MJ,NDIM), VPS(MJ,MJ,NDIM)
cc      DIMENSION  WRK(40,40), VH(40), GAIN(40)
cc      DIMENSION  I0(10), MTYPE(NCM)
cxx      DIMENSION  XF(MJ), VF(MJ,MJ), XP(MJ), VP(MJ,MJ)
cxx      DIMENSION  XFS(MJ,NMAX), XPS(MJ,NMAX), Q(NCM,NCM)
cxx      DIMENSION  VFS(MJ,MJ,NMAX), VPS(MJ,MJ,NMAX)
cxx      DIMENSION  WRK(MJ,MJ), VH(MJ), GAIN(MJ)
cxx      DIMENSION  I0(NCM), MTYPE(NCM)
C
      INTEGER NMAX, MMAX, NCM, NC, NS, N, NPE, MJ, NDIM, MTYPE(NCM),
     1        M(NCM), IER
      DOUBLE PRECISION Y(NDIM), XF(MJ), VF(MJ,MJ), A(MMAX,NCM),
     1                 B(MMAX,NCM), C(MMAX,NCM), Q(NCM,NCM), SIG2,
     2                 REG(NMAX,7), OUTMIN, OUTMAX, VFS(MJ,MJ,NMAX),
     3                 VPS(MJ,MJ,NMAX), XFS(MJ,NMAX), XPS(MJ,NMAX), FF,
     4                 OVAR
c local
      INTEGER I, II, I0(NCM), J, L, MM, NSUM
      DOUBLE PRECISION XP(MJ), VP(MJ,MJ), WRK(MJ,MJ), VH(MJ), GAIN(MJ),
     1                 PI, SDET, SUM, PVAR, PERR
C
      DATA   PI  /3.1415926535D0/
C
      IER = 0
      OVAR = 0.0D0
      SDET = 0.0D0
      NSUM = 0
      I0(1) = 0
      DO 10 I=2,NC
cxx   10 I0(I) = I0(I-1) + M(I-1)
      I0(I) = I0(I-1) + M(I-1)
   10 CONTINUE
      MM = I0(NC) + M(NC)
C
      DO 300  II=NS,NPE
C
C  ...  ONE STEP AHEAD PREDICTION  ...
C
cxx      DO 100  L=1,NC
      DO 101  L=1,NC
      XP(I0(L)+M(L)) = A(M(L),L)*XF(I0(L)+1)
      DO 100  I=1,M(L)-1
cxx  100 XP(I0(L)+I) = A(I,L)*XF(I0(L)+1) + XF(I0(L)+I+1)
      XP(I0(L)+I) = A(I,L)*XF(I0(L)+1) + XF(I0(L)+I+1)
  100 CONTINUE
  101 CONTINUE
C
cxx      DO 110  J=1,MM
cxx      DO 110  L=1,NC
      DO 112  J=1,MM
      DO 111  L=1,NC
      WRK(I0(L)+M(L),J) = A(M(L),L)*VF(I0(L)+1,J)
      DO 110  I=1,M(L)-1
cxx  110 WRK(I0(L)+I,J) = A(I,L)*VF(I0(L)+1,J) + VF(I0(L)+I+1,J)
      WRK(I0(L)+I,J) = A(I,L)*VF(I0(L)+1,J) + VF(I0(L)+I+1,J)
  110 CONTINUE
  111 CONTINUE
  112 CONTINUE
C
cxx      DO 120  I=1,MM
cxx      DO 120  L=1,NC
      DO 122  I=1,MM
      DO 121  L=1,NC
      VP(I,I0(L)+M(L)) = WRK(I,I0(L)+1)*A(M(L),L)
      DO 120  J=1,M(L)-1
cxx  120 VP(I,I0(L)+J) = WRK(I,I0(L)+1)*A(J,L) + WRK(I,I0(L)+J+1)
      VP(I,I0(L)+J) = WRK(I,I0(L)+1)*A(J,L) + WRK(I,I0(L)+J+1)
  120 CONTINUE
  121 CONTINUE
  122 CONTINUE
C
cxx      DO 130  J=1,NC
cxx      DO 130  L=1,NC
      DO 132  J=1,NC
      DO 131  L=1,NC
      DO 130  I=1,M(L)
cxx  130 WRK(I0(L)+I,J) = B(I,L)*Q(L,J)
      WRK(I0(L)+I,J) = B(I,L)*Q(L,J)
  130 CONTINUE
  131 CONTINUE
  132 CONTINUE
C
cxx      DO 140  I=1,MM
cxx      DO 140  L=1,NC
      DO 142  I=1,MM
      DO 141  L=1,NC
      DO 140  J=1,M(L)
cxx  140 VP(I,I0(L)+J) = VP(I,I0(L)+J) + WRK(I,L)*B(J,L)
      VP(I,I0(L)+J) = VP(I,I0(L)+J) + WRK(I,L)*B(J,L)

  140 CONTINUE
  141 CONTINUE
  142 CONTINUE
C
C  ...  FILTERING  ...
C
cc      IF( Y(II).GT.OUTMIN .AND. Y(II).LT.OUTMAX .AND. II.LE.N )  THEN
      IF( II.LE.N .AND. (Y(II).GT.OUTMIN .AND. Y(II).LT.OUTMAX) )  THEN
C
      DO 210  I=1,MM
      SUM = 0.0D0
      DO 205  L=1,NC
      IF( MTYPE(L).EQ.0 )  THEN
         DO 200  J=1,M(L)
cxx  200    SUM = SUM + VP(I,I0(L)+J)*C(J,L)
         SUM = SUM + VP(I,I0(L)+J)*C(J,L)
  200    CONTINUE
      ELSE
         SUM = SUM + VP(I,I0(L)+1)*REG(II,MTYPE(L))
      END IF
  205 CONTINUE
cxx  210 VH(I) = SUM
      VH(I) = SUM
  210 CONTINUE
C
      PVAR = SIG2
      DO 225 L=1,NC
      IF( MTYPE(L).EQ.0 )  THEN
         DO 220 J=1,M(L)
cxx  220    PVAR = PVAR + VH(I0(L)+J)*C(J,L)
         PVAR = PVAR + VH(I0(L)+J)*C(J,L)
  220    CONTINUE
      ELSE
         PVAR = PVAR + VH(I0(L)+1)*REG(II,MTYPE(L))
      END IF
  225 CONTINUE
c------------------   2013/07/01
cdel      if( PVAR.LE.0 ) GO TO 400
c------------------
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      IF( PVAR.LE.1.0D-30 )  THEN
         FF = -1.0D20
         RETURN
      END IF
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      DO 230  I=1,MM
cxx  230 GAIN(I) = VH(I)/PVAR
      GAIN(I) = VH(I)/PVAR
  230 CONTINUE
C
      PERR = Y(II)
      DO 245 L=1,NC
      IF( MTYPE(L).EQ.0 )  THEN
         DO 240 J=1,M(L)
cxx  240    PERR = PERR - C(J,L)*XP(I0(L)+J)
         PERR = PERR - C(J,L)*XP(I0(L)+J)
  240    CONTINUE
      ELSE
         PERR = PERR - REG(II,MTYPE(L))*XP(I0(L)+1)
      END IF
  245 CONTINUE
C
      DO 250  I=1,MM
cxx  250 XF(I) = XP(I) + GAIN(I)*PERR
      XF(I) = XP(I) + GAIN(I)*PERR
  250 CONTINUE
C
cxx      DO 260  J=1,MM
      DO 261  J=1,MM
      DO 260  I=1,MM
cxx  260 VF(I,J) = VP(I,J) - GAIN(I)*VH(J)
      VF(I,J) = VP(I,J) - GAIN(I)*VH(J)
  260 CONTINUE
  261 CONTINUE
C
      OVAR = OVAR + PERR**2/PVAR
      SDET = SDET + DLOG(PVAR)
      NSUM = NSUM + 1
C
C  ...  MISSING OBSERVATION  ...
C
      ELSE
cxx      DO 270  I=1,MM
      DO 271  I=1,MM
      XF(I) = XP(I)
      DO 270  J=1,MM
cxx  270 VF(I,J) = VP(I,J)
      VF(I,J) = VP(I,J)
  270 CONTINUE
  271 CONTINUE
      END IF
C
C  ...  SAVE MEAN AND COVARIANCE  ...
C
      IF( NDIM.GT.1 )  THEN
cxx      DO 280  I=1,MM
      DO 281  I=1,MM
      XPS(I,II) = XP(I)
      XFS(I,II) = XF(I)
      DO 280  J=1,MM
      VPS(I,J,II) = VP(I,J)
cxx  280 VFS(I,J,II) = VF(I,J)
      VFS(I,J,II) = VF(I,J)
  280 CONTINUE
  281 CONTINUE
      END IF
C
  300 CONTINUE
      OVAR = OVAR/NSUM
      FF = -0.5D0*(NSUM*DLOG(PI*2*OVAR) + SDET + NSUM)
C     WRITE(6,*)  'FF =',FF, OVAR, SDET, NSUM
C
      RETURN
c------------------   2013/07/01
cdel  400 FF = -1.0D30
cdel      IER = 400
cdel      RETURN
c------------------
      E N D
      SUBROUTINE  ISTAT1( NC,M,MJ,MAXM,A,XMEAN,XVAR,XF,VF )
C
C  ...  Initial state ...
C
C     Inputs:
C        NC:    Number of components
C        M:     Dimension of the state vector
C        MJ:    Adjustable dimension of F
C        MAXM:  Adjustable dimension of A
C        A:     Parameter of matrix F
C        XMEAN: Mean value
C        XVAR:  Varaince
C     Outputs:
C         XF:   State vector, X(0|0)
C         VF:   State covarance matrix, V(0|0)
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  M(NC), A(MAXM,NC), XMEAN(NC), XVAR(MAXM,NC)
cc      DIMENSION  XF(MJ), VF(MJ,MJ), I0(10)
cxx      DIMENSION  XF(MJ), VF(MJ,MJ), I0(NC)
C
      INTEGER NC, M(NC), MJ, MAXM
      DOUBLE PRECISION A(MAXM,NC), XMEAN(NC), XVAR(MAXM,NC), XF(MJ),
     1                 VF(MJ,MJ)
c local
      INTEGER I, I1, I0(NC), J, J1, L
      DOUBLE PRECISION SUM
C
      I0(1) = 0
      DO 5 I=2,NC
cxx    5 I0(I) = I0(I-1) + M(I-1)
      I0(I) = I0(I-1) + M(I-1)
    5 CONTINUE
cxx      DO 10  I=1,MJ
cxx      DO 10  J=1,MJ
cxx   10 VF(I,J) = 0.0D0
      VF(1:MJ,1:MJ) = 0.0D0
C
      DO 70  L=1,NC
C
      XF(I0(L)+1) = XMEAN(L)
      SUM = 0.0D0
      DO 20  I=M(L),2,-1
      SUM = SUM + A(I,L)
cxx   20 XF(I0(L)+I) = SUM*XMEAN(L)
      XF(I0(L)+I) = SUM*XMEAN(L)
   20 CONTINUE
C
      VF(I0(L)+1,I0(L)+1) = XVAR(1,L)
      DO 40 I=2,M(L)
      SUM = 0.0D0
      DO 30 J=I,M(L)
cxx   30 SUM = SUM + A(J,L)*XVAR(J,L)
      SUM = SUM + A(J,L)*XVAR(J,L)
   30 CONTINUE
      VF(I0(L)+1,I0(L)+I) = SUM
cxx   40 VF(I0(L)+I,I0(L)+1) = SUM
      VF(I0(L)+I,I0(L)+1) = SUM
   40 CONTINUE
cxx      DO 60 I=2,M(L)
      DO 61 I=2,M(L)
      DO 60 J=I,M(L)
      SUM = 0.0D0
cxx      DO 50 I1=I,M(L)
      DO 51 I1=I,M(L)
      DO 50 J1=J,M(L)
cxx   50 SUM = SUM + A(I1,L)*A(J1,L)*XVAR(IABS(J1-J-I1+I)+1,L)
      SUM = SUM + A(I1,L)*A(J1,L)*XVAR(IABS(J1-J-I1+I)+1,L)
   50 CONTINUE
   51 CONTINUE
      VF(I0(L)+I,I0(L)+J) = SUM
cxx   60 VF(I0(L)+J,I0(L)+I) = SUM
      VF(I0(L)+J,I0(L)+I) = SUM
   60 CONTINUE
   61 CONTINUE
   70 CONTINUE
C
      RETURN
      E N D
cc      SUBROUTINE  SETABC( M,L,NC,MTYPE,TAU2,MAXM,MM,AR,Y,N,A,B,C,Q,
cc     *                    XMEAN,XVAR )
      SUBROUTINE  SETABC1( M1,M2,M3,M4,MPER,M,L,NC,MTYPE,TAU2,MAXM,
     *                                MM,AR,Y,N,A,B,C,Q,XMEAN,XVAR,ier )
C
C  ...  State space model for Seasonal Adjustment  ...
C
C     Input:
C       M:     Order of the trend
C       L:     Number of components
C       NC:    Adjustable dimension
C       MTYPE: Model type
C       TAU2:  System noise variance
C       MAXM:  Adjustable dimension of A, B, C
C       MM:    State dimension
C       AR:    AR coefficient
C       Y(I):  Time series
C       N:     Data length
C     Outputs:
C       A,B,C: Parameters of F, G, H
C       Q:     System noise covarance matrix
C       XMEAN: Initial value of each component
C       XVAR:  Initial variance of each component
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  A(MAXM,NC), B(MAXM,NC), C(MAXM,NC), Q(NC,NC)
cc      DIMENSION  M(NC), TAU2(NC), AR(NC), MTYPE(NC)
cxx      DIMENSION  M(NC), TAU2(4), AR(M3), MTYPE(NC)
cc      DIMENSION  Y(N), XMEAN(NC), XVAR(MAXM,NC), DUM(1), COV(0:10)
cxx      DIMENSION  Y(N), XMEAN(NC), XVAR(MAXM,NC), DUM(1), COV(0:M3)
cc      COMMON  /C92827/  M1, M2, M3, M4, MPER
C
      INTEGER M1, M2, M3, M4, MPER, NC, L, MAXM, MM, N, M(NC),
     1        MTYPE(NC), ier
      DOUBLE PRECISION TAU2(4), AR(M3), Y(N), A(MAXM,NC), B(MAXM,NC),
     1                 C(MAXM,NC), Q(NC,NC), XMEAN(NC), XVAR(MAXM,NC)
c local
      INTEGER I, MSUM
      DOUBLE PRECISION DUM(1), COV(0:M3), YMEAN, YVAR
C
      ier = 0
      L = 0
      CALL  MOMENT( Y,N/4,YMEAN,YVAR )
      YVAR = 1.0D4
C
cxx      DO 10  J=1,NC
cxx      DO 10  I=1,MAXM
cxx      XVAR(I,J) = 0.0D0
cxx      C(I,J) = 0.0D0
cxx      A(I,J) = 0.0D0
cxx   10 B(I,J) = 0.0D0
cxx      DO 20  I=1,NC
cxx      DO 20  J=1,NC
cxx   20 Q(I,J) = 0.0D0
      XVAR(1:MAXM,1:NC) = 0.0D0
      C(1:MAXM,1:NC) = 0.0D0
      A(1:MAXM,1:NC) = 0.0D0
      B(1:MAXM,1:NC) = 0.0D0
      Q(1:NC,1:NC) = 0.0D0
C
C  ...  TREND MODEL  ...
C
      IF( M1.GT.0 )  THEN
        L = 1
        M(L) = M1
        MTYPE(L) = 0
        IF( M1.EQ.1 )  THEN
          A(1,1) = 1.0D0
        END IF
        IF( M1.EQ.2 )  THEN
          A(1,1) =  2.0D0
          A(2,1) = -1.0D0
        END IF
        IF( M1.EQ.3 )  THEN
          A(1,1) =  3.0D0
          A(2,1) = -3.0D0
          A(3,1) =  1.0D0
        END IF
        B(1,1) = 1.0D0
        C(1,1) = 1.0D0
        Q(1,1) = TAU2(1)
        XMEAN(1) = YMEAN
        XVAR(1,1) = YVAR
      END IF
C
C  ...  SEASONAL MODEL  ...
C
      IF( M2.GT.0 )  THEN
        L = L+1
        M(L) = M2*(MPER-1)
        MTYPE(L) = 0
        IF( M2.NE.2 )  THEN
          DO 30  I=1,M(L)
cxx   30     A(I,L) = -1.0D0
          A(I,L) = -1.0D0
   30     CONTINUE
        END IF
        IF( M2.EQ.2 )  THEN
          DO 40 I=1,MPER-1
          A(I,L) = -(I+1)
cxx   40     A(M(L)-I+1,L) = -I
          A(M(L)-I+1,L) = -I
   40     CONTINUE
        END IF
        B(1,L) =  1.0D0
        C(1,L) =  1.0D0
        Q(L,L) =  TAU2(L)
        XMEAN(L) = 0.0D0
        XVAR(1,L) = YVAR
      END IF
C
C  ...  AR MODEL  ...
C
      IF( M3.GT.0 )  THEN
        L = L+1
        M(L) = M3
        MTYPE(L) = 0
        DO 50  I=1,M3
cxx   50   A(I,L) = AR(I)
        A(I,L) = AR(I)
   50   CONTINUE
        B(1,L) = 1.0D0
        C(1,L) = 1.0D0
        Q(L,L) = TAU2(L)
cc        CALL  ARMCOV( M3,0,AR,DUM,1.0D0,M3,COV )
        CALL  ARMCOV( M3,0,AR,DUM,1.0D0,M3,COV,M3,IER )
        if( ier.ne.0 ) return
        XMEAN(L) = 0.0D0
        DO 60 I=1,M3
cxx   60   XVAR(I,L) = COV(I-1)
        XVAR(I,L) = COV(I-1)
   60   CONTINUE
      END IF
C
C  ...  TRADING DAY EFFECT MODEL  ...
C
      IF( M4.GT.0 )  THEN
      DO 70  I=L+1,L+6
      M(I) = 1
      MTYPE(I) = I-L
      XMEAN(I) = 0.0D0
      XVAR(1,I) = YVAR
      A(1,I) = 1.0D0
      B(1,I) = 1.0D0
      C(1,I) = 1.0D0
cxx   70 Q(I,I) = 0.0D0
      Q(I,I) = 0.0D0
   70 CONTINUE
      L = L + 6
      END IF
C
C  ...  STATE DIMENSION  ...
C
      MSUM = 0
      DO 80  I=1,L
cxx   80 MSUM = MSUM + M(I)
      MSUM = MSUM + M(I)
   80 CONTINUE
      MM = MSUM
C
      RETURN
      E N D
cc      SUBROUTINE  FFSEAS( K,AA,FF,IFG )
      SUBROUTINE  FFSEAS( Y,N,M1,M2,M3,M4,NPER,REG,K,AA,
     * OUTMIN,OUTMAX,ALIMIT,M,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,
     * FF,IFG,ier )
C
C  ... Function for Maximum likelihood estimation (seasonal model) ...
C
C     Inputs:
C        K:      Number of parameters
C        AA(I):  Parameter vector
C     Outputs:
C        FF:     -(Log likelihood)
C        IFG:    =1 if some conditions are violated
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  M(10), TAU2(10), PAR(10)
cc      DIMENSION  XMEAN(10), XVAR(22,10), AA(20), AR(10)
cc      DIMENSION  A(22,9), B(22,9), C(22,9), Q(9,9)
cc      DIMENSION  XPS(25,300), XFS(25,300)
cc      DIMENSION  VPS(25,25,300), VFS(25,25,300)
cc      DIMENSION  XF(40), VF(40,40), MTYPE(10)
cc      DIMENSION  Y(NMAX), REG(N,7)
cxx      DIMENSION  M(NC), TAU2(4), PAR(M3)
cxx      DIMENSION  XMEAN(NC), XVAR(MAXM,NC), AA(K), AR(M3)
cxx      DIMENSION  A(MAXM,NC), B(MAXM,NC), C(MAXM,NC), Q(NC,NC)
cxx      DIMENSION  XPS(MJ,NMAX), XFS(MJ,NMAX)
cxx      DIMENSION  VPS(MJ,MJ,NMAX), VFS(MJ,MJ,NMAX)
cxx      DIMENSION  XF(MJ), VF(MJ,MJ), MTYPE(NC)
cxx      DIMENSION  Y(N), REG(NMAX,7)
C
      INTEGER N, M1, M2, M3, M4, NPER, K, NS, NFE, NPE, NMAX, MJ, MAXM,
     1        NC, IFG, ier, M(NC)
      DOUBLE PRECISION Y(N), REG(NMAX,7), AA(K), OUTMIN, OUTMAX, ALIMIT,
     1                 XMEAN(NC), XVAR(MAXM,NC), FF
c local
      INTEGER I, IFPLIM, L, MM, MTYPE(NC), NP, ID
      DOUBLE PRECISION TAU2(4), PAR(M3), AR(M3), A(MAXM,NC), B(MAXM,NC),
     1                 C(MAXM,NC), Q(NC,NC), XPS(MJ,NMAX), XFS(MJ,NMAX),
     2                 VPS(MJ,MJ,NMAX), VFS(MJ,MJ,NMAX), XF(MJ),
     3                 VF(MJ,MJ), SIG2, OVAR
C
cc      COMMON  /C92827/  M1, M2, M3, M4, NPER
cc      COMMON  /C92826/   Y(160), REG(160,7)
cc      COMMON  /C92825/   OUTMIN, OUTMAX
cc      COMMON  /C92907/  ALIMIT
cc      COMMON  /CMMODL/  M, XMEAN, XVAR, N, NS, NFE, NPE
cc      NMAX= 300
cc      MJ  = 25
cc      MAXM= 22
cc      NC  = 9
c
C avoid floating-point exceptions
cc      IFPLIM = 709
      IFPLIM = 87
c
      NP  = ID(M1) + ID(M2) + ID(M3)
      IFG = 0
      ier = 0
C
      DO 10 I=1,K
         IF( DABS(AA(I)).GT.IFPLIM ) IER = 400
cxx   10 IF( DABS(AA(I)).GT.20.0D0 )  GO TO 100
cxx      IF( DABS(AA(I)).GT.20.0D0 )  GO TO 100
      IF( DABS(AA(I)).GT.30.0D0 )  GO TO 100
   10 CONTINUE
      DO 20 I=1,NP
cxx   20 TAU2(I) = DEXP( AA(I) )/(1.0D0 + DEXP( AA(I) ))
      TAU2(I) = DEXP( AA(I) )/(1.0D0 + DEXP( AA(I) ))
   20 CONTINUE
      DO 30 I=1,M3
cxx   30 PAR(I)  = ALIMIT*(DEXP(AA(NP+I))-1.0D0)/(DEXP(AA(NP+I))+1.0D0)
      PAR(I)  = ALIMIT*(DEXP(AA(NP+I))-1.0D0)/(DEXP(AA(NP+I))+1.0D0)
   30 CONTINUE
      CALL  ARCOEF( PAR,M3,AR )
      SIG2 = 1.0D0
      IFG = 0
C
cc      CALL  SETABC( M,L,NC,MTYPE,TAU2,MAXM,MM,AR,Y,N,A,B,C,Q,
cc     *              XMEAN,XVAR )
      CALL  SETABC1( M1,M2,M3,M4,NPER,M,L,NC,MTYPE,TAU2,MAXM,MM,AR,
     *                     Y,N,A,B,C,Q,XMEAN,XVAR,ier )
      if( ier.ne.0 ) return
      CALL  ISTAT1( L,M,MJ,MAXM,A,XMEAN,XVAR,XF,VF )
cc      CALL  FILTR1( Y,XF,VF,A,B,C,Q,SIG2,REG,MTYPE,NMAX,M,MAXM,NC,L,
      CALL  FILTR1( Y,XF,VF,A,B,C,Q,SIG2,REG,MTYPE,NMAX,M,MAXM,NC,L,
     *      NS,NFE,NPE,MJ,N,OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,OVAR,ier )
cc      WRITE(6,*) FF
      FF = -FF
      RETURN
C
  100 IFG = 1
      FF = 1.0D20
      RETURN
      E N D
      SUBROUTINE  TRADE( JYEAR,MONTH,N,MJ,TDAY )
C
C  ...  This subroutine computes the number of days of the week
C       in each month,
C
C     Inputs:
C        JYEAR:  Starting year of the data set
C        MONTH:  Starting month of the data set
C        N:      Data length
C        MJ:     Adjustable dimension of TDAY
C     Output:
C        TDAY(I,J):  Number of J-th days of the week in I-th month
C     Written by F.K.  Nov.1981
C
cxx      REAL*8  TDAY
cxx      DIMENSION  TDAY(MJ,7), IX(12)
C
      INTEGER JYEAR, MONTH, N, MJ
      DOUBLE PRECISION TDAY(MJ,7)
c local
      INTEGER I, II, I0, I1, I2, IE, J, JJ, JS, IX(12)
C
      DATA   IX  /3,0,3,2,3,2,3,3,2,3,2,3/
C
      JS = JYEAR - 1900
      I0 = MOD( JS+(JS-1)/4,7 ) + 1
         I2 = 0
      JJ = 2 - MONTH
      II = 0
    5 II = II + 1
      I1 = II + JS - 1
      IX(2) = 0
      IF( MOD(I1,4) .EQ. 0 )  IX(2) = 1
      IF( MOD(I1,100).EQ.0 )  IX(2) = 0
      IF( MOD(I1,400).EQ.0 )  IX(2) = 1
cxx      DO 30  J=1,12
      DO 31  J=1,12
      DO 10  I=1,7
cc   10 IF( JJ.GT.0 )  TDAY(JJ,I) = 4
cxx   10 IF( JJ.GT.0 .AND. JJ.LE.MJ )  TDAY(JJ,I) = 4
      IF( JJ.GT.0 .AND. JJ.LE.MJ )  TDAY(JJ,I) = 4
   10 CONTINUE
C
      IE = IX(J)
      IF( IE .EQ. 0 )  GO TO 30
      DO 20  I=1,IE
      I2 = I0 + I
      IF( I2 .GT. 7 ) I2 = I2 - 7
cc   20 IF( JJ.GT.0 )  TDAY(JJ,I2) = 5
cxx   20 IF( JJ.GT.0 .AND. JJ.LE.MJ )  TDAY(JJ,I2) = 5
      IF( JJ.GT.0 .AND. JJ.LE.MJ )  TDAY(JJ,I2) = 5
   20 CONTINUE
      I0 = I2
   30 JJ = JJ + 1
   31 CONTINUE
      IF( JJ .GT.N ) RETURN
      GO TO 5
C
      E N D
cc      SUBROUTINE  DAVIDN( FUNCT,X,N,NDIF )
      SUBROUTINE  DAVIDN1( FUNCT,X,N,NDIF,
     * YY,NN,M1,M2,M3,M4,NPER,REG,OUTMIN,OUTMAX,ALIMIT,ISW,IDIF,
     * M,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,IG,ier )
C
C  ...  6/20/83, 12/19/92
C
cxx      IMPLICIT  REAL*8( A-H,O-Z )
cc      DIMENSION  X(40), DX(40), G(40), G0(40), Y(40)
cc      DIMENSION  H(40,40), WRK(40), S(40)
cxx      DIMENSION  X(N), DX(N), G(N), G0(N), Y(N)
cxx      DIMENSION  H(N,N), WRK(N), S(N)
cxx      DIMENSION  YY(NN), REG(NMAX,7)
cxx      DIMENSION  M(NC), XMEAN(NC), XVAR(MAXM,NC)
C
      INTEGER N, NDIF, NN, M1, M2, M3, M4, NPER, ISW, IDIF, NS, NFE,
     1        NPE, NMAX, MJ, MAXM, NC, M(NC), IG, ier
      DOUBLE PRECISION X(N), YY(NN), REG(NMAX,7), OUTMIN, OUTMAX,
     1                 ALIMIT, XMEAN(NC), XVAR(MAXM,NC)
c local
      INTEGER I, ICC, IPR, ICOUNT, J
      DOUBLE PRECISION DX(N), G(N), G0(N), Y(N), H(N,N), WRK(N), S(N),
     1                 TAU2, EPS1, EPS2, RAMDA, CONST1, XM, SUM, S1, S2,
     2                 SS, STEM, ED, XMB
C
cc      COMMON     / CCC /  ISW, IPR, ISMT, IDIF
cc      COMMON     / DDD /  XM , AIC , SD
      EXTERNAL  FUNCT
      DATA        TAU2  /          1.0D-6  /
      DATA  EPS1, EPS2  / 1.0D-6 , 1.0D-6  /
      RAMDA  = 0.5D0
      CONST1 = 1.0D-30
      IPR = 0
c-----
      IDIF = NDIF
c-----
C
C          INITIAL ESTIMATE OF INVERSE OF HESSIAN
C
      ICOUNT = 0
cxx 1000 DO 20  I=1,N
cxx      DO 10  J=1,N
cxx   10 H(I,J) = 0.0D0
cxx      S(I)   = 0.0D0
cxx      DX(I)  = 0.0D0
cxx   20 H(I,I) = 1.0D0
 1000 CONTINUE
      H(1:N,1:N) = 0.0D0
      S(1:N)   = 0.0D0
      DX(1:N)  = 0.0D0
      DO 20  I=1,N
      H(I,I) = 1.0D0
   20 CONTINUE

      ISW = 0
C
cc      IF( NDIF.EQ.0 )  CALL  FUNCT( N,X,XM,G,IG )
cc      IF( NDIF.GE.1 )  CALL  FUNCND( FUNCT,N,X,XM,G,IG )
      IF( NDIF.EQ.0 )  CALL  FUNCT( YY,NN,M1,M2,M3,M4,NPER,REG,N,X,
     * OUTMIN,OUTMAX,ALIMIT,M,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,
     * MAXM,NC,XM,IG,ier )
      if( ier.ne.0 ) return
      IF( NDIF.GE.1 )  CALL  FUNCND1( FUNCT,N,X,XM,G,IG,IDIF,
     * YY,NN,M1,M2,M3,M4,NPER,REG,OUTMIN,OUTMAX,ALIMIT,M,XMEAN,XVAR,
     * NS,NFE,NPE,ISW,NMAX,MJ,MAXM,NC,ier )
      if( ier.ne.0 ) return
C
cc      IF( IPR .GE. 2 )   WRITE( 6,640 )   XM, SD, AIC
cc      WRITE(6,650) (X(I),I=1,N), XM, (G(I),I=1,N)
C
      ICC = 0
C      ITERATION
 2000 CONTINUE
      ICC = ICC + 1
      IF( ICC .EQ. 1 )  GO TO 120
C
      DO 40  I=1,N
cxx   40 Y(I) = G(I) - G0(I)
      Y(I) = G(I) - G0(I)
   40 CONTINUE
      DO 60  I=1,N
      SUM = 0.0D0
      DO 50  J=1,N
cxx   50 SUM = SUM + Y(J)*H(I,J)
      SUM = SUM + Y(J)*H(I,J)
   50 CONTINUE
cxx   60 WRK(I) = SUM
      WRK(I) = SUM
   60 CONTINUE
      S1 = 0.0D0
      S2 = 0.0D0
      DO 70  I=1,N
      S1 = S1 + WRK(I)*Y(I)
cxx   70 S2 = S2 + DX(I) *Y(I)
      S2 = S2 + DX(I) *Y(I)
   70 CONTINUE
      IF( S1.LE.CONST1 .OR. S2.LE.CONST1 )  GO TO 900
C     IF( S1 .LE. S2 )   GO TO 100
C
C          UPDATE THE INVERSE OF HESSIAN MATRIX
C
C               ---  BROYDEN-FLETCHER-GOLDFARB-SHANNO TYPE CORRECTION  -
C
cxx  100 CONTINUE
      STEM = S1 / S2 + 1.0D0
cxx      DO 110  I=1,N
      DO 111  I=1,N
      DO 110  J=I,N
      H(I,J) = H(I,J)-(DX(I)*WRK(J)+WRK(I)*DX(J)-DX(I)*DX(J)*STEM)/S2
cxx  110 H(J,I) = H(I,J)
      H(J,I) = H(I,J)
  110 CONTINUE
  111 CONTINUE
C
  120 CONTINUE
      SS = 0.0D0
      DO 140  I=1,N
      SUM = 0.0D0
      DO 130  J=1,N
cxx  130 SUM = SUM + H(I,J)*G(J)
      SUM = SUM + H(I,J)*G(J)
  130 CONTINUE
      SS  = SS + SUM**2
cxx  140 S(I) = -SUM
      S(I) = -SUM
  140 CONTINUE
C
      S1 = 0.0D0
      S2 = 0.0D0
      DO 150  I=1,N
      S1 = S1 + S(I)*G(I)
cxx  150 S2 = S2 + G(I)*G(I)
      S2 = S2 + G(I)*G(I)
  150 CONTINUE
C     DS2 = DSQRT(S2)
C     GTEM = DABS(S1)/DS2
C     IF( GTEM.LE.TAU1 .AND. DS2.LE.TAU2 )  GO TO 900
      IF( S1.LT.0.0D0 )  GO TO 200
      DO 170  I=1,N
      DO 160  J=1,N
cxx  160 H(I,J) = 0.0D0
      H(I,J) = 0.0D0
  160 CONTINUE
      H(I,I) = 1.0D0
cxx  170 S(I) = -S(I)
      S(I) = -S(I)
  170 CONTINUE
  200 CONTINUE
C
      ED = XM
C
C          LINEAR  SEARCH
C
cc      CALL  LINEAR( FUNCT,X,S,RAMDA,ED,N,IG )
      CALL  LINEAR1( FUNCT,X,S,RAMDA,ED,N,IG,
     * YY,NN,M1,M2,M3,M4,NPER,REG,OUTMIN,OUTMAX,ALIMIT,M,XMEAN,XVAR,
     * NS,NFE,NPE,ISW,NMAX,MJ,MAXM,NC,ier )
      if( ier.ne.0 ) return
C
cc      IF( IPR .GE. 2 )  WRITE( 6,630 )  RAMDA, ED, SD, AIC
C
      S1 = 0.0D0
      DO 210  I=1,N
      DX(I) = S(I)*RAMDA
      S1 = S1 + DX(I)*DX(I)
      G0(I) = G(I)
cxx  210 X(I) = X(I) + DX(I)
      X(I) = X(I) + DX(I)
  210 CONTINUE
      XMB = XM
      ISW = 0
C
cc      IF( NDIF.EQ.0 )  CALL  FUNCT( N,X,XM,G,IG )
cc      IF( NDIF.GE.1 )  CALL  FUNCND( FUNCT,N,X,XM,G,IG )
      IF( NDIF.EQ.0 )  CALL  FUNCT( YY,NN,M1,M2,M3,M4,NPER,REG,N,X, 
     * OUTMIN,OUTMAX,ALIMIT,M,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,
     * MAXM,NC,XM,IG,ier )
      if( ier.ne.0 ) return
      IF( NDIF.GE.1 )  CALL  FUNCND1( FUNCT,N,X,XM,G,IG,IDIF,
     * YY,NN,M1,M2,M3,M4,NPER,REG,OUTMIN,OUTMAX,ALIMIT,M,XMEAN,XVAR,
     * NS,NFE,NPE,ISW,NMAX,MJ,MAXM,NC,ier )
      if( ier.ne.0 ) return
cc      WRITE(6,650) (X(I),I=1,N), XM, (G(I),I=1,N)
C
      S2 = 0.D0
      DO 220  I=1,N
cxx  220 S2 = S2 + G(I)*G(I)
      S2 = S2 + G(I)*G(I)
  220 CONTINUE
      IF( DSQRT(S2) .LT. TAU2 )  GO TO 900
      IF( XMB/XM-1.D0.LT.EPS1 .AND. DSQRT(S1).LT.EPS2 )  GO TO 900
C     IF( ICC .GE. 5 )  GO TO 900
      GO TO 2000
  900 CONTINUE
      IF( IPR .LE. 0 )  RETURN
cc      WRITE( 6,600 )
cc      WRITE( 6,610 )  (X(I),I=1,N)
cc      WRITE( 6,620 )
cc      WRITE( 6,610 )  (G(I),I=1,N)
C
      ICOUNT  = ICOUNT + 1
      S2 = 0.D0
      DO 910  I=1,N
cxx  910 S2 = S2 + G(I)*G(I)
      S2 = S2 + G(I)*G(I)
  910 CONTINUE
      IF( S2.GT.1.0D0.AND.ICOUNT.LE.1 )   GO TO 1000
      RETURN
cxx  600 FORMAT( 1H0,'-----  X  -----' )
cxx  610 FORMAT( 1H ,10D13.5 )
cxx  620 FORMAT( 1H0,'***  GRADIENT  ***' )
cxx  630 FORMAT( 1H ,'LAMBDA =',D15.7,3X,'(-1)LOG LIKELIHOOD =',D23.15,
cxx     *        3X,'SD =',D22.15,5X,'AIC =',D23.15 )
cxx  640 FORMAT( 1H ,26X,'(-1)LOG-LIKELIHOOD =',D23.15,3X,'SD =',D22.15,
cxx     *        5X,'AIC =',D23.15 )
cxx  650 FORMAT( 10X,5F12.7 )
      E N D
cc      SUBROUTINE  FUNCND( FUNCT,M,A,F,G,IFG )
      SUBROUTINE  FUNCND1( FUNCT,M,A,F,G,IFG,IDIF,Y,N,M1,M2,M3,M4,
     * NPER,REG,OUTMIN,OUTMAX,ALIMIT,MM,XMEAN,XVAR,NS,NFE,NPE,ISW,
     * NMAX,MJ,MAXM,NC,ier )
C
C  ...  FUNCTION EVALUATION AND NUMERICAL DIFFERENCING  ...
C
cxx      IMPLICIT   REAL*8( A-H,O-Z )
cc      DIMENSION  A(M) , G(M) , B(20),GD(5)
cxx      DIMENSION  A(M) , G(M) , B(M)
cxx      DIMENSION  Y(N), REG(NMAX,7), MM(NC), XMEAN(NC), XVAR(MAXM,NC)
C
      INTEGER M, IFG, IDIF, N, M1, M2, M3, M4, NPER, NS, NFE, NPE,
     1           ISW, NMAX, MJ, MAXM, NC, MM(NC), ier
      DOUBLE PRECISION A(M), F, G(M), Y(N), REG(NMAX,7), OUTMIN, OUTMAX,
     1                ALIMIT, XMEAN(NC), XVAR(MAXM,NC)
c local
      INTEGER I, II
      DOUBLE PRECISION B(M), CONST, FF, FB
C
cc      COMMON  / CCC /  ISW , IPR, ISMT, IDIF
cc      COMMON  /CMFUNC/ DJACOB,FC,SIG2,AIC,FI,SIG2I,AICI,GI(20),GC(20)
C     DATA       ICNT /0/
      CONST = 0.00001D0
C
cc      CALL  FUNCT( M,A,F,GD,IFG )
      CALL FUNCT( Y,N,M1,M2,M3,M4,NPER,REG,M,A,OUTMIN,OUTMAX,
     * ALIMIT,MM,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,F,IFG,ier )
      if( ier.ne.0 ) return

      FB = F
      IF( ISW .GE. 1 )   RETURN
C
C     WRITE( 6,600 )   (A(I),I=1,M)
      DO 10  I=1,M
cxx   10 B(I) = A(I)
      B(I) = A(I)
   10 CONTINUE
C
      DO 30  II=1,M
      B(II) = A(II) + CONST
cc      CALL  FUNCT( M,B,FF,GD,IFG )
      CALL FUNCT( Y,N,M1,M2,M3,M4,NPER,REG,M,B,OUTMIN,OUTMAX,
     * ALIMIT,MM,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,FF,IFG,ier )
      if( ier.ne.0 ) return
      IF( IDIF .EQ. 1 )  GO TO 20
      B(II) = A(II) - CONST
cc      CALL  FUNCT( M,B,FB,GD,IFG )
      CALL FUNCT( Y,N,M1,M2,M3,M4,NPER,REG,M,B,OUTMIN,OUTMAX,
     * ALIMIT,MM,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,FB,IFG,ier )
      if( ier.ne.0 ) return
   20 G(II) = (FF-FB)/(CONST*IDIF)
      IF( G(II) .GT. 1.0D20 )  G(II) = (F-FB)/CONST
      IF( G(II) .LT.-1.0D20 )  G(II) = (FF-F)/CONST
      IF( FB.GT.F .AND. FF.GT.F )  G(II) = 0.0D0
cxx   30 B(II) = A(II)
      B(II) = A(II)
   30 CONTINUE
C
      RETURN
C
cxx  600 FORMAT( 3X,'---  PARAMETER  ---',(/,3X,5D13.5) )
cxx  610 FORMAT( 3X,'---  GRADIENT  ---',(/,3X,5D13.5) )
      E N D
cc      SUBROUTINE  LINEAR( FUNCT,X,H,RAM,EE,K,IG )
      SUBROUTINE  LINEAR1( FUNCT,X,H,RAM,EE,K,IG, Y,N,M1,M2,M3,M4,
     * NPER,REG,OUTMIN,OUTMAX,ALIMIT,M,XMEAN,XVAR,NS,NFE,NPE,ISW,
     * NMAX,MJ,MAXM,NC,ier )
C
C  ...  LINEAR SEARCH  ...
C
cxx      IMPLICIT  REAL*8( A-H,O-Z )
cc      INTEGER  RETURN,SUB
cc      DIMENSION  X(1), H(1), X1(40)
cc      DIMENSION  G(40)
cc      COMMON     / CCC /  ISW , IPR, ISMT, IDIF
cxx      DIMENSION  X(K), H(K), X1(K)
cxx      DIMENSION  Y(N), REG(NMAX,7)
cxx      DIMENSION  M(NC), XMEAN(NC), XVAR(MAXM,NC)
C
      INTEGER K, IG, N, M1, M2, M3, M4, NPER, NS, NFE, NPE, ISW, NMAX,
     1        MJ, MAXM, NC, M(NC), ier
      DOUBLE PRECISION X(K), H(K), RAM, EE, Y(N), REG(NMAX,7), OUTMIN,
     1                 OUTMAX, ALIMIT, XMEAN(NC), XVAR(MAXM,NC)
c local
      INTEGER I, IFG, IRET, ISUB, ire510
      DOUBLE PRECISION X1(K), CONST2, HNORM, RAM1, RAM2, RAM3, E1, E2,
     1                 E3, A1, A2, A3, B1, B2
C
C     IPR = 7
C     C1 = 1.0D-7
C     C2 = 1.0D-30
C
      ISW = 1
      IG  = 0
      RAM = 0.1D0
      CONST2 = 1.0D-60
   14 DO 15 I=1,K
cxx   15 IF( DABS(H(I)) .GT. 1.0D10 )  GO TO 16
      IF( DABS(H(I)) .GT. 1.0D10 )  GO TO 16
   15 CONTINUE
      GO TO 18
   16 DO 17 I=1,K
cxx   17 H(I) = H(I)*1.0D-10
      H(I) = H(I)*1.0D-10
   17 CONTINUE
      GO TO 14
   18 CONTINUE
      HNORM = 0.D0
      DO 10  I=1,K
cxx   10 HNORM = HNORM + H(I)**2
      HNORM = HNORM + H(I)**2
   10 CONTINUE
      HNORM = DSQRT( HNORM )
C
      RAM2 = RAM
      E1 =EE
      RAM1 = 0.D0
C
      DO 20  I=1,K
cxx   20 X1(I) = X(I) + RAM2*H(I)
      X1(I) = X(I) + RAM2*H(I)
   20 CONTINUE
cc      CALL  FUNCT( K,X1,E2,G,IG )
      CALL  FUNCT( Y,N,M1,M2,M3,M4,NPER,REG,K,X1,OUTMIN,OUTMAX,
     * ALIMIT,M,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,E2,IG,ier )
      if( ier.ne.0 ) return
cc      IF(IPR.GE.7)  WRITE(6,2)  RAM2,E2
C
      IF( IG .EQ. 1  )  GO TO 50
      IF( E2 .GT. E1 )  GO TO 50
   30 RAM3 = RAM2*4.D0
      DO 40  I=1,K
cxx   40 X1(I) = X(I) + RAM3*H(I)
      X1(I) = X(I) + RAM3*H(I)
   40 CONTINUE
cc      CALL  FUNCT( K,X1,E3,G,IG )
      CALL  FUNCT( Y,N,M1,M2,M3,M4,NPER,REG,K,X1,OUTMIN,OUTMAX,
     * ALIMIT,M,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,E3,IG,ier )
      if( ier.ne.0 ) return
      IF( IG.EQ.1 )  GO TO  500
cc      IF( IPR.GE.7 )  WRITE(6,3)  RAM3,E3
      IF( E3 .GT. E2 )  GO TO 70
      IF(RAM3.GT.1.0D10 .AND. E3.LT.E1)  GO TO 45
      IF(RAM3.GT.1.0D10 .AND. E3.GE.E1)  GO TO 46
      RAM1 = RAM2
      RAM2 = RAM3
      E1 = E2
      E2 = E3
      GO TO 30
C
   45 RAM = RAM3
      EE = E3
      RETURN
C
   46 RAM = 0.0D0
      RETURN
C
   50 RAM3 = RAM2

      E3 = E2
      RAM2 = RAM3*0.1D0
      IF( RAM2*HNORM .LT. CONST2 )  GO TO  400
      DO 60  I=1,K
cxx   60 X1(I) = X(I) + RAM2*H(I)
      X1(I) = X(I) + RAM2*H(I)
   60 CONTINUE
cc      CALL  FUNCT( K,X1,E2,G,IG )
      CALL  FUNCT( Y,N,M1,M2,M3,M4,NPER,REG,K,X1,OUTMIN,OUTMAX,
     * ALIMIT,M,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,E2,IG,ier )
      if( ier.ne.0 ) return
cc      IF(IPR.GE.7)  WRITE(6,4)  RAM2,E2
      IF( E2.GT.E1 )  GO TO 50
C
cc   70 ASSIGN 80 TO RETURN
   70 CONTINUE
      IRET = 80
      GO TO 200
C
   80 DO 90  I=1,K
cxx   90 X1(I) = X(I) + RAM*H(I)
      X1(I) = X(I) + RAM*H(I)
   90 CONTINUE
cc      CALL  FUNCT( K,X1,EE,G,IG )
      CALL  FUNCT( Y,N,M1,M2,M3,M4,NPER,REG,K,X1,OUTMIN,OUTMAX,
     * ALIMIT,M,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,EE,IG,ier )
      if( ier.ne.0 ) return
cc      IF(IPR.GE.7)  WRITE(6,5)  RAM,EE
C
      IFG = 0
cc      ASSIGN  300 TO  SUB
cc      ASSIGN 200 TO SUB
cc   95 ASSIGN 130 TO RETURN
      ISUB = 200
   95 CONTINUE
      IRET = 130
      IF( RAM .GT. RAM2 )  GO TO 110
      IF( EE .GE. E2 )  GO TO 100
      RAM3 = RAM2
      RAM2 = RAM
      E3 =E2
      E2 =EE
cc      GO TO  SUB,( 200,300 )
      IF( ISUB.EQ.200 ) GO TO 200
      IF( ISUB.EQ.300 ) GO TO 300
C
  100 RAM1 = RAM
      E1 = EE
cc      GO TO  SUB,( 200,300 )
      IF( ISUB.EQ.200 ) GO TO 200
      IF( ISUB.EQ.300 ) GO TO 300
C
  110 IF( EE .LE. E2 )  GO TO 120
      RAM3 = RAM
      E3 = EE
cc      GO TO  SUB,( 200,300 )
      IF( ISUB.EQ.200 ) GO TO 200
      IF( ISUB.EQ.300 ) GO TO 300
C
  120 RAM1 = RAM2
      RAM2 = RAM
      E1 = E2
      E2 = EE
cc      GO TO  SUB,( 200,300 )
      IF( ISUB.EQ.200 ) GO TO 200
      IF( ISUB.EQ.300 ) GO TO 300
C
  130 DO 140  I=1,K
cxx  140 X1(I) = X(I) + RAM*H(I)
      X1(I) = X(I) + RAM*H(I)
  140 CONTINUE
cc      CALL  FUNCT( K,X1,EE,G,IG )
      CALL  FUNCT( Y,N,M1,M2,M3,M4,NPER,REG,K,X1,OUTMIN,OUTMAX,
     * ALIMIT,M,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,EE,IG,ier )
      if( ier.ne.0 ) return
cc      IF( IPR.GE.7 )  WRITE(6,6)  RAM,EE
cc      ASSIGN 200 TO SUB
      ISUB = 200
      IFG = IFG+1
cxx  600 FORMAT( 1H ,6D20.13 )
C     IF( HNORM*(RAM3-RAM1) .GT. 1.0D-12 )  GO TO 95
      IF( RAM3-RAM1 .GT. RAM2*1.0D-1 )  GO TO 95
C     IF( E1-EE .GT. 1.0D-15 )  GO TO 95
C     IF( E3-EE .GT. 1.0D-15 )  GO TO 95
C     IF(DX.LE.C1*XN+C2 .AND. DF.LE.C1*DABS(EE)+C2)  RETURN
C     IF( IFG .LE. 2 )  GO TO 95
C
      IF( E2 .LT. EE )  RAM = RAM2
      RETURN
C
C      -------  INTERNAL SUBROUTINE SUB1  -------
  200 IF( RAM3-RAM2 .GT. 5.0D0*(RAM2-RAM1) )  GO TO 202
      IF( RAM2-RAM1 .GT. 5.0D0*(RAM3-RAM2) )  GO TO 204
      A1 = (RAM3-RAM2)*E1
      A2 = (RAM1-RAM3)*E2
      A3 = (RAM2-RAM1)*E3
      B2 = (A1+A2+A3)*2.D0
      B1 = A1*(RAM3+RAM2) + A2*(RAM1+RAM3) + A3*(RAM2+RAM1)
      IF( B2 .EQ. 0.D0 )  GO TO 210
      RAM = B1 /B2
C
      IF( RAM .LE. RAM1 )  RAM = (RAM1 + RAM2)/2.0D0
      IF( RAM .GE. RAM3 )  RAM = (RAM2 + RAM3)/2.0D0
      IF( DABS(RAM-RAM2) .LE. 1.0D-15 )  RAM = (RAM2*4.D0 + RAM3)/5.0D0
cc      GO TO RETURN ,( 80,130 )
      IF( IRET.EQ.80 ) GO TO 80
      IF( IRET.EQ.130 ) GO TO 130
  202 RAM = (4.0D0*RAM2 + RAM3)/5.0D0
cc      GO TO RETURN, (80,130)
      IF( IRET.EQ.80 ) GO TO 80
      IF( IRET.EQ.130 ) GO TO 130
  204 RAM = (RAM1 + 4.0D0*RAM2)/5.0D0
cc      GO TO RETURN, (80,130)
      IF( IRET.EQ.80 ) GO TO 80
      IF( IRET.EQ.130 ) GO TO 130
C
  210 IG = 1
      RAM = RAM2
      RETURN
C
C      -------  INTERNAL SUBROUTINE SUB2  -------
C
  300 IF( RAM3-RAM2 .GT. RAM2-RAM1 )  GO TO 310
      RAM = (RAM1+RAM2)*0.5D0
cc      GO TO RETURN ,( 80,130 )
      IF( IRET.EQ.80 ) GO TO 80
      IF( IRET.EQ.130 ) GO TO 130
C
  310 RAM = (RAM2+RAM3)*0.5D0
cc      GO TO RETURN ,( 80,130 )
      IF( IRET.EQ.80 ) GO TO 80
      IF( IRET.EQ.130 ) GO TO 130
C
  400 RAM = 0.D0
      RETURN
C
  500 RAM = (RAM2+RAM3)*0.5D0
cxx 2019/07/27
         ire510 = 0
cxx
  510 DO 520  I=1,K
cxx  520 X1(I) = X(I) + RAM*H(I)
      X1(I) = X(I) + RAM*H(I)
  520 CONTINUE
cc      CALL  FUNCT( K,X1,E3,G,IG )
      CALL  FUNCT( Y,N,M1,M2,M3,M4,NPER,REG,K,X1,OUTMIN,OUTMAX,
     * ALIMIT,M,XMEAN,XVAR,NS,NFE,NPE,NMAX,MJ,MAXM,NC,E3,IG,ier )
      if( ier.ne.0 ) return
cc      IF( IPR.GE.7 )  WRITE(6,7)  RAM,E3
      IF( IG.EQ.1 )  GO TO 540
      IF( E3.GT.E2 )  GO TO 530
      RAM1 = RAM2
      RAM2 = RAM
      E1 = E2
      E2 = E3
      GO TO 500
C
  530 RAM3 = RAM
      GO TO 70
C
  540 RAM = (RAM2+RAM)*0.5D0
cxx      GO TO 510
cxx 2019/07/27
         if (RAM .EQ. RAM2) return
         ire510 = ire510 + 1
         if (ire510 .le. 100) GO TO 510
C
         return
C
C
C ------------------------------------------------------------
cxx    1 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E1 =',D25.17 )
cxx    2 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E2 =',D25.17 )
cxx    3 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E3 =',D25.17 )
cxx    4 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E4 =',D25.17 )
cxx    5 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E5 =',D25.17 )
cxx    6 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E6 =',D25.17 )
cxx    7 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E7 =',D25.17 )
      E N D

# PROGRAM 12.1
season <- function(y, trend.order = 1, seasonal.order = 1, ar.order = 0,
                   trade = FALSE, period = NULL, tau2.ini = NULL,
                   filter = c(1,length(y)), predict = length(y),
                   arcoef.ini = NULL, log = FALSE, log.base = "e",
                   minmax = c(-1.0e+30, 1.0e+30), plot = TRUE, ...) 
{
  if (trend.order < 0 || trend.order > 3)
    stop("'trend.order' must be 0, 1, 2 or 3.")

  if (seasonal.order < 0 || seasonal.order > 2)
    stop("'seasonal.order' must be 0, 1 or 2.")

  if (ar.order < 0 || ar.order > 5)
    stop("'ar.order' must be 0, 1, 2, 3, 4 or 5.")

  n <- length(y)      # data length
  m1 <- trend.order
  m2 <- seasonal.order
  m3 <- ar.order
  m4 <- 0
  m123 <- m1 + m2 + m3

  if (m123 == 0)
    stop("Total order is 0.")

  if (m2 !=0 || trade == TRUE)
    if (is.null(tsp(y)) == TRUE && is.null(period) == TRUE)
      stop("Neither 'frequency' and 'period' is not specified.")

  period.in <- period
  if (is.null(tsp(y)) == TRUE) {
	year <- 0
	month <- 1
  } else if (is.null(tsp(y)) == FALSE) {
	year <- start(y)[1]
	month <- start(y)[2]
    period <- tsp(y)[3]
    if (is.null(period.in) == FALSE && period != period.in) {
      warning(gettextf("The tsp attribute is not NULL, the value of 'period' is ignored.
                       \n'period' was set to 'frequency' %d.", period),
                       domain=NA)
    }
  }

  if (m2 != 0) {
    if (is.element(period, c(4, 5, 7, 12, 24)) == FALSE) {
      warning("'period' must be 4, 5, 7, 12 or 24.\n")
      m2 <- 0
    }
  }

  if (trade == TRUE) {
    if(period == 4 || period ==12) {
      m4 <- 6
    } else {
      warning("In the case of trade = TRUE, 'period' must be 4 or 12.\n")
    }
  }

  jyear <- year
  jmonth <- month
  logt <- 0
  if (log == TRUE) {
    if (log.base == "10") {
      logt <- 1
    } else if (log.base == "e") {
      logt <- 2
    } else {
      warning('log.base must be "e" (default) or "10".\n')
    }
  }
  iopt <- 1
  outmin <- minmax[1]
  outmax <- minmax[2]
  ns <- filter[1]
  nfe <- filter[2]
  npe <- predict
  nmax <- max(n, npe)

  np <- 0
  mj <- 0
  if (m1 > 0) {
    np <- 1
    mj <- m1
  }
  if (m2 > 0) {
    np <- np+1
    mj <- mj + m2*(period-1)
  }
  if (m3 > 0) {
    np <- np+1
    mj <- mj + m3
  }
  if (m4 > 0)
    mj <- mj + 6

  tau2 <- rep(0, 4)
  ll <- 0
  if (is.null(tau2.ini) ) {
    if (m1 > 0) {
      ll <- 1
      tau2[1] <- 0.05
    }
    if (m2 > 0) {
      ll <- ll+1
      tau2[ll] <- 0.11e-7
    }
    if (m3 > 0) {
      ll <- ll+1
      tau2[ll] <- 0.9
    }
    if (m4 > 0) {
      ll <- ll+1
      tau2[ll] <- 0.0
    }
  } else {
    nc <- length(tau2.ini)
    tau2[1:nc] <- tau2.ini[1:nc]
    for (i in 1:nc)
      if (tau2[i] == 1)
        stop("variance of the system noise tau2 is not equal to 1")
  }

  arcoef <- 0
  if (m3 > 0) {
    if (is.null(arcoef.ini)) {
      par <- rep(0,m3)
      a <- rep(0,m3)
      arcoef <- rep(0,m3)
      for (i in 1:m3)
        par[i] <- -(-0.8)**i
      for (i in 1:m3 ) {
        arcoef[i] <- par[i]
        a[i] <- par[i]
        if (i > 1 ) {
          im1 <- i-1
          for (j in 1:im1)
            arcoef[j] <- a[j] - par[i]*a[i-j] 
          if (i < m3 ) for (j in 1: im1 ) a[j] <- arcoef[j]
        }
      }
    } else {
      if (length(arcoef.ini) < m3 )
        stop(gettextf( "%d specify initial AR coefficients", m3 ), domain = NA)
      arcoef <- arcoef.ini
    }
  }

  z <- .Fortran(C_season,
                as.double(y),
                as.integer(n),
                as.integer(m1),
                as.integer(m2),
                as.integer(m3),
                as.integer(m4),
                as.integer(period),
                as.integer(jyear),
                as.integer(jmonth),
                tau2 = as.double(tau2),
                as.integer(ns),
                as.integer(nfe),
                as.integer(npe),
                as.double(arcoef),
                as.integer(logt),
                as.integer(iopt),
                as.double(outmin),
                as.double(outmax),
                as.integer(nmax),
                as.integer(mj),
                ff = double(1),
                over = double(1),
                aic = double(1),
                xss = double(nmax * mj),
                vss = double(nmax * mj * mj),
                deff = double(nmax),
                ier1 = integer(1),
                ier2 = integer(1))

  ier1 <- z$ier1
  if (ier1 == -1) 
    stop("Log-transformation cannot be applied to zeros and nagative numbers")

  ier2 <- z$ier2
  if (ier2 == 1 ) {
    stop(" Matrix with zero row in decompose" )
  } else if (ier2 == -1) {
    stop(" PARCOR for AR coefficients > 0.9" )
  } else if (ier2 == 2) {
    stop(" Singular matrix in decompose.zero divide in solve" )
  } else if (ier2 == 3) {
    stop(" Convergence in impruv.matrix is nearly singular" )
  } else if (ier2 ==400) {
    stop(" This model is unsuitable. The condition of the filtering routine is violated" )
  }

  trend <- NULL
  seasonal <- NULL
  ar <- NULL
  if (m3 == 0) arcoef <- NULL
  deff <- NULL

  sig2 <- 
  m12 <- m1 + m2*(period-1)
  xss <- array(z$xss, dim = c(mj, nmax))
  if (m1 > 0)
    trend <- xss[1,1:nmax]
  if (m2 > 0)
    seasonal <- xss[m1+1,1:nmax]
  if (m3 > 0)
    ar <- xss[m12+1, 1:nmax]
  if (m4 > 0)
    deff <- z$deff
  vss <- array(z$vss, dim = c(mj,mj,nmax))
  vss <- vss[1,1,1:nmax]

  yy <- y
  if (logt == 1) {
    yy <- log10(y)
  } else if (logt == 2) {
    yy <- log(y)
  }
  
  noise <- yy
  if (m1 > 0)
    noise <- noise - trend[1:n]
  if (m2 > 0)
    noise <- noise - seasonal[1:n]
  if (m3 > 0)
    noise <- noise - ar[1:n]
  if (m4 > 0)
    noise <- noise - deff[1:n]
  noise.list <- list(val = noise, range = c(ns, nfe), period = period)

  season.out <- list(tau2 = z$tau2[1:np], sigma2 = z$over, llkhood = z$ff,
                     aic = z$aic, trend = trend, seasonal = seasonal,
                     arcoef = arcoef, ar = ar, day.effect = deff,
                     noise = noise.list, cov = vss)
  class(season.out) <- "season"

  if (plot == TRUE) {
    rdata <- deparse(substitute(y))
    eval(parse(text=paste(rdata, "<- y")))
    if (logt == 1) {
      eval(parse(text=paste("plot.season(season.out, log10(", rdata, "), ...)")))
    } else if (logt == 2) {
      eval(parse(text=paste("plot.season(season.out, log(", rdata, "), ...)")))
    } else {
      eval(parse(text=paste("plot.season(season.out,", rdata, ", ...)")))
    }
  }

  return(season.out)
}


print.season <- function(x, ...)
{
  n <- length(x$tau2)
  message("\n tau2\t", appendLF=FALSE)
  for (i in 1:n)
    message(gettextf("\t%12.5e", x$tau2[i]), appendLF=FALSE, domain = NA)
  message(gettextf("\n sigma2\t\t%12.5e", x$sigma2), domain = NA)
  message(gettextf(" log-likelihood\t%12.3f", x$llkhood), domain = NA)
  message(gettextf(" aic\t\t%12.3f\n", x$aic), domain = NA)
}


plot.season <- function(x, rdata = NULL, ...)
{
  period <- x$noise$period

  if (is.null(rdata) == TRUE) {
    y <- NULL
    ts.atr <- c(1, 1, period)
    xtime <- "time"
  } else {
    y <- rdata
    tsname <- deparse(substitute(rdata))
    if (is.null(tsp(rdata)) == TRUE){
      y <- ts(y, start = 0, frequency = period)
    }
    ts.atr <- tsp(y)
    if (period == 4 || period == 12) {
      xtime <- "year"
    } else if (period == 24) {
      xtime <- "day"
    } else if (period == 5 || period == 7) {
      xtime <- "week"
    } else {
      xtime <- "time"
    }
  }

  trend <- x$trend
  seasonal <- x$seasonal
  ar <- x$ar
  day.effect <- x$day.effect
  noise <- x$noise$val
  noise <- ts(noise, start = ts.atr[1], frequency = ts.atr[3])

  n <- length(noise)
  range <- x$noise$range
  if (is.null(range) == TRUE) {
    ns <- 1
    nfe <- n
  } else {
    ns <- range[1]
    nfe <- range[2]
  }
  npe <- max(length(trend), length(seasonal), length(ar), length(day.effect))

  old.par <- par(no.readonly = TRUE)
  par(mfrow = c(3, 1), xaxs = "i", yaxs = "i")
  nplot <- 0
  new.mar <- old.par$mar
  new.mar[1] <- new.mar[1] * 0.75
  new.mar[3] <- new.mar[3] * 0.5
  new.mgp <- old.par$mgp
  new.mgp[1] <- new.mgp[1] * 0.75

  nmax <- max(n, npe)
  xlim <- c(1, nmax)

### original and trend
  par(mar = new.mar, mgp = new.mgp)
  k <- NULL
  if (is.null(trend) == TRUE) {
    if (is.null(y) == FALSE) {
      ylim1 <- get.ylim(y)$minmax
      k <- get.ylim(y)$k
      mtitle <- paste(tsname)
      y <- ts(y, start = ts.atr[1], frequency = ts.atr[3])
      plot(y, type = "l", ylim = ylim1, xlab = xtime, ylab = "", main = mtitle, ...)
      nplot <- nplot + 1
    }

  } else {  # is.null(trend) == FALSE
    tt <- rep(NA, nmax)
    tt[ns:npe] <- trend[ns:npe]
    tt <- ts(tt, start = ts.atr[1], frequency = ts.atr[3])
    s2 <- x$sigma2
    t1 <- tt[1:nmax]
    t2 <- tt[1:nmax]
    for (i in 1:nmax) {
      t1[i] <- t1[i] - sqrt(x$cov[i] * s2)
      t2[i] <- t2[i] + sqrt(x$cov[i] * s2)
    }
    yrange <- range(t1[ns:nmax], t2[ns:nmax], y, na.rm = TRUE)
    ylim1 <- get.ylim(yrange)$minmax
    k <- get.ylim(yrange)$k

    trend <- ts(trend, start = ts.atr[1], frequency = ts.atr[3])
    if (is.null(y) == TRUE) {
      mtitle <- paste("Trend component")
      plot(trend, type = "n", ylim = ylim1, xlab = "", ylab = "",
           main = mtitle, ...)
      par(new = TRUE)
    } else {
      mtitle <- paste(tsname, "and trend component")
      y <- ts(y, start = ts.atr[1], frequency = ts.atr[3])
      yyy <- rep(NA, nmax)
      yyy[1:n] <- y[1:n]
      yyy <- ts(yyy, start = ts.atr[1], frequency = ts.atr[3])
      plot(yyy, type = "l", ylim = ylim1, xlab = "", ylab = "", main = mtitle, ...)
      par(new = TRUE)
    }
    plot(tt, type = "l", col = 2, 
         ylim = ylim1, xlab = xtime, ylab = "", main = "", xaxt = "n", ...) 
    par(new = TRUE)
    plot(t1, type = "l", lty = 3, col = 4,
         ylim = ylim1, xlab = "", ylab = "", main = "", xaxt = "n", ...)
    par(new = TRUE)
    plot(t2, type = "l", lty = 3, col = 4,
         ylim = ylim1, xlab = "", ylab = "", main = "", xaxt = "n", ...)
    nplot <- nplot + 1
  }
  par(mar = old.par$mar, mgp = old.par$mgp)


### seasonal
  if (is.null(seasonal) == FALSE) {
 
    if (is.null(k) == TRUE)
      k <- get.ylim(seasonal)$k
	  
    if (k == 0) {
	  ylim2 <- get.ylim(seasonal)$minmax * 2
	} else {
      ylim2 <- get.ylim(seasonal)$minmax
      if (ylim2[2] < 10 ** k) {
        ylim2[1] <- -10 ** k
        ylim2[2] <- 10 ** k
      }
    }

    seasonal <- ts(seasonal, start = ts.atr[1], frequency = ts.atr[3])
    ss <- rep(NA, nmax)
    ss[ns:npe] <- seasonal[ns:npe]
    ss <- ts(ss, start = ts.atr[1], frequency = ts.atr[3])
    plot(ss, type = "h", ylim = ylim2, xlab = xtime, ylab = "",
         main = "Seasonal component", ...) 
    nplot <- nplot + 1
  }


### AR
  if (is.null(ar) == FALSE) {

    if (is.null(k) == TRUE)
      k <- get.ylim(ar)$k
	  
    if (k == 0) {
      ylim3 <- get.ylim(ar)$minmax * 2
    } else {
      ylim3 <- get.ylim(ar)$minmax
      if (ylim3[2] < 10 ** k) {
        ylim3[1] <- -10 ** k
        ylim3[2] <- 10 ** k
      }
    }

    if (ylim3[1] > 0)
      ylim3[1] <- 0

    ar <- ts(ar, start = ts.atr[1], frequency = ts.atr[3])
    aa <- rep(NA, nmax)
    aa[ns:npe] <- ar[ns:npe]
    aa <- ts(aa, start = ts.atr[1], frequency = ts.atr[3])
    plot(aa, type = "h", xlab = xtime, ylim = ylim3, ylab = "",
         main = "AR component", ...)
    nplot <- nplot + 1
  }


### day.effect
  if (is.null(day.effect) == FALSE) {
  
    if (is.null(k) == TRUE)
      k <- get.ylim(day.effect)$k

    if (k == 0) {
      ylim4 <- get.ylim(day.effect)$minmax * 2
    } else {
      ylim4 <- get.ylim(day.effect)$minmax
      if (ylim4[2] < 10 ** k) {
        ylim4[1] <- -10 ** k
        ylim4[2] <- 10 ** k
      }
    }

    if (nplot == 3) {
      dev.new()
      par(mfrow = c(3, 1), xaxs = "i", yaxs = "i")
      nplot <- 0
    }
    day.effect <- ts(day.effect, start = ts.atr[1], frequency = ts.atr[3])
    dd <- rep(NA, nmax)
    dd[ns:npe] <- day.effect[ns:npe]
    dd <- ts(dd, start = ts.atr[1], frequency = ts.atr[3])
    plot(dd, type = "h", xlab = xtime, ylim = ylim4, ylab = "",
         main = "Trading day effect", ...)
    nplot <- nplot + 1
  }

### noise

  if (is.null(k) == TRUE)
    k <- get.ylim(noise)$k

  if (k == 0) {
    ylim5 <- get.ylim(noise)$minmax * 2
  } else {
    ylim5 <- get.ylim(noise)$minmax
    if (ylim5[2] < 10 ** k) {
      ylim5[1] <- -10 ** k
      ylim5[2] <- 10 ** k
    }
  }

  if (ylim5[1] > 0)
    ylim5[1] <- 0

  if (nplot == 3) {
    dev.new()
    par(mfrow = c(3, 1), xaxs = "i", yaxs = "i")
    nplot <- 0
  }

  sse <- rep(NA, nmax)
  sse[ns:nfe] <- noise[ns:nfe]
  sse <- ts(sse, start = ts.atr[1], frequency = ts.atr[3])

  plot(sse, type = "n", ylim = ylim5, xlab = xtime, ylab = "", main = "Noise", ...)
  par(new = TRUE)
  plot(sse, type = "h", xlab = "", ylim = ylim5, ylab = "", xaxt = "n", ...) 
  nplot <- nplot + 1

### Predicted values
  if (nfe < npe) {
    nps <- nfe + 1
    yy <- rep(NA, npe)
    for (i in nps:npe) {
      yy[i] <- 0
      if (is.null(trend) == FALSE)
        yy[i] <- yy[i] + trend[i]
      if (is.null(seasonal) == FALSE)
        yy[i] <- yy[i] + seasonal[i]
      if (is.null(ar) == FALSE)
        yy[i] <- yy[i] + ar[i]
      if (is.null(day.effect) == FALSE)
        yy[i] <- yy[i] + day.effect[i]
    }
    yy <- ts(yy, start = ts.atr[1], frequency = ts.atr[3])
    ydummy <- ts(rep(NA, nps), start=ts.atr[1], frequency = ts.atr[3])
    xlim <- c(0, nmax)

    if (nplot == 3) {
      dev.new()
      par(mfrow = c(3, 1), xaxs = "i", yaxs = "i")
    }
    par(mar = new.mar, mgp = new.mgp)

    if (is.null(trend) == FALSE) {
      t1 <- yy
      t2 <- yy
      for (i in nps:npe) {
        t1[i] <- t1[i] - sqrt(x$cov[i] * s2)
        t2[i] <- t2[i] + sqrt(x$cov[i] * s2)
      }
      ylim6 <- range(ylim1, t1[nps:npe], t2[nps:npe], na.rm = TRUE)

      plot(yy, type = "l", col = 2, ylim = ylim6,
           xlab = xtime, ylab = "", main = "Predicted values", xaxt = "n", ...)
      par(new = TRUE)

      if (is.null(y) == FALSE) {
        y0 <- rep(NA, npe)
#        y0[ns:nfe] <- y[ns:nfe]
        y0[1:length(y)] <- y
        y0 <- ts(y0, start = ts.atr[1], frequency = ts.atr[3])
        plot(y0, type = "l", ylim = ylim6, xlab = "", ylab = "", main = "", ...)
        par(new = TRUE)
      }

      plot(t1, type = "l", col = 4, ylim = ylim6,
           xlab = "", ylab = "", main = "", xaxt = "n", ...)
      par(new = TRUE)
      plot(t2, type = "l", col = 4, ylim = ylim6,
           xlab = "", ylab = "", main = "", xaxt = "n", ...)
      abline(v = tsp(ydummy)[2], lty = 2)
	  
    } else {
      ylim6 <- range(ylim1, yy[nps:npe], na.rm = TRUE)
      if (is.null(y) == FALSE) {
        y0 <- rep(NA, npe)
        y0[ns:nfe] <- y[ns:nfe]
        y0 <- ts(y0, start = ts.atr[1], frequency = ts.atr[3])
        plot(y0, type = "l", ylim = ylim6, xlab = "", ylab = "", main = "", ...) 
        par(new = TRUE)
      }
      plot(yy, type = "l", col = 2, ylim = ylim6, xaxt = "n",
           xlab = xtime, ylab = "", main = "Predicted values", ...)
      abline(v = tsp(ydummy)[2], lty = 2)
    }
    par(mar = old.par$mar, mgp = old.par$mgp)
  }
  par(mfrow = old.par$mfrow, xaxs = old.par$xaxs)
}


get.ylim <- function(y) {
  ymax <- max(y)
  ymin <- min(y)
  yd <- ymax - ymin
  k <- floor(log10(yd))
  ymax <- ((ymax %/% 10^k) + 1) * (10 ** k)
  ymin <- (ymin %/% 10^k) * (10 ** k)
  if (ymin < 0) {
    ys <- max(abs(ymin), abs(ymax))
    ymin <- -ys
    ymax <- ys
  }
  return(list(minmax = c(ymin, ymax), k = k))
}

# PROGRAM 8.1  LSAR1
lsar <- function (y, max.arorder = 20, ns0, plot = TRUE, ...)
{
  n <- length(y)         # Data length
# max.arorder            # Highest AR order
  lag <- max.arorder
# ns                     # Basic local span
  nb <- as.integer(n / ns0)
  nf0 <- 100             # Initial number of frequencies for computing spectrum

  z <- .Fortran(C_lsar1,
                as.double(y),
	            as.integer(n),
                as.integer(lag),
                as.integer(ns0),
                as.integer(nb),
                as.integer(nf0),
                ns = integer(nb),
                n0 = integer(nb),
                n1 = integer(nb),
                iflag = integer(nb),
                ms = integer(nb),
                sds = double(nb),
                aics = double(nb),
                mp = integer(nb),
                sdp = double(nb),
                aicp = double(nb),
                as = double(nb*lag),
                mfs = integer(nb),
                sig2s = double(nb),
                nnf = integer(nb),
                ier = integer(1))

  if (z$ier == 0) {
    model <- z$iflag
    sub <- list(start = z$n0, end = z$n1)
    as <- array(z$as, dim = c(lag, nb))
    nnf <- z$nnf

    spec <- sub.arsp(model, sub, as, z$mfs, z$sig2s, nnf)
    sdp <- z$sdp
    sdp[1] <- NA
    aicp <- z$aicp
    aicp[1] <- NA


    if (ns0 < max.arorder * 2)
      warning("the length of basic local span (ns0) is too small. 
ns0 should be at least twice as large as the maximum AR order")

    lsar.out <- list(model = model, ns = z$ns, span = sub, nf = nnf, ms = z$ms,
                     sds = z$sds, aics = z$aics, mp = z$mp, sdp = sdp, aicp = aicp,
                     spec = spec, tsname = deparse(substitute(y)))
    class(lsar.out) <- "lsar"

    if (plot)
      plot.lsar(lsar.out, ...)

    return(lsar.out)

  } else {
    stop("estimation error: the length of basic local span (ns0) is too small. 
ns0 should be at least twice as large as the maximum AR order")
  }
}

sub.arsp <- function(model, sub, as, mfs, sig2s, nf)
{
  st <- sub$start[1]
  ed <- NULL

  spec <- list()
  b <- 0
  l <- 0
  nb <- length(model)
  k <- 0
  for (i  in 1:(nb-1))
    if (model[i + 1] == 2) {
      a <- as[, i]
      m <- mfs[i]
      sig2 <- sig2s[i]
##      nnf <- nf[i]
      nnf <- max(nf[i], 100)

      z <- .Fortran(C_armasp,
                   as.double(a),
                   as.integer(m),
                   as.double(b),
                   as.integer(l),
                   as.double(sig2),
                   as.integer(nnf),
                   spec = double(nnf + 1))

      k <- k + 1
      spec[[k]] <- z$spec
      ed <- c(ed, sub$end[i])
      st <- c(st, sub$start[i + 1])
    }

  a <- as[, nb]
  m <- mfs[nb]
  sig2 <- sig2s[nb]
  nnf <- nf[nb]

  z <- .Fortran(C_armasp,
                as.double(a),
                as.integer(m),
                as.double(b),
                as.integer(l),
                as.double(sig2),
                as.integer(nnf),
                spec = double(nnf + 1))

  k <- k + 1
  spec[[k]] <- z$spec
  ed[k] <- sub$end[nb]

  spec <- list(subspec = spec, start = st, end = ed)
  return(spec = spec)
}

plot.lsar <- function(x, ...)
{
  spec <- x$spec$subspec
  st <- x$spec$start
  ed <- x$spec$end

  k <- length(spec)
  sp <- unlist(spec)
  ylim <- c(floor(min(sp)), ceiling(max(sp)))

  old.par <- par(no.readonly = TRUE)
  par(mfrow = c(3, 3), xaxs = "i", yaxs = "i")
  ic <- 0
  for (i in 1:k) {
#    if (ic == 4) {
    if (ic == 9) {
      par(ask = TRUE)
      ic <- 0
    }
    nf1 <- length(spec[[i]])
    xx <- rep(0, nf1)
    for (j in 1:nf1)
      xx[j] <- (j - 1) / (2 * (nf1-1))

    mtitle <- paste(st[i], " - ", ed[i])
    if (i == 1)
      mtitle <- paste(x$tsname, "\n", st[i], " - ", ed[i])
    plot(xx, spec[[i]], type = "l", main = mtitle, ylab = "", xlab = "f",
         ylim = ylim, ...)
    ic <- ic + 1
  }
  par(old.par)
}

print.lsar <- function(x, ...)
{
  nb <- length(x$ns)
  n0 <- x$span$start
  n1 <- x$span$end

  message(gettextf("<<< new data ( n = %d --- %d ) >>>", n0[1], n1[1]),
          domain = NA)
  message(gettextf(" initial model : NS = %5d", x$ns[1]), domain = NA)
  message(gettextf("\t\tms = %d   sds = %13.6e   aics = %12.3f",
          x$ms[1], x$sds[1], x$aics[1]), domain = NA)


  for (i in 2:nb) {
    np <- x$ns[i] + x$nf[i - 1]
    message(gettextf("\n<<< new data ( n = %d --- %d ) >>>", n0[i], n1[i]),
            domain = NA)
    message(gettextf(" switched model : ( nf = %d, ns = %d )", x$nf[i-1],
            x$ns[i]), domain = NA)
    message(gettextf("\t ms = %d   sds = %13.6e   aics = %12.3f", x$ms[i],
            x$sds[i], x$aics[i]), domain = NA)

    message(gettextf(" pooled model : ( np = %d )", np), domain = NA)
    message(gettextf("\t mp = %d   sdp = %13.6e   aicp = %12.3f", x$mp[i],
            x$sdp[i], x$aicp[i]), domain = NA)

    if (x$model[i] == 1)
      message("\n ***  pooled model accepted   ***")
    if (x$model[i] == 2)
      message("\n ***  switched model accepted   ***")
  }
}

# PROGRAM 8.2  LSAR2
lsar.chgpt <- function (y, max.arorder = 20, subinterval, candidate,
                        plot = TRUE, ...)
{
  n <- length(y)      # Data length
# max.arorder         # Highest AR order
  k <- max.arorder
  if (is.null(subinterval))
    subinterval <- c(1, n)
# n0 < n1 < n2 < ne <= n
  n0 <- subinterval[1]
  ne <- subinterval[2]      # [n0,ne] Time interval used for model fitting
  n1 <- candidate[1]
  n2 <- candidate[2]        # [n1,n2] Candidate for change point

  if (ne > n)
    stop("ne is less than or equal to n (data length).")
  n02k <- n0 + 2 * k
  if ((n02k == n1) || (n02k > n1))
    stop("\n the following condition 'n0 + 2k < n1' is not satisfied, where n0
    is the start point of 'subinterval', n1 the start point of 'candidate' and
    k is max.arorder.")
  n2k <- n2 + k
  if ((n2k == ne) || (n2k > ne))
    stop("\n the following condition 'n2 + k < ne' is not satisfied, where n2 is
    the end point of 'candidate', \n k is max.arorder and ne is the end point of
    'subinterval'.")

  m <- n2 - n1

  z <- .Fortran(C_lsar2,
                as.double(y),
                as.integer(n),
                as.integer(max.arorder),
                as.integer(n0),
                as.integer(n1),
                as.integer(n2),
                as.integer(ne),
                aics = double(m),
                aicmin = double(1),
                nmin = integer(1))

  change.pt <- n1 + z$nmin
  subint <- list(tsname = deparse(substitute(y)), st = n1+1, ed = n2,
                 y = y[(n1+1):n2])

  lsar.chgpt.out <- list(aic = z$aics, aicmin = z$aicmin,
                         change.point = change.pt, subint = subint )
  class(lsar.chgpt.out) <- "chgpt"

  if (plot) {
    plot.chgpt(lsar.chgpt.out, ...)
    invisible(lsar.chgpt.out)
  } else lsar.chgpt.out
}

plot.chgpt <- function(x, ...)
{
  cpt <- x$change.point
  n1 <- x$subint$st
  n2 <- x$subint$ed
  xx <- c(n1:n2)
  yy <- x$subint$y

  old.par <- par(no.readonly = TRUE)
  par(mfrow = c(2, 1), xaxs = "i")

  plot(xx, yy, type = "l", main = paste(x$subint$tsname, "sub-interval"),
       xlab = "", ylab = "y", cex.main = 1.0, ...)

  aicmin <- format(round(x$aicmin, 2), nsmall=2)
  mtitle <- paste("AICs of the model in the candidate range\n minimum aic =",
                  aicmin, " at ", cpt)
  plot(xx, x$aic, type = "l", main = mtitle, xlab = "time", ylab = "aic",
       cex.main = 1.0, ...)
  points(cpt, x$aicmin, col = 2, pch = 20)

  par(old.par)
}

C     PROGRAM 10.1  ARMAFT
      SUBROUTINE ARMAFT2( Y0,N,MMAX,LMAX,MLMAX,TSIG2,TFF,TAIC,AR,CMA,
     1                    IER )
C
C  ...  ARMA MODEL FITTING  ...
C
C     Inputs:
C        M:       AR Order (M <= 20)
C        L:       MA Order (M <= 20)
C        IPARAM:  =0    Use defalt initail values
C                 =1    Read intial values
C        AR(I):   AR coefficients (I=1,M3)
C     Parameters:
C        NMAX:    Adjustable dimension of Y
C        MJ:      Adjustable dimension of XF, VF, etc.
C     @TEST.FILTER2O    NOV.29,1990, SEP.02,1992
C
cc      PARAMETER( NMAX=1000,MJ=40 )
cc      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  AA(40), AR(40), PAR(40), CMA(40)
cc      DIMENSION  TAIC(0:20,0:20), TFF(0:20,0:20)
cc      DIMENSION  SAA(40,0:20,0:20)
cc      COMMON  /C92825/  OUTMIN, OUTMAX
cc      COMMON  /C92826/  Y(NMAX)
cc      COMMON  /C92907/  ALIMIT
cc      COMMON  /C92908/  M, L, N
cc      COMMON  /C92909/  FLK, SIG2
cc      COMMON  / CCC /  ISW, IPR, ISMT, IDIF
c
      INTEGER N, MMAX, LMAX, MLMAX, IER(3)
      DOUBLE PRECISION Y0(N), TSIG2(0:MMAX, 0:LMAX), TFF(0:MMAX,0:LMAX),
     1                 TAIC(0:MMAX, 0:LMAX), AR(MMAX, 0:MMAX, 0:LMAX),
     2                 CMA(LMAX, 0:MMAX, 0:LMAX)
c local
      INTEGER I, J, M, L, IPRAM, IOPT, IDIF, NSUM
      DOUBLE PRECISION Y(N), PI, SUM, YMEAN, YVAR, ALIMIT, OUTMIN,
     1                 OUTMAX, PAR(MLMAX), AA(MMAX+LMAX), SIG2, FLK,
     2                 AIC, SAA(MMAX+LMAX, 0:MMAX, 0:LMAX)
c
      EXTERNAL  FFARMA
      PI = 3.1415926530D0
cc      IPR = 0
c
      TSIG2(0:MMAX, 0:LMAX) = 0.0D0
      TFF(0:MMAX,0:LMAX) = 0.0D0
      TAIC(0:MMAX, 0:LMAX) = 0.0D0
      AR(1:MMAX, 0:MMAX, 0:LMAX) = 0.0D0
      CMA(1:LMAX, 0:MMAX, 0:LMAX) = 0.0D0
      PAR(1:MLMAX) = 0.0D0
      AA(1:(MMAX+LMAX)) = 0.0D0
      SAA(1:(MMAX+LMAX), 0:MMAX, 0:LMAX) = 0.0D0
      IER(1:3) = 0
C
C  ...  Read Time Series  ...
C
cc      CALL  READTS( 1,Y,N )
      DO 5 I = 1,N
      Y(I) = Y0(I)
    5 CONTINUE
C
C  ...  Subtrac Mean Value  ...
C
cc      SUM = 0.0D0
cc      DO 10 I=1,N
cccc   10 SUM = SUM + Y(I)
cc      SUM = SUM + Y(I)
cc   10 CONTINUE
cc      YMEAN = SUM/N
      CALL  MEAN( Y,N,-1.0D30,1.0D30,NSUM,YMEAN )
      DO 20 I=1,N
cc   20 Y(I) = Y(I) - YMEAN
      Y(I) = Y(I) - YMEAN
   20 CONTINUE

cc      DO 25 I=0,20
cc      DO 25 J=0,20
cc   25 TFF(I,J) = -1.0D10
      DO 26 I=0,MMAX
      DO 25 J=0,LMAX
      TFF(I,J) = -1.0D10
   25 CONTINUE
   26 CONTINUE
C
C  ...  Read Model Orders  ...
C
C      READ( 5,* )  M, L, IPARAM
cc      open( 3,file='temp.dat' )
cc      MMAX = 10
ccc     LMAX = 10
cc      DO 100  M=0,MMAX
      DO 101  M=0,MMAX
      DO 100  L=0,LMAX
      
cc      write(3,*) M, L
      IF( M.EQ.0 .and. L.EQ.0 )  THEN
cc         SUM = 0.0D0
cc         DO 110 I=1,N
cccc  110    SUM = SUM + Y(I)**2
cc         SUM = SUM + Y(I)**2
cc  110    CONTINUE
         CALL  MEAN2( Y,N,-1.0D30,1.0D30,SUM )
         YVAR = SUM/N
         TSIG2(0,0) = YVAR
         TFF(0,0) = -0.5D0*N*(DLOG(2*PI*YVAR) + 1)
         TAIC(0,0) = -2*TFF(0,0) + 2
         GO TO 100
      END IF
C
C  ...  Set Defalt Parameters  ...
C
cc      IPR   = 0
      IPRAM = 0
      IOPT  = 1
      IDIF  = 2
      ALIMIT = 0.95D0
      
      IF( L.EQ.0 .OR. M.EQ.0 ) THEN 
      
cc      CALL  SPARA1( M,L,AR,CMA,OUTMIN,OUTMAX,IOPT )
      CALL  SPARA1( M,L,MLMAX,AR(1,M,L),CMA(1,M,L),OUTMIN,OUTMAX,IOPT )
C      IF( IPARAM.EQ.1 )  THEN
C         READ( 5,* )  (AR(I),I=1,M)
C         READ( 5,* )  (CMA(I),I=1,L)
C      END IF
C
C      WRITE(6,*) M, L
C      WRITE(6,*) (AR(I),I=1,M)
C      WRITE(6,*) (CMA(I),I=1,L)
C
cc      CALL  PARCOR( AR,M,PAR )
      CALL  PARCOR( AR(1,M,L),M,PAR )
C      write(6,*) (PAR(I),I=1,M)
      DO 30 I=1,M
cc   30 AA(I) = DLOG( (ALIMIT+PAR(I))/(ALIMIT-PAR(I)) )
      AA(I) = DLOG( (ALIMIT+PAR(I))/(ALIMIT-PAR(I)) )
   30 CONTINUE
cc      CALL  PARCOR( CMA,L,PAR )
      CALL  PARCOR( CMA(1,M,L),L,PAR )
C      write(6,*) (PAR(I),I=1,L)
      DO 40 I=1,L
cc   40 AA(M+I) = DLOG( (ALIMIT+PAR(I))/(ALIMIT-PAR(I)) )
      AA(M+I) = DLOG( (ALIMIT+PAR(I))/(ALIMIT-PAR(I)) )
   40 CONTINUE
   
      ELSE
         AA(M+L) = 0.0000D0
C         IF( M.EQ.0.AND.L.EQ.1 )  AA(1) = 0.18D0
C         IF( M.GT.0 )  THEN
         IF( TFF(M-1,L).GT.TFF(M,L-1) )  THEN
            DO 70 I=1,M-1
cc   70       AA(I) = SAA(I,M-1,L)
            AA(I) = SAA(I,M-1,L) 
   70       CONTINUE
           AA(M) = 0.0000D0
            DO 80 I=1,L
cc   80       AA(M+I) = SAA(M-1+I,M-1,L)         
            AA(M+I) = SAA(M-1+I,M-1,L)         
   80       CONTINUE         
         END IF
C         END IF
      END IF
      
cc      WRITE(6,*) M, L
cc      WRITE(6,666) (AA(I),I=1,M+L)
cc  666 FORMAT( 10F10.5 )
  
C
C  ...  Maximum Likelihood Method  ...
C
cc      write(3,*)  (AA(I),I=1,M+L)
cc      IF( IOPT.EQ.1 )  CALL  DAVIDN( FFARMA,AA,M+L,2 )
      IF( IOPT.EQ.1 )  CALL  DAVIDN( FFARMA,AA,M+L,2,
     *    Y,N,M,L,MLMAX,OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER(1) )
      if( ier(1).ne.0 ) then
         ier(2) = m
         ier(3) = l
         return
      end if
C
C
      DO 50 I=1,M
cc   50 PAR(I)  = ALIMIT*(DEXP(AA(I))-1.0D0)/(DEXP(AA(I))+1.0D0)
      PAR(I)  = ALIMIT*(DEXP(AA(I))-1.0D0)/(DEXP(AA(I))+1.0D0)
   50 CONTINUE
cc      CALL  ARCOEF( PAR,M,AR )
      CALL  ARCOEF( PAR,M,AR(1,M,L) )
      DO 60 I=1,L
cc   60 PAR(I)  = ALIMIT*(DEXP(AA(M+I))-1.0D0)/(DEXP(AA(M+I))+1.0D0)
      PAR(I)  = ALIMIT*(DEXP(AA(M+I))-1.0D0)/(DEXP(AA(M+I))+1.0D0)
   60 CONTINUE
cc      CALL  ARCOEF( PAR,L,CMA )
      CALL  ARCOEF( PAR,L,CMA(1,M,L) )
C
      AIC = -2*FLK + 2*(M+L+1)
C
C  ...  Print out the Maximum Likelihood Estimates  ...
C
cc      WRITE(6,666) (AA(I),I=1,M+L)

cc      CALL  PRARMA( M,L,AR,CMA,SIG2,FLK,AIC )
      TFF(M,L) = FLK
      TAIC(M,L) = AIC
      TSIG2(M,L) = SIG2
      DO 90 I=1,M+L
cc   90 SAA(I,M,L) = AA(I)
      SAA(I,M,L) = AA(I)
   90 CONTINUE
cc      WRITE(3,33)  FLK, AIC
cc   33 FORMAT( 5f12.4 )
cc      WRITE(3,*)  (AA(I),I=1,M+L)
  100 CONTINUE
  101 CONTINUE
C
cc      open( 2,file='temp_table.dat' )
cc      WRITE(6,*)  'log-Likelihood(M,L)'
cc      WRITE(2,*)  'log-Likelihood(M,L)'
cc      DO 200 I=0,MMAX
cc      WRITE(2,600)  (TFF(I,J),J=0,LMAX)
cc  200 WRITE(6,600)  (TFF(I,J),J=0,LMAX)
cc      WRITE(2,*)  'AIC(M,L)'
cc      WRITE(6,*)  'AIC(M,L)'
cc      DO 210 I=0,MMAX
cc      WRITE(2,600)  (TAIC(I,J),J=0,LMAX)
cc  210 WRITE(6,600)  (TAIC(I,J),J=0,LMAX)
cc      close( 2 )
cc      close( 3 )
cc  600 FORMAT( 11F10.4 )
cc      STOP
      RETURN
      E N D
      SUBROUTINE  MEAN2( Y,N,OUTMIN,OUTMAX,YSUM )
C
C     Inputs:
C        Y(I):    time series
C        N:       data length
C        OUTMIN:  bound for outliers in low side
C        OUTMAX:  bound for outliers in high side
C     Outputs:
C        YSUM:    sum of non-outlier observations
C
      INTEGER N
      DOUBLE PRECISION Y(N), OUTMIN, OUTMAX, YSUM
c local
      INTEGER I
C
      YSUM = 0.0D0
      DO 10 I=1,N
      IF( Y(I) .GT.OUTMIN .AND. Y(I).LT.OUTMAX ) THEN
         YSUM = YSUM + Y(I)**2
      END IF
   10 CONTINUE
C
      RETURN
      E N D

C     PROGRAM 6.2  MARSPC
      SUBROUTINE MARSPCF( M,L,A,E,NF,P,AMP,ANG,COH,FNC,FRNC )
C
C  ...  This program computes cross spectra and noise contribution  ...
C
C     Inputs:
C        IDEV:    Input device
C        M:       AR order
C        L:       Dimension of the time series
C        E(I,J):  Innovation covariance matrix
C        A(II,I,J):  AR coefficient matrices
C     Parametrs:
C        NF:      Number of frequencies
C        MJ:      Adjustable dimension
C        MJ1:     Adjustable dimension
C     Written by G.K.
cc      PARAMETER( NF=200,MJ=3,MJ1=7,IDEV=1 )
cxx      IMPLICIT  REAL*8(A-H)
cxx      IMPLICIT  COMPLEX*16(O-Z)
cc      DIMENSION  A(MJ,MJ,MJ1), E(MJ,MJ)
cc      DIMENSION  P(0:NF,MJ,MJ), FNC(0:NF,MJ,MJ), COH(0:NF,MJ,MJ)
cc      DIMENSION  AMP(0:NF,MJ,MJ), ANG(0:NF,MJ,MJ)
cxx      DIMENSION  A(L,L,M), E(L,L), FRNC(0:NF,L,L)
cxx      DIMENSION  P(0:NF,L,L), FNC(0:NF,L,L), COH(0:NF,L,L)
cxx      DIMENSION  AMP(0:NF,L,L), ANG(0:NF,L,L)
C
      INTEGER M, L, NF
      DOUBLE PRECISION A(L,L,M), E(L,L), AMP(0:NF,L,L), ANG(0:NF,L,L),
     1                 COH(0:NF,L,L), FNC(0:NF,L,L), FRNC(0:NF,L,L)
c local
      COMPLEX(KIND(0.0D0)) P(0:NF,L,L)
c
      AMP(:,:,:) = 0.0d0
      ANG(:,:,:) = 0.0d0
      COH(:,:,:) = 0.0d0
C
cc      open( IDEV,file='ar2.dat' )
cc      READ(IDEV,*)  M, L
cc      DO 10 I=1,L 
cc   10 READ(IDEV,*)  (E(I,J),J=1,L)
cc      DO 20 II=1,M
cc      DO 20  I=1,L
cc   20 READ(IDEV,*)  (A(I,J,II),J=1,L)
cc      close( IDEV )
C
C  ...  Cross spectrum  ...
C
cc      CALL  MARSPC( M,L,A,E,NF,MJ,MJ1,P,FNC,AMP,ANG,COH )
      CALL  MARSPC( M,L,A,E,NF,P,FNC,AMP,ANG,COH )
C
C  ...  Print out plot cross spectra, coherency and noise
C       contribution  ...
C
cc      CALL  PRMSPC( M,L,NF,MJ,P,FNC,COH )
cxx      CALL  PRMSPC( M,L,NF,P,FNC,COH,FRNC )
      CALL  PRMSPC( L,NF,FNC,FRNC )
C      CALL  PTCSPC( M,L,NF,MJ,P,FNC,AMP,ANG,COH )
C
cc      STOP
      RETURN
      E N D
cc      SUBROUTINE  PRMSPC( M,L,NF,MJ,P,FNC,COH )
cxx      SUBROUTINE  PRMSPC( M,L,NF,P,FNC,COH,FRNC )
      SUBROUTINE  PRMSPC( L,NF,FNC,FRNC )
C
C  ...  Print out cross spectra and relative noise contribution  ...
C
C     Inputs:
C        M:    AR order
C        L:    Dimension of the time series
C        NF:   Number of frequencies
C        MJ:   Adjustable dimension of P and FNC
C        P(II,I,J):   Cross spectrum of Y(I) and Y(J) at II-th frequency
C        FNC(II,I,J): Noise contribution of Y(J) to Y(I)
C        COH(II,I,J): Simple coherency
C
cxx      IMPLICIT COMPLEX*16(O-Z)
cxx      IMPLICIT REAL*8(A-H)
cc      DIMENSION  P(0:NF,MJ,MJ), FNC(0:NF,MJ,MJ), FRNC(0:200)
cc      DIMENSION  COH(0:NF,MJ,MJ)
cxx      DIMENSION  P(0:NF,L,L), FNC(0:NF,L,L), FRNC(0:NF,L,L)
cxx      DIMENSION  COH(0:NF,L,L)
C
      INTEGER L, NF
      DOUBLE PRECISION FNC(0:NF,L,L), FRNC(0:NF,L,L)
c local
      INTEGER I, II, J
C
cc      WRITE(6,600)
cc      WRITE(6,610)  M, L, NF
cc      WRITE(6,620)
cc      DO 10 I=1,L
cc      DO 10 J=1,L
cc      WRITE(6,630)  I, J
cc   10 WRITE(6,640)  (P(II,I,J),II=0,NF)
cc      WRITE(6,650)
cc      DO 20 I=1,L-1
cc      DO 20 J=I+1,L
cc      WRITE(6,630)  I, J
cc   20 WRITE(6,660)  (COH(II,I,J),II=0,NF)
cc      WRITE(6,670)
cxx      DO 40 I=1,L
      DO 41 I=1,L
      DO 40 J=1,L
cc      WRITE(6,630)  I, J
      DO 30 II=0,NF
cc      IF(J.EQ.1)  FRNC(II) = FNC(II,I,J)/FNC(II,I,L)
cc   30 IF(J.NE.1)  FRNC(II) = (FNC(II,I,J)-FNC(II,I,J-1))/FNC(II,I,L)
      IF(J.EQ.1)  FRNC(II,I,J) = FNC(II,I,J)/FNC(II,I,L)
cxx   30 IF(J.NE.1)  FRNC(II,I,J) = (FNC(II,I,J)-FNC(II,I,J-1))/FNC(II,I,L)
      IF(J.NE.1)  FRNC(II,I,J) = (FNC(II,I,J)-FNC(II,I,J-1))/FNC(II,I,L)
   30 CONTINUE
cc   40 WRITE(6,660)  (FRNC(II),II=0,NF)
   40 CONTINUE
   41 CONTINUE
C
cxx  600 FORMAT( 1H ,'PROGRAM 6.2:  SPECTRUM ANALYSIS BY MAR MODEL' )
cxx  610 FORMAT( 1H ,'M(ORDER) =',I3,5X,'L(DIMENSION) =',I2,5X,
cxx     *            'NF(NUMBER OF FREQUENCIES =',I3 )
cxx  620 FORMAT( 1H ,'CROSS SPECTRUM' )
cxx  630 FORMAT( 1H ,'*** I =',I2,2X,'J =',I2,' ***' )
cxx  640 FORMAT( 2(' (',D13.5,',',D13.5,')',4X) )
cxx  650 FORMAT( 1H ,'COHERENCY' )
cxx  660 FORMAT( 1H ,10F7.4 )
cxx  670 FORMAT( 1H ,'RELATIVE NOISE CONTRIBUTION' )
      RETURN
      E N D
cc      SUBROUTINE  MARSPC( M,L,A,E,NF,MJ,MJ1,P,FNC,AMP,ANG,COH )
      SUBROUTINE  MARSPC( M,L,A,E,NF,P,FNC,AMP,ANG,COH )
C
C  ...  Cross spectra, amplitude, phase and noise contribution  ...
C
C     Inputs:
C        M:       AR order
C        L:       Dimension of the time series
C        E(I,J):  Innovation covariance matrix
C        A(I,J,II):  AR coefficient matrices
C        NF:      Number of frequencies
C        MJ:      Adjustable dimension
C        MJ1:     Adjustable dimension
C     Outputs:
C        P(II,I,J):    Cross spectra
C        FNC(II,I,J):  Noise contribution of Y(J) to Y(I)
C        COH(II,I,J):  Simple coherency
C        AMP(II,I,J):  Amplitude spectra
C        ANG(II,I,J):  Phase spectra
C
cxx      IMPLICIT  REAL*8(A-H)
cxx      IMPLICIT  COMPLEX*16(O-Z)
cc      DIMENSION  A(MJ,MJ,MJ1), E(MJ,MJ), C(0:20)
cc      DIMENSION  P(0:NF,MJ,MJ), FNC(0:NF,MJ,MJ), COH(0:NF,MJ,MJ)
cc      DIMENSION  AMP(0:NF,MJ,MJ), ANG(0:NF,MJ,MJ)
cc      DIMENSION  ZA(0:200,10,10), ZB(10,10)
cc      DIMENSION  FC(0:200), FS(0:200), WRK(10,10)
cxx      DIMENSION  A(L,L,M), E(L,L), C(0:M)
cxx      DIMENSION  P(0:NF,L,L), FNC(0:NF,L,L), COH(0:NF,L,L)
cxx      DIMENSION  AMP(0:NF,L,L), ANG(0:NF,L,L)
cxx      DIMENSION  ZA(0:NF,L,L), ZB(L,L)
cxx      DIMENSION  FC(0:NF), FS(0:NF), WRK(L,L)
C
      INTEGER M, L, NF
      DOUBLE PRECISION A(L,L,M), E(L,L), FNC(0:NF,L,L), AMP(0:NF,L,L),
     1                 ANG(0:NF,L,L), COH(0:NF,L,L)
      COMPLEX(KIND(0.0D0)) P(0:NF,L,L)
c local
      INTEGER I, II, IJ, J
      DOUBLE PRECISION C(0:M), FC(0:NF), FS(0:NF), FSUM
      COMPLEX(KIND(0.0D0)) ZA(0:NF,L,L), ZB(L,L), WRK(L,L), ZDET, SUM
C
cxx      DO 30 I=1,L
      DO 31 I=1,L
      DO 30 J=1,L
      C(0) = 0.0D0
      IF( I.EQ.J )  C(0) = 1.0D0
      DO 10 II=1,M
cxx   10 C(II) = -A(I,J,II)
      C(II) = -A(I,J,II)
   10 CONTINUE
C
      CALL  FOURIE( C(0),M+1,NF+1,FC(0),FS(0) )
C
      DO 20 II=0,NF
cxx   20 ZA(II,I,J) = DCMPLX( FC(II),FS(II) )
cxxx      ZA(II,I,J) = DCMPLX( FC(II),FS(II) )
      ZA(II,I,J) = CMPLX( FC(II),FS(II),KIND(0.0D0) )
   20 CONTINUE
   30 CONTINUE
   31 CONTINUE
C
      DO 200  II=0,NF
cxx      DO 40 I=1,L
      DO 41 I=1,L
      DO 40 J=1,L
cxx   40 ZB(I,J) = ZA(II,I,J)
      ZB(I,J) = ZA(II,I,J)
   40 CONTINUE
   41 CONTINUE
C
cc      CALL  CINV( ZB,ZDET,L,10 )
      CALL  CINV( ZB,ZDET,L )
C
cxx      DO 60 I=1,L
      DO 61 I=1,L
      DO 60 J=1,L
      SUM = (0.0D0,0.0D0)
      DO 50 IJ=1,L
cxx   50 SUM = SUM + ZB(I,IJ)*E(IJ,J)
      SUM = SUM + ZB(I,IJ)*E(IJ,J)
   50 CONTINUE
cxx   60 WRK(I,J) = SUM
      WRK(I,J) = SUM
   60 CONTINUE
   61 CONTINUE
cxx      DO 80 I=1,L
      DO 81 I=1,L
      DO 80 J=1,L
      SUM = 0.0D0
      DO 70 IJ=1,L
cxx   70 SUM = SUM + WRK(I,IJ)*DCONJG( ZB(J,IJ) )
cxx      SUM = SUM + WRK(I,IJ)*DCONJG( ZB(J,IJ) )
      SUM = SUM + WRK(I,IJ)*CONJG( ZB(J,IJ) )
   70 CONTINUE
cxx   80 P(II,I,J) = SUM
      P(II,I,J) = SUM
   80 CONTINUE
   81 CONTINUE
C
C  ...  Amplitude and phase  ...
C
cxx      DO 90 I=1,L-1
      DO 91 I=1,L-1
      DO 90 J=I+1,L
cxx        AMP(II,I,J) = DSQRT( DREAL(P(II,I,J))**2 + DIMAG(P(II,I,J))**2)
cxx        ANG(II,I,J) = DATAN( DIMAG(P(II,I,J))/DREAL(P(II,I,J)) )
        AMP(II,I,J) = DSQRT( REAL(P(II,I,J))**2 + AIMAG(P(II,I,J))**2)
        ANG(II,I,J) = DATAN( AIMAG(P(II,I,J))/REAL(P(II,I,J)) )
cxx        IF( DIMAG(P(II,I,J)).GT.0.0D0.AND.DREAL(P(II,I,J)).LT.0.0D0 )
        IF( AIMAG(P(II,I,J)).GT.0.0D0 .AND. REAL(P(II,I,J)).LT.0.0D0 )
     *        ANG(II,I,J) = ANG(II,I,J) + 3.1415926535D0
cxx        IF( DIMAG(P(II,I,J)).LT.0.0D0.AND.DREAL(P(II,I,J)).LT.0.0D0 )
        IF( AIMAG(P(II,I,J)).LT.0.0D0 .AND. REAL(P(II,I,J)).LT.0.0D0 )
     *        ANG(II,I,J) = ANG(II,I,J) - 3.1415926535D0
   90 CONTINUE
   91 CONTINUE
C
C  ...  Simple coherency  ...
C
cxx      DO 100 I=1,L-1
      DO 101 I=1,L-1
      DO 100 J=I+1,L
cxx        COH(II,I,J) = (DREAL(P(II,I,J))**2 + DIMAG(P(II,I,J))**2)/
cxx     *                (P(II,I,I)*P(II,J,J))
cxx        COH(II,I,J) = DREAL((DREAL(P(II,I,J))**2 + DIMAG(P(II,I,J))**2)/
cxx     *                (P(II,I,I)*P(II,J,J)))
        COH(II,I,J) = DBLE((REAL(P(II,I,J))**2 + AIMAG(P(II,I,J))**2)/
     *                (P(II,I,I)*P(II,J,J)))
  100 CONTINUE
  101 CONTINUE
C
C  ...  Power contribution  ...
C
      DO 120 I=1,L
      FSUM = 0.0D0
      DO 110 J=1,L
cxx      FSUM = FSUM + ZB(I,J)*DCONJG( ZB(I,J) )*E(J,J)
cxx      FSUM = FSUM + DREAL(ZB(I,J)*DCONJG( ZB(I,J) )*E(J,J))
      FSUM = FSUM + REAL(ZB(I,J)*CONJG( ZB(I,J) )*E(J,J))
cxx  110 FNC(II,I,J) = FSUM
      FNC(II,I,J) = FSUM
  110 CONTINUE
  120 CONTINUE
C
  200 CONTINUE
C
      RETURN
      E N D
cc      SUBROUTINE CINV( X,DET,M,MJ )
      SUBROUTINE CINV( X,DET,M )
C
C  ...  Inverse and determinant of a complex matrix X  ...
C
C       INPUTS:
C          X:     M*M SQUARE MATRIX
C          M:     DIMENSION OF X
C          MJ:    ABSOLUTE DIMENSION OF X IN THE MAIN PROGRAM
C
C       OUTPUTS:
C          X:     INVERSE OF X
C          DET:   DETERMINANT OF X
C
cxx      IMPLICIT  COMPLEX*16 (A-H,O-Z)
cc      DIMENSION  X(MJ,MJ), IND(100)
cxx      DIMENSION  X(M,M), IND(M)
C
      INTEGER M
      COMPLEX(KIND(0.0D0)) DET, X(M,M)
c local
      INTEGER I, IMAX, J, JJ, L, IND(M)
      COMPLEX(KIND(0.0D0)) XMAX, XTEMP
C
      DET = 1.0D0
      DO 60 L=1,M
      XMAX = 0.1D-10
      IMAX = 0
      DO 10 I=L,M
cxx      IF( CDABS( X(I,L) ).GT.CDABS( XMAX ) )  THEN
      IF( ABS( X(I,L) ).GT.ABS( XMAX ) )  THEN
         XMAX = X(I,L)
         IMAX = I
      END IF
   10 CONTINUE
      IND(L) = IMAX
      IF( IMAX.NE.L )  THEN
         IF( IMAX.LE.0 )  THEN
            DET = 0.0D0
            RETURN
         END IF
C
C     ROW INTERCHANGE
         DO 20 J=1,M
         XTEMP = X(IMAX,J)
         X(IMAX,J) = X(L,J)
cxx   20    X(L,J) = XTEMP
         X(L,J) = XTEMP
   20    CONTINUE
         DET = -DET
      END IF
      DET = DET*XMAX
      X(L,L) = 1.0D0
      DO 30 J=1,M
cxx   30 X(L,J) = X(L,J)/XMAX
      X(L,J) = X(L,J)/XMAX
   30 CONTINUE
      DO 50 I=1,M
      IF(I.NE.L)  THEN
         XTEMP  =X(I,L)
         X(I,L) = 0.0D0
         DO 40 J=1,M
cxx   40    X(I,J) = X(I,J) - XTEMP*X(L,J)
         X(I,J) = X(I,J) - XTEMP*X(L,J)
   40    CONTINUE
      END IF
   50 CONTINUE
   60 CONTINUE
C
C     COLUMN INTERCHANGE
      IF( M.GT.1 )  THEN
         DO 110 J=1,M-1
         JJ = IND(M-J)
         IF( JJ.NE.M-J )  THEN
            DO 100 I=1,M
            XTEMP    = X(I,JJ)
            X(I,JJ)  = X(I,M-J)
cxx  100       X(I,M-J) = XTEMP
            X(I,M-J) = XTEMP
  100       CONTINUE
         END IF
  110    CONTINUE
      END IF
      RETURN
      E N D

C     PROGRAM 9.1  SMOOTH
      SUBROUTINE SMOOTHF(Y,N,M,K,F,G,H,Q,R,XF,VF,NFE,NPE,OUTMIN,
     *                   OUTMAX,NMISS,N0,NN,XSS,VSS,FLK,AIC)
C
C  ...  Prediction and Interpolation of Time Series  ...
C
C     Inputs:
C        NFE:      End point of filtering
C        NPE:      End point of prediction
C        M:        Order of the AR model
C        TAU2:     Innovation variance of the AR model
C        AR(I):    AR coefficients (II=1,M)
C        OUTMIN,OUTMAX:  Lower and upper limits of observations
C        NMISS:    Number of missed intervals
C        N0(I):    Start position of missed intervals (I=1,NMISS)
C        NN(I):    Number of missed observations (I=1,NMISS)
C     Inputs required in subroutine READTS
C        TITLE:    Title of the data set
C        N:        Data length
C        Y(I):     Tiem series (I=1,N)
C     Parameters:
C        IDEV:     Input device specification
C        NMAX:     Adjustable dimension of Y, YMISS (NMAX >= N)
C        MJ:       Highest dimension of the state (MJ >= M)
C        K:        Dimension of the system noise
C        ISW:      =1 (R is specified)
C     @TEST.FILTER2:  SEP.08,1990, SEP.02,1992
C     MODIFIED  2/15/93
C
cc      PARAMETER( NMAX=1000,MJ=20,K=1,ISW=1,IDEV=1 )
cxxx      PARAMETER( ISW=1 )
      INTEGER, PARAMETER :: ISW=1
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(NMAX), YMISS(NMAX)
cc      DIMENSION  F(MJ,MJ), G(MJ), H(MJ), Q(K,K)
cc      DIMENSION  XPS(MJ,NMAX), XFS(MJ,NMAX), XSS(MJ,NMAX)
cc      DIMENSION  VPS(MJ,MJ,NMAX), VFS(MJ,MJ,NMAX), VSS(MJ,MJ,NMAX)
cc      DIMENSION  XF(MJ), VF(MJ,MJ)
cc      DIMENSION  N0(10), NN(10), AR(MJ)
cxx      DIMENSION  Y(N,L), YMISS(N,L)
cxx      DIMENSION  F(M,M), G(M,K), H(L,M), Q(K,K), R(L,L)
cxx      DIMENSION  XPS(M,NPE), XFS(M,NPE), XSS(M,NPE)
cxx      DIMENSION  VPS(M,M,NPE), VFS(M,M,NPE), VSS(M,M,NPE)
cxx      DIMENSION  XF(M), VF(M,M)
cxx      DIMENSION  N0(NMISS), NN(NMISS)
cxx      DIMENSION  YMEAN(L), YVAR(L)
C
      INTEGER N, M, K, NFE, NPE, NMISS, N0(NMISS), NN(NMISS)
      DOUBLE PRECISION Y(N), F(M,M), G(M,K), H(M), Q(K,K), R, XF(M),
     1                 VF(M,M), OUTMIN, OUTMAX, XSS(M,NPE),
     2                 VSS(M,M,NPE), FLK, AIC
c local
      INTEGER I, J, MJ, NMAX, NDIM, NS
      DOUBLE PRECISION YMISS(N), XPS(M,NPE), XFS(M,NPE), VPS(M,M,NPE),
     1                 VFS(M,M,NPE), YMEAN, YVAR, FF, SIG2
C
      NMAX = N
      MJ = M
      NDIM = NPE
C
cc      READ( 5,* )  NFE, NPE
cc      READ( 5,* )  M
cc      READ( 5,* )  TAU2
cc      READ( 5,* )  (AR(I),I=1,M)
cc      READ( 5,* )  OUTMIN, OUTMAX
cc      READ( 5,* )  NMISS
cc      DO 10 I=1,NMISS
cc   10 READ( 5,* )  N0(I), NN(I)
      NS = 1
C
C  ...  Read Time Series  ...
C
cc      CALL  READTS( IDEV,Y,N )
      CALL  MOMENT( Y,N,YMEAN,YVAR )
C
C  ...  SET MISSING OBSERVATIONS  ...
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      DO 20 I=1,N
cc   20 YMISS(I) = Y(I) - YMEAN
      YMISS(I) = Y(I) - YMEAN
   20 CONTINUE
cxx      DO 30 J=1,NMISS
      DO 31 J=1,NMISS
      DO 30 I=1,NN(J)
cc   30 YMISS(N0(J)+I-1) = OUTMIN
      YMISS(N0(J)+I-1) = OUTMIN
   30 CONTINUE
   31 CONTINUE
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  ...  Set State Space Model  ...
C
cc      CALL  SETFGH( M,MJ,K,AR,TAU2,F,G,H,Q,R )
C
C  ...  Set Initial State  ...
C
cc      CALL  ISTATE( M,MJ,0.0D0,YVAR,XF,VF )
C
C  ...  Kalman Filtering  ...
C
cc      CALL  FILTER( YMISS,XF,VF,F,G,H,Q,R,M,K,ISW,NS,NFE,NPE,MJ,NMAX,
cc     *              NMAX,OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,SIG2 )
      CALL  MFILTER( YMISS,N,XF,VF,F,G,H,Q,R,M,K,ISW,NS,NFE,NPE,
     *               NDIM,OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,SIG2 )
C
C  ...  Fixed Interval Smoothing  ...
C
cc      CALL  SMOOTH( F,M,MJ,NDIM,NS,NFE,NPE,VFS,VPS,XFS,XPS,
cc     *              VSS,XSS )
      CALL SMOOTH( F,M,NDIM,NS,NFE,NPE,VFS,VPS,XFS,XPS,VSS,XSS )
      FLK = -FF
      AIC = -2*FF + 2*(M+1)
C
C  ...  Plot and Print  ...
C
C      CALL  PTSTAT( Y,M,TAU2,FLK,AIC,XSS,VSS,N,NFE,NPE,MJ,
C     *              YMEAN,NMISS,N0,NN )
cc      CALL  PRSTAT( M,TAU2,FLK,AIC,XSS,VSS,YMEAN,MJ,NFE,NPE,
cc     *              NMISS,N0,NN )
C
      RETURN
      E N D
cc      SUBROUTINE  FILTER( Y,XF,VF,F,G,H,Q,R,M,K,ISW,NS,NFE,NPE,MJ,NMAX,
      SUBROUTINE  MFILTER( Y,N,XF,VF,F,G,H,Q,R,M,K,ISW,NS,NFE,NPE,NDIM,
     *                     OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,SIG2 )
C
C  ...  Kalman Filter (General Form, L>1)  ...
C
C     Inputs:
C        Y:      time series
C        NS:     Start position of filtering
C        NFE:    End position of filtering
C        NPE:    End position of prediction
C        XF:     Initial state vector
C        VF:     Initial covariance matrix
C        M:      Dimension of the state vector
C        K:      Dimension of the system noise
C        F:      M*M matrix
C        G:      M*K matrix
C        H:      M vector
C        Q:      K*K matrix, system noise covariance
C        R:      observation variance
C        ISW:    = 0;   R will be estimated
C                = 1;   R is given
C        MJ:     Adjustable dimension of XF, VF
C        NDIM:   Adjustable dimension of XFS, XPS, VFS, VPS
C                = 0   XF, XP, VF, VP are not stored
C                > 0   They are stored for smoothing
C        NMAX    Adjustable dimension of Y
C        OUTMIN: Lower limit for detecting outliers
C        OUTMAX: Upper limit for detecting outliers
C     Outputs:
C        VFS:    Covariance matrices of the filter
C        VPS:    Covariance matrices of the predictor
C        XFS:    Mean vectors of the filter
C        XPS:    Mean vectors of the predictor
C        FF:     Log likelihood
C        SIG2:   Estimated observational noise variance
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(NMAX)
cc      DIMENSION  F(MJ,MJ), G(MJ,K), H(MJ), Q(K,K)
cc      DIMENSION  XF(MJ), VF(MJ,MJ), XP(40), VP(40,40)
cc      DIMENSION  XFS(MJ,NDIM),    XPS(MJ,NDIM)
cc      DIMENSION  VFS(MJ,MJ,NDIM), VPS(MJ,MJ,NDIM)
cc      DIMENSION  WRK(40,40), VH(40), GAIN(40)
cxx      DIMENSION  Y(N,L)
cxx      DIMENSION  F(M,M), G(M,K), H(L,M), Q(K,K)
cxx      DIMENSION  XF(M), VF(M,M), XP(M), VP(M,M)
cxx      DIMENSION  XFS(M,NDIM), XPS(M,NDIM)
cxx      DIMENSION  VFS(M,M,NDIM), VPS(M,M,NDIM)
cxx      DIMENSION  WRK(M,M), WRK1(M,K), WRK2(L), VH(M,L), GAIN(M,L)
cxx      DIMENSION  R(L,L), PVAR(L,L), PERR(L)
C
      INTEGER N, M, K, ISW, NS, NFE, NPE, NDIM
      DOUBLE PRECISION Y(N), XF(M), VF(M,M), F(M,M), G(M,K), H(M),
     1                 Q(K,K),R, OUTMIN, OUTMAX, VFS(M,M,NDIM),
     2                 VPS(M,M,NDIM),XFS(M,NDIM), XPS(M,NDIM), FF, SIG2
c local
      INTEGER I, II, J, JJ, NSUM
      DOUBLE PRECISION XP(M), VP(M,M), WRK(M,M), WRK1(M,K), VH(M),
     1                 GAIN(M), PVAR, PERR, PI, SDET, SUM
C
      DATA   PI  /3.1415926535D0/
C
      SIG2 = 0.0D0
      SDET = 0.0D0
      NSUM = 0
C
      DO 500  II=NS,NPE
C
C  ...  ONE STEP AHEAD PREDICTION  ...
C
      DO 20  I=1,M
      SUM = 0.0D0
      DO 10  J=1,M
cxx   10 SUM = SUM + F(I,J)*XF(J)
      SUM = SUM + F(I,J)*XF(J)
   10 CONTINUE
cxx   20 XP(I) = SUM
      XP(I) = SUM
   20 CONTINUE
C
cxx      DO 40  I=1,M
      DO 41  I=1,M
      DO 40  J=1,M
      SUM = 0.0D0
      DO 30  JJ=1,M
cxx   30 SUM = SUM + F(I,JJ)*VF(JJ,J)
      SUM = SUM + F(I,JJ)*VF(JJ,J)
   30 CONTINUE
cxx   40 WRK(I,J) = SUM
      WRK(I,J) = SUM
   40 CONTINUE
   41 CONTINUE
C
cxx      DO 60  I=1,M
      DO 61  I=1,M
      DO 60  J=1,M
      SUM = 0.0D0
      DO 50 JJ=1,M
cxx   50 SUM = SUM + WRK(I,JJ)*F(J,JJ)
      SUM = SUM + WRK(I,JJ)*F(J,JJ)
   50 CONTINUE
cxx   60 VP(I,J) = SUM
      VP(I,J) = SUM
   60 CONTINUE
   61 CONTINUE
C
cxx      DO 80  I=1,M
      DO 81  I=1,M
      DO 80  J=1,K
      SUM = 0.0D0
      DO 70 JJ=1,K
cxx   70 SUM = SUM + G(I,JJ)*Q(JJ,J)
      SUM = SUM + G(I,JJ)*Q(JJ,J)
   70 CONTINUE
cc   80 WRK(I,J) = SUM
cxx   80 WRK1(I,J) = SUM
      WRK1(I,J) = SUM
   80 CONTINUE
   81 CONTINUE
C
cxx      DO 100  I=1,M
      DO 101  I=1,M
      DO 100  J=1,M
      SUM = VP(I,J)
      DO 90  JJ=1,K
cc   90 SUM = SUM + WRK(I,JJ)*G(J,JJ)
cxx   90 SUM = SUM + WRK1(I,JJ)*G(J,JJ)
      SUM = SUM + WRK1(I,JJ)*G(J,JJ)
   90 CONTINUE
cxx  100 VP(I,J) = SUM
      VP(I,J) = SUM
  100 CONTINUE
  101 CONTINUE
C
C  ...  FILTERING  ...
C
cc      IF( Y(II).GT.OUTMIN.AND.Y(II).LT.OUTMAX.AND. II.LE.NFE ) THEN
      IF( II.LE.NFE ) THEN
         IF  ( Y(II).GT.OUTMIN .AND. Y(II).LT.OUTMAX )  THEN
C
      DO 210  I=1,M
      SUM = 0.0D0
cc      DO 200  J=1,M
cc  200 SUM = SUM + VP(I,J)*H(J)
cc  210 VH(I) = SUM
      DO 200  JJ=1,M
cxx  200 SUM = SUM + VP(I,JJ)*H(J,JJ)
      SUM = SUM + VP(I,JJ)*H(JJ)
  200 CONTINUE
cxx  210 VH(I,J) = SUM
      VH(I) = SUM
  210 CONTINUE
C
      PERR = Y(II)
      PVAR = R
      DO 220  I=1,M
      PERR = PERR - H(I)*XP(I)
cxx  220 PVAR = PVAR + H(I)*VH(I)
      PVAR = PVAR + H(I)*VH(I)
  220 CONTINUE
C
      DO 250  I=1,M
cc  250 GAIN(I) = VH(I)/PVAR
      GAIN(I) = VH(I)/PVAR
  250 CONTINUE
C
      DO 290  I=1,M
cxx  290 XF(I) = XP(I) + GAIN(I)*PERR
      XF(I) = XP(I) + GAIN(I)*PERR
  290 CONTINUE
C
cxx      DO 310  I=1,M
      DO 311  I=1,M
      DO 310  J=1,M
cc  310 VF(I,J) = VP(I,J) - GAIN(I)*VH(J)
      VF(I,J) = VP(I,J) - GAIN(I)*VH(J)
  310 CONTINUE
  311 CONTINUE
C
      SIG2 = SIG2 + PERR**2/PVAR
      SDET = SDET + DLOG(PVAR)
      NSUM = NSUM + 1
C
C  ...  MISSING OBSERVATION  ...
C
      ELSE
cxx      DO 350  I=1,M
      DO 352  I=1,M
      XF(I) = XP(I)
      DO 350  J=1,M
cxx  350 VF(I,J) = VP(I,J)
      VF(I,J) = VP(I,J)
  350 CONTINUE
  352 CONTINUE
      END IF
C
      ELSE
cxx      DO 351  I=1,M
      DO 353  I=1,M
      XF(I) = XP(I)
      DO 351  J=1,M
cxx  351 VF(I,J) = VP(I,J)
      VF(I,J) = VP(I,J)
  351 CONTINUE
  353 CONTINUE
      END IF
C
C  ...  SAVE MEAN AND COVARIANCE  ...
C
      IF( NDIM.GT.1 )  THEN
cxx      DO 360  I=1,M
      DO 361  I=1,M
      XPS(I,II) = XP(I)
      XFS(I,II) = XF(I)
      DO 360  J=1,M
      VPS(I,J,II) = VP(I,J)
cxx  360 VFS(I,J,II) = VF(I,J)
      VFS(I,J,II) = VF(I,J)
  360 CONTINUE
  361 CONTINUE
      END IF
C
  500 CONTINUE
cc      SIG2 = SIG2/NSUM
cxxx      SIG2 = SIG2/DFLOAT(NSUM)
      SIG2 = SIG2/DBLE(NSUM)
      IF(ISW.EQ.0)  FF = -0.5D0*(NSUM*(DLOG(PI*2*SIG2) + 1) + SDET)
      IF(ISW.EQ.1)  FF = -0.5D0*(NSUM*(DLOG(PI*2) + SIG2) + SDET)
C
      RETURN
      E N D

C     PROGRAM 13.3  TVSPC
      SUBROUTINE TVSPC( N, M, NOBS, NF, IVAR, SIG2, AR, VAR, SP )
C
C  ...  CHANGING SPECTRUM ...
C
C     INPUTS:
C        N:    NUMBER OF POINTS
C        M:    AR ORDER
C        K:    TREND ORDER
C        NOBS: SAMPLING INTERVAL
C        IVAR=1:  FOR VARIANCE CORRECTION
C        AR(I,J): TIME VARYING AR COEFFICIENT
C        VAR(I):  TIME VARYING VARIANCE
C     PARAMETRS:
C        NMAX: ADJUSTABLE DIMENSION OF AR (NMAX >= N)
C        MJ:   ADJUSTABLE DIMENSION OF AR (MJ   >= M)
C        NF:   NUMBER OF FREQUENCIES
C     @TEST.UNI:  5/16/89  1/07/91  9/09/92
C
cc      PARAMETER( NMAX=200,NF=200,MJ=10,IDEV1=2,IDEV2=3 )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      CHARACTER  TITLE*72,  VNAME*8
cc      DIMENSION  AR(MJ,NMAX), VNAME(5), VALUE(5), VAR(3000)
cxx      DIMENSION  AR(M,N), VAR(N*NOBS), SP(0:NF,N)
cc      COMMON  /CMDATA/  TITLE
cc      DATA    VAR /3000*1.0D0/
C
      INTEGER N, M, NOBS, NF, IVAR
      DOUBLE PRECISION SIG2, AR(M,N), VAR(N*NOBS), SP(0:NF,N)
c local
      INTEGER NN
C
cc      OPEN( IDEV1,FILE='A:$LASERLIB$TEMP3.DAT' )
cc      READ(IDEV1,1) TITLE
cc      READ(IDEV1,*)  N, M, K, NOBS, IVAR
cc      READ(IDEV1,*)  SIG2
cc      DO 10 J=1,N
cc   10 READ(IDEV1,*) (AR(I,J),I=1,M)
cc      CLOSE( IDEV1 )
      NN = N*NOBS
cc      IF(IVAR.EQ.1 )  OPEN( IDEV2,FILE='A:$LASERLIB$TEMP4.DAT' )
cc      IF(IVAR.EQ.1 )  READ( IDEV2,*) (VAR(I),I=1,NN)
cc      IF(IVAR.EQ.1 )  CLOSE( IDEV2 )
cc    1 FORMAT( A72 )
      IF( IVAR.NE.1 )  THEN
cxx         DO 100 I = 1,NN
cxx  100    VAR(I) = 1.0D0
         VAR(1:NN) = 1.0D0
      END IF
C
C  ...  PLOT CHANGING SPECTRUM  ...
C
C
cc      VNAME(1) = 'N      ='
cc      VNAME(2) = 'M      ='
cc      VNAME(3) = 'K      ='
cc      VNAME(4) = 'NOBS   ='
cc      VNAME(5) = 'SIG2   ='
cc      VALUE(1) = N
cc      VALUE(2) = M
cc      VALUE(3) = K
cc      VALUE(4) = NOBS
cc      VALUE(5) = SIG2
cc      CALL  PLOTS
C     call  plots( 1,0,0,1,0 )
C     call  form( 1 )
C     call  factor( 10.0 )
cc      CALL  HEADER( 'PROGRAM 12.2:  CHANGING SPECTRUM',32,5,VNAME,
cc     *              VALUE )
cc      CALL  PLOT( 3.0,2.0,-3 )
cc      CALL  PT3DSP( AR,SIG2,MJ,M,N,NOBS,VAR )
      CALL  PT3DSP( AR,SIG2,M,N,NOBS,NF,VAR,SP )

cc      CALL  PLOTE
C     call  plot( 0.0,0.0,999 )
      RETURN
      E N D
cc      SUBROUTINE  PT3DSP( A,SIG2,MJ,M,N,NOBS,VAR )
      SUBROUTINE  PT3DSP( A,SIG2,M,N,NOBS,NF,VAR,SP )
C
C  ...  COMPUTE AND DRAW TIME VARYING SPECTRUM  ...
C
C     INPUTS:
C        A,SIG2,M:  AR MODELS
C        N:         DETA LENGTH
C        NOBS:      LOCAL STATIONARY SPAN
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION A(MJ,N), SP(0:200), YH(2000), VAR(3000), B(20)
cxx      DIMENSION A(M,N), SP(0:NF,N), VAR(N*NOBS)
C
      INTEGER M, N, NOBS, NF
      DOUBLE PRECISION A(M,N), SIG2, VAR(N*NOBS), SP(0:NF,N)
c local
      INTEGER I, J
      DOUBLE PRECISION B
C
cc      ANGLE = 70.0
cc      NF = 200
C
cc      WX = 5.0
cc      WY = 3.0
cc      WZ =10.0
cc      X0 = 0.0
cc      X1 = 0.5
cc      DX = 0.5
cc      Z0 =  0.0
cc      Z1 =  N*NOBS
cc      DZ =  500.0
cc      K = 2
cc      CALL  ARMASP( A,M,B,0,SIG2,NF,SP )
      CALL  ARMASP( A,M,B,0,SIG2,NF,SP(0,1) )
cc      CALL  MAXMIN( SP(0),NF+1,Y0,Y1,DY )
C
C  ...  DRAW 3D-AXIS  ...
C
cc      CALL  PLOT3A( NF,M,WX,WY,WZ,ANGLE,X0,X1,DX,Y0,Y1,DY,Z0,Z1,
cc     *              DZ,KN,K,YH )
      DO 100 J=1,N
C
C  ...  SPECTRUM OF AR MODEL AT TIME J*NOBS   ...
C
cc      CALL  ARMASP( A(1,J),M,B,0,SIG2,NF,SP )
      CALL  ARMASP( A(1,J),M,B,0,SIG2,NF,SP(0,J) )
C
C  ...  DRAW SPECTRUM AND SHIFT THE ORIGIN  ...
C
      DO 10 I=0,NF
cc   10 SP(I) = SP(I) + DLOG10( VAR(J*NOBS-NOBS/2) )
cxx   10 SP(I,J) = SP(I,J) + DLOG10( VAR(J*NOBS-NOBS/2) )
      SP(I,J) = SP(I,J) + DLOG10( VAR(J*NOBS-NOBS/2) )
   10 CONTINUE
cc      CALL  PLOT3B( SP(0),NF+1,N,WX,WY,WZ,ANGLE,Y0,Y1,K,YH,ZS,ZC )
  100 CONTINUE
C      CALL  PLOT( -SNGL(ZC),-SNGL(ZS),-3 )
C
      RETURN
      E N D
     

C     PROGRAM 10.1  ARMAFT
      SUBROUTINE ARMAFT( Y0,N,M,L,MLMAX,IPARAM,AR0,CMA0,SIG2,FLK,AIC,
     *                   AR,CMA,IER )
C
C  ...  ARMA MODEL FITTING  ...
C
C     Inputs:
C        L:       AR Order (M <= 20)
C        M:       MA Order (M <= 20)
C        IPARAM:  =0    Use defalt initail values
C                 =1    Read intial values
C        AR(I):   AR coefficients (I=1,M3)
C     Parameters:
C        NMAX:    Adjustable dimension of Y
C        MJ:      Adjustable dimension of XF, VF, etc.
C     @TEST.FILTER2O    NOV.29,1990, SEP.02,1992
C
cc      PARAMETER( NMAX=1000,MJ=20 )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  AA(20), AR(20), PAR(20), CMA(20)
cxx      DIMENSION  AR0(M), CMA0(L)
cxx      DIMENSION  AA(M+L), AR(M), PAR(MLMAX), CMA(L)
cxx      DIMENSION  Y0(N), Y(N)
cc      COMMON  /C92825/  OUTMIN, OUTMAX
cc      COMMON  /C92826/  Y(NMAX)
cc      COMMON  /C92907/  ALIMIT
cc      COMMON  /C92908/  M, L, N
cc      COMMON  /C92909/  FLK, SIG2
cc      COMMON  / CCC /  ISW, IPR, ISMT, IDIF
C
      INTEGER N, M, L, MLMAX, IPARAM, IER 
      DOUBLE PRECISION Y0(N), AR0(M), CMA0(L), SIG2, FLK, AIC, AR(M),
     1                 CMA(L)
c local
      INTEGER I, IOPT, NSUM
      DOUBLE PRECISION Y(N), AA(M+L), PAR(MLMAX), ALIMIT, OUTMIN,
     1                 OUTMAX, YMEAN
      EXTERNAL  FFARMA
C
C  ...  Read Model Orders  ...
C
cc      READ( 5,* )  M, L, IPARAM
C
C  ...  Set Defalt Parameters  ...
C
cc      IPR  = 7
      ALIMIT = 0.95D0
cc      CALL  SPARA1( M,L,AR,CMA,OUTMIN,OUTMAX,IOPT )
      CALL  SPARA1( M,L,MLMAX,AR,CMA,OUTMIN,OUTMAX,IOPT )
      IF( IPARAM.EQ.1 )  THEN
         DO 5 I=1,M
cxx    5    AR(I) = AR0(I)
         AR(I) = AR0(I)
    5    CONTINUE
         DO 6 I=1,L
cxx    6    CMA(I) = CMA0(I)
         CMA(I) = CMA0(I)
    6    CONTINUE
cc         READ( 5,* )  (AR(I),I=1,M)
cc         READ( 5,* )  (CMA(I),I=1,L)
      END IF
C
C  ...  Read Time Series  ...
C
cc      CALL  READTS( 1,Y,N )
      DO 7 I = 1,N
cxx    7 Y(I) = Y0(I)
      Y(I) = Y0(I)
    7 CONTINUE
C
C  ...  Subtrac Mean Value  ...
C
cc      SUM = 0.0D0
cc      DO 10 I=1,N
cccxx   10 SUM = SUM + Y(I)
cc      SUM = SUM + Y(I)
cc   10 CONTINUE
cc      YMEAN = SUM/N
cxx      CALL  MEAN( Y,N,-1.0D30,1.0D30,SUM,YMEAN )
      CALL  MEAN( Y,N,-1.0D30,1.0D30,NSUM,YMEAN )
      DO 20 I=1,N
cxx   20 Y(I) = Y(I) - YMEAN
      Y(I) = Y(I) - YMEAN
   20 CONTINUE
C
cc      WRITE(6,*) M, L
cc      WRITE(6,*) (AR(I),I=1,M)
C
      CALL  PARCOR( AR,M,PAR )
      DO 30 I=1,M
cxx   30 AA(I) = DLOG( (ALIMIT+PAR(I))/(ALIMIT-PAR(I)) )
      AA(I) = DLOG( (ALIMIT+PAR(I))/(ALIMIT-PAR(I)) )
   30 CONTINUE
      CALL  PARCOR( CMA,L,PAR )
      DO 40 I=1,L
cxx   40 AA(M+I) = DLOG( (ALIMIT+PAR(I))/(ALIMIT-PAR(I)) )
      AA(M+I) = DLOG( (ALIMIT+PAR(I))/(ALIMIT-PAR(I)) )
   40 CONTINUE
C
C  ...  Maximum Likelihood Method  ...
C
cc      IF( IOPT.EQ.1 )  CALL  DAVIDN( FFARMA,AA,M+L,2 )
      ier = 0
      IF( IOPT.EQ.1 )  CALL  DAVIDN( FFARMA,AA,M+L,2,
     *    Y,N,M,L,MLMAX,OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
      if( ier.ne.0 ) return
C
      DO 50 I=1,M
cxx   50 PAR(I)  = ALIMIT*(DEXP(AA(I))-1.0D0)/(DEXP(AA(I))+1.0D0)
      PAR(I)  = ALIMIT*(DEXP(AA(I))-1.0D0)/(DEXP(AA(I))+1.0D0)
   50 CONTINUE
      CALL  ARCOEF( PAR,M,AR )
      DO 60 I=1,L
cxx   60 PAR(I)  = ALIMIT*(DEXP(AA(M+I))-1.0D0)/(DEXP(AA(M+I))+1.0D0)
      PAR(I)  = ALIMIT*(DEXP(AA(M+I))-1.0D0)/(DEXP(AA(M+I))+1.0D0)
   60 CONTINUE
      CALL  ARCOEF( PAR,L,CMA )
C
      AIC = -2*FLK + 2*(M+L+1)
C
C  ...  Print out the Maximum Likelihood Estimates  ...
C
cc      CALL  PRARMA( M,L,AR,CMA,SIG2,FLK,AIC )
C
cc      STOP
      RETURN
      E N D
C
C------------------------------
C  armafit, armafit2 commoon 
C------------------------------
C
cc      SUBROUTINE  SPARA1( M,L,AR,CMA,OUTMIN,OUTMAX,IOPT )
      SUBROUTINE  SPARA1( M,L,MLMAX,AR,CMA,OUTMIN,OUTMAX,IOPT )
C
C  ...  Set Default Parameters  ...
C
C     Inputs:
C       M:       AR order
C       L:       MA order
C     Outputs:
C       TAU2:    System noise variance
C       AR:      AR coefficients
C       OUTMIN:  Lower bound for outliers
C       OUTMAX:  Upper bound for outliers
C       IOPT:    (=1  MLE by numerical optimization)
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  AR(20), PAR(20), CMA(20)
cxx      DIMENSION  AR(M), PAR(MLMAX), CMA(L)
C
      INTEGER M, L, MLMAX, IOPT
      DOUBLE PRECISION AR(M), CMA(L), OUTMIN, OUTMAX
c local
      INTEGER I
      DOUBLE PRECISION PAR(MLMAX)
C
      DO 10 I=1,M
cxx   10 PAR(I) = -(-0.6D0)**I
      PAR(I) = -(-0.6D0)**I
   10 CONTINUE
      CALL  ARCOEF( PAR,M,AR )
      DO 20 I=1,L
cxx   20 PAR(I) = -(-0.6D0)**I
ccc      PAR(I) = -(-0.6D0)**I
      PAR(I) = -(-0.5D0)**I
   20 CONTINUE
      CALL  ARCOEF( PAR,L,CMA )
C
      OUTMIN = -1.0D30
      OUTMAX =  1.0D30
      IOPT = 1
C
      RETURN
      E N D
cc      SUBROUTINE  FILTR3( Y,XF,VF,A,B,C,M,MJ,NS,N,OUTMIN,OUTMAX,
cxx      SUBROUTINE  FILTR3( Y,XF,VF,A,B,C,M,NS,N,OUTMIN,OUTMAX,
      SUBROUTINE  FILTR3( Y,XF,VF,A,B,M,NS,N,OUTMIN,OUTMAX,
     *                    FF,OVAR )
C
C  ...  Kalman filter  ...
C
C     Inputs:
C        Y:      time series
C        N:      data length
C        NS:     Start position of filtering
C        XF:     Initial state vector
C        VF:     Initial covariance matrix
C        A:      Parameters of the matrix F
C        B:      Parameters of the matrix G
C        C:      Parameters of the matrix H
C        K:      Dimension of the state vector
C        MJ:     Adjustable dimension of XF, VF
C        OUTMIN: Lower limit for detecting outliers
C        OUTMAX: Upper limit for detecting outliers
C     Outputs:
C        FF:     Log likelihood
C        SIG2:   Estimated variance
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  A(MJ), B(MJ), C(MJ), Y(N)
cc      DIMENSION  XF(MJ), VF(MJ,MJ), XP(40), VP(40,40)
cc      DIMENSION  WRK(40,40), VH(40), GAIN(40)
cxx      DIMENSION  A(M), B(M), C(M), Y(N)
cxx      DIMENSION  XF(M), VF(M,M), XP(M), VP(M,M)
cxx      DIMENSION  WRK(M,M), VH(M), GAIN(M)
C
      INTEGER M, NS, N
      DOUBLE PRECISION Y(N), VF(M,M), A(M), B(M), OUTMIN, OUTMAX, FF,
     1                 OVAR
c local
      INTEGER I, II, J, NSUM
      DOUBLE PRECISION XF(M), XP(M), VP(M,M), WRK(M,M), VH(M), GAIN(M),
     1                 PI, SDET, PVAR, PERR
C
      DATA   PI  /3.1415926535D0/
C
      OVAR = 0.0D0
      SDET = 0.0D0
      NSUM = 0
C
      DO 300  II=NS,N
C
C  ...  ONE STEP AHEAD PREDICTION  ...
C
      XP(M) = A(M)*XF(1)
      DO 100  I=1,M-1
cxx  100 XP(I) = A(I)*XF(1) + XF(I+1)
      XP(I) = A(I)*XF(1) + XF(I+1)
  100 CONTINUE
C
cxx      DO 110  J=1,M
      DO 111  J=1,M
      WRK(M,J) = A(M)*VF(1,J)
      DO 110  I=1,M-1
cxx  110 WRK(I,J) = A(I)*VF(1,J) + VF(I+1,J)
      WRK(I,J) = A(I)*VF(1,J) + VF(I+1,J)
  110 CONTINUE
  111 CONTINUE
C
cxx      DO 120  I=1,M
      DO 121  I=1,M
      VP(I,M) = WRK(I,1)*A(M)
      DO 120  J=1,M-1
cxx  120 VP(I,J) = WRK(I,1)*A(J) + WRK(I,J+1)
      VP(I,J) = WRK(I,1)*A(J) + WRK(I,J+1)
  120 CONTINUE
  121 CONTINUE
C
cxx      DO 140  I=1,M
      DO 141  I=1,M
      DO 140  J=1,M
cxx  140 VP(I,J) = VP(I,J) + B(I)*B(J)
      VP(I,J) = VP(I,J) + B(I)*B(J)
  140 CONTINUE
  141 CONTINUE
C
C  ...  FILTERING  ...
C
      IF( Y(II).GT.OUTMIN .AND. Y(II).LT.OUTMAX .AND. II.LE.N )  THEN
C
      DO 210  I=1,M
cxx  210 VH(I) = VP(I,1)
      VH(I) = VP(I,1)
  210 CONTINUE
C
      PVAR = VH(1)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      IF( PVAR.LE.1.0D-30 )  THEN
         FF = -1.0D20
         RETURN
      END IF
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      PERR = Y(II) - XP(1)
C
      DO 230  I=1,M
cxx  230 GAIN(I) = VH(I)/PVAR
      GAIN(I) = VH(I)/PVAR
  230 CONTINUE
C
      DO 250  I=1,M
cxx  250 XF(I) = XP(I) + GAIN(I)*PERR
      XF(I) = XP(I) + GAIN(I)*PERR
  250 CONTINUE
C
cxx      DO 260  J=1,M
      DO 261  J=1,M
      DO 260  I=1,M
cxx  260 VF(I,J) = VP(I,J) - GAIN(I)*VH(J)
      VF(I,J) = VP(I,J) - GAIN(I)*VH(J)
  260 CONTINUE
  261 CONTINUE
C
      OVAR = OVAR + PERR**2/PVAR
      SDET = SDET + DLOG(PVAR)
      NSUM = NSUM + 1
C
C  ...  MISSING OBSERVATION  ...
C
      ELSE
cxx      DO 270  I=1,M
      DO 271  I=1,M
      XF(I) = XP(I)
      DO 270  J=1,M
cxx  270 VF(I,J) = VP(I,J)
      VF(I,J) = VP(I,J)
  270 CONTINUE
  271 CONTINUE
      END IF
C
  300 CONTINUE
      OVAR = OVAR/NSUM
      FF = -0.5D0*(NSUM*DLOG(PI*2*OVAR) + SDET + NSUM)
C
      RETURN
      E N D
cc      SUBROUTINE  ISTAT3( M,L,AR,CMA,MJ,XF,VF )
      SUBROUTINE  ISTAT3( M,L,MM,AR,CMA,XF,VF,IER )
C
C  ...  Initial state ...
C
C     Inputs:
C        M:     AR order
C        L:     MA order
C        AR:    AR coefficient
C        CMA:   MA coefficient
C        MJ:    Adjustable dimension of F
C     Outputs:
C         XF:   State vector, X(0|0)
C         VF:   State covarance matrix, V(0|0)
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  AR(*), CMA(*)
cc      DIMENSION  XF(MJ), VF(MJ,MJ), COV(0:20), G(0:20)
cxx      DIMENSION  AR(M), CMA(L)
cxx      DIMENSION  XF(MM), VF(MM,MM), COV(0:MM), G(0:MM)
C
      INTEGER M, L, MM, IER 
      DOUBLE PRECISION AR(M), CMA(L), XF(MM), VF(MM,MM)
c local
      INTEGER I, I1, J, J1, JMIN
      DOUBLE PRECISION COV(0:MM), G(0:MM), SUM
C
cc      MM = MAX0( M,L+1 )
cc      DO 10  I=1,MJ
cxx      DO 10  I=1,MM
      DO 11  I=1,MM
      XF(I) = 0.0D0
cc      DO 10  J=1,MJ
      DO 10  J=1,MM
cxx   10 VF(I,J) = 0.0D0
      VF(I,J) = 0.0D0
   10 CONTINUE
   11 CONTINUE
C
cc      CALL  ARMCOV( M,L,AR,CMA,1.0D0,MM,COV )
      CALL  ARMCOV( M,L,AR,CMA,1.0D0,MM,COV,MM,IER )
      if( ier.ne.0 ) return
      CALL  IMPULS( M,L,AR,CMA,MM,G )
      VF(1,1) = COV(0)
      DO 50 I=2,MM
      SUM = 0.0D0
      DO 30 J=I,M
cxx   30 SUM = SUM + AR(J)*COV(J-I+1)
      SUM = SUM + AR(J)*COV(J-I+1)
   30 CONTINUE
      DO 40 J=I-1,L
cxx   40 SUM = SUM - CMA(J)*G(J-I+1)
      SUM = SUM - CMA(J)*G(J-I+1)
   40 CONTINUE
      VF(1,I) = SUM
cxx   50 VF(I,1) = SUM
      VF(I,1) = SUM
   50 CONTINUE
cxx      DO 100 I=2,MM
      DO 101 I=2,MM
      DO 100 J=I,MM
      SUM = 0.0D0
cxx      DO 60 I1=I,M
      DO 61 I1=I,M
      DO 60 J1=J,M
cxx   60 SUM = SUM + AR(I1)*AR(J1)*COV(IABS(J1-J-I1+I))
      SUM = SUM + AR(I1)*AR(J1)*COV(IABS(J1-J-I1+I))
   60 CONTINUE
   61 CONTINUE
cxx      DO 70 I1=I,M
      DO 71 I1=I,M
c  modified 2019/12/09 =============
ccc   DO 70 J1=J-I+I1,L
      JMIN = MAX(J-1,J-I+I1)
      DO 70 J1=JMIN,L
c ==================================
cxx   70 SUM = SUM - AR(I1)*CMA(J1)*G(IABS(J1-J-I1+I))
      SUM = SUM - AR(I1)*CMA(J1)*G(IABS(J1-J-I1+I))
   70 CONTINUE
   71 CONTINUE
cxx      DO 80 I1=J,M
      DO 81 I1=J,M
c  modified 2019/12/09 =============
ccc   DO 80 J1=I-J+I1,L
      JMIN = MAX( I-1,I-J+I1)
      DO 80 J1=JMIN,L
c ==================================
cxx   80 SUM = SUM - AR(I1)*CMA(J1)*G(IABS(J1-I-I1+J))
      SUM = SUM - AR(I1)*CMA(J1)*G(IABS(J1-I-I1+J))
   80 CONTINUE
   81 CONTINUE
c  modified 2019/12/09 =============
ccc      DO 90 I1=I-1,L
      DO 90 I1=I-1,L+I-J
ccc      SUM = SUM + CMA(I1)*CMA(J1)
      SUM = SUM + CMA(I1)*CMA(J-I+I1)
   90 CONTINUE
c ==================================
      VF(I,J) = SUM
cxx  100 VF(J,I) = SUM
      VF(J,I) = SUM
  100 CONTINUE
  101 CONTINUE
C
      RETURN
      E N D
cc      SUBROUTINE  SETABC( M,L,AR,CMA,A,B,C )
      SUBROUTINE  SETABC( M,L,AR,CMA,A,B,C,MM )
C
C  ...  State space model for Seasonal Adjustment  ...
C
C     Input:
C       M:     AR Order
C       L:     MA Order
C       AR:    AR coefficient
C       CMA:   MA coefficient
C     Outputs:
C       A,B,C: Parameters of F, G, H
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  A(*), B(*), C(*)
cc      DIMENSION  AR(*), CMA(*)
cxx      DIMENSION  A(MM), B(MM), C(MM)
cxx      DIMENSION  AR(M), CMA(L)
C
      INTEGER M, L, MM
      DOUBLE PRECISION AR(M), CMA(L), A(MM), B(MM), C(MM)
c local
      INTEGER I
C
cc      MM = MAX0( M,L+1 )
cxx      DO 10  I=1,MM
cxx      C(I) = 0.0D0
cxx      A(I) = 0.0D0
cxx   10 B(I) = 0.0D0
      C(1:MM) = 0.0D0
      A(1:MM) = 0.0D0
      B(1:MM) = 0.0D0

C
      DO 20  I=1,M
cxx   20 A(I) = AR(I)
      A(I) = AR(I)
   20 CONTINUE
      B(1) = 1.0D0
      DO 30 I=1,L
cxx   30 B(I+1) =-CMA(I)
      B(I+1) =-CMA(I)
   30 CONTINUE
      C(1) = 1.0D0
C
      RETURN
      E N D
cc      SUBROUTINE  FFARMA( K,AA,FF,GDUMMY,IFG )
cxx      SUBROUTINE  FFARMA( K,AA,FF,GDUMMY,IFG,
      SUBROUTINE  FFARMA( K,AA,FF,IFG,
     * Y,N,M,L,MM,OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
C
C  ... Function for Maximum likelihood estimation (seasonal model) ...
C
C     Inputs:
C        K:      Number of parameters
C        AA(I):  Parameter vector
C     Outputs:
C        FF:     -(Log likelihood)
C        IFG:    =1 if some conditions are violated
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  PAR(20), GDUMMY(20)
cc      DIMENSION  AA(20), AR(20), CMA(20)
cc      DIMENSION  A(20), B(20), C(20)
cc      DIMENSION  XF(40), VF(40,40)
cc      COMMON  /C92826/   Y(1000)
cc      COMMON  /C92825/   OUTMIN, OUTMAX
cc      COMMON  /C92907/  ALIMIT
cc      COMMON  /C92908/   M, L, N
cc      COMMON  /C92909/   FLK, SIG2
cxx      DIMENSION  PAR(MM)
cxx      DIMENSION  AA(K), AR(M), CMA(L)
cxx      DIMENSION  A(MM), B(MM), C(MM)
cxx      DIMENSION  XF(MM), VF(MM,MM)
cxx      DIMENSION  Y(N)
C
      INTEGER K, IFG, N, M, L, MM, IER
      DOUBLE PRECISION AA(K), FF, Y(N), OUTMIN, OUTMAX, ALIMIT, FLK,
     1                 SIG2
c local
      INTEGER I
      DOUBLE PRECISION PAR(MM), AR(M), CMA(L), A(MM), B(MM), C(MM),
     1                 XF(MM), VF(MM,MM)
C avoid floating-point exceptions
      INTEGER IFPLIM
cc      IFPLIM = 709
      IFPLIM = 87
C
cc      MJ  = 20
C
cc      MM = MAX0( M,L+1 )
c
      IER = 0
      DO 101 I=1,M
      IF( DABS(AA(I)).GT.IFPLIM ) IER = -1
cxx      IF( DABS(AA(I)).GT.20.0D0 )  GO TO 100
      IF( DABS(AA(I)).GT.30.0D0 )  GO TO 100
  101 CONTINUE
c
      DO 10 I=1,M
cxx   10 PAR(I)  = ALIMIT*(DEXP(AA(I))-1.0D0)/(DEXP(AA(I))+1.0D0)
      PAR(I)  = ALIMIT*(DEXP(AA(I))-1.0D0)/(DEXP(AA(I))+1.0D0)
   10 CONTINUE
      CALL  ARCOEF( PAR,M,AR )
      DO 20 I=1,L
cxx   20 PAR(I)  = ALIMIT*(DEXP(AA(M+I))-1.0D0)/(DEXP(AA(M+I))+1.0D0)
      PAR(I)  = ALIMIT*(DEXP(AA(M+I))-1.0D0)/(DEXP(AA(M+I))+1.0D0)
   20 CONTINUE
      CALL  ARCOEF( PAR,L,CMA )
      IFG = 0
C
cc      CALL  SETABC( M,L,AR,CMA,A,B,C )
cc      CALL  ISTAT3( M,L,AR,CMA,MJ,XF,VF )
cc      CALL  FILTR3( Y,XF,VF,A,B,C,MM,MJ,1,N,OUTMIN,OUTMAX,FLK,SIG2 )
      CALL  SETABC( M,L,AR,CMA,A,B,C,MM )
      CALL  ISTAT3( M,L,MM,AR,CMA,XF,VF,IER )
      if( ier.ne.0 ) return
cxx      CALL  FILTR3( Y,XF,VF,A,B,C,MM,1,N,OUTMIN,OUTMAX,FLK,SIG2 )
      CALL  FILTR3( Y,XF,VF,A,B,MM,1,N,OUTMIN,OUTMAX,FLK,SIG2 )
      FF = -FLK
      RETURN
C
  100 IFG = 1
      FF = 1.0D20
      RETURN
      E N D
cc      SUBROUTINE  DAVIDN( FUNCT, X, N, NDIF )
      SUBROUTINE  DAVIDN( FUNCT,X,N,NDIF, YY,NN,M,L,MLMAX,
     *                   OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
C
C  ...  6/20/83, 12/19/92
C
cxx      IMPLICIT  REAL*8( A-H,O-Z )
cc      DIMENSION  X(40), DX(40), G(40), G0(40), Y(40)
cc      DIMENSION  H(40,40), WRK(40), S(40)
cc      COMMON     / CCC /  ISW, IPR, ISMT, IDIF
cc      COMMON     / DDD /  XM , AIC , SD
cxx      DIMENSION  X(N), DX(N), G(N), G0(N), Y(N)
cxx      DIMENSION  H(N,N), WRK(N), S(N)
cxx      DIMENSION  YY(NN)
C
      INTEGER N, NDIF, NN, M, L, MLMAX, IER
      DOUBLE PRECISION X(N), YY(NN), OUTMIN, OUTMAX, ALIMIT, FLK, SIG2
c local
      INTEGER I, IDIF, IPR, ICOUNT, ICC, IG, ISW, J
      DOUBLE PRECISION DX(N), G(N), G0(N), Y(N), H(N,N), WRK(N), S(N),
     1                 TAU2, EPS1, EPS2, RAMDA, CONST1, SUM, S1, S2,
     2                 STEM, SS, ED, XM, XMB
C
      EXTERNAL  FUNCT
      DATA        TAU2  /          1.0D-6  /
      DATA  EPS1, EPS2  / 1.0D-6 , 1.0D-6  /
      RAMDA  = 0.5D0
      CONST1 = 1.0D-30
      IPR = 0
c-------   
      IDIF = NDIF
c-------
C
C          INITIAL ESTIMATE OF INVERSE OF HESSIAN
C
      ICOUNT = 0
 1000 CONTINUE
cxx 1000 DO 20  I=1,N
cxx      DO 10  J=1,N
cxx   10 H(I,J) = 0.0D0
cxx      S(I)   = 0.0D0
cxx      DX(I)  = 0.0D0
cxx   20 H(I,I) = 1.0D0
      H(1:N,1:N) = 0.0D0
      S(1:N)   = 0.0D0
      DX(1:N)  = 0.0D0
      DO 20  I=1,N
      H(I,I) = 1.0D0
   20 CONTINUE
      ISW = 0
C
cc      IF( NDIF.EQ.0 )  CALL  FUNCT( N,X,XM,G,IG )
cc      IF( NDIF.GE.1 )  CALL  FUNCND( FUNCT,N,X,XM,G,IG )
cxx      IF( NDIF.EQ.0 ) CALL  FUNCT ( N,X,XM,G,IG, YY,NN,M,L,MLMAX,
      IF( NDIF.EQ.0 ) CALL  FUNCT ( N,X,XM,IG, YY,NN,M,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
      IF( NDIF.GE.1 ) CALL  FUNCND( FUNCT,N,X,XM,G,IG,YY,NN,M,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,ISW,IDIF,FLK,SIG2,IER )
      if( ier.ne.0 ) return
C
cc      IF( IPR .GE. 2 )   WRITE( 6,640 )   XM, SD, AIC
cc      WRITE(6,650) (X(I),I=1,N), XM, (G(I),I=1,N)
C
      ICC = 0
C      ITERATION
 2000 CONTINUE
      ICC = ICC + 1
      IF( ICC .EQ. 1 )  GO TO 120
C
      DO 40  I=1,N
cxx   40 Y(I) = G(I) - G0(I)
      Y(I) = G(I) - G0(I)
   40 CONTINUE
      DO 60  I=1,N
      SUM = 0.0D0
      DO 50  J=1,N
cxx   50 SUM = SUM + Y(J)*H(I,J)
      SUM = SUM + Y(J)*H(I,J)
   50 CONTINUE
cxx   60 WRK(I) = SUM
      WRK(I) = SUM
   60 CONTINUE
      S1 = 0.0D0
      S2 = 0.0D0
      DO 70  I=1,N
      S1 = S1 + WRK(I)*Y(I)
cxx   70 S2 = S2 + DX(I) *Y(I)
      S2 = S2 + DX(I) *Y(I)
   70 CONTINUE
      IF( S1.LE.CONST1 .OR. S2.LE.CONST1 )  GO TO 900
C     IF( S1 .LE. S2 )   GO TO 100
C
C          UPDATE THE INVERSE OF HESSIAN MATRIX
C
C               ---  BROYDEN-FLETCHER-GOLDFARB-SHANNO TYPE CORRECTION  -
C
cxx  100 CONTINUE
      STEM = S1 / S2 + 1.0D0
cxx      DO 110  I=1,N
      DO 111  I=1,N
      DO 110  J=I,N
      H(I,J) = H(I,J)-(DX(I)*WRK(J)+WRK(I)*DX(J)-DX(I)*DX(J)*STEM)/S2
cxx  110 H(J,I) = H(I,J)
      H(J,I) = H(I,J)
  110 CONTINUE
  111 CONTINUE
C
  120 CONTINUE
      SS = 0.0D0
      DO 140  I=1,N
      SUM = 0.0D0
      DO 130  J=1,N
cxx  130 SUM = SUM + H(I,J)*G(J)
      SUM = SUM + H(I,J)*G(J)
  130 CONTINUE
      SS  = SS + SUM**2
cxx  140 S(I) = -SUM
      S(I) = -SUM
  140 CONTINUE
C
      S1 = 0.0D0
      S2 = 0.0D0
      DO 150  I=1,N
      S1 = S1 + S(I)*G(I)
cxx  150 S2 = S2 + G(I)*G(I)
      S2 = S2 + G(I)*G(I)
  150 CONTINUE
C     DS2 = DSQRT(S2)
C     GTEM = DABS(S1)/DS2
C     IF( GTEM.LE.TAU1 .AND. DS2.LE.TAU2 )  GO TO 900
      IF( S1.LT.0.0D0 )  GO TO 200
      DO 170  I=1,N
      DO 160  J=1,N
cxx  160 H(I,J) = 0.0D0
      H(I,J) = 0.0D0
  160 CONTINUE
      H(I,I) = 1.0D0
cxx  170 S(I) = -S(I)
      S(I) = -S(I)
  170 CONTINUE
  200 CONTINUE
C
      ED = XM
C
C          LINEAR  SEARCH
C
cc      CALL  LINEAR( FUNCT,X,S,RAMDA,ED,N,IG )
      CALL  LINEAR( FUNCT,X,S,RAMDA,ED,N,IG, YY,NN,M,L,
cxx     *         MLMAX,OUTMIN,OUTMAX,ALIMIT,ISW,IDIF,FLK,SIG2,IER )
     *         MLMAX,OUTMIN,OUTMAX,ALIMIT,ISW,FLK,SIG2,IER )
      if( ier.ne.0 ) return
C
cc      IF( IPR .GE. 2 )  WRITE( 6,630 )  RAMDA, ED, SD, AIC
C
      S1 = 0.0D0
      DO 210  I=1,N
      DX(I) = S(I)*RAMDA
      S1 = S1 + DX(I)*DX(I)
      G0(I) = G(I)
cxx  210 X(I) = X(I) + DX(I)
      X(I) = X(I) + DX(I)
  210 CONTINUE
      XMB = XM
      ISW = 0
C
cc      IF( NDIF.EQ.0 )  CALL  FUNCT( N,X,XM,G,IG )
cc      IF( NDIF.GE.1 )  CALL  FUNCND( FUNCT,N,X,XM,G,IG )
cxx      IF( NDIF.EQ.0 )  CALL  FUNCT ( N,X,XM,G,IG, YY,NN,M,L,MLMAX,
      IF( NDIF.EQ.0 )  CALL  FUNCT ( N,X,XM,IG, YY,NN,M,L,MLMAX,
     *                     OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
      IF( NDIF.GE.1 )  CALL  FUNCND( FUNCT,N,X,XM,G,IG,YY,NN,M,L,MLMAX,
     *                     OUTMIN,OUTMAX,ALIMIT,ISW,IDIF,FLK,SIG2,IER )
      if( ier.ne.0 ) return
cc      WRITE(6,650) (X(I),I=1,N), XM, (G(I),I=1,N)
C
      S2 = 0.D0
      DO 220  I=1,N
cxx  220 S2 = S2 + G(I)*G(I)
      S2 = S2 + G(I)*G(I)
  220 CONTINUE
      IF( DSQRT(S2) .LT. TAU2 )  GO TO 900
      IF( XMB/XM-1.D0.LT.EPS1 .AND. DSQRT(S1).LT.EPS2 )  GO TO 900
C     IF( ICC .GE. 5 )  GO TO 900
      GO TO 2000
  900 CONTINUE
      IF( IPR .LE. 0 )  RETURN
cc      WRITE( 6,600 )
cc      WRITE( 6,610 )  (X(I),I=1,N)
cc      WRITE( 6,620 )
cc      WRITE( 6,610 )  (G(I),I=1,N)
C
      ICOUNT  = ICOUNT + 1
      S2 = 0.D0
      DO 910  I=1,N
cxx  910 S2 = S2 + G(I)*G(I)
      S2 = S2 + G(I)*G(I)
  910 CONTINUE
      IF( S2.GT.1.0D0.AND.ICOUNT.LE.1 )   GO TO 1000
      RETURN
cxx  600 FORMAT( 1H0,'-----  X  -----' )
cxx  610 FORMAT( 1H ,10D13.5 )
cxx  620 FORMAT( 1H0,'***  GRADIENT  ***' )
cxx  630 FORMAT( 1H ,'LAMBDA =',D15.7,3X,'(-1)LOG LIKELIHOOD =',D23.15,
cxx     *        3X,'SD =',D22.15,5X,'AIC =',D23.15 )
cxx  640 FORMAT( 1H ,26X,'(-1)LOG-LIKELIHOOD =',D23.15,3X,'SD =',D22.15,
cxx     *        5X,'AIC =',D23.15 )
cxx  650 FORMAT( 10X,5F12.7 )
      E N D
cc      SUBROUTINE  FUNCND( FUNCT,M,A,F,G,IFG )
      SUBROUTINE  FUNCND( FUNCT,M,A,F,G,IFG, Y,N,MM,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,ISW,IDIF,FLK,SIG2,IER )
C
C  ...  FUNCTION EVALUATION AND NUMERICAL DIFFERENCING  ...
C
cxx      IMPLICIT   REAL*8( A-H,O-Z )
cc      DIMENSION  A(M) , G(M) , B(20),GD(5)
cc      COMMON  / CCC /  ISW , IPR, ISMT, IDIF
cc      COMMON  /CMFUNC/ DJACOB,FC,SIG2,AIC,FI,SIG2I,AICI,GI(20),GC(20)
cxx      DIMENSION  A(M) , G(M) , B(M)
cxx      DIMENSION  Y(N)
C
      INTEGER M, IFG, N, MM, L, MLMAX, ISW, IDIF, IER
      DOUBLE PRECISION A(M), F, G(M), Y(N), OUTMIN, OUTMAX, ALIMIT,
     1                 FLK, SIG2
c local
      INTEGER I, II
      DOUBLE PRECISION B(M), CONST, FB, FF
      EXTERNAL FUNCT
C
C     DATA       ICNT /0/
      CONST = 0.00001D0
C
cc      CALL  FUNCT( M,A,F,GD,IFG )
cxx      CALL  FUNCT( M,A,F,GD,IFG, Y,N,MM,L,MLMAX,
      CALL  FUNCT( M,A,F,IFG, Y,N,MM,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
      if( ier.ne.0 ) return
      FB = F
      IF( ISW .GE. 1 )   RETURN
C
C     WRITE( 6,600 )   (A(I),I=1,M)
      DO 10  I=1,M
cxx   10 B(I) = A(I)
      B(I) = A(I)
   10 CONTINUE
C
      DO 30  II=1,M
      B(II) = A(II) + CONST
cc      CALL  FUNCT( M,B,FF,GD,IFG )
cxx      CALL  FUNCT( M,B,FF,GD,IFG, Y,N,MM,L,MLMAX,
      CALL  FUNCT( M,B,FF,IFG, Y,N,MM,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
      if( ier.ne.0 ) return
      IF( IDIF .EQ. 1 )  GO TO 20
      B(II) = A(II) - CONST
cc      CALL  FUNCT( M,B,FB,GD,IFG )
cxx      CALL  FUNCT( M,B,FB,GD,IFG, Y,N,MM,L,MLMAX,
      CALL  FUNCT( M,B,FB,IFG, Y,N,MM,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
   20 G(II) = (FF-FB)/(CONST*IDIF)
      IF( G(II) .GT. 1.0D20 )  G(II) = (F-FB)/CONST
      IF( G(II) .LT.-1.0D20 )  G(II) = (FF-F)/CONST
      IF( FB.GT.F .AND. FF.GT.F )  G(II) = 0.0D0
cxx   30 B(II) = A(II)
      B(II) = A(II)
   30 CONTINUE
C
      RETURN
C
cxx  600 FORMAT( 3X,'---  PARAMETER  ---',(/,3X,5D13.5) )
cxx  610 FORMAT( 3X,'---  GRADIENT  ---',(/,3X,5D13.5) )
      E N D
cc      SUBROUTINE  LINEAR( FUNCT,X,H,RAM,EE,K,IG )
      SUBROUTINE  LINEAR( FUNCT,X,H,RAM,EE,K,IG, Y,N,M,L,
cxx     * MLMAX,OUTMIN,OUTMAX,ALIMIT,ISW,IDIF,FLK,SIG2,IER )
     * MLMAX,OUTMIN,OUTMAX,ALIMIT,ISW,FLK,SIG2,IER )

C
C  ...  LINEAR SEARCH  ...
C
cxx      IMPLICIT  REAL*8( A-H,O-Z )
cc      INTEGER  RETURN,SUB
cc      DIMENSION  X(1), H(1), X1(K)
cc      DIMENSION  G(40)
cc      COMMON     / CCC /  ISW , IPR, ISMT, IDIF
cxx      DIMENSION  X(K), H(K), X1(K)
cxx      DIMENSION Y(N)
C
      INTEGER K, IG, N, M, L, MLMAX, ISW, IER
      DOUBLE PRECISION X(K), H(K), RAM, EE, Y(N), OUTMIN, OUTMAX,
     1                 ALIMIT, FLK, SIG2
c local
      INTEGER I, IRET, ISUB, IFG, ire510
      DOUBLE PRECISION X1(K), CONST2, HNORM, RAM1, RAM2, RAM3, E1, E2,
     1                 E3, A1, A2, A3, B1, B2
      EXTERNAL FUNCT
C
C     IPR = 7
C     C1 = 1.0D-7
C     C2 = 1.0D-30
C
      ISW = 1
      IG  = 0
      RAM = 0.1D0
      CONST2 = 1.0D-60
   14 DO 15 I=1,K
cxx   15 IF( DABS(H(I)) .GT. 1.0D10 )  GO TO 16
      IF( DABS(H(I)) .GT. 1.0D10 )  GO TO 16
   15 CONTINUE
      GO TO 18
   16 DO 17 I=1,K
cxx   17 H(I) = H(I)*1.0D-10
      H(I) = H(I)*1.0D-10
   17 CONTINUE
      GO TO 14
   18 CONTINUE
      HNORM = 0.D0
      DO 10  I=1,K
cxx   10 HNORM = HNORM + H(I)**2
      HNORM = HNORM + H(I)**2
   10 CONTINUE
      HNORM = DSQRT( HNORM )
C
      RAM2 = RAM
      E1 =EE
      RAM1 = 0.D0
C
      DO 20  I=1,K
cxx   20 X1(I) = X(I) + RAM2*H(I)
      X1(I) = X(I) + RAM2*H(I)
   20 CONTINUE
cc      CALL  FUNCT( K,X1,E2,G,IG )
cxx      CALL  FUNCT( K,X1,E2,G,IG, Y,N,M,L,MLMAX,
      CALL  FUNCT( K,X1,E2,IG, Y,N,M,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
      if( ier.ne.0 ) return
cc      IF(IPR.GE.7)  WRITE(6,2)  RAM2,E2
C
      IF( IG .EQ. 1  )  GO TO 50
      IF( E2 .GT. E1 )  GO TO 50
   30 RAM3 = RAM2*4.D0
      DO 40  I=1,K
cxx   40 X1(I) = X(I) + RAM3*H(I)
      X1(I) = X(I) + RAM3*H(I)
   40 CONTINUE
cc      CALL  FUNCT( K,X1,E3,G,IG )
cxx      CALL  FUNCT( K,X1,E3,G,IG, Y,N,M,L,MLMAX,
      CALL  FUNCT( K,X1,E3,IG, Y,N,M,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
      if( ier.ne.0 ) return
       IF( IG.EQ.1 )  GO TO  500
cc      IF( IPR.GE.7 )  WRITE(6,3)  RAM3,E3
      IF( E3 .GT. E2 )  GO TO 70
c----- for debug
c         IF( E3 .EQ. E2 )  GO TO 70
c-----
      IF(RAM3.GT.1.0D10 .AND. E3.LT.E1)  GO TO 45
      IF(RAM3.GT.1.0D10 .AND. E3.GE.E1)  GO TO 46
      RAM1 = RAM2
      RAM2 = RAM3
      E1 = E2
      E2 = E3
      GO TO 30
C
   45 RAM = RAM3
      EE = E3
      RETURN
C
   46 RAM = 0.0D0
      RETURN
C
   50 RAM3 = RAM2
      E3 = E2
      RAM2 = RAM3*0.1D0
      IF( RAM2*HNORM .LT. CONST2 )  GO TO  400
      DO 60  I=1,K
cxx   60 X1(I) = X(I) + RAM2*H(I)
      X1(I) = X(I) + RAM2*H(I)
   60 CONTINUE
cc      CALL  FUNCT( K,X1,E2,G,IG )
cxx      CALL  FUNCT( K,X1,E2,G,IG, Y,N,M,L,MLMAX,
      CALL  FUNCT( K,X1,E2,IG, Y,N,M,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
      if( ier.ne.0 ) return
cc      IF(IPR.GE.7)  WRITE(6,4)  RAM2,E2
      IF( E2.GT.E1 )  GO TO 50
C
cc   70 ASSIGN 80 TO RETURN
   70 CONTINUE
      IRET = 80
      GO TO 200
C
   80 DO 90  I=1,K
cxx   90 X1(I) = X(I) + RAM*H(I)
      X1(I) = X(I) + RAM*H(I)
   90 CONTINUE
cc      CALL  FUNCT( K,X1,EE,G,IG )
cxx      CALL  FUNCT( K,X1,EE,G,IG, Y,N,M,L,MLMAX,
      CALL  FUNCT( K,X1,EE,IG, Y,N,M,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
      if( ier.ne.0 ) return
cc      IF(IPR.GE.7)  WRITE(6,5)  RAM,EE
C
      IFG = 0
cc      ASSIGN  300 TO  SUB
cc      ASSIGN 200 TO SUB
cc   95 ASSIGN 130 TO RETURN
      ISUB = 300
      ISUB = 200
   95 CONTINUE
      IRET = 130
      IF( RAM .GT. RAM2 )  GO TO 110
      IF( EE .GE. E2 )  GO TO 100
      RAM3 = RAM2
      RAM2 = RAM
      E3 =E2
      E2 =EE
cc      GO TO  SUB,( 200,300 )
      IF( ISUB.EQ.200 ) GO TO 200
      IF( ISUB.EQ.300 ) GO TO 300
C
  100 RAM1 = RAM
      E1 = EE
cc      GO TO  SUB,( 200,300 )
      IF( ISUB.EQ.200 ) GO TO 200
      IF( ISUB.EQ.300 ) GO TO 300
C
  110 IF( EE .LE. E2 )  GO TO 120
      RAM3 = RAM
      E3 = EE
cc      GO TO  SUB,( 200,300 )
      IF( ISUB.EQ.200 ) GO TO 200
      IF( ISUB.EQ.300 ) GO TO 300
C
  120 RAM1 = RAM2
      RAM2 = RAM
      E1 = E2
      E2 = EE
cc      GO TO  SUB,( 200,300 )
      IF( ISUB.EQ.200 ) GO TO 200
      IF( ISUB.EQ.300 ) GO TO 300
C
  130 DO 140  I=1,K
cxx  140 X1(I) = X(I) + RAM*H(I)
      X1(I) = X(I) + RAM*H(I)
  140 CONTINUE
cc      CALL  FUNCT( K,X1,EE,G,IG )
cxx      CALL  FUNCT( K,X1,EE,G,IG, Y,N,M,L,MLMAX,
      CALL  FUNCT( K,X1,EE,IG, Y,N,M,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
      if( ier.ne.0 ) return
cc      IF( IPR.GE.7 )  WRITE(6,6)  RAM,EE
cc      ASSIGN 200 TO SUB
      ISUB = 200
      IFG = IFG+1
cxx  600 FORMAT( 1H ,6D20.13 )
C     IF( HNORM*(RAM3-RAM1) .GT. 1.0D-12 )  GO TO 95
      IF( RAM3-RAM1 .GT. RAM2*1.0D-1 )  GO TO 95
C     IF( E1-EE .GT. 1.0D-15 )  GO TO 95
C     IF( E3-EE .GT. 1.0D-15 )  GO TO 95
C     IF(DX.LE.C1*XN+C2 .AND. DF.LE.C1*DABS(EE)+C2)  RETURN
C     IF( IFG .LE. 2 )  GO TO 95
C
      IF( E2 .LT. EE )  RAM = RAM2
      RETURN
C
C      -------  INTERNAL SUBROUTINE SUB1  -------
  200 IF( RAM3-RAM2 .GT. 5.0D0*(RAM2-RAM1) )  GO TO 202
      IF( RAM2-RAM1 .GT. 5.0D0*(RAM3-RAM2) )  GO TO 204
      A1 = (RAM3-RAM2)*E1
      A2 = (RAM1-RAM3)*E2
      A3 = (RAM2-RAM1)*E3
      B2 = (A1+A2+A3)*2.D0
      B1 = A1*(RAM3+RAM2) + A2*(RAM1+RAM3) + A3*(RAM2+RAM1)
      IF( B2 .EQ. 0.D0 )  GO TO 210
      RAM = B1 /B2
C
      IF( RAM .LE. RAM1 )  RAM = (RAM1 + RAM2)/2.0D0
      IF( RAM .GE. RAM3 )  RAM = (RAM2 + RAM3)/2.0D0
      IF( DABS(RAM-RAM2) .LE. 1.0D-15 )  RAM = (RAM2*4.D0 + RAM3)/5.0D0
cc      GO TO RETURN ,( 80,130 )
      IF( IRET.EQ.80 ) GO TO 80
      IF( IRET.EQ.130 ) GO TO 130
  202 RAM = (4.0D0*RAM2 + RAM3)/5.0D0
cc      GO TO RETURN, (80,130)
      IF( IRET.EQ.80 ) GO TO 80
      IF( IRET.EQ.130 ) GO TO 130
  204 RAM = (RAM1 + 4.0D0*RAM2)/5.0D0
cc      GO TO RETURN, (80,130)
      IF( IRET.EQ.80 ) GO TO 80
      IF( IRET.EQ.130 ) GO TO 130
C
  210 IG = 1
      RAM = RAM2
      RETURN
C
C      -------  INTERNAL SUBROUTINE SUB2  -------
C
  300 IF( RAM3-RAM2 .GT. RAM2-RAM1 )  GO TO 310
      RAM = (RAM1+RAM2)*0.5D0
cc      GO TO RETURN ,( 80,130 )
      IF( IRET.EQ.80 ) GO TO 80
      IF( IRET.EQ.130 ) GO TO 130
C
  310 RAM = (RAM2+RAM3)*0.5D0
cc      GO TO RETURN ,( 80,130 )
      IF( IRET.EQ.80 ) GO TO 80
      IF( IRET.EQ.130 ) GO TO 130
C
  400 RAM = 0.D0
      RETURN
C
  500 RAM = (RAM2+RAM3)*0.5D0
cxx 19/12/07
         ire510 = 0
cxx
  510 DO 520  I=1,K
cxx  520 X1(I) = X(I) + RAM*H(I)
      X1(I) = X(I) + RAM*H(I)
  520 CONTINUE
cc      CALL  FUNCT( K,X1,E3,G,IG )
cxx      CALL  FUNCT( K,X1,E3,G,IG, Y,N,M,L,MLMAX,
      CALL  FUNCT( K,X1,E3,IG, Y,N,M,L,MLMAX,
     *                    OUTMIN,OUTMAX,ALIMIT,FLK,SIG2,IER )
      if( ier.ne.0 ) return
cc      IF( IPR.GE.7 )  WRITE(6,7)  RAM,E3
      IF( IG.EQ.1 )  GO TO 540
      IF( E3.GT.E2 )  GO TO 530
      RAM1 = RAM2
      RAM2 = RAM
      E1 = E2
      E2 = E3
      GO TO 500
C
  530 RAM3 = RAM
      GO TO 70
C
  540 RAM = (RAM2+RAM)*0.5D0
cxx      GO TO 510
cxx 19/12/07
         if (RAM .EQ. RAM2) return
         ire510 = ire510 + 1
         if (ire510 .le. 100) GO TO 510
C
C ------------------------------------------------------------
cxx    1 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E1 =',D25.17 )
cxx    2 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E2 =',D25.17 )
cxx    3 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E3 =',D25.17 )
cxx    4 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E4 =',D25.17 )
cxx    5 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E5 =',D25.17 )
cxx    6 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E6 =',D25.17 )
cxx    7 FORMAT( 1H ,'LAMBDA =',D18.10, 10X,'E7 =',D25.17 )
      E N D

C     PROGRAM 7.1  ARFIT
ccx      SUBROUTINE ARFITF( Y,N,LAG,NF,ISW,SIG2,AIC,MAR,A,PAR,SP )
      SUBROUTINE ARFIT( Y,N,LAG,NF,MJ2,ISW,SIG2,AIC,MAR,A,PAR,SP )
C
C  ...  AR model fitting  ...
C
C     Inputs:
C        IDEV:    Input device for time series
C        LAG:     Highest order of AR model
C        ISW:     Estimation procedure
C                 = 1:  Yule-Walker method
C                 = 2:  Least squares (Householder) method
C                 = 3:  Partial autoregression method
C                 = 4:  PARCOR method
C                 = 5:  Burg's algorithm (MEM)
C     The following inputs are required in the subroutine READTS.
C        TITLE:   Caption of the data set
C        N:       Data length
C        FORMAT:  Reading format
C        Y(I):    Time series, (i=1,...,N)
C     Parameters:
C        NMAX:    Adjustable dimension of Y (NMAX.GE.N)
C        MJ,MJ2:  Adjustable dimensions
C        NF:      Number of frequencies for computing spectrum
C     @TEST.PN71:  12/12/90,1/7/91 Y.I. and G.K. 9/4/91
C
cc      PARAMETER( NMAX=200,MJ=20,MJ2=100,NF=200,IDEV=1 )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION Y(NMAX), COV(0:MJ,4), SIG2(0:MJ), SP(0:NF)
cc      DIMENSION D(MJ2), X(MJ2,MJ+1)
cc      DIMENSION AIC(0:MJ), A(MJ,MJ), B(MJ), PAR(MJ)
cc      DIMENSION FE(NMAX), BE(NMAX)
cxx      DIMENSION Y(N), COV(0:LAG,4), SIG2(0:LAG), SP(0:NF)
cxx      DIMENSION X(MJ2,LAG+1)
cxx      DIMENSION AIC(0:LAG), A(LAG,LAG), B(LAG+1), PAR(LAG)
cxx      DIMENSION FE(N), BE(N)
c
      INTEGER N, LAG, NF, MJ2, ISW, MAR
      DOUBLE PRECISION Y(N), SIG2(0:LAG), AIC(0:LAG), A(LAG,LAG), 
     1                 PAR(LAG), SP(0:NF)
c local
      INTEGER I, NSUM
      DOUBLE PRECISION COV(0:LAG,4), X(MJ2,LAG+1), B(LAG+1), FE(N),
     1                 BE(N), OUTMIN, OUTMAX, YMEAN
c
      EXTERNAL  SETXAR
      DATA  OUTMIN/-1.0D30/, OUTMAX/1.0D30/
c
      X(1:MJ2,1:LAG+1) = 0.0D0
      PAR(1:LAG) = 0.0D0
cc      READ(5,*)  LAG, ISW
cc      CALL  READTS( IDEV,Y,N )
      CALL  MEAN( Y,N,-1.0D30,1.0D30,NSUM,YMEAN )
C
C  ...  Yule-Walker method  ...
C
      IF( ISW.EQ.1 )  THEN
cc         CALL  UNICOR( Y,N,LAG,OUTMIN,OUTMAX,COV )
         CALL  UNICOR( Y,N,LAG,OUTMIN,OUTMAX,COV,YMEAN )
         CALL  ARYULE( COV,N,LAG,SIG2,AIC,PAR,A,MAR)
      END IF
C
C  ...  Householder method  ...
C
      IF( ISW.EQ.2 )  THEN
cc         CALL  REDUCT( SETXAR,Y,D,N-LAG,0,LAG,MJ2,X )
cc         CALL  REGRES( X,LAG,N-LAG,MJ2,MJ,A,SIG2,AIC,MAR )
         CALL  REDUCT( SETXAR,Y,N-LAG,0,LAG,MJ2,X )
         CALL  REGRES( X,LAG,N-LAG,MJ2,A,SIG2,AIC,MAR )
cxx         CALL  PARCOR( A(1,MAR),MAR,PAR )
         CALL  PARCOR( A(1,LAG),LAG,PAR )
      END IF
C
C  ...  PARCOR method  ...
C
      IF( ISW.GE.3 )  THEN
         CALL  ARPCOR( Y,FE,BE,SIG2,AIC,LAG,N,PAR,ISW-2,MAR )
         DO 10 I=1,LAG
cxx   10    CALL  ARCOEF( PAR,I,A(1,I) )
            CALL  ARCOEF( PAR,I,A(1,I) )
   10    CONTINUE
      END IF
C
C  ...  Power spectrum  ...
C
      CALL  ARMASP( A(1,MAR),MAR,B,0,SIG2(MAR),NF,SP )
C
cc      CALL  PRARSP( N,LAG,A,MAR,PAR,SIG2,AIC,SP,MJ,NF,ISW )
C      CALL  PTAR( PAR,AIC,SP,LAG,NF,MAR,SIG2,ISW )
cc      STOP
      RETURN
      E N D
      SUBROUTINE  ARPCOR( Y,FE,BE,SIG2,AIC,K,N,PARCOR,ISW,MAR )
C
C  ...  PARCOR method  ...
C
C     Inputs:
C        Y(I):    Time series
C        N:       Data length
C        K:       Highest AR order
C        ISW:     Estimation procedure
C                 = 1:   Partial autoregression
C                 = 2:   PARCOR
C                 = 3:   Burg's algorithm (MEM)
C        FE,BE:   Working area
C     Outputs:
C        SIG2(I): Innovation variance
C        AIC(I):  AIC
C        PARCOR(I):  PARCOR
C        MAR:     MAICE order
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  Y(N), FE(N), BE(N)
cxx      DIMENSION  SIG2(0:K), AIC(0:K), PARCOR(K)
cc      DIMENSION  A(50), B(50), FA(50), BA(50)
cxx      DIMENSION  A(K), B(K), FA(K), BA(K)
C
      INTEGER K, N, ISW, MAR
      DOUBLE PRECISION Y(N), FE(N), BE(N), SIG2(0:K), AIC(0:K),
     1                 PARCOR(K)
c local
      INTEGER I, L, M
      DOUBLE PRECISION A(K), B(K), FA(K), BA(K), PI, SUM, AICM, FB, FF,
     1                 BB, FE0, X
C
      PI = 3.1415926535D0
C
      SUM = 0.0D0
      DO 5 I=K+1,N
cxx    5 SUM = SUM + Y(I)**2
      SUM = SUM + Y(I)**2
    5 CONTINUE
      SIG2(0) = SUM/(N-K)
      AIC(0) = (N-K)*( DLOG(2*PI) + 1+ DLOG(SIG2(0))) + 2
      MAR = 0
      AICM = AIC(0)
C
      DO 10 I=1,N
      FE(I) = Y(I)
cxx   10 BE(I) = Y(I)
      BE(I) = Y(I)
   10 CONTINUE
C
      DO 100 M=1,K
      FB = 0.0D0
      FF = 0.0D0
      BB = 0.0D0
      DO 20 I=M+1,N
      FB = FB + FE(I)*BE(I-M)
      FF = FF + BE(I-M)**2
cxx   20 BB = BB + FE(I)**2
      BB = BB + FE(I)**2
   20 CONTINUE
C
      IF( ISW.EQ.1) THEN
        A(M) = FB/FF
        B(M) = FB/BB
      END IF
      IF( ISW.EQ.2 ) THEN
        A(M) = FB/(DSQRT(FF*BB))
        B(M) = FB/(DSQRT(FF*BB))
      END IF
      IF( ISW.EQ.3 ) THEN
        A(M) = FB/((FF+BB)/2)
        B(M) = FB/((FF+BB)/2)
      END IF
C
      DO 50 L=1,M-1
      A(L) = FA(L)-A(M)*BA(M-L)
cxx   50 B(L) = BA(L)-B(M)*FA(M-L)
      B(L) = BA(L)-B(M)*FA(M-L)
   50 CONTINUE
      DO 60 L=1,M
      FA(L) = A(L)
cxx   60 BA(L) = B(L)
      BA(L) = B(L)
   60 CONTINUE
      DO 70 I=M+1,N
      FE0 = FE(I)
      FE(I) = FE(I)-A(M)*BE(I-M)
cxx   70 BE(I-M) = BE(I-M)-B(M)*FE0
      BE(I-M) = BE(I-M)-B(M)*FE0
   70 CONTINUE
C
      SUM = 0.0D0
      DO 80 I=K+1,N
cxx   80 SUM = SUM + FE(I)**2
      SUM = SUM + FE(I)**2
   80 CONTINUE
      X = SUM
      PARCOR(M) = A(M)
      SIG2(M) = X/(N-K)
      AIC(M) = (N-K)*( DLOG(2*PI) + 1+ DLOG(SIG2(M)))+ 2*(M+1)
      IF( AIC(M).LT.AICM )  THEN
         MAR  = M
         AICM = AIC(M)
      END IF
C
  100 CONTINUE
C
      RETURN
      E N D

# PROGRAM 11.2
trend <- function (y, trend.order = 1, tau2.ini = NULL, delta, plot = TRUE, ...)     
{
  n <- length(y)      # data length
  m <- trend.order
  if (m < 1)
    stop("Trend order must be a positive integer.")


# tau2.ini                 # initial variance of systen noise
# delta                  # delta for computing variance of system noise
  iopt <- 1               # search method
  if (is.null(tau2.ini)) {
    iopt  <- 0
    tau2.ini <- 0
    delta <- 0
  }

  z <- .Fortran(C_trend,
                as.double(y),
                as.integer(n),
                as.integer(m),
                as.integer(iopt),
                as.double(tau2.ini),
                as.double(delta),
                tau2 = double(1),
                sig2 = double(1),
                lkhood = double(1),
                aic = double(1),
                xss = double(n * m),
                vss = double(n * m * m),
                rs = double(n))

  xss <- array(z$xss, dim = c(m, n))
  trend <- xss[1, 1:n]
  vss <- array(z$vss, dim = c(m,m,n))
  vss <- vss[1,1,1:n]
  res <- z$rs

  ts.atr <- tsp(y)
    if (is.null(ts.atr) == FALSE) {
      trend <- ts(trend, start = ts.atr[1], frequency = ts.atr[3])
      res <- ts(res, start = ts.atr[1], frequency = ts.atr[3])
    }

  trend.out <- list(trend = trend, residual = res, tau2 = z$tau2,
                    sigma2 = z$sig2, llkhood = z$lkhood, aic = z$aic, cov = vss)
  class(trend.out) <- "trend"

  if(plot) {
    rdata <- deparse(substitute(y))
    eval(parse(text=paste(rdata, "<- y")))
    eval(parse(text=paste("plot.trend(trend.out,", rdata, ", ...)")))
  }

  return(trend.out)
}

print.trend <- function(x, ...)
{
  message(gettextf("\n tau2 \t\t%12.5e", x$tau2), domain = NA)
  message(gettextf(" sigma2 \t%12.5e", x$sigma2), domain = NA)
  message(gettextf(" log-likelihood\t%12.3f", x$llkhood), domain = NA)
  message(gettextf(" aic\t\t%12.3f\n", x$aic), domain = NA)
}

plot.trend <- function(x, rdata = NULL, ...)
{

    trend <- x$trend
    res <- x$res
    ts.atr <- tsp(trend)

    if (is.null(ts.atr) == TRUE) {
      xtime <- "time"
    } else {
      freq <- ts.atr[3]
      if (freq == 4 || freq == 12) {
        xtime <- "year"
      } else if (freq == 24) {
        xtime <- "day"
      } else if (freq == 52 || freq == 365/7) {
        xtime <- "week"
      } else if (freq == 365.25/7 || freq == 52.18) {
        xtime <- "week"
      } else {
        xtime <- "time"
      }
    }

    old.par <- par(no.readonly = TRUE)
    par(mfcol = c(2, 1), xaxs = "i")

    ylim <- range(x$trend, rdata, na.rm = TRUE)
    if (is.null(rdata) == TRUE) {
      mtitle <- paste("Trend component")
    } else {
      tsname <- deparse(substitute(rdata))
      mtitle <- paste(tsname, "and trend component")
      plot(rdata, type = "l", ylim = ylim, xlab = "", ylab = "", main = "", xaxt = "n", ...)
      par(new = TRUE)
    }
    plot(trend, type = "l", col = 2, ylim = ylim, xlab = xtime,
         ylab = "", main = mtitle, ...)

    plot(res, type = "h", xlab = xtime,  ylab = "",  main = "Residuals", ...)
    abline(h = 0)

    par(old.par)
}

C     PROGRAM  8.2  LSAR2
      SUBROUTINE LSAR2( Y,N,K,N0,N1,N2,NE, AICS,AICMIN,MMIN )
C
C  ...  Estimation of the change point  ...
C
C     Inputs:
C        K:       Highest order od AR model
C        [N0,NE]: Time interval used for model fitting
C        [N1,N2]: Candidate for change point
C     The following inputs are required in the subroutine READTS.
C        TITLE:   Caption of the data set
C        N:       Data length
C        FORMAT:  Reading format
C        Y(I):    Time series, (i=1,...,N)
C     Parameters:
C        IDEV:    Input device for time series
C        NMAX:    Adjustable dimension of Y (NMAX.GE.N)
C        MJ,K,NSPAN:  Adjustable dimensions
C     MODIFIED  2/15/93
C
cc      PARAMETER(NMAX=3000,MJ=100,NSPAN=1000,KMAX=20,K1=KMAX+1,IDEV=1)
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(NMAX)
cc      DIMENSION  X(MJ,K1), AIC1(NSPAN), AIC2(NSPAN), AICS(NSPAN)
cxx      DIMENSION  Y(N)
cxx      DIMENSION  AIC1(N2-N1), AIC2(N2-N1), AICS(N2-N1)
C     DIMENSION  DATA(200)
C
      INTEGER N, K, N0, N1, N2, NE, MMIN
      DOUBLE PRECISION Y(N), AICS(N2-N1), AICMIN
c local
      INTEGER I, NS, M, MJ
      DOUBLE PRECISION AIC1(N2-N1), AIC2(N2-N1)
C
cc      READ( 5,* )  K, N0, N1, N2, NE
      NS = 1
      M = N2 - N1
c-----
cc      MJ = NE-N0
       MJ = NS + K + 1
c-----
C
C  ...  Read time series  ...
C
cc      CALL  READTS( IDEV,Y,N )
C
C  ...   First models and AIC1's  ...
C
cc      CALL  UPDATE( X,Y,N0,N1,M,NS,K,MJ,AIC1 )
      CALL  UPDATE( Y,N,N0,N1,M,NS,K,MJ,AIC1 )
C
C  ...   Second models and AIC2's  ...
C
cc      CALL  BUPDAT( X,Y,N2,NE,M,NS,K,MJ,AIC2 )
      CALL  BUPDAT( Y,N2,NE,M,NS,K,MJ,AIC2 )
C
C  ...   AICs of the locally stationary AR models  ...
C
      DO 20 I=1,M
cxx   20 AICS(I) = AIC1(I) + AIC2(I)
      AICS(I) = AIC1(I) + AIC2(I)
   20 CONTINUE
C
      AICMIN = 1.0D30
c-----
      MMIN = 1
c-----
      DO 60 I=1,M
      IF( AICS(I) .GT. AICMIN )  GO TO 60
         AICMIN = AICS(I)
         MMIN = I
   60 CONTINUE
C
C  ...   Print out and plot results  ...
C
cc      CALL  PRLSAR( AICS,AICMIN,MMIN,M,N0,N1,N2,NE,K )
C      CALL  PTLSAR( Y,AICS,AICMIN,MMIN,N,M,N0,N1,N2,NE,K )
C
cc      STOP
      RETURN
      E N D
cc      SUBROUTINE  UPDATE( X,Z,N0,N1,M,NS,K,MJ,AIC )
      SUBROUTINE  UPDATE( Z,N,N0,N1,M,NS,K,MJ,AIC )
C
C  ...  Fit AR models to first part of data  ...
C
C     Inputs:
C        X:     Working area
C        Z:     Time series
C        N0,N1,M,NS:  Fit AR models on [N0,N1],[N0,N1+NS],...,
C                                                    [N0,N1+(M-1)*NS]
C        K:     Highest order of AR models
C        MJ:    Adjustable dimension
C     Output:
C        AIC(I):  AIC of the AR model fitted on [N0,N1+(I-1)*NS]
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  X(MJ,1), AIC(1), D(200), A(20,20), AICS(0:20)
cc      DIMENSION  Z(N1), SIG2(0:20)
cxx      DIMENSION  X(MJ,K+1), AIC(M), A(K,K), AICS(0:K)
cxx      DIMENSION  Z(N), SIG2(0:K)
C
C
      INTEGER N, N0, N1, M, NS, K, MJ
      DOUBLE PRECISION Z(N), AIC(M)
c local
      INTEGER I, II, IMIN, NMK
      DOUBLE PRECISION SIG2(0:K), X(MJ,K+1), A(K,K), AICS(0:K)
      EXTERNAL  SETXAR
C
      NMK = N1 - K - N0
C
cc      CALL  REDUCT( SETXAR,Z,D,NMK,N0,K,MJ,X )
      CALL  REDUCT( SETXAR,Z,NMK,N0,K,MJ,X )   
C
      DO 100  I=1,M
      II = N1 + (I-1)*NS
c      CALL  REGRES( X,K,II-K-N0,MJ,20,A,SIG2,AICS,IMIN )
      CALL  REGRES( X,K,II-K-N0,MJ,A,SIG2,AICS,IMIN )
      AIC(I) = AICS(IMIN)
C
      CALL  SETXAR( Z,II-K,NS,K,MJ,1,X )
cc      CALL  HUSHL2( X,D,MJ,K+1+NS,K+1 )
      CALL  HUSHL2( X,MJ,K+1+NS,K+1 )
C
  100 CONTINUE
      RETURN
      E N D
cc      SUBROUTINE  BUPDAT( X,Z,N2,N,M,NS,K,MJ,AIC )
      SUBROUTINE  BUPDAT( Z,N2,N,M,NS,K,MJ,AIC )
C
C  ...  Fit AR models to the second part of data  ...
C
C     Inputs:
C        X:     Working area
C        Z:     Time series
C        N0,N2,N,M,NS:  Fit AR models on [N2-(M-1)*NS,N],...,
C                                           [N2-NS,S],[N2,N]
C        K:     Highest order of AR models
C        MJ:    Adjustable dimension
C     Output:
C        AIC(I):  AIC of the AR model fitted on [N0,N1+(I-1)*NS]
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  X(MJ,1), AIC(1), D(200), A(20,20), AICS(0:20)
cc      DIMENSION  Z(1), SIG2(0:20)
cxx      DIMENSION  X(MJ,K+1), AIC(M), A(K,K), AICS(0:K)
cx      DIMENSION  Z(1), SIG2(0:K)
cxx      DIMENSION  Z(N), SIG2(0:K)
C
      INTEGER N2, N, M, NS, K, MJ
      DOUBLE PRECISION Z(N), AIC(M)
c local
      INTEGER I, II, J, IMIN, NMK
      DOUBLE PRECISION SIG2(0:K), X(MJ,K+1), A(K,K), AICS(0:K)
      EXTERNAL  SETXAR
C
      NMK = N - N2
      J = M
C
cc      CALL  REDUCT( SETXAR,Z,D,NMK,N2-K-NS,K,MJ,X )
      CALL  REDUCT( SETXAR,Z,NMK,N2-K-NS,K,MJ,X )
C
      DO 100  I=1,M
      II = N2 - (I-2)*NS
cc      CALL  REGRES( X,K,N-II,MJ,20,A,SIG2,AICS,IMIN )
      CALL  REGRES( X,K,N-II,MJ,A,SIG2,AICS,IMIN )
      AIC(J) = AICS(IMIN)
      J = J - 1
C
      CALL  SETXAR( Z,II-K-NS,NS,K,MJ,1,X )
cc      CALL  HUSHL2( X,D,MJ,K+1+NS,K+1 )
      CALL  HUSHL2( X,MJ,K+1+NS,K+1 )
C
  100 CONTINUE
      RETURN
      E N D
cc      SUBROUTINE  HUSHL2( X,D,MJ1,N,K )
      SUBROUTINE  HUSHL2( X,MJ1,N,K )
C
C  ...  Householder transformation for adding new observations  ...
C
C     Inputs:
C        X:     Data matrix (Householder reduced form augmented with
C                            new observations)
C        D:     Working area
C        MJ1:   Adjustable dimension
C        N:     Number of rows of X
C        K:     Number of columns of X
C     Output:
C        X:     Householder reduced form
C
cxx      IMPLICIT  REAL*8 (A-H,O-Z)
cxx      DIMENSION  X(MJ1,K) , D(MJ1)
C
      INTEGER MJ1, N, K
      DOUBLE PRECISION X(MJ1,K)
c local
      INTEGER I, II, II1, J
      DOUBLE PRECISION D(MJ1), TOL, D1, H, G, S
C
           TOL = 1.0D-30
C
cxx      DO 100  II=1,K
      DO 101  II=1,K
         D1 = X(II,II)
         H  = D1**2
         DO 10  I=K+1,N
            D(I) = X(I,II)
cxx   10       H = H + D(I)**2
            H = H + D(I)**2
   10    CONTINUE
         IF( H .GT. TOL )  GO TO 20
         G = 0.0D00
         GO TO 100
   20    G = DSQRT( H )
         IF( D1 .GE. 0.0D00 )   G = -G
         H  = H - D1*G
         D1 = D1 - G
C
         IF( II .EQ. K )  GO TO 100
         II1 = II+1
         DO 60  J=II1,K
            S = D1*X(II,J)
            DO 40  I=K+1,N
cxx   40       S = S + D(I)*X(I,J)
               S = S + D(I)*X(I,J)
   40       CONTINUE
            S = S/H
            X(II,J) = X(II,J) - D1*S
            DO 50  I=K+1,N
cxx   50       X(I,J) = X(I,J) - D(I)*S
               X(I,J) = X(I,J) - D(I)*S
   50       CONTINUE
   60    CONTINUE
  100 X(II,II) = G
  101 CONTINUE
C
      RETURN
      E N D

# PROGRAM 9.1  SMOOTH
tsmooth <- function( y, f, g, h, q, r, x0 = NULL, v0 = NULL, filter.end = NULL,
                     predict.end = NULL, minmax = c(-1.0e+30, 1.0e+30),
                     missed = NULL, np = NULL, plot = TRUE, ...)
{
# y                    # time series
  n <- length(y)
  ymean <- mean(y)

  ff <- f
  if (is.matrix(ff) == FALSE)
    ff <- as.matrix(ff)
  m <- dim(ff)[1]
  if (dim(ff)[2] != m)
    stop("Error : state transition matrix F(n) is invalid.")

  gg <- g
  if (is.matrix(gg) == FALSE)
    gg <- as.matrix(gg)
  if (dim(gg)[1] != m)
    stop("Error : matrix G(n) is invalid.")
  k <- dim(gg)[2]      # dimension of the system noise

  qq <- q
  if (is.matrix(qq) == FALSE)
    qq <- as.matrix(qq)
  if (dim(qq)[1] != k || dim(qq)[2] != k)
    stop("Error : matrix Q(n) is invalid.")

  if (is.null(x0))
    x0 <- rep(0.0e0, m)
  if (is.null(v0)) {
    v0 <- matrix(0.0e0, m, m )
    for (i in 1:m)
      v0[i, i] <- 10.0e-5
  }

  if (is.null(filter.end))
    filter.end <- n    # end point of filtering
  if (is.null(predict.end))
    predict.end <- n   # end point of prediction
  if (filter.end > predict.end)
    stop("'filter.end' is less than or equal to n (data length)")

  outmin <- minmax[1]   # lower limits of observations
  outmax <- minmax[2]   # upper limits of observations

  startp <- missed
  if (is.null(startp) || is.null(np)) {
    nmiss <- 0
    startp <- 0
    np <- 0
  } else {
    nmiss <- min(length(startp), length(np)) # number of missed intervals
    startp <- startp[1:nmiss]               # start position of missed intervals
    np <- np[1:nmiss]                       # number of missed observations
  }

  npe <- predict.end

  z <- .Fortran(C_smoothf,
             as.double(y),
             as.integer(n),
             as.integer(m),
             as.integer(k),
             as.double(ff),
             as.double(gg),
             as.double(h),
             as.double(qq),
             as.double(r),
             as.double(x0),
             as.double(v0),
             as.integer(filter.end),
             as.integer(npe),
             as.double(outmin),
             as.double(outmax),
             as.integer(nmiss),
             as.integer(startp),
             as.integer(np),
             xss = double(m * npe),
             vss = double(m * m * npe),
             lkhood = double(1),
             aic = double(1))

  xss <- array(z$xss, dim = c(m, npe)) + ymean
  vss <- array(z$vss, dim = c(m, m, npe))
  cov.smooth <- array(,dim = c(m, npe))
  for (i in 1:m)
    cov.smooth[i, ] <- vss[i, i, ]
  err <- NULL

  if (filter.end < n) {
    err <- rep(0, npe)
    nps <- filter.end + 1
    for (i in nps:npe)
      err[i] <- y[i] - xss[1, i]
  }

  if (nmiss != 0) {
    err <- rep(0, npe)
    for (i in 1:n)
      err[i] <- y[i] - xss[1, i]
  }

  tsmooth.out <- list(mean.smooth = xss, cov.smooth = cov.smooth, 
                      esterr = err, llkhood = z$lkhood, aic = z$aic)
  class(tsmooth.out) <- c("smooth")

  if (plot) {
    rdata <- deparse(substitute(y))
    eval(parse(text=paste(rdata, "<- y")))
    eval(parse(text=paste("plot.smooth(tsmooth.out,", rdata, ", ...)")))
  }

  return(tsmooth.out)
}

print.smooth <- function(x, ...) {
  message(gettextf("\n log-likelihood\t%12.3f", x$llkhood), domain = NA)
  message(gettextf(" aic\t\t%12.3f\n", x$aic), domain = NA)
}

plot.smooth <- function(x, rdata = NULL, ...)
{
  if (is.null(rdata) == FALSE)
    tsname <- deparse(substitute(rdata))

  old.par <- par(no.readonly = TRUE)
  new.mar <- old.par$mar
  new.mar[3] <- new.mar[3] + 1

  xss <- x$mean.smooth
  xss1 <- xss[1, ]
  cov <- x$cov.smooth
  npe <- dim(xss)[2]

  ts.atr <- tsp(rdata)
  if (is.null(ts.atr) == FALSE)
    xss1 <- ts(xss1, start = ts.atr[1], frequency = ts.atr[3])

  c1 <- xss1 + sqrt(cov[1, ])
  c2 <- xss1 - sqrt(cov[1, ])

  err <- x$esterr
  if (is.null(rdata) == FALSE && is.null(err)) {
    err <- rep(0, npe)
    n <- length(rdata)
    for (i in 1:n)
      err[i] <- rdata[i] - xss[1, i]
    if (min(err) == 0 && max(err) == 0)
      err <- NULL
  }

  if (is.null(err) == FALSE)
    par(mfcol=c(2,1))
  par(mar = new.mar, xpd=T)

  mtitle <- paste("Mean vectors of the smoother and standard deviation")
  ylim <- c(floor(min(rdata, c1, c2)), ceiling((max(rdata, c1, c2))))
  plot(c1, type='l', ylim = ylim, col=4, xlab="", ylab="", ...)
  par(new = TRUE)
  plot(c2, type='l', ylim = ylim, col=4, xlab="", ylab="", ...)
  par(new = TRUE)
  plot(xss1, type = 'l', ylim = ylim, col = 2, main = mtitle, xlab="", ylab="",
       ...)
  par(new = TRUE)

  pos.y <- par()$usr[4]
  if (is.null(rdata) == FALSE) {
    if (is.null(ts.atr) == TRUE) {
      xlim <- range(1, npe)
      plot(rdata, type = 'l', xlim = xlim, ylim = ylim, xlab = "", ylab = "", ...)
    } else {
      plot(rdata, type = 'l', ylim = ylim, xlab = "", ylab = "", ...)
    }
    lg3 <- legend("topright", legend = c("mean", "+/- SD", paste(tsname)),
                 lty = c(1, 1, 1), x.intersp = 0.3,ncol = 3, cex = 0.7, 
                 plot = FALSE)
    pos.y <- pos.y + lg3$rect$h
    legend(lg3$rect$left, pos.y, legend = c("mean", "+/- SD", paste(tsname)),
                 lty = c(1, 1, 1), col = c(2, 4, 1), x.intersp = 0.3, ncol = 3,
                 cex = 0.7, bty = "n")
  } else {
    lg2 <- legend("topright", legend = c("mean", "+/- SD"), lty = c(1, 1),
                 x.intersp = 0.3, ncol = 2, cex = 0.7, plot = FALSE)
    pos.y <- pos.y + lg2$rect$h
    legend(lg2$rect$left, pos.y, legend = c("mean", "+/- SD"), lty = c(1, 1),
           col = c(2, 4), x.intersp = 0.3,  ncol = 2, cex = 0.7, bty = "n")
  }

  if (is.null(err) == FALSE) {
    if (is.null(ts.atr) == FALSE)
      err <- ts(err, start = ts.atr[1], frequency = ts.atr[3])
    plot(err, type = 'h', main = "estimation error", xlab = "",
         ylab = "", ...)
  }
  par(old.par)
}

C     PROGRAM 4.2  KLINFO
      SUBROUTINE KLINFOF( IDISTG,PG,IDISTF,PF,XMIN,XMAX,NINT,DX,
     *                   FKLI,GINT )
C
C  ...  Driver program of the subroutine KLINFO  ...
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  PARAMG(3), PARAMF(3)
cxx      DIMENSION PG(2), PF(2)
cxx      DIMENSION NINT(4), DX(4), FKLI(4), GINT(4)
C
      INTEGER IDISTG, IDISTF, NINT(4)
      DOUBLE PRECISION PG(2), PF(2), XMIN, XMAX, DX(4), FKLI(4),
     1                 GINT(4), GAUSS, CAUCHY
c local
      INTEGER II
C
      EXTERNAL  GAUSS
      EXTERNAL  CAUCHY
cc      DATA  PARAMG /0.0D0, 1.0D0, 0.0D0/
cc      DATA  PARAMF /0.1D0, 1.5D0, 0.0D0/
C
cc      XMIN = -8.0D0
cc      XMAX =  8.0D0
cc      WRITE(6,600)  XMIN, XMAX
cc      WRITE(6,610)
      DO 10 II=1,4
cc      NINT = (XMAX-XMIN+1.0D-5)*2**(II-1)
cc   10 CALL KLINFO(GAUSS,GAUSS ,PAG,PAF,XMIN,XMAX,NINT,FKLI,GINT)
cxx         NINT(II) = (XMAX-XMIN+1.0D-5)*2**(II-1)
         NINT(II) = INT((XMAX-XMIN+1.0D-5)*2**(II-1))
         IF( IDISTG .EQ. 1 ) THEN
            IF( IDISTF .EQ. 1 ) THEN
               CALL KLINFO( GAUSS,GAUSS ,PG,PF,XMIN,XMAX,
     *                            NINT(II),DX(II),FKLI(II),GINT(II) )
            ELSE
               CALL KLINFO( GAUSS, CAUCHY,PG,PF,XMIN,XMAX,
     *                            NINT(II),DX(II),FKLI(II),GINT(II) )
            END IF
         ELSE
            IF( IDISTF .EQ. 1 ) THEN
               CALL KLINFO( CAUCHY,GAUSS ,PG,PF,XMIN,XMAX,
     *                            NINT(II),DX(II),FKLI(II),GINT(II) )
            ELSE
               CALL KLINFO( CAUCHY,CAUCHY,PG,PF,XMIN,XMAX,
     *                            NINT(II),DX(II),FKLI(II),GINT(II) )
            END IF
         END IF
   10 CONTINUE
cc      STOP
      RETURN
cxx  600 FORMAT( 1H ,'XMIN =',F5.1,3X,'XMAX =',F5.1 )
cxx  610 FORMAT( 1H ,'  XMIN  XMAX   NINT',3X,'DX',9X,'FKLI',11X,'GINT' )
      E N D
      SUBROUTINE  KLINFO( DISTG,DISTF,PARAMG,PARAMF,XMIN,XMAX,NINT,
cc     *                    FKLI,GINT )
     *                     DX,FKLI,GINT )  
C
C  ...  This subroutine computes Kullback-Leibler information  ...
C
C     Inputs:
C        DISTG:    function name for the true density
C        DISTF:    function name for the model density
C        PARAMG:   parameter vector of the true density
C        PARAMF:   parameter vector of the model density
C        XMIN:     lower limit of integration
C        XMAX:     upper limit of integration
C        NINT:     number of function evaluation
C     Outputs:
C        FKLI:     Kullback-Leibler information number, I(g;f)
C        GINT:     integration of g(y) over [XMIN,XMAX]
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cx      DIMENSION  PARAMG(*), PARAMF(*)
cxx      DIMENSION  PARAMG(2), PARAMF(2)
C
      INTEGER NINT
      DOUBLE PRECISION DISTG, DISTF, PARAMG(2), PARAMF(2), XMIN, XMAX,
     1                 DX, FKLI, GINT
c local 
      INTEGER I
      DOUBLE PRECISION XX, GX, FX 
C
cxxx      DX = (XMAX-XMIN)/DFLOAT(NINT)
      DX = (XMAX-XMIN)/DBLE(NINT)
      FKLI = 0.0D0
      GINT = 0.0D0
C
      DO 10  I=0,NINT
      XX = XMIN + DX*I
      GX = DISTG( XX,PARAMG )
      FX = DISTF( XX,PARAMF )
      IF( I.GE.1 .AND. I.LT.NINT )  THEN
         FKLI = FKLI + DLOG( GX/FX )*GX
         GINT = GINT + GX
      ELSE
         FKLI = FKLI + DLOG( GX/FX )*GX/2.0D0
         GINT = GINT + GX/2.0D0
      END IF
   10 CONTINUE
C
      FKLI = FKLI*DX
      GINT = GINT*DX
cc      WRITE(6,600) XMIN, XMAX, NINT, DX, FKLI, GINT
C
      RETURN
cxx  600 FORMAT( 1H ,2F6.2,I5,F9.4,D17.8,F12.8 )
      E N D

C     PROGRAM 7.3  MARLSQ
      SUBROUTINE MARLSQ( Y,N,L,LAG,A,V,LMAX,AIC )
C
C  ...  MAR model fitting (least squares method)  ...
C
C     Inputs:
C        MT:    Input device
C        LAG:   Heighest AR order
C     Inputs for READMD
C        TITLE: Tile of the data set
C        N:     Data length
C        L:     Dimension
C        IFM:   Control variable for reading order
C        FORM:  Reading format
C        Y(N,J):   ID-dimensional time series
C
cc      PARAMETER( MJ=1000,MJ1=200,MJ2=5,MJ3=20,MJ4=MJ2*(MJ3+1),IDEV=1 )
ccxx      PARAMETER( MJ1=200 )
cxx      IMPLICIT  REAL*8 (A-H,O-Z)
cc      DIMENSION  Y(MJ,MJ2), X(MJ1,MJ4), D(MJ1) 
cc      DIMENSION  A(MJ2,MJ2,MJ3) , V(MJ2,MJ2)
cc      DIMENSION  Y(N,L), X(MJ1,L*(LAG+1))
cxx      DIMENSION  Y(N,L), X((L+1)*(LAG+1),L*(LAG+1))
cxx      DIMENSION  A(L,L,LAG) , V(L,L)
C
      INTEGER N, L, LAG, LMAX
      DOUBLE PRECISION Y(N,L), A(L,L,LAG), V(L,L), AIC
c local
      INTEGER MJ, MJ1, MJ2, MJ3, MJ4, N0, NMK, IPR
      DOUBLE PRECISION X((L+1)*(LAG+1),L*(LAG+1))
C
      EXTERNAL   SETMAR
C
      MJ1 = (L+1)*(LAG+1)
C
      MJ = N
      MJ2 = L
      MJ3 = LAG
      MJ4 = L*(LAG+1)
C
cc      IPR = 3
cc      READ( 5,* )  LAG
C
C  ...  Read multi-variate time series  ...
C
cc      CALL  READMD( IDEV,MJ,Y,N,L )
      N0 = 0
      NMK = N - LAG
C
C  ...  Housholder reduction  ...
C
cc      CALL  MREDCT( SETMAR,Y,D,NMK,N0,LAG,L,MJ,MJ1,X )
      CALL  MREDCT( SETMAR,Y,NMK,N0,LAG,L,MJ,MJ1,X )
C
C  ...  Least squares method for MAR model  ...
C
cc      CALL  MARFIT( X,D,NMK,L,LAG,MJ1,MJ2,MJ3,IPR,A,V,LMAX,AIC )
      CALL  MARFIT2( X,NMK,L,LAG,MJ1,IPR,A,V,LMAX,AIC )
C
C  ...  Print out fitted model  ...
C
cc      CALL  PRMAR2( A,V,L,LMAX,LAG,N,MJ2,MJ3 )
C     WRITE(2,*)  LMAX
C     DO 97 I=1,L
C  97 WRITE(2,600) (V(I,J),J=1,L)
C     DO 98 II=1,LMAX
C     DO 98 I=1,L
C  98 WRITE(2,600) (A(I,J,II),J=1,L)
C 600 FORMAT( 3D15.7 )
C
cc      STOP
      RETURN
      E N D
cc      SUBROUTINE  MARFIT( X,D,N,ID,M,MJ1,MJ2,MJ3,IPR,B,E,LMAX,AICS )
      SUBROUTINE  MARFIT2( X,N,ID,M,MJ1,IPR,B,E,LMAX,AICS )
C
C  ...  Multivariate AR model fitting (Householder method)  ...
C
C       Inputs:
C          X:     Householder reduced form
C          D:     Working area
C          N:     Data length
C          ID:    Dimension of the series
C          M:     Highest AR order
C          IPR:   Print out control
C          MJ1, MJ2, MJ3:  Adjustable dimensions
C       Outputs:
C          B:     AR-coefficient matrices
C          E:     innovation covariance matrix
C          LMAX:  Order of the MAICE model
C          AICS:  Total AIC of the model
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cc      DIMENSION  X(MJ1,1), D(1), E(MJ2,1), B(MJ2,MJ2,MJ3)
cc      DIMENSION  A(100), AIC(51), SD(51), EX(10)
cc      DIMENSION  IND(100), JND(100)
cxx      DIMENSION  X(MJ1,(M+1)*ID), E(ID,ID), B(ID,ID,M)
cxx      DIMENSION  A((M+1)*ID), AIC(M+1), SD(M+1), EX(ID)
cxx      DIMENSION  IND((M+1)*ID), JND((M+1)*ID)
C
      INTEGER N, ID, M, MJ1, IPR, LMAX
      DOUBLE PRECISION X(MJ1,(M+1)*ID), B(ID,ID,M), E(ID,ID), AICS
c local
      INTEGER I, II, IJ, IMIN, J, JJ, K, KK, K0, MJ2, MJ3, MD, MD2,
     1        IND((M+1)*ID), JND((M+1)*ID)
      DOUBLE PRECISION A((M+1)*ID), AIC(M+1), SD(M+1), EX(ID), PI2, SUM,
     1                 AICSUM, AICMIN, SDMIN, AICX
C
      DATA  PI2/6.28318531D0/
C
      MJ2 = ID
      MJ3 = M
C
      MD = M*ID
      MD2 = (M+1)*ID
      AICSUM = 0.D0
      LMAX = 0
cxx      DO 20  I=1,MJ2
cxx      DO 20  J=1,MJ2
cxx      DO 10  K=1,MJ3
cxx   10 B(I,J,K) = 0.D0
cxx   20 E(I,J) = 0.D0
      B(1:MJ2,1:MJ2,1:MJ3) = 0.D0
      E(1:MJ2,1:MJ2) = 0.D0
      DO 30  I=1,MD2
cxx   30 JND(I) = I
      JND(I) = I
   30 CONTINUE
C
      DO 200     II=1,ID
C
      JJ = II - 1
      KK = MD + JJ
      IF( II .GT. 1 )  THEN
C
C  ...  Addition of instantaneous variable  ...
C
      DO 110  I=1,MD2
      J = JND(I)
cxx  110 IND(J) = I
      IND(J) = I
  110 CONTINUE
      JND(1) = 1
      DO 120  I=1,JJ
cxx  120 JND(I) = MD+I
      JND(I) = MD+I
  120 CONTINUE
      DO 130  I=1,MD
cxx  130 JND(I+JJ) = I
      JND(I+JJ) = I
  130 CONTINUE
C     I1 = JJ + 1
      DO 140  I=JJ+1,ID
      J = MD + I
cxx  140 JND(J) = J
      JND(J) = J
  140 CONTINUE
C
cc      CALL  HUSHL1( X,D,MJ1,MD2,MD2,1,IND,JND )
      CALL  HUSHL1( X,MJ1,MD2,MD2,1,IND,JND )
      END IF
C
C  ...   AIC's of the models for II-th variable  ...
C
      DO 160  I=1,M+1
      K = (I-1)*ID + JJ
      SUM = 0.0D0
      DO 150 IJ=K+1,KK+1
cxx  150 SUM = SUM + X(IJ,KK+1)**2
      SUM = SUM + X(IJ,KK+1)**2
  150 CONTINUE
      SD(I) = SUM/N
cxx  160 AIC(I) = N*DLOG( PI2*SD(I) ) + N + 2*(K+1)
      AIC(I) = N*DLOG( PI2*SD(I) ) + N + 2*(K+1)
  160 CONTINUE
C
C  ...  Order determination by AIC  ...
C
      CALL  MAICE( AIC,SD,M,IPR-2,AICMIN,SDMIN,IMIN )
C
      K0 = IMIN*ID + JJ
      CALL  SRCOEF( X,K0,KK,N,MJ1,JND,A,EX(II),AICX )
      DO 170 I=1,II-1
cxx  170 E(II,I)   = -A(I)
      E(II,I)   = -A(I)
  170 CONTINUE
      E(II,II)  = 1.0D0
cxx      DO 180 IJ=1,IMIN
      DO 181 IJ=1,IMIN
      DO 180 J=1,ID
cxx  180 B(II,J,IJ) = A((IJ-1)*ID+J+II-1)
      B(II,J,IJ) = A((IJ-1)*ID+J+II-1)
  180 CONTINUE
  181 CONTINUE
      AICSUM = AICSUM + AICMIN
      IF( IMIN.GT.LMAX )  LMAX = IMIN
C
  200 CONTINUE
C
cc      IF(IPR.GE.1)  WRITE( 6,3 )
cxx      CALL  MCOEF( B,E,EX,ID,LMAX,MJ2,MJ3 )
      CALL  MCOEF( B,E,EX,ID,LMAX,MJ3 )
cc      IF(IPR.GE.1)  WRITE( 6,611 )  AICSUM
      AICS = AICSUM
C
      RETURN
cxx    3 FORMAT\( 1H0,132(1H-) )
cxx   15 FORMAT( 1H0,'*****  FINAL ESTIMATE  *****' )
cxx  611 FORMAT( 1H0,'AIC =',F15.3 )
      END
cxx      SUBROUTINE  MCOEF( B,E,EX,ID,LMAX,MJ2,MJ3 )
      SUBROUTINE  MCOEF( B,E,EX,ID,LMAX,MJ3 )
C
C  ...  AR coefficient matrices and innovation covariance matrix  ...
C
C       Inputs:
C          B:     Regression coefficient matrix
C          E:     B(0)
C          EX:    Residual variances of orthogonarized model
C          ID:    Dimension
C          LMAX:  AR-order
C          MJ2:   Adjustable dimension of B and E
C          MJ3:   djustable dimension of B
C       Outputs:
C          B:     AR-coefficient matrix
C          E:     Innovation covaraince matrix
C
cxx      IMPLICIT  REAL*8 ( A-H,O-Z )
cc      DIMENSION  B(MJ2,MJ2,MJ3), E(MJ2,1), EX(1), EE(24,24)
cc      MJ5 = 24
cxx      DIMENSION  B(ID,ID,MJ3), E(ID,ID), EX(ID), EE(ID,ID)
C
      INTEGER ID, LMAX, MJ3 
      DOUBLE PRECISION B(ID,ID,MJ3), E(ID,ID), EX(ID)
c local
      INTEGER I, II, J, JJ
      DOUBLE PRECISION EE(ID,ID), SUM
C
cc      CALL  TRIINV( E,ID,MJ2,MJ5,EE )
      CALL  TRIINV( E,ID,EE )
C
cxx      DO 30  II=1,LMAX
cxx      DO 20  I=1,ID
      DO 32  II=1,LMAX
      DO 21  I=1,ID
      DO 20  J=1,ID
      SUM = 0.D0
      DO 10  JJ=1,I
cxx   10 SUM = SUM + EE(I,JJ)*B(JJ,J,II)
      SUM = SUM + EE(I,JJ)*B(JJ,J,II)
   10 CONTINUE
cxx   20 E(I,J) = SUM
      E(I,J) = SUM
   20 CONTINUE
   21 CONTINUE
cxx      DO 30  I=1,ID
      DO 31  I=1,ID
      DO 30  J=1,ID
cxx   30 B(I,J,II) = E(I,J)
      B(I,J,II) = E(I,J)
   30 CONTINUE
   31 CONTINUE
   32 CONTINUE
cxx      DO 50  I=1,ID
      DO 51  I=1,ID
      DO 50  J=1,I
      SUM = 0.D0
      DO 40  II=1,J
cxx   40 SUM = SUM + EE(I,II)*EE(J,II)*EX(II)
      SUM = SUM + EE(I,II)*EE(J,II)*EX(II)
   40 CONTINUE
      E(I,J) = SUM
cxx   50 E(J,I) = SUM
      E(J,I) = SUM
   50 CONTINUE
   51 CONTINUE
cc      DO 99 I=1,ID
cc   99 WRITE(6,*) (E(I,J),J=1,ID)
C
      RETURN
      E N D
cc      SUBROUTINE  MREDCT( MSETX,Z,D,NMK,N0,LAG,ID,MJ,MJ1,X )
      SUBROUTINE  MREDCT( MSETX,Z,NMK,N0,LAG,ID,MJ,MJ1,X )
C
C  ...  This subroutine computes Householder reduced form of the
C       data matrix for fitting MAR model  ...
C
C       Inputs:
C          MSETX:  Function name
C          Z:      Multivariate time series
C          D:      Working area
C          NMK:    Actual data length
C          N0:     Origin of new data
C          LAG:    Highest order of the AR model
C          ID:     Dimension of observations
C          MJ,MJ1: Adjustable dimension
C         OUTPUT:
C          X:      Householder reduced form (upper triangular form)
C
cxx      IMPLICIT  REAL*8 (A-H,O-Z)
cc      DIMENSION  X(MJ1,1) , D(1), Z(MJ,1)
cx      DIMENSION  X(MJ1,1) , Z(MJ,1)
cxx      DIMENSION  X(MJ1,(LAG+1)*ID) , Z(MJ,ID)
C
      INTEGER NMK, N0, LAG, ID, MJ, MJ1
      DOUBLE PRECISION Z(MJ,ID), X(MJ1,(LAG+1)*ID)
c local
      INTEGER KD1, L, LK, N1, N2
C
      L = MIN0( NMK,MJ1 )
      KD1 = (LAG+1)*ID
      N1 = L
C
      CALL  MSETX( Z,N0,L,LAG,ID,MJ,MJ1,0,X )
cc      CALL  HUSHLD( X,D,MJ1,L,KD1 )
      CALL  HUSHLD( X,MJ1,L,KD1 )
C
      IF( N1 .GE. NMK )     RETURN
  100 L = MIN0( NMK-N1,MJ1-KD1 )
      LK = L + KD1
      N2 = N0 + N1
C
      CALL  MSETX( Z,N2,L,LAG,ID,MJ,MJ1,1,X )
cc      CALL  HUSHLD( X,D,MJ1,LK,KD1 )
      CALL  HUSHLD( X,MJ1,LK,KD1 )
      N1 = N1 + L
      IF( N1 .LT. NMK )     GO TO 100
C
      RETURN
      E N D
      SUBROUTINE  SETMAR( Z,N0,L,LAG,ID,MJ,MJ1,JSW,X )
C
C  ...  Make matrix X for multi-variate AR model  ...
C
C       Inputs:
C          Z:      Multivariate time series
C          N0:     Origin of new data
C          L:      Number of new observations
C          LAG:    Highest order of AR model
C          ID:     Dimension of time series
C          MJ,MJ1: Adjustable dimension
C          JSW:    =0   To make initail L*(LAG+1) data matrix
C                  =1   To augment original (LAG+1)*(LAG+1) matrix X
C                       by L*(LAG+1) data matrix of additional obs.
C       Outputs
C          X:      L*(LAG+1) matrix            if  JSW = 0
C                  (LAG+1+L)*(LAG+1) matrix    if  JSW = 1
C
cc      REAL*8  X(MJ1,1), Z(MJ,1)
cxx      REAL*8  X(MJ1,(LAG+1)*ID), Z(MJ,ID)
C
      INTEGER N0, L, LAG, ID, MJ, MJ1, JSW
      DOUBLE PRECISION Z(MJ,ID), X(MJ1,(LAG+1)*ID)
c local
      INTEGER II, I0, I1, I2, J, JJ, J1, J2, KD, KD1
C
      KD = LAG*ID
      KD1 = (LAG+1)*ID
      I0 = 0
      IF( JSW.EQ.1 )  I0 = KD1
C
cxx      DO 30  II=1,L
      DO 31  II=1,L
      I1 = N0 + LAG + II
      I2 = I0 + II
      DO 10  J=1,ID
      J2 = KD + J
cxx   10 X(I2,J2) = Z(I1,J)
      X(I2,J2) = Z(I1,J)
   10 CONTINUE
      DO 30  JJ=1,LAG
      I1 = I1 - 1
      J1 = (JJ-1)*ID
      DO 20 J=1,ID
      J2 = J1 + J
cxx   20 X(I2,J2) = Z(I1,J)
      X(I2,J2) = Z(I1,J)
   20 CONTINUE
   30 CONTINUE
   31 CONTINUE
C
      RETURN
      E N D
cc      SUBROUTINE  TRIINV( X,M,MJ,MJ1,Y )
      SUBROUTINE  TRIINV( X,M,Y )
C
C  ...  Lower triangular matrix inversion  ...
C
C       Inputs:
C          X:    Triangular matrix with unit diagonal elements
C          M:    Dimension of matrix X
C          MJ:   Adjustable dimension of X
C          MJ1:  Adjustable dimension of Y
C       Output:
C          Y:    Inverse of X
C
cxx      IMPLICIT  REAL * 8  ( A-H , O-Z )
cc      DIMENSION  X(MJ,1) , Y(MJ1,1)
cxx      DIMENSION  X(M,M) , Y(M,M)
C
      INTEGER M
      DOUBLE PRECISION X(M,M), Y(M,M)
c local
      INTEGER I, II, J, JJ
      DOUBLE PRECISION SUM
C
cxx      DO 10  I=1,M-1
      DO 11  I=1,M-1
      DO 10  J=I,M
cxx   10 Y(I,J) = 0.0D0
      Y(I,J) = 0.0D0
   10 CONTINUE
   11 CONTINUE
      DO 20  I=1,M
cxx   20 Y(I,I) = 1.0D0
      Y(I,I) = 1.0D0
   20 CONTINUE
cxx      DO 40  J=1,M-1
      DO 41  J=1,M-1
      DO 40  I=J+1,M
      SUM = 0.0D0
      DO 30  II=1,I-J
      JJ = II + J - 1
cxx   30 SUM = SUM + X(I,JJ) * Y(JJ,J)
      SUM = SUM + X(I,JJ) * Y(JJ,J)
   30 CONTINUE
cxx   40 Y(I,J) = -SUM
      Y(I,J) = -SUM
   40 CONTINUE
   41 CONTINUE
      RETURN
      E N D
      SUBROUTINE  MAICE( AIC,SD,K,ISW,AICM,SDM,IMIN )
C
C       INPUTS:
C          AIC:   VECTOR OF AIC'S
C          SD:    VECTOR OF INNOVATION VARIANCES
C          K:     UPPER LIMIT OF THE ORDER
C          ISW:   =0   OUTPUTS ARE SUPPRESSED
C                 >0   AIC'S ARE DIPLAIED
C       OUTPUTS:
C          AICM:  MINIMUM AIC
C          SDM:   MAICE INNOVATION VARIANCE
C          IMIN:  MAICE ORDER
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cc      DIMENSION  AIC(1), SD(1)
cxx      DIMENSION  AIC(K+1), SD(K+1)
C
      INTEGER K, ISW, IMIN 
      DOUBLE PRECISION AIC(K+1), SD(K+1), AICM, SDM
c local
      INTEGER I
C
C       SEARCH FOR THE MINIMUM OF AIC(I)
C
      IMIN = 0
      SDM  = SD(1)
      AICM = AIC(1)
      DO 20 I=1,K
      IF( AIC(I+1).LT.AICM )  THEN
         IMIN = I
         SDM  = SD(I+1)
         AICM = AIC(I+1)
      END IF
   20 CONTINUE
      IF( ISW .LE. 0 )     RETURN
C
C       DISPLAY OF AIC'S
C
cc      WRITE( 6,5 )
cc      DO 30 I=1,K+1
cc      II = I - 1
cc      DIC = AIC(I) - AICM
cc   30 WRITE( 6,6 )  II, SD(I), AIC(I), DIC
cc      WRITE( 6,7 )  IMIN, AICM, SDM
C
      RETURN
cxx    5 FORMAT( 1H ,4X,'M',9X,'SIG2(M)',13X,'AIC(M)',7X,'AIC(M)-AICMIN' )
cxx    6 FORMAT( 1H ,I5,D20.10,2F16.3 )
cxx    7 FORMAT( 1H0,'MAICE ORDER =',I3,5X,'AIC =',F15.3,5X,'SIG2 =',
cxx     *             D17.10 )
      E N D
cc      SUBROUTINE  HUSHL1( X,D,MJ,K,L,M,IND,JND )
      SUBROUTINE  HUSHL1( X,MJ,K,L,M,IND,JND )
C
C  ...  Householder trasnformation of the matrix X  ...
C
C       Inputs:
C         X(I,J): (K+1)*(K+1) matrix with the form specified by IND(I)
C         D(I):   Working area
C         MJ:     Adjustable dimension of X
C         K:      Number of columns of X
C         M:      Starting position of the Householder transformation
C         L:      End position of the Householder transformation
C         IND:    Specification of the present form of X
C                 << IND(J) = I means J-th regressor is variable I>>
C         JND:    Specification of the required form of X
C                 << JND(I) = J means I-th regressor is variable J >>
C       Output:
C         X(I,J)  Transformed matrix with the form specified by JND(I)
C
cxx      IMPLICIT  REAL*8 (A-H,O-Z)
cc      DIMENSION  X(MJ,1), D(40), IND(40), JND(40)
cxx      DIMENSION  X(MJ,K), D(MJ), IND(K), JND(K)
C
      INTEGER MJ, K, L, M, IND(K), JND(K)
      DOUBLE PRECISION X(MJ,K)
c local
      INTEGER I, II, J, JJ, J1, NN
      DOUBLE PRECISION D(MJ), TOL, F, G, H, S
C
      TOL = 1.0D-60
C
      NN = 0
      DO 100  II=M,L
      JJ = JND(II)
      NN = MAX0( NN,IND(JJ) )
      H = 0.0D0
      DO 10  I=II,NN
      D(I) = X(I,JJ)
cxx   10 H = H + D(I)**2
      H = H + D(I)**2
   10 CONTINUE
      IF( H.GT.TOL )  GO TO 20
      G = 0.0D0
      GO TO 100
   20 G = DSQRT( H )
      F = X(II,JJ)
      IF( F.GE.0.0D0 )  G = -G
      D(II) = F - G
      H = H - F * G
C
      DO 30  I=II+1,NN
cxx   30 X(I,JJ) = 0.D0
      X(I,JJ) = 0.D0
   30 CONTINUE
      IF( II.EQ.K )  GO TO 100
      DO 60  J1=II+1,K
      J = JND(J1)
      S = 0.0D0
      DO 40  I=II,NN
cxx   40 S = S + D(I)*X(I,J)
      S = S + D(I)*X(I,J)
   40 CONTINUE
      S = S / H
      DO 50  I=II,NN
cxx   50 X(I,J) = X(I,J) - D(I)*S
      X(I,J) = X(I,J) - D(I)*S
   50 CONTINUE
   60 CONTINUE
cxx  100 X(II,JJ) = G
      X(II,JJ) = G
  100 CONTINUE
      RETURN
      E N D
      SUBROUTINE  SRCOEF( X,M,K,N,MJ,JND,A,SIG2,AIC )
C
C  ...  Subset regression coefficients, residual variance and AIC  ...
C
C       Iuputs:
C         X:     Householder reduced form with index vector JND(I)
C         M:     Number of regressors
C         K:     Heighest order of the models
C         N:     Data length
C         MJ:    Adjustable dimension of X
C         JND(I): Specification of the I-th regressor
C       Outputs:
C         A(I):  Regression coefficients
C         SIG2:  Innovation variance
C         AIC:   AIC
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cc      DIMENSION  X(MJ,1), A(1), JND(1)
cxx      DIMENSION  X(MJ,K+1), A(M), JND(M)
C
      INTEGER M, K, N, MJ, JND(M)
      DOUBLE PRECISION X(MJ,K+1), A(M), SIG2, AIC
c local
      INTEGER I, II, J, L
      DOUBLE PRECISION PI2, SUM, SD
C
      DATA  PI2/6.28318531D0/
C
C  ...  REGRESSION COEFFICIENTS...
C
      L = JND(M)
      A(M) = X(M,K+1)/X(M,L)
      DO 10  II=1,M-1
      I = M - II
      SUM = X(I,K+1)
      DO 20  J=I+1,M
      L = JND(J)
cxx   20 SUM = SUM - A(J)*X(I,L)
      SUM = SUM - A(J)*X(I,L)
   20 CONTINUE
      L = JND(I)
cxx   10 A(I) = SUM/X(I,L)
      A(I) = SUM/X(I,L)
   10 CONTINUE
C
C  ...  RESIDUAL VARIANCE AND AIC  ...
C
      SD = 0.0D0
      DO 30  I=M+1,K+1
cxx   30 SD = SD + X(I,K+1)**2
      SD = SD + X(I,K+1)**2
   30 CONTINUE
      SIG2 = SD/N
      AIC  = N*DLOG( PI2*SIG2 ) + N + 2.0D0*(M+1)
C
      RETURN
      E N D

# PROGRAM 6.2 MARSPC
marspc <- function(arcoef, v, plot = TRUE, ...)
{
  m <- dim(arcoef)[3]      # ar order
  l <- dim(v)[1]           # dimension of the time series

  nf <- 200                # number of frequencies
  nf1 <- nf + 1
  llnf1 <- l * l * nf1

  z <- .Fortran(C_marspcf,
             as.integer(m),
             as.integer(l),
             as.double(arcoef),
             as.double(v),
             as.integer(nf),
             p = complex(llnf1),
             amp = double(llnf1),
             ang = double(llnf1),
             coh = double(llnf1),
             fnc = double(llnf1),
             frnc = double(llnf1))

  p <- array(z$p, dim = c(nf1, l, l))
  amp <- array(z$amp, dim = c(nf1, l, l))
  ang <- array(z$ang, dim = c(nf1, l, l))
  coh <- array(z$coh, dim = c(nf1, l, l))
  fnc <- array(z$fnc, dim = c(nf1, l, l))
  frnc <- array(z$frnc, dim = c(nf1, l, l))

  marspc.out <- list(spec = p, amp = amp, phase = ang, coh = coh, power = fnc,
                     rpower = frnc)

  if (plot) {
    plot.marspc(marspc.out, ...)
    invisible(marspc.out)
  } else marspc.out
}

plot.marspc <- function(x, ...)
{
  spec <- x$spec
  amp <- x$amp
  phase <- x$phase
  coh <- x$coh
  fnc <- x$power
#  frnc <- x$rpower

  old.par <- par(no.readonly = TRUE)

  d <- dim(spec)[3]
  d1 <- d - 1
  nf1 <- dim(spec)[1]
  fsp <- array(0, dim = c(nf1, d, d))

  x <- rep(0, nf1)
  for (i in 1:nf1)
    x[i] <- (i - 1) / (2 * (nf1 - 1))

# cross spectra

  for (i in 1:d)
    for(k in 1:nf1) 
      fsp[k, i, i] <- log10(Re(spec[k, i, i]))

  for (i in 1:d1) 
    for (j in (i+1):d)
      for(k in 1:nf1) {
        fsp[k, i, j] <- log10(amp[k, i, j])
        fsp[k, j, i] <- phase[k, i, j]
    }

  min1 <- min(fsp[, 1, 1])
  max1 <- max(fsp[, 1, 1])
  min2 <- min(fsp[, 1, 2])
  max2 <- max(fsp[, 1, 2])
  min3 <- min(fsp[, 2, 1])
  max3 <- max(fsp[, 2, 1])

  for (i in 2:d) {
    min1 <- min(min1, fsp[, i, i])
    max1 <- max(max1, fsp[, i, i])
  }
  if (d > 2) {
    for (i in 1:d1) 
      for (j in (i+1):d) {
          min2 <- min(min2, fsp[, i, j])
          max2 <- max(max2, fsp[, i, j])
          min3 <- min(min3, fsp[, j, i])
          max3 <- max(max3, fsp[, j, i])
      }
  }

  ylim1 <- c(floor(min1), ceiling(max1))
  ylim2 <- c(floor(min2), ceiling(max2))
  ylim3 <- c(floor(min3), ceiling(max3))

  par(mfrow = c(d, d), xaxs = "i", yaxs = "i")
  for (i in 1:d)
    for (j in 1:d) 
      if (i == j) {
        plot(x, fsp[, i, i], type = "l", ylim = ylim1, 
             xlab = "Frequency", ylab = paste("Spec (", i, ",", i, ")"), ...)
      } else if (i < j) {
        plot(x, fsp[, i, j], type = "l", ylim = ylim2,
             xlab = "Frequency", ylab = paste("Amp (", i, ",", j, ")"), ...)
      } else {
        plot(x, fsp[, i, j], type = "l", ylim = ylim3,
             xlab = "Frequency", ylab = paste("Phase (", i, ",", j, ")"), ...)
      }

# Power spectre and coherency
  par(ask = TRUE)
  par(mfrow = c(d, d), xaxs = "i", yaxs = "i")
  ymax <- max(coh)
  for (i in 1:d)
    for (j in i:d) {
      if (j != 1) par(mfg = c(i, j, d, d))
      if(i == j) {
        plot(x, fsp[, i, i], type = "l", ylim = ylim1,
             xlab = "Frequency", ylab = paste("Spec (", i, ",", i, ")"), ...)
      } else {
        plot(x, coh[, i, j], type = "l", ylim = c(0, ymax),
             xlab = "Frequency", ylab = paste("Coh (", i, ",", j, ")"), ...)
      }
    }

  par(ask=TRUE)
  par(mfrow = c(d,2))
  ymin <- 0
  for(i in 1:d) {
    ymax <- max(fnc[,i,])*1.1
    for(j in 1:d) {
      if( i == 1 )
        plot(x, fnc[,i,j], type='l', ylim=c(ymin,ymax), xlab="", ylab="",
             main="Power contribution", ...)
      if( i != 1 )
        plot(x, fnc[,i,j], type='l', ylim=c(ymin,ymax), xlab="", ylab="", ...)
      par(new=TRUE)
    }
    par(new=FALSE)

    for(j in 1:d) {
      rnc <- fnc[, i, j] / fnc[, i, d]
      if (i == 1)
        plot(x, rnc, type = "l", ylim = c(0, 1), xlab = "", ylab = "",
             main = "Relative power contribution", ...)
      if (i != 1)
        plot(x, rnc, type = "l", ylim = c(0, 1), xlab = "", ylab = "", ...)
      par(new = TRUE)
    }
    par(new = FALSE)
  }
  par(old.par)
}

# PROGRAM 4.1
pdfunc <- function(model = "norm", mean=0, sigma2 = 1, mu = 0, tau2 = 1, shape,
                 lambda = 1, side = 1, df, xmin = 0, xmax = 1, plot = TRUE, ...)
{
  k <- 201

  if (model == "norm") {
    model.type <- 1
    param <- c(mean, sigma2, 0)

  } else if (model == "Cauchy") {
    model.type <- 2
    param <- c(mu, tau2, 0)

  } else if (model == "Pearson") {
    model.type <- 3
#    if ((shape < 0) || (shape == 0))
    if ((shape < 0.5) || (shape == 0.5))
      stop("'shape' is greater than 0.5")
    param <- c(mu, tau2, shape)

  } else if (model == "exp") {
    if (side == 1) {
      model.type <- 4
    } else if (side == 2) {
      model.type <- 0
    } else {
      stop("'side' is 1 or 2")
    }
    param <- c(lambda, side, 0)

  } else if (model == "Chi2") {
    model.type <- 5
    param <- c(df, 0, 0)

  } else if (model == "dexp") {
    model.type <- 6
    param <- rep(0, 3)

  } else if (model == "unif") {
    model.type <- 7
    param <- c(xmin, xmax, 0)

  } else {
    stop("the model type is invalid.")
  }

  z <- .Fortran(C_denstyf,
                as.integer(model.type),
                as.double(param),
                as.double(xmin),
                as.double(xmax),
                as.integer(k),
                f = double(k))

  pdfunc.out <- list(model = model, density = z$f, interval=c(xmin, xmax),
                     param=param)
  class(pdfunc.out) <- c("pdfunc")

  if (plot) {
    plot.pdfunc(pdfunc.out, ...)
    invisible(pdfunc.out)
  } else pdfunc.out
}

plot.pdfunc <- function(x, ...)
{
  k <- 201
  model <- x$model
  f <- x$density
  xmin <- x$interval[1]
  xmax <- x$interval[2]
  ddx <- xmax - xmin
  pa1 <- x$param[1]
  pa2 <- x$param[2]
  pa3 <- x$param[3]

  if (model == "norm") {
    mtitle = paste("Normal distribution N ( ", pa1, ",", pa2, ")")

  } else if (model == "Cauchy") {
    mtitle = paste("Cauchy distribution\n ( mu =", pa1, ", tau2 =", pa2, ")")

  } else if (model == "Pearson") {
    mtitle = paste("Pearson family of distributions\n ( mu =", pa1, ", tau2 =",
                    pa2, ", shape =", pa3, ")")
  } else if (model == "exp") {
    if (pa2 == 1)
     mtitle = paste("Exponential distribution\n ( lambda =", pa1, ")")
    if (pa2 == 2)
     mtitle = paste("Two-sided exponential distribution\n ( lambda =", pa1, ")")

  } else if (model == "Chi2") {
    mtitle = paste("Chi-square distribution\n ( k =", pa1, ")")

  } else if (model == "dexp") {
    mtitle = "Double exponential distribution"

  } else if (model == "unif") {
    mtitle = "Uniform distribution"
  }

  if (model != "unif") {
    xx <- xmin + c(0:(k-1)) * ddx/(k-1)
    plot(xx, f, type = "l", main = mtitle, xlab = "x", ylab = "density", ...)
  } else {
    xx <- xmin + c(1:(k-1)) * ddx/(k-1)
    plot(xx, f[2:k], type = "l", ylim=c(0, max(f)*2), main = mtitle,
         xlab = "x", ylab = "density", ...)
    lines(c(xx[1],xx[1]), c(0,f[2]))
    lines(c(xx[k-1],xx[k-1]), c(0,f[k]))
  }
}

C     PROGRAM 7.2  MARFIT
      SUBROUTINE MARFIT( Y,N,L,LAG,AMIN,VMIN,AIC,MMIN )
C
C  ...  This program fits multivariate AR model  ...
C
C     The following inputs are required in the subroutine READMD.
C        TITLE:   title of the data
C        N:       data length
C        ID:      dimension of the observation
C        IFM:     = 1   ((Y(I,J),J=1,ID),I=1,N)
C                 = 2   ((Y(I,J),I=1,N),J=1,ID)
C        Y(I,J):  multi-variate time series
C     Parameters:
C        MJ:      adjustable dimension of Y; (MJ.GE.N)
C        MJ1:     adjustable dimension of Y, C, R; (MJ1.GE.ID)
C        LAG:     maximum lag of the cross-covariance function
C
cc      PARAMETER( MJ0=500,MJ=4,MJ1=10,LAG=MJ1,IDEV=1 )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(MJ0,MJ), OUTMIN(10), OUTMAX(10)
cc      DIMENSION  C(0:MJ1,MJ,MJ), R(0:MJ1,MJ,MJ), YMEAN(10)
cc      DIMENSION  AMIN(MJ1,MJ,MJ), VMIN(MJ,MJ), AIC(0:MJ1)
cc      DATA  OUTMIN/10*-1.0D30/, OUTMAX/10*1.0D30/
cxx      DIMENSION  Y(N,L), OUTMIN(L), OUTMAX(L)
cxx      DIMENSION  C(0:LAG,L,L), R(0:LAG,L,L), YMEAN(L)
cxx      DIMENSION  AMIN(LAG,L,L), VMIN(L,L), AIC(0:LAG)
C
      INTEGER N, L, LAG, MMIN
      DOUBLE PRECISION Y(N,L), AMIN(LAG,L,L), VMIN(L,L), AIC(0:LAG)
c local
      DOUBLE PRECISION OUTMIN(L), OUTMAX(L), C(0:LAG,L,L), R(0:LAG,L,L),
     1                 YMEAN(L)
C
cxx      DO 100 I = 1,L
cxx         OUTMIN(I) = -1.0D30
cxx         OUTMAX(I) = 1.0D30
cxx  100 CONTINUE
      OUTMIN(1:L) = -1.0D30
      OUTMAX(1:L) = 1.0D30
C
C  ...  read in multivariate time series  ...
C
cc      CALL  READMD( IDEV,MJ0,Y,N,L )
C
C  ...  cross-covariance function  ...
C
cc      CALL  CRSCOR( Y,N,L,LAG,MJ0,MJ,OUTMIN,OUTMAX,C,R,YMEAN )
      CALL  CRSCOR( Y,N,L,LAG,OUTMIN,OUTMAX,C,R,YMEAN )
C
C  ...  Yule-Walker method for MAR model  ...
C
cc      CALL  MYULE( L,LAG,N,C,MJ,MJ1,AMIN,VMIN,MMIN,AIC )
      CALL  MYULE( L,LAG,N,C,AMIN,VMIN,MMIN,AIC )
C
C  ...  print out estimated model  ...
C
cc      CALL  PRMAR( L,N,LAG,MJ,MJ1,AMIN,VMIN,MMIN,AIC )
cc      STOP
      RETURN
      E N D
cc      SUBROUTINE  MYULE( L,LAG,N,C,MJ,MJ1,AMIN,VMIN,MMIN,AIC )
      SUBROUTINE  MYULE( L,LAG,N,C,AMIN,VMIN,MMIN,AIC )
C
C  ...  Yule-Walker method for fitting multivariate AR model  ...
C
C     Inputs:
C        L:         Dimension of the time series
C        N:         Data length
C        LAG:       Highest order of fitted AR models
C        C(K,I,J):  Crosscovariance function
C        MJ, MJ1:   Adjustable dimensions
C     Outputs:
C        AMIN(K,I,J):  AR coefficient of the AIC best model
C        VMIN(I,J):    Innovation covariance matrix of AIC best model
C        MMIN:         MAICE order
C        AIC(I):       AIC's of the AR models
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  C(0:MJ1,MJ,MJ)
cc      DIMENSION  AMIN(MJ1,MJ,MJ), VMIN(MJ,MJ), AIC(0:MJ1)
cc      DIMENSION  A(20,10,10), A0(20,10,10), B(20,10,10), B0(20,10,10)
cc      DIMENSION  V(10,10), U(10,10), W(10,10)
cxx      DIMENSION  C(0:LAG,L,L)
cxx      DIMENSION  AMIN(LAG,L,L), VMIN(L,L), AIC(0:LAG)
cxx      DIMENSION  A(LAG,L,L), A0(LAG,L,L), B(LAG,L,L), B0(LAG,L,L)
cxx      DIMENSION  V(L,L), U(L,L), W(L,L)
C
      INTEGER L, LAG, N, MMIN
      DOUBLE PRECISION C(0:LAG,L,L), AMIN(LAG,L,L), VMIN(L,L),
     1                 AIC(0:LAG)
c local
      INTEGER I, IJ, J, K, M, MJ2
      DOUBLE PRECISION A(LAG,L,L), A0(LAG,L,L), B(LAG,L,L), B0(LAG,L,L),
     1                 V(L,L), U(L,L), W(L,L), UDET, VDET, AICMIN, SUM,
     2                 SUM1, SUM2, PI2
C
      DATA  PI2 /6.28318531D0/
cc      MJ2 = 10
      MJ2 = L
C
cxx      DO 10 I=1,L
      DO 11 I=1,L
      DO 10 J=1,L
      V(I,J) = C(0,I,J)
      U(I,J) = C(0,I,J)
cxx   10 VMIN(I,J) = V(I,J)
      VMIN(I,J) = V(I,J)
   10 CONTINUE
   11 CONTINUE
      CALL  INVDET( U,UDET,L,MJ2 )
      CALL  INVDET( V,VDET,L,MJ2 )
      AIC(0) = N*L*(DLOG(PI2) + 1) + N*DLOG(VDET) + L*(L+1)
      AICMIN = AIC(0)
      MMIN = 0
C
      DO 300 M=1,LAG
C
cxx      DO 110 I=1,L
      DO 111 I=1,L
      DO 110 J=1,L
      SUM = C(M,I,J)
cxx      DO 100 K=1,M-1
      DO 101 K=1,M-1
      DO 100 IJ=1,L
cxx  100 SUM = SUM - A0(K,I,IJ)*C(M-K,IJ,J)
      SUM = SUM - A0(K,I,IJ)*C(M-K,IJ,J)
  100 CONTINUE
  101 CONTINUE
cxx  110 W(I,J) = SUM
      W(I,J) = SUM
  110 CONTINUE
  111 CONTINUE
C
cxx      DO 130 I=1,L
      DO 131 I=1,L
      DO 130 J=1,L
      SUM1 = 0.0D0
      SUM2 = 0.0D0
      DO 120 IJ=1,L
      SUM1 = SUM1 + W(I,IJ)*U(IJ,J)
cxx  120 SUM2 = SUM2 + W(IJ,I)*V(IJ,J)
      SUM2 = SUM2 + W(IJ,I)*V(IJ,J)
  120 CONTINUE
      A(M,I,J) = SUM1
cxx  130 B(M,I,J) = SUM2
      B(M,I,J) = SUM2
  130 CONTINUE
  131 CONTINUE
C
cxx      DO 150 K=1,M-1
cxx      DO 150 I=1,L
      DO 152 K=1,M-1
      DO 151 I=1,L
      DO 150 J=1,L
      SUM1 = A0(K,I,J)
      SUM2 = B0(K,I,J)
      DO 140 IJ=1,L
      SUM1 = SUM1 - A(M,I,IJ)*B0(M-K,IJ,J)
cxx  140 SUM2 = SUM2 - B(M,I,IJ)*A0(M-K,IJ,J)
      SUM2 = SUM2 - B(M,I,IJ)*A0(M-K,IJ,J)
  140 CONTINUE
      A(K,I,J) = SUM1
cxx  150 B(K,I,J) = SUM2
      B(K,I,J) = SUM2
  150 CONTINUE
  151 CONTINUE
  152 CONTINUE
C
cxx      DO 170 I=1,L
      DO 171 I=1,L
      DO 170 J=1,L
      SUM1 = C(0,I,J)
      SUM2 = C(0,I,J)
cxx      DO 160 K=1,M
      DO 161 K=1,M
      DO 160 IJ=1,L
      SUM1 = SUM1 - A(K,I,IJ)*C(K,J,IJ)
cxx  160 SUM2 = SUM2 - B(K,I,IJ)*C(K,IJ,J)
      SUM2 = SUM2 - B(K,I,IJ)*C(K,IJ,J)
  160 CONTINUE
  161 CONTINUE
      V(I,J) = SUM1
      W(I,J) = SUM1
cxx  170 U(I,J) = SUM2
      U(I,J) = SUM2
  170 CONTINUE
  171 CONTINUE
C
cxx      DO 180 K=1,M
cxx      DO 180 J=1,L
      DO 182 K=1,M
      DO 181 J=1,L
      DO 180 I=1,L
      A0(K,I,J) = A(K,I,J)
cxx  180 B0(K,I,J) = B(K,I,J)
      B0(K,I,J) = B(K,I,J)
  180 CONTINUE
  181 CONTINUE
  182 CONTINUE

C
      CALL  INVDET( U,UDET,L,MJ2 )
      CALL  INVDET( V,VDET,L,MJ2 )
      AIC(M) = N*L*(DLOG(PI2) + 1) + N*DLOG(VDET) + L*(L+1) + 2*L*L*M
C
      IF( AIC(M).LT.AICMIN )  THEN
      AICMIN = AIC(M)
      MMIN   = M
cxx      DO 200 I=1,L
cxx      DO 200 J=1,L
      DO 202 I=1,L
      DO 201 J=1,L
      VMIN(I,J) = W(I,J)
      DO 200 K=1,M
cxx  200 AMIN(K,I,J) = A(K,I,J)
      AMIN(K,I,J) = A(K,I,J)
  200 CONTINUE
  201 CONTINUE
  202 CONTINUE
      END IF
  300 CONTINUE
C
      RETURN
      E N D
      SUBROUTINE INVDET( X,DET,M,MJ )
C
C  ...  Inverse and determinant of matrix X  ...
C
C       Inputs:
C          X:     M*M square matrix
C          M:     Dimension of X
C          MJ:    Adjustable dimension of X
C       Outputs:
C          X:     Inverse of X
C          DET:   Determinant of X
C
cxx      IMPLICIT  REAL*8 (A-H,O-Z)
cc      DIMENSION  X(MJ,MJ), IND(100)
cxx      DIMENSION  X(MJ,MJ), IND(M)
C
      INTEGER M, MJ
      DOUBLE PRECISION X(MJ,MJ), DET
c local
      INTEGER I, IMAX, J, JJ, L, IND(M)
      DOUBLE PRECISION XMAX, XTEMP
C
      DET = 1.0D0
      DO 60 L=1,M
      XMAX = 0.1D-10
      IMAX = 0
      DO 10 I=L,M
      IF( DABS( X(I,L) ).GT.DABS( XMAX ) )  THEN
         XMAX = X(I,L)
         IMAX = I
      END IF
   10 CONTINUE
      IND(L) = IMAX
      IF( IMAX.NE.L )  THEN
         IF( IMAX.LE.0 )  THEN
            DET = 0.0D0
            RETURN
         END IF
C
C     Row interchange
         DO 20 J=1,M
         XTEMP = X(IMAX,J)
         X(IMAX,J) = X(L,J)
cxx   20    X(L,J) = XTEMP
         X(L,J) = XTEMP
   20    CONTINUE
         DET = -DET
      END IF
      DET = DET*XMAX
      X(L,L) = 1.0D0
      DO 30 J=1,M
cxx   30 X(L,J) = X(L,J)/XMAX
      X(L,J) = X(L,J)/XMAX
   30 CONTINUE
      DO 50 I=1,M
      IF(I.NE.L)  THEN
         XTEMP  =X(I,L)
         X(I,L) = 0.0D0
         DO 40 J=1,M
cxx   40    X(I,J) = X(I,J) - XTEMP*X(L,J)
         X(I,J) = X(I,J) - XTEMP*X(L,J)
   40 CONTINUE
      END IF
   50 CONTINUE
   60 CONTINUE
C
C     Column interchange
      IF( M.GT.1 )  THEN
         DO 110 J=1,M-1
         JJ = IND(M-J)
         IF( JJ.NE.M-J )  THEN
            DO 100 I=1,M
            XTEMP    = X(I,JJ)
            X(I,JJ)  = X(I,M-J)
cxx  100       X(I,M-J) = XTEMP
            X(I,M-J) = XTEMP
  100       CONTINUE
         END IF
  110    CONTINUE
      END IF
      RETURN
      E N D

C     PROGRAM  3.2  FFTPER
cx      SUBROUTINE FFTPERF( Y,N,IWINDW,PE,SPE,NP )
      SUBROUTINE FFTPERF( Y,N,IWINDW,PE,SPE,NP,IFG )
C
C  ...  This program computes periodogram via FFT  ...
C
C     The following inputs are required in READTS.
C        TITLE:   title of the data
C        N:       data length
C        Y(I):    time series
C     Parameters:
C        NMAX:    adjustable dimension of Y (NMAX.GE.N)
C        NF:      adjustable dimension of PE and SPE
C        MJ:      adjustable dimension of COV
C     @TEST.PN31:  5/16/89, 12/5/90, 12/21/90, 8/5/91, 7/19/92
C
cc      PARAMETER( NMAX=3000,NF=512,IDEV=1,JDEV=6,IWINDW=1 )
cxxx      PARAMETER( NF=1024 )
      INTEGER, PARAMETER :: NF=1024
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(NMAX), PE(0:NF), SPE(0:NF)
cxx      DIMENSION  Y(N), PE(0:NF), SPE(0:NF)
C
      INTEGER N, IWINDW, NP, IFG
      DOUBLE PRECISION Y(N), PE(0:NF), SPE(0:NF)
c local
      INTEGER LAG, NN
C
cc      CALL  READTS( IDEV,Y,N )
C
      IF( IWINDW.EQ.0 )  LAG = N-1
cxx      IF( IWINDW.GT.0 )  LAG = 2*DSQRT( DFLOAT(N) )
cxxx      IF( IWINDW.GT.0 )  LAG = INT(2*DSQRT( DFLOAT(N) ))
      IF( IWINDW.GT.0 )  LAG = INT(2*DSQRT( DBLE(N) ))
      IF( IWINDW.EQ.0 )  NP  = (LAG+1)/2
      IF( IWINDW.GT.0 )  NP  = LAG
C
c----------
      NN = 0
c----------
      CALL  FFTPER( Y,N,NN,PE,NP )
cx      CALL  WINDOW( PE,NP,IWINDW,SPE )
      CALL  WINDOW( PE,NP,IWINDW,SPE,IFG )
cc      CALL  PRPER( JDEV,PE,SPE,N,NP,IWINDW )
C
cc      STOP
      RETURN
      E N D
      SUBROUTINE  FFTPER( Y,N,NN,P,N2 )
C
C ... periodogram computation by FFT ...
C
C     Inputs:
C        Y(I):   data
C        N:      data length
C        NN:     basic span for computing peiodogram (if NN>0)
C     Outputs:
C        P(J):   periodogram
C        NN:     basic span for computing peiodogram
C        N2:     number of frequencies
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION Y(N),P(0:1024),X(1024),FY(1024),WK(1024)
C
      INTEGER N, NN, N2
      DOUBLE PRECISION Y(N), P(0:1024)
c local
      INTEGER I, II, NP, NPOOL, ND
      DOUBLE PRECISION X(1024), FY(1024), WK(1024)
C
      IF(NN.LE.0) THEN
         IF( N.LE.1024 ) THEN
cxx           NP = ALOG( N-0.01 )/ALOG(2.0) + 1
           NP = INT(ALOG( N-0.01 )/ALOG(2.0) + 1)
           NN = 2**NP
           NPOOL = 1
         ELSE
           NN = 1024
           NPOOL = (N-1)/NN + 1
         END IF
      ELSE
cxx         NP = ALOG(NN-0.01)/ALOG(2.0) +1
         NP = INT(ALOG(NN-0.01)/ALOG(2.0) +1)
         NN = 2**NP
      IF( NN.GT.1024) NN=1024
         NPOOL = (N-1)/NN +1
      END IF
C
      N2 = NN/2
cxx      DO 10 I=0,N2
cxx   10 P(I) = 0.0D0
      P(0:N2) = 0.0D0
C
      DO 100 II=1,NPOOL
      ND = MIN(N,II*NN) - (II-1)*NN
      DO 20 I=1,ND
cxx   20 X(I) = Y((II-1)*NN+I)
      X(I) = Y((II-1)*NN+I)
   20 CONTINUE
      DO 30 I=ND+1,NN
cxx   30 X(I) = 0.0D0
      X(I) = 0.0D0
   30 CONTINUE
C
      CALL  FFTR2 ( X,NN,1,FY,WK )
C
      P(0)  = P(0) + FY(1)**2
      P(N2) = P(N2) + FY(N2+1)**2
      DO 40 I=0,N2-1
cxx   40 P(I) = P(I) + FY(I+1)**2 + FY(N2+1)**2
      P(I) = P(I) + FY(I+1)**2 + FY(N2+1)**2
   40 CONTINUE
  100 CONTINUE
C
      DO 50 I=0,N2
cxx   50 P(I) = P(I)/N 
      P(I) = P(I)/N
   50 CONTINUE

C
      RETURN
      E N D
      SUBROUTINE  FFTR2( X,N,ISW,FX,WRK )
C
C  ...  Fast Fourier Transform  ...
C
C     Inputs:
C        X(I):   data
C        N:      data length
C        ISW:
C        WRK:    working area
C     Output:
C        FX(J):  Fourier transform
C                  J=1,...,N/2+1   cosine transform
C                  J=N/2+2,...,N   sine transform
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  X(N), FX(N), WRK(N/4)
C
      INTEGER N, ISW
      DOUBLE PRECISION X(N), FX(N), WRK(N/4)
c local
      INTEGER I, IFG, JFG, K, K2, NP, N2, N4, M, M2
      DOUBLE PRECISION PI2
C
      DATA  PI2 /6.2831853071796D0/
C
cxx      NP = DLOG( DFLOAT(N) )/DLOG( 2.0D0 ) + 1.0D-5
cxxx      NP = INT(DLOG( DFLOAT(N) )/DLOG( 2.0D0 ) + 1.0D-5)
      NP = INT(DLOG( DBLE(N) )/DLOG( 2.0D0 ) + 1.0D-5)
      N2 = N/2
      N4 = N/4
C
      IF( ISW.EQ.1 )  THEN
        DO 10 I=2,N4
cxx   10   WRK(I) = DSIN( (I-1)*PI2/N )
        WRK(I) = DSIN( (I-1)*PI2/N )
   10   CONTINUE
      END IF
C
      DO 20  I=1,N2
      FX(I)    = X(I) + X(I+N2)
cxx   20 FX(I+N2) = X(I) - X(I+N2)
      FX(I+N2) = X(I) - X(I+N2)
   20 CONTINUE
C
      IFG = 1
      JFG = 1
      K =  1
      M = N4
C
      DO 30  I=1,NP-1
      M2 = M*2
      K2 = K*2
      IF( M.GE.K )  THEN
        IF( IFG.LT.0 )  THEN
          CALL  FFTSB1( X,WRK,K,M,M2,K2,FX )
        ELSE
          CALL  FFTSB1( FX,WRK,K,M,M2,K2,X )
        END IF
      ELSE
        IF( JFG.EQ.1 )  THEN
          IF( IFG.LT.0 )  THEN
            CALL  FFTSB2( X,K2,M2,FX )
          ELSE
            CALL  FFTSB2( FX,K2,M2,X )
          END IF
          JFG = 2
        END IF
        IF( IFG.LT.0 )  THEN
          CALL  FFTSB3( FX,WRK,K,M,X )
        ELSE
          CALL  FFTSB3( X,WRK,K,M,FX )
        END IF
      END IF
      K = K*2
      M = M/2
cxx   30 IFG = -IFG
      IFG = -IFG
   30 CONTINUE
C
      IF( IFG.GT.0 )  THEN
         DO 40  I=1,N
cxx   40    FX(I) = X(I)
         FX(I) = X(I)
   40    CONTINUE
      END IF
C
      RETURN
      E N D
      SUBROUTINE  FFTSB2( X,M,L,Y )
C
C  ...  slave subroutine for FFTR2  ...
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  X(L,M),  Y(M,L)
C
      INTEGER M, L
      DOUBLE PRECISION X(L,M), Y(M,L)
c local
      INTEGER I, J
C
      IF( M.GE.L )  THEN
cxx      DO 10  J=1,L
      DO 11  J=1,L
      DO 10  I=1,M
cxx   10 Y(I,J) = X(J,I)
      Y(I,J) = X(J,I)
   10 CONTINUE
   11 CONTINUE
      ELSE
cxx      DO 20  I=1,M
      DO 21  I=1,M
      DO 20  J=1,L
cxx   20 Y(I,J) = X(J,I)
      Y(I,J) = X(J,I)
   20 CONTINUE
   21 CONTINUE
      END IF
C
      RETURN
      E N D
      SUBROUTINE  FFTSB1( X,SINE,K,M,MJ1,MJ2,Y )
C
C  ...  slave subroutine for FFTR2  ...
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  X(MJ1,MJ2), Y(M,K,4), SINE(M,K)
C
      INTEGER K, M, MJ1, MJ2
      DOUBLE PRECISION X(MJ1,MJ2), SINE(M,K), Y(M,K,4)
c local
      INTEGER I, J
      DOUBLE PRECISION SUM1, SUM2
C
      DO 10  I=1,M
      Y(I,1,1) = X(I,1) + X(I+M,1)
      Y(I,1,3) = X(I,1) - X(I+M,1)
      Y(I,1,2) = X(I,K+1)
cxx   10 Y(I,1,4) = X(I+M,K+1)
      Y(I,1,4) = X(I+M,K+1)
   10 CONTINUE
C
cxx      DO 20  J=2,K
      DO 21  J=2,K
      DO 20  I=1,M
      SUM1 = SINE(1,K-J+2)*X(I+M,J) - SINE(1,J)    *X(I+M,J+K)
      SUM2 = SINE(1,J)    *X(I+M,J) + SINE(1,K-J+2)*X(I+M,J+K)
      Y(I,J,1)     = X(I,J) + SUM1
      Y(I,K-J+2,2) = X(I,J) - SUM1
      Y(I,J,3)     = SUM2 + X(I,J+K)
cxx   20 Y(I,K-J+2,4) = SUM2 - X(I,J+K)
      Y(I,K-J+2,4) = SUM2 - X(I,J+K)
   20 CONTINUE
   21 CONTINUE
C
      RETURN
      E N D
      SUBROUTINE  FFTSB3( X,SINE,K,M,Y )
C
C  ...  slave subroutine for FFTR2  ...
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  X(K,2,M,2),  SINE(M,K),  Y(K,4,M)
C
      INTEGER K, M
      DOUBLE PRECISION X(K,2,M,2), SINE(M,K), Y(K,4,M)
c local
      INTEGER I, J
      DOUBLE PRECISION SUM1, SUM2
C
cxx      DO 10  J=1,M
      DO 11  J=1,M
      Y(1,1,J) = X(1,1,J,1) + X(1,1,J,2)
      Y(1,3,J) = X(1,1,J,1) - X(1,1,J,2)
      Y(1,2,J) = X(1,2,J,1)
      Y(1,4,J) = X(1,2,J,2)
      DO 10  I=2,K
      SUM1 = SINE(1,K-I+2)*X(I,1,J,2) - SINE(1,I)    *X(I,2,J,2)
      SUM2 = SINE(1,I)    *X(I,1,J,2) + SINE(1,K-I+2)*X(I,2,J,2)
      Y(I,1,J)     = X(I,1,J,1) + SUM1
      Y(K-I+2,2,J) = X(I,1,J,1) - SUM1
      Y(I,3,J)     = SUM2 + X(I,2,J,1)
cxx   10 Y(K-I+2,4,J) = SUM2 - X(I,2,J,1)
      Y(K-I+2,4,J) = SUM2 - X(I,2,J,1)
   10 CONTINUE
   11 CONTINUE
C
      RETURN
      E N D

C     PROGRAM 11.2  TREND
      SUBROUTINE TREND(Y,N,M,IOPT,TAU20,DELTA,TAUMAX,SIG2M,FFMAX,AIC,
ccc     &                 XSS,RS)
     &                 XSS,VSS,RS)
C
C  ...  State space model for trend estimation  ...
C
C     Inputs:
C        M:       Order of the trend model
C        IOPT:    Search method
C        TAU20:   Initial estimate of TAU2
C        DELTA:   Search width
C     Parameters:
C        NMAX:    Adjustable dimension of Y
C        MJ:      Adjustable dimension of XF, VF, etc.
C        MAXM:    Adjustable dimension of A, B, C
C        NC:      Number of components
C     @TEST.FILTER2:  SEP.08,1990
C     MODIFIED  2/15/93
C
cc      PARAMETER( NMAX=1000,MJ=2,K=1,NDIM=NMAX,IDEV=1 )
cxxx      PARAMETER( K=1 )
      INTEGER, PARAMETER :: K=1
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(NMAX)
cc      DIMENSION  F(MJ,MJ), G(MJ), H(MJ), Q(K,K)
cc      DIMENSION  XPS(MJ,NMAX), XFS(MJ,NMAX), XSS(MJ,NMAX)
cc      DIMENSION  VPS(MJ,MJ,NMAX), VFS(MJ,MJ,NMAX), VSS(MJ,MJ,NMAX)
cc      DIMENSION  XF(MJ), VF(MJ,MJ)
cxx      DIMENSION  Y(N)
cxx      DIMENSION  F(M,M), G(M), H(M), Q(K,K)
cxx      DIMENSION  XPS(M,N), XFS(M,N), XSS(M,N)
cxx      DIMENSION  VPS(M,M,N), VFS(M,M,N), VSS(M,M,N)
cxx      DIMENSION  XF(M), VF(M,M)
cxx      DIMENSION  RS(N)
C
      INTEGER N, M, IOPT
      DOUBLE PRECISION Y(N), TAU20, DELTA, TAUMAX, SIG2M, FFMAX, AIC,
     1                 XSS(M,N), RS(N)
c local
      INTEGER II, NDIM, NS, NFE, NPE, MJ 
      DOUBLE PRECISION F(M,M), G(M), H(M), Q(K,K), XPS(M,N), XFS(M,N),
     1                 VPS(M,M,N), VFS(M,M,N), VSS(M,M,N), XF(M),
     2                 VF(M,M), OUTMIN, OUTMAX, SIG2, YMEAN, VAR, TAU2,
     3                 R, FF
C
      DATA  OUTMIN, OUTMAX /-1.0D30, 1.0D30/
C
      NDIM = N
      MJ = M
CCCCCC  TREND MODEL  CCCCC
      SIG2 =1.0D0
CCCCCCCCCCCCCCCCCCCCCCCCCC
cc      READ( 5,* )  M, IOPT
cc      READ( 5,* )  TAU20, DELTA
C
C  ...  Read Time Series  ...
C
cc      CALL  READTS( IDEV,Y,N )
C
      NS = 1
      NFE = N
      NPE = N
      CALL  MOMENT( Y,N/10,YMEAN,VAR )
C
      FFMAX  = -1.0D30
      TAU2 = 0.0D0
C
      DO 100  II=1,19
      IF( IOPT.NE.0 )  TAU2 = TAU20 + DELTA*(II-9)
      IF( IOPT.EQ.0 .AND. M.EQ.1 )  TAU2 = 2.0D0**(-II)
      IF( IOPT.EQ.0 .AND. M.GT.1 )  TAU2 = 2.0D0**(-II-5)
C
C  ...  Set Up State Space Model for Trend Estimation  ...
C
cc      CALL  SETTRN( M,MJ,F,G,H,R )
      CALL  SETTRN( M,F,G,H,R )
C
C  ...  Set initial values  ...
C
cc      CALL  ISTATE( M,MJ,YMEAN,VAR,XF,VF )
      CALL  ISTATE( M,YMEAN,VAR,XF,VF )
C
C  ...  Kalman Filter  ...
C
      Q(1,1) = TAU2
cc      CALL  FILTER( Y,XF,VF,F,G,H,Q,R,M,K,0,NS,NFE,NPE,MJ,NMAX,
      CALL  FILTER( Y,XF,VF,F,G,H,Q,R,M,K,0,NS,NFE,NPE,
     *              NDIM,OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,SIG2 )
      IF( FF.GT.FFMAX )  THEN
         FFMAX  = FF
         TAUMAX = TAU2
         SIG2M  = SIG2
      END IF
cc  100 WRITE(6,600)  TAU2, SIG2, FF
  100 CONTINUE
      AIC = -2*FFMAX + 2*(M+2)
cc      WRITE(6,610)  TAUMAX, SIG2M, FFMAX, AIC
C
C  ...  Filtering with the best parameter  ...
C
cc      CALL  ISTATE( M,MJ,YMEAN,VAR,XF,VF )
      CALL  ISTATE( M,YMEAN,VAR,XF,VF )
      Q(1,1) = TAUMAX
cc      CALL  FILTER( Y,XF,VF,F,G,H,Q,R,M,K,0,NS,NFE,NPE,MJ,NMAX,
      CALL  FILTER( Y,XF,VF,F,G,H,Q,R,M,K,0,NS,NFE,NPE,
     *              NDIM,OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,SIG2 )
C
C  ... Fixed Interval Smoother  ...
C
cc      CALL  SMOOTH( F,M,MJ,NDIM,NS,NFE,NPE,VFS,VPS,XFS,XPS,
cc     *              VSS,XSS )
      CALL  SMOOTH( F,M,NDIM,NS,NFE,NPE,VFS,VPS,XFS,XPS,VSS,XSS )
C
C  ...  Plot and print data, trend and noise  ...
C
C      CALL  PTTRND( Y,XSS,VSS,N,MJ,M,TAUMAX,SIG2,FF,AIC )
cc      CALL  PRTRND( Y,M,TAUMAX,SIG2,FF,AIC,XSS,MJ,N )
      CALL  PRTRND( Y,XSS,MJ,N,RS )
C
cc      STOP
      RETURN
cxx  600 FORMAT( 1H ,5X,F15.10,F12.6,F13.5 )
cxx  610 FORMAT( 1H ,'TAUMAX =',F15.10,3X,'SIG2 =',F15.10,3X,
cxx     *            'FF =',F13.5,3X,'AIC =',F13.4 )
      E N D
cc      SUBROUTINE  PRTRND( Y,M,TAU2,SIG2,FF,AIC,XSS,MJ,N )
      SUBROUTINE  PRTRND( Y,XSS,MJ,N,DATA )
C
C  ...  Print out Estimates  ...
C
C     Inputs:
C        Y:      Time Series
C        M:      Trend order
C        TAU2:   Variance of the system noise
C        SIG2:   Variance of the observational noise
C        FF:     Log-Likelihood of the mode
C        AIC:    AIC of the model
C        XSS:    Smoothed state
C        MJ:     Adjustable dimension of F and G
C        N:      Data LENGTH
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      CHARACTER  TITLE*72
cc      DIMENSION  XSS(MJ,*), Y(*), DATA(400)
cx      DIMENSION  XSS(MJ,*), Y(N), DATA(N)
cxx      DIMENSION  XSS(MJ,N), Y(N), DATA(N)
cc      COMMON  /CMDATA/  TITLE
C
      INTEGER MJ, N
      DOUBLE PRECISION Y(N), XSS(MJ,N), DATA(N)
c local
      INTEGER I
C
cc      WRITE(6,600)
cc      WRITE(6,610)  TITLE
cc      WRITE(6,620)  M, TAU2, SIG2, FF, AIC
cc      WRITE(6,630)
cc      WRITE(6,640)  (XSS(1,I),I=1,N)
      DO 10 I=1,N
cxx   10 DATA(I) = Y(I) - XSS(1,I)
      DATA(I) = Y(I) - XSS(1,I)
   10 CONTINUE
cc      WRITE(6,650)
cc      WRITE(6,640)  (DATA(I),I=1,N)
      RETURN
C
cxx  600 FORMAT( 1H1,'PROGRAM 9.1:  PREDICTION AND INTERPOLATION' )
cxx  610 FORMAT( 1H ,'DATA: ',A72 )
cxx  620 FORMAT( 1H ,'M =',I3,3X,'TAU2 =',D12.5,3X,'SIG2 =',D12.5,
cxx     *        3X,'LOG-L =',F12.4,3X,'AIC =',F12.4 )
cxx  630 FORMAT( 1H ,'***  TREND COMPONENT  ***' )
cxx  640 FORMAT( 1H ,5D13.6 )
cxx  650 FORMAT( 1H ,'***  RESIDUALS  ***' )
      E N D

# PROGRAM 15.1
simssm <- function(n = 200, trend = NULL, seasonal.order = 0, seasonal = NULL, arcoef = NULL,
                   ar = NULL, tau1 = NULL, tau2 = NULL, tau3 = NULL, 
                   sigma2 = 1.0, seed = NULL, plot = TRUE, ...)
{
  k <- 0
  if (is.null(trend)) {
    m1 <- 0
    x <- NULL
    tau1 <- 0
  } else {
    m1 <- length(trend)      # trend order   1 or 2
    if (is.null(tau1))
      stop("'tau1' is not specified")
    if (m1 > 2)
      stop("trend order is 0, 1 or 2")
    x <- trend
    k <- k + 1
  }

  m2 <- seasonal.order 
  if (m2 == 0) {
    period <- 0
    tau2 <- 0
  } else if (m2 == 1 || m2 == 2) {
    if (is.null(seasonal))
      stop("'seasonal' is not specified")
    if (is.null(tau2))
      stop("'tau2' is not specified")
    period <- length(seasonal) + 1
    x <- c(x, seasonal)
    k <- k + 1
  } else {
    stop("seasonal.order is 0, 1 or 2")
  }

  m3 <- length(arcoef)
  if (m3 == 0) {
    ar <- 0
    tau3 <- 0
  } else {
    if (is.null(tau3))
      stop("'tau3' is not specified")
    if (is.null(ar)) {
      ar <- rep(0, m3)
    } else {
      if(length(ar) != m3)
        stop("'ar' is invalid")
    }
    x <- c(x, ar)
    k <- k + 1
  }

  m <- m1 + m2 * (period - 1) + m3

# n:    simulation interval
# seed :     for random number generator
  ix <- seed
  if (is.null(ix))
    ix <- -1

  z <- .Fortran(C_simssmf,
                as.integer(m1),
                as.integer(m2),
                as.integer(m3),
                as.integer(m),
                as.integer(k),
                as.integer(n),
                as.integer(ix),
                as.double(sigma2),
                as.integer(period),
                as.double(tau1),
                as.double(tau2),
                as.double(tau3),
                as.double(arcoef),
                as.double(x),
                y = double(n))

  out <- z$y
  class(out) <- "simulate"

  if (plot) {
    plot.simulate(out, c(1, n), ...)
    invisible(out)
  } else out
}


# PROGRAM 15.2
ngsim <- function(n = 200, trend = NULL, seasonal.order = 0, seasonal = NULL, arcoef = NULL,
                  ar = NULL, noisew = 1, wminmax = NULL, paramw = NULL,
                  noisev = 1, vminmax = NULL, paramv = NULL, seed = NULL,
                  plot = TRUE, ... )
{
  k <- 0
  if (is.null(trend)) {
    m1 <- 0
    x <- NULL
  } else {
    m1 <- length(trend)     # trend order   1 or 2
    if (m1 > 2)
      stop("trend order is 0, 1 or 2")
    x <- trend
    k <- k + 1
  }

  m2 <- seasonal.order      # seasonal order
  if (m2 == 0) {
    period <- 0
  } else if (m2 == 1 || m2 == 2) { 
    if (is.null(seasonal))
      stop("'seasonal' is not specified") 
    period <- length(seasonal) + 1
    x <- c(x, seasonal)
    k <- k + 1
  } else {
    stop("seasonal.order is 0, 1 or 2")
  }

  m3 <- length(arcoef)
  if (m3 == 0) {
    ar <- 0
  } else {
    if (is.null(ar)) {
      ar <- rep(0, m3)
    } else {
      if(length(ar) != m3)
        stop("'ar' is invalid")
    }
    x <- c(x, ar)
    k <- k + 1
  }

  m <- m1 + m2 * (period - 1) + m3

# n:    simulation interval
# seed :     for random number generator
  ix <- seed
  if (is.null(ix))
    ix <- -1

# noisew:    type of observational noise
  if (noisew < -3 || noisew > 3)
    stop("type of observational noise is one of -3, -2, -1, 0, 1, 2, 3")
# noisev:    type of system noise
  if (noisev < -3 || noisev > 3)
    stop("type of system noise is one of -3, -2, -1, 0, 1, 2, 3")

# wparam:      parameter of the observational noise density
# vparam:      parameter of the system noise density
  pw <- c(0.0, 1.0, 1.0)
  pv <- c(0.0, 1.0, 1.0)
    
  if (noisew == 1) {
    if (is.null(paramw))
      stop("'paramw' is not specified")
    if (paramw[1] < 0)
      stop("the value of 'paramw' (variance) is non-negative")
    pw[2] <- paramw[1]
  } else if (noisew == 2) {
    if (is.null(paramw))
      stop("'paramw' is not specified")
    if (length(paramw) != 2)
      stop("'paramw' is not specified")
    if ((paramw[1] == 0) || (paramw[1] < 0))
      stop("dispersion parameter of the observational noise density is greater
            than 0")
    if ((paramw[2] == 0) || (paramw[2] < 0))
      stop("shape parameter of the observational noise density is greater than
            0")
    pw[2:3] <- paramw[1:2]
  }

  if (noisev == 1) {
    if (is.null(paramv))
      stop("'paramv' is not specified")
    if (paramv[1] < 0)
      stop("the value of 'paramv' (variance) is non-negative")
    pv[2] <- paramv[1]
  } else if (noisev == 2) {
    if (is.null(paramv))
      stop("'paramv' is not specified")
    if (length(paramv) != 2)
      stop("'paramv' is not specified")
    if ((paramv[1] == 0) || (paramv[1] < 0))
      stop("dispersion parameter of the system noise density is greater than 0")
    if ((paramv[2] == 0.5) || (paramv[2] < 0.5))
      stop("shape parameter of the system noise density is greater than 0.5")
    pv[2:3] <- paramv[1:2]
  }

# wminmax:     lower and upper bound of observational  noise
  if (is.null(wminmax)) {
    if (pw[3] > 1.6) {
      sd4 <- 4 * sqrt(pw[2]**2 / (2*pw[3] - 3))
      wmin <- -sd4
      wmax <- sd4
    } else {
      wmin <- -20
      wmax <- 20
    }
  } else {
    wmin <- wminmax[1]
    wmax <- wminmax[2]
  }
# vminmax:     lower and upper bound of system noise
  if (is.null(vminmax)) {
    if (pv[3] > 1.6) {
      sd4 <- 4 * sqrt(pv[2]**2 / (2*pv[3] - 3))
      vmin <- -sd4
      vmax <- sd4
    } else {
      vmin <- -20
      vmax <- 20
    }
  } else {
    vmin <- vminmax[1]
    vmax <- vminmax[2]
  }

  z <- .Fortran(C_ngsimf,
                as.integer(m1),
                as.integer(m2),
                as.integer(m3),
                as.integer(m),
                as.integer(k),
                as.integer(n),
                as.integer(ix),
                as.integer(noisew),
                as.double(wmin),
                as.double(wmax),
                as.double(pw),
                as.integer(noisev),
                as.double(vmin),
                as.double(vmax),
                as.double(pv),
                as.integer(period),
                as.double(arcoef),
                as.double(x),
                y = double(n))

  out <- z$y
  class(out) <- "simulate"

  if (plot) {
    plot.simulate(out, c(1, n), ...)
    invisible(out)
  } else out

}

plot.simulate <- function(x, use = NULL, ...)
{
  n <- length(x)
  if (is.null(use) == TRUE) {
    st <- 1
    ed <- n
  } else {
    st <- use[1]
    ed <- use[2]
  }
  if (st < 1)
      stop("start time must be between 1 and n.\n")
  if (ed > n)
      stop("end time must be between 1 and n.\n")
  if (ed > n)
      stop("used time interval is invalid.\n")

  xx <- c(st:ed)
  yy <- x[st:ed]
  plot(xx, yy, type = "l", xlab = "", ylab = "y(n)",
       main = "Simulated time series", xaxs = "i", ...)
}

# PROGRAM 3.1
period <-
  function(y, window = 1, lag = NULL, minmax = c(-1.0e+30, 1.0e+30),
           plot = TRUE, ...) {

  n <- length(y)

  if (window == 0) {
    if (is.null(lag))
      lag <- n - 1
    np <- (lag + 1) /2
  } else if (window == 1 || window == 2) {
    if (is.null(lag))
      lag <- as.integer(2 * sqrt(n))
    np <- lag
  } else {
    stop("'window' is 0, 1 or 2")
  }

  outmin <- minmax[1]
  outmax <- minmax[2]
  np1 <- np + 1

  z <- .Fortran(C_periodf,
                as.double(y),
                as.integer(n),
                as.integer(np),
                as.integer(window),
                as.integer(lag),
                as.double(outmin),
                as.double(outmax),
                pe = double(np1),
                spe = double(np1),
                ifg = integer(1))

  log <- "TRUE"
  if (z$ifg != 0) log <- "FALSE"

  period.out <- list(period = z$pe, smoothed.period = z$spe, window = window,
                     log.scale = log, tsname = deparse(substitute(y)))
  class(period.out) <- "spg"

  if (plot) {
    plot.spg(period.out, ...)
    invisible(period.out)
  } else period.out
}


# PROGRAM 3.2
fftper <- 
  function(y, window = 1, plot = TRUE, ...) {

  n <- length(y)
  if ((window < 0) || (window > 2 ))
    stop("'window' is 0, 1 or 2")

  nf <- 1024
  nf1 <- nf +1

  z <- .Fortran(C_fftperf,
                as.double(y),
                as.integer(n),
                as.integer(window),
                pe = double(nf1),
                spe = double(nf1),
                np = integer(1),
                ifg = integer(1))

  np1 <- z$np +1
  pe <- z$pe[1:np1]      # periodogram (raw spectrum)
  spe <- z$spe[1:np1]    # logarithm of smoothed periodogram

  log <- "TRUE"
  if (z$ifg != 0) log <- "FALSE"

  fftper.out <- list(period = pe, smoothed.period = spe, window = window,
                     log.scale = log, tsname = deparse(substitute(y)))
  class(fftper.out) <- "spg"

  if (plot) {
    plot.spg(fftper.out, ...)
    invisible(fftper.out)
  } else fftper.out
}

plot.spg <- function(x, type = "vl", ...)
{
  spe <- x$smoothed.period
  np1 <- length(spe)
  np <- np1 - 1
  spe <- spe[2:np1]
  xx <- rep(0, np)
  for (i in 1:np)
    xx[i] <- i / (np * 2)
  ymin <- floor(min(spe))
  ylim <- c(ymin, ceiling(max(spe)))

  mtitle <- paste(x$tsname)
  if (x$log.scale == "TRUE") {
    if (x$window == 0)
      ylabel <- "log-periodgram"
    if (x$window != 0)
      ylabel <- "smoothed log-periodgram"
  } else {
    if (x$window == 0)
      ylabel <- "periodgram"
    if (x$window != 0)
      ylabel <- "smoothed periodgram"
  }

  if (type == "l") {
    plot(xx, spe, type = "l", xlim = c(0, 0.5), main = mtitle, ylim = ylim,
         xlab = "f", ylab = ylabel, ...)
  } else {
    plot(xx, spe, type = "n", xlim = c(0, 0.5), main = mtitle, ylim = ylim,
         xlab = "f", ylab = ylabel, ...)
    for (i in 1:np)
      segments(xx[i], ymin, xx[i], spe[i], lwd = 0.7, ...)
  }
  
}

C     PROGRAM 2.1  UNICOR
      SUBROUTINE UNICORF(Y,N,LAG,OUTMIN,OUTMAX,COV,YMEAN)
C
C  ...  This program computes sample autocovariance and sample
C       autocorrelation function  ...
C
C     The following inputs are required in the subroutine READTS.
C        TITLE:   caption of the data set
C        N:       data length
C        FORMAT:  reading format
C        Y(I):    time series, (i=1,...,N)
C     Parameters:
C        NMAX:    adjustable dimension of Y (NMAX.GE.N)
C        LAG:     maximum lag of autocovariance function
C        IDEV:    input file specification
C        JDEV:    output file specification
C     5/16,1989 modified 12/05/90, 12/06/90, 7/18/92
C
cc      PARAMETER( NMAX=1000,LAG=30,IDEV=1,JDEV=6 )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(NMAX), COV(0:LAG,4)
cxx      DIMENSION  Y(N), COV(0:LAG,4)
cc      DATA  OUTMIN/-1.0D30/, OUTMAX/ 1.0D30/
C
      INTEGER N, LAG
      DOUBLE PRECISION Y(N), OUTMIN, OUTMAX, COV(0:LAG,4), YMEAN
C
C  ...  read in data  ...
C
cc      CALL  READTS( IDEV,Y,N )
C
C  ...  compute autocovariance, autocorrelation and their error bound  .
C
cc      CALL  UNICOR( Y,N,LAG,OUTMIN,OUTMAX,COV )
      CALL  UNICOR( Y,N,LAG,OUTMIN,OUTMAX,COV,YMEAN )
C
C  ...  print out results  ...
C
cc      CALL  PRACOR( JDEV,COV,N,LAG )
cc      STOP
      RETURN
      E N D

# PROGRAM 4.2 KLINFO
klinfo <- function(distg = 1, paramg = c(0, 1), distf = 1, paramf, xmax = 10)
{
  xmin <- -xmax

  z <- .Fortran(C_klinfof,
                as.integer(distg),
                as.double(paramg),
                as.integer(distf),
                as.double(paramf),
                as.double(xmin),
                as.double(xmax),
                nint = integer(4),
                dx = double(4),
                fkli = double(4),
                gint = double(4))
			 
  klinfo.out <- list(nint = z$nint, dx = z$dx, KLI = z$fkli,
                     gint = z$gint, xmax = xmax)
  class(klinfo.out) <- "klinfo"
  return(klinfo.out)
}

print.klinfo <- function(x, ...)
{
  n <- length(x$nint)
  message("\n  xmax\t    k\t    delta\t     KL info\t        gint")
  for (i in 1:n) 
    message(gettextf("%6.2f\t%5d\t%9.4f\t%12.8f\t%12.8f", x$xmax, x$nint[i],
                x$dx[i], x$KLI[i], x$gint[i]), domain = NA)
}

C     PROGRAM 15.2  NGSIM
      SUBROUTINE NGSIMF( M1,M2,M3,M,K,N,INI0,NOISEW,WMIN,WMAX,
     &                   PARAMW,NOISEV,VMIN,VMAX,PARAMV,PERIOD,AR,X,Y )
C
C  ...  SIMULATION BY NON-GAUSSIAN STATE SPACE MODEL  ...
C
C     INPUTS:
C        M1:   TREND ORDER (0,1,2)
C        M2:   SEASONAL ORDER (0 OR 1)
C        M3:   AR ORDER (0,...,20)
C        N:    SIMULATION INTERVAL
C        NS:   Y(NS+1),...,Y(N) ARE ACTUALLY USED
C        INI:  INITIAL VALUE FOR RANDOM NUMBER GENERATOR
C        NOISEW:    TYPE OF OBSERVATIONAL NOISE
C        NOISEV:    TYPE OF SYSTEM NOISE
C        WMIN,WMAX: LOWER AND UPPER BOUND OF OBS. NOISE
C        VMIN,VMAX: LOWER AND UPPER BOUND OF SYSTEM NOISE
C        X(I): INITIAL STATE
C     PARAMETERS:
C        MJ:   MAXIMUM STATE DIMENSION
C        NN:   MAXIMUM DATA LENGTH
C     MODIFIED  2/16/93
C
cc      PARAMETER( MJ=20,NN=1000 )
cxxx      PARAMETER( L=1 )
      INTEGER, PARAMETER :: L=1
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      INTEGER    PERIOD
cc      DIMENSION  X(MJ), F(MJ,MJ), G(MJ,MJ), H(MJ)
cc      DIMENSION  Q(MJ,MJ), Y(NN), AR(20), R(1,1)
cxx      DIMENSION  X(M), F(M,M), G(M,K), H(M)
cxx      DIMENSION  Q(K,K), Y(N), AR(M3), R(L,L)
cxx      DIMENSION  PARAMV(3), PARAMW(3)
C
      INTEGER M1, M2, M3, M, K, N, INI0, NOISEW, NOISEV, PERIOD
      DOUBLE PRECISION WMIN, WMAX, PARAMW(3), VMIN, VMAX, PARAMV(3),
     1                 AR(M3), X(M), Y(N)
c local
      INTEGER INI
      DOUBLE PRECISION F(M,M), G(M,K), H(M), Q(K,K), R(L,L), TAU1, TAU2,
     1                 TAU3, SIG2
C
      DATA   TAU1/1.0D0/, TAU2/1.0D0/, TAU3/1.0D0/, SIG2/1.0D0/
cc      INI = 1992092521
C
cc      READ( 5,* )  M1, M2, M3, N, NS, INI
C
cc      READ( 5,* )  NOISEW, WMIN, WMAX
      PARAMW(1) = 0.0D0
cc      IF( NOISEW.NE.2 )  READ(5,*)  PARAMW(2)
cc      IF( NOISEW.EQ.2 )  READ(5,*)  PARAMW(2), PARAMW(3)
C
cc      READ( 5,* )  NOISEV, VMIN, VMAX
      PARAMV(1) = 0.0D0
cc      IF( NOISEV.NE.2 )  READ(5,*)  PARAMV(2)
cc      IF( NOISEV.EQ.2 )  READ(5,*)  PARAMV(2), PARAMV(3)
C
cc      MM = 0
cc      IF( M1.GT.0 )  THEN
cc         READ( 5,* )  (X(I),I=1,M1)
cc         MM = M1
cc      END IF
cc      IF( M2.GT.0 )  THEN
cc         READ( 5,* )  PERIOD
cc         READ( 5,* )  (X(MM+I),I=1,PERIOD-1)
cc         MM = MM + PERIOD-1
cc      END IF
cc      IF( M3.GT.0 )  THEN
cc         READ( 5,* )  (AR(I),I=1,M3)
cc         READ( 5,* )  (X(MM+I),I=1,M3)
cc      END IF
      INI  = INI0
C
C  ...  SET STATE SPACE MODEL  ...
C
      CALL  SETSEA( M1,M2,M3,PERIOD,AR,TAU1,TAU2,TAU3,SIG2,F,G,H,
cc     *              Q,R,M,K,MJ )
     *              Q,R,M,K )
C
C  ...  SIMULATION  ...
C
      CALL  NGSIM( NOISEV,NOISEW,PARAMV,PARAMW,VMIN,VMAX,WMIN,WMAX,
cxx     *             F,G,H,Q,R,X,N,M,1,K,INI,MJ,NN,Y )
     *             F,G,H,Q,R,X,N,M,1,K,INI,Y )
C
C  ...  PLOT AND PRINT OUT SIMULATED TIME SERIES  ...
C
C      CALL  PTSIM( Y,N,NS,K,M1,M2,M3,INI0 )
cc      CALL  PRSIM( Y,N,NS,M1,M2,M3,INI0 )
C
cc      STOP
      RETURN
      E N D
      SUBROUTINE NGSIM( NOISEV,NOISEW,PV,PW,X0,X1,Y0,Y1,F,G,H,Q,R,
cxx     *                  X,N,M,L,K,IX,MJ,NN,Y )
     *                  X,N,M,L,K,IX,Y )
C
C  ...  SIMULATION BY NON-GAUSSIAN STATE SPACE MODEL  ...
C
C     INPUTS:
C        NOISEV:  TYPE OF THE SYSTEM NOISE
C        NOISEW:  TYPE OF THE OBSERVATIONAL NOISE
C        PV:      PARAMETER OF THE SYSTEM NOISE DENSITY
C        PW:      PARAMETER OF THE OBSER. NOISE DENSITY
C        X0,X1:   DOMAIN OF SYSTEM NOISE DENSITY
C        Y0,Y1:   DOMAIN OF OBSER. NOISE DENSITY
C        F,G,H:   COEFFICIENT MATRICES OF STATE SPACE MODEL
C        Q,R:     SYSTEM NOISE AND OBS. NOISE VARIANCES
C        X:       INITIAL STATE VECTOR
C        M,L,K:   DIMENSIONS OF THE STATE, SYSTEM NOISE AND OBS.
C        IX:      INITAL SEED FOR RANDOM NUMBER GENERATOR
C        MJ:      ADJUSTABLE DIMENSION
C        NN:      NUMBER OF DATA
C     OUTPUT:
C        Y(I):    SIMULATED TIME SERIES
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION X(MJ), F(MJ,MJ), G(MJ,MJ), H(L,MJ)
cc      DIMENSION Q(MJ,MJ), R(L,L), SQ(10,10), SR(10,10)
cc      DIMENSION XT(40), T(20), V(20), W(10), Y(NN,L)
cxx      DIMENSION X(M), F(M,M), G(M,K), H(L,M)
cxx      DIMENSION Q(K,K), R(L,L), SQ(K,K), SR(L,L)
cxx      DIMENSION XT(M), T(L), V(K), W(L), Y(N,L)
cxx      DIMENSION FV(0:400), FW(0:400), XV(0:400), XW(0:400)
cxx      DIMENSION PV(3), PW(3)
C
      INTEGER NOISEV, NOISEW, N, M, L, K, IX
      DOUBLE PRECISION PV(3), PW(3), X0, X1, Y0, Y1, F(M,M), G(M,K),
     1                 H(L,M), Q(K,K), R(L,L), X(M), Y(N,L)
c local
      INTEGER I, II, J
      DOUBLE PRECISION GAUSS, PEARSN, DBLEXP, USERV1, USERW1, SQ(K,K),
     1                 SR(L,L), XT(M), T(L), V(K), W(L), FV(0:400),
     2                 FW(0:400), XV(0:400), XW(0:400), DXV, DXW
C
      EXTERNAL  GAUSS
      EXTERNAL  PEARSN
      EXTERNAL  DBLEXP
cc      EXTERNAL  USERV
cc      EXTERNAL  USERW
      EXTERNAL  USERV1
      EXTERNAL  USERW1
C
C  ...  DISTRIBUTION FUNCTION OF SYSTEM NOISE AND OBS. NOISE  ...
C
cc      IF(NOISEV.EQ.0)  CALL  DISTRI( USERV,PV,X0,X1,FV,XV,DXV )
cxx      IF(NOISEV.EQ.0)  CALL  DISTRI( USERV1,PV,X0,X1,FV,XV,DXV )
      IF(NOISEV.EQ.0)  CALL  DISTRI0( USERV1,X0,X1,FV,XV,DXV )
      IF(NOISEV.EQ.1)  CALL  DISTRI( GAUSS ,PV,X0,X1,FV,XV,DXV )
      IF(NOISEV.EQ.2)  CALL  DISTRI( PEARSN,PV,X0,X1,FV,XV,DXV )
      IF(NOISEV.EQ.3)  CALL  DISTRI( DBLEXP,PV,X0,X1,FV,XV,DXV )
C
cc      IF(NOISEW.EQ.0)  CALL  DISTRI( USERW,PW,Y0,Y1,FW,XW,DXW )
cxx      IF(NOISEW.EQ.0)  CALL  DISTRI( USERW1,PW,Y0,Y1,FW,XW,DXW )
      IF(NOISEW.EQ.0)  CALL  DISTRI0( USERW1,Y0,Y1,FW,XW,DXW )
      IF(NOISEW.EQ.1)  CALL  DISTRI( GAUSS ,PW,Y0,Y1,FW,XW,DXW )
      IF(NOISEW.EQ.2)  CALL  DISTRI( PEARSN,PW,Y0,Y1,FW,XW,DXW )
      IF(NOISEW.EQ.3)  CALL  DISTRI( DBLEXP,PW,Y0,Y1,FW,XW,DXW )
C
cc      CALL  CHOLES( Q,MJ,K,SQ,10 )
cc      CALL  CHOLES( R,L,L,SR,10 )
      CALL  CHOLES( Q,K,K,SQ,K )
      CALL  CHOLES( R,L,L,SR,L )
C
C----- Initialize Mersenne Twister with given seed value
      call init(IX)
c-----
C
      DO 100 II= 1,N
C
C  ...  X = F*X + V  ...
C
ccxx      CALL  NGNOIS( NOISEV,FV,XV,DXV,IX,SQ,K,V )
      CALL  NGNOIS( NOISEV,FV,XV,DXV,SQ,K,V )
C
cxx      DO 10 I=1,M
cxx   10 XT(I) = 0.0D0
      XT(1:M) = 0.0D0
cxx      DO 20 J=1,M
      DO 21 J=1,M
      DO 20 I=1,M
cxx   20 XT(I) = XT(I) + F(I,J)*X(J)
      XT(I) = XT(I) + F(I,J)*X(J)
   20 CONTINUE
   21 CONTINUE
cxx      DO 30 J=1,K
      DO 31 J=1,K
      DO 30 I=1,M
cxx   30 XT(I) = XT(I) + G(I,J)*V(J)
      XT(I) = XT(I) + G(I,J)*V(J)
   30 CONTINUE
   31 CONTINUE
      DO 40 I=1,M
cxx   40 X(I) = XT(I)
      X(I) = XT(I)
   40 CONTINUE
C
C  ...  T = H*X + W  ...
C
ccxx      CALL  NGNOIS( NOISEW,FW,XW,DXW,IX,SR,L,W )
      CALL  NGNOIS( NOISEW,FW,XW,DXW,SR,L,W )
cxx      DO 50 I=1,L
cxx   50 T(I) = 0.0D0
      T(1:L) = 0.0D0
cxx      DO 60 I=1,L
      DO 61 I=1,L
      DO 60 J=1,M
cxx   60 T(I) = T(I) + H(I,J)*X(J)
      T(I) = T(I) + H(I,J)*X(J)
   60 CONTINUE
   61 CONTINUE
      DO 80 I=1,L
cxx   80 Y(II,I) = T(I) + W(I)
      Y(II,I) = T(I) + W(I)
   80 CONTINUE
  100 CONTINUE
C
      RETURN
      E N D
ccxx      SUBROUTINE  NGNOIS( NOISE,F,X,DX,IX,Q,K,V )
      SUBROUTINE  NGNOIS( NOISE,F,X,DX,Q,K,V )
C
C  ...  K-DIMENSIONAL NON-GAUSSIAN RANDOM NUMBER  ...
C
C     INPUTS:
C        NOISE:NOISE TYPE
C        F:    DISTRIBUTION FUNCTION OF NON-GAUSSIAN DISTRIBUTION
C        IX:   INITIAL SEED FOR RANDOM NUMBER GENERATOR
C        Q:    COVARIANCE
C        K:    DIMENSION OF THE RANDOM NUMBER
C     OUTPUT:
C        V:    K-DIMENSIONAL RANDOM NUMBER WITH COVARIANCE Q
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Q(10,10), W(10), V(K), F(0:400), X(0:400)
cxx      DIMENSION  Q(K,K), W(K), V(K), F(0:400), X(0:400)
C
      INTEGER NOISE, K
      DOUBLE PRECISION F(0:400), X(0:400), DX, Q(K,K), V(K)
c local
      INTEGER I, J
      DOUBLE PRECISION RNG, RNG2, W(K), SUM
C
      DO 10 I=1,K
ccxx      IF( NOISE.LT.0 )  W(I) = RNG2( IX,NOISE )
      IF( NOISE.LT.0 )  W(I) = RNG2( NOISE )
cxx   10 IF( NOISE.GE.0 )  W(I) = RNG( IX,F,X,DX )
ccxx      IF( NOISE.GE.0 )  W(I) = RNG( IX,F,X,DX )
      IF( NOISE.GE.0 )  W(I) = RNG( F,X,DX )
   10 CONTINUE
      DO 30 I=1,K
      SUM = 0.0D0
      DO 20 J=1,K
cxx   20 SUM = SUM + Q(J,I)*W(J)
      SUM = SUM + Q(J,I)*W(J)
   20 CONTINUE
cxx   30 V(I) = SUM
      V(I) = SUM
   30 CONTINUE
      RETURN
      E N D
      SUBROUTINE  DISTRI( FUNCT,PARAM,XMIN,XMAX,F,X,DX )
C
C  ...  DISTRIBUTION FUNCTION  ...
C
C     INPUTS:
C        FUNCT:  FUNCTION NAME
C        PARAM:  PARAMETER OF THE DENSITY
C        XMIN,XMAX:  DOMAIN OF THE  DENSITY
C     OUTPUTS:
C        F:      DISTRIBUTION FUNCTION
C        X:      LOCATION OF NODE
C        DX:     SIZE OF EACH BIN
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  F(0:400), P(0:400), X(0:400), PARAM(3)
C
      DOUBLE PRECISION FUNCT, PARAM(3), XMIN, XMAX, F(0:400), X(0:400),
     1                 DX
c local
      INTEGER I, K
      DOUBLE PRECISION P(0:400)
      EXTERNAL FUNCT
C
      K = 400
      DX = (XMAX-XMIN)/K
C
      DO 10 I=0,K
      X(I) = XMIN + I*DX
cxx   10 P(I) = FUNCT( X(I),PARAM )
      P(I) = FUNCT( X(I),PARAM )
   10 CONTINUE
cxx      DO 20 I=0,K
cxx   20 F(I) = 0.0D0
      F(0:K) = 0.0D0
      DO 30 I=1,K
cxx   30 F(I) = F(I-1) + ( P(I-1) + P(I) )*DX/2.0D0
      F(I) = F(I-1) + ( P(I-1) + P(I) )*DX/2.0D0
   30 CONTINUE
      DO 40 I=1,K
cxx   40 F(I) = F(I)/F(K)
      F(I) = F(I)/F(K)
   40 CONTINUE
C
      RETURN
      E N D
cxx      SUBROUTINE  DISTRI( FUNCT,PARAM,XMIN,XMAX,F,X,DX )
      SUBROUTINE  DISTRI0( FUNCT,XMIN,XMAX,F,X,DX )
C
C  ...  DISTRIBUTION FUNCTION  ...
C
C     INPUTS:
C        FUNCT:  FUNCTION NAME
C        PARAM:  PARAMETER OF THE DENSITY
C        XMIN,XMAX:  DOMAIN OF THE  DENSITY
C     OUTPUTS:
C        F:      DISTRIBUTION FUNCTION
C        X:      LOCATION OF NODE
C        DX:     SIZE OF EACH BIN
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  F(0:400), P(0:400), X(0:400), PARAM(3)
C
      DOUBLE PRECISION FUNCT, XMIN, XMAX, F(0:400), X(0:400), DX
c local
      INTEGER I, K
      DOUBLE PRECISION P(0:400)
      EXTERNAL FUNCT
C
      K = 400
      DX = (XMAX-XMIN)/K
C
      DO 10 I=0,K
      X(I) = XMIN + I*DX
cxx   10 P(I) = FUNCT( X(I),PARAM )
      P(I) = FUNCT( X(I) )
   10 CONTINUE
cxx      DO 20 I=0,K
cxx   20 F(I) = 0.0D0
      F(0:K) = 0.0D0
      DO 30 I=1,K
cxx   30 F(I) = F(I-1) + ( P(I-1) + P(I) )*DX/2.0D0
      F(I) = F(I-1) + ( P(I-1) + P(I) )*DX/2.0D0
   30 CONTINUE
      DO 40 I=1,K
cxx   40 F(I) = F(I)/F(K)
      F(I) = F(I)/F(K)
   40 CONTINUE
C
      RETURN
      E N D
ccxx      DOUBLE PRECISION FUNCTION  RNG( IX,F,X,DX )
      DOUBLE PRECISION FUNCTION  RNG( F,X,DX )
C
C  ...  NON-GAUSSIAN RANDON NUMBER GENERATOR  ...
C
C     INPUTS:
C        IX:     SEED
C        F:      DISTRIBUTION FUNCTION
C        X:      LOCATION
C        DX:     SIZE OF EACH BIN
cxx      IMPLICIT REAL*8( A-H,O-Z)
cxx      DIMENSION  F(0:400), X(0:400)
C
ccxx      INTEGER IX
      DOUBLE PRECISION F(0:400), X(0:400), DX
c local
      INTEGER I
cxx      DOUBLE PRECISION RUNI, U, V
      DOUBLE PRECISION U, V, random

C
cxx      IF( IX.EQ.0 )  IX = 1990103011
cxx      U = RUNI(IX)
      U = random()
C
      I = 0
   10 I = I+1
      IF( U.GT.F(I) )  GO TO 10
      IF( U.EQ.F(I) )  THEN
        V = X(I)
      ELSE
        V =((U-F(I-1))/( F(I)-F(I-1)) )*DX+X(I-1)
      END IF
      RNG = V
C
      RETURN
      E N D
ccxx      DOUBLE PRECISION FUNCTION  RNG2( IX,NOISE )
      DOUBLE PRECISION FUNCTION RNG2( NOISE )
C
C  ...  NON-GAUSSIAN RANDON NUMBER GENERATOR  ...
C
C     NOISE = -1:  CAUCHY RANDOM NUMBER
C           = -2:  EXPONENTIAL DISTRIBUTION
C           = -3:  DOUBLE EXPONENTIAL DISTRIBUTION
C
cxx      IMPLICIT REAL*8( A-H,O-Z)
C
ccxx      INTEGER IX, NOISE
      INTEGER NOISE
cxx      DOUBLE PRECISION RUNI, PI, U
      DOUBLE PRECISION PI, U, random
C
      DATA  PI /3.1415926535D0/
C
cxx      IF( IX.EQ.0 )  IX = 1990103011
cxx   10 U = RUNI()
      U = random()
C
C  ...  CAUCHY DISTRIBUTION  ...
C
ccxx      IF( NOISE.EQ.-1 )  THEN
ccxx         IF( U.EQ.0.5D0 )  GO TO 10
         RNG2 = DTAN( PI*U )
ccxx      END IF
C
C  ...  EXPONENTIAL DISTRIBUTION  ...
C
      IF( NOISE.EQ.-2 )  THEN
ccxx         IF( U.EQ.0.0D0 )  GO TO 10
         RNG2 = -DLOG( U )
      END IF
C
C  ...  DOUBLE EXPONENTIAL DISTRIBUTION  ...
C
      IF( NOISE.EQ.-3 )  THEN
ccxx         IF( U.EQ.0.0D0 )  GO TO 10
         RNG2 = DEXP( -DEXP( U ) )
      END IF
C
      RETURN
      E N D
cc      DOUBLE PRECISION FUNCTION  USERV( X,PARAM )
cxx      DOUBLE PRECISION FUNCTION  USERV1( X,PARAM )
      DOUBLE PRECISION FUNCTION  USERV1( X )
C
C  ...  User supplied density function  ...
C       (The double exp. dist. with mean 0 )
C     Inputs:
C        X:
C        PARAM(1):  lambda
C     Output:
C        USERV:     density at X
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  PARAM(2)
C
      DOUBLE PRECISION X, C
      DATA  C/0.5771D0/
C
cc      USERV = DEXP( X-C - DEXP(X-C) )
      USERV1 = DEXP( X-C - DEXP(X-C) )
      RETURN
      E N D
cc      DOUBLE PRECISION FUNCTION  USERW( X,PARAM )
cxx      DOUBLE PRECISION FUNCTION  USERW1( X,PARAM )
      DOUBLE PRECISION FUNCTION  USERW1( X )
C
C  ...  User supplied density function  ...
C       (The double exp. dist. with mean 0 )
C     Inputs:
C        X:
C        PARAM(1):  lambda
C     Output:
C        USERW:     density at X
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  PARAM(2)
C
      DOUBLE PRECISION X, C
      DATA  C/0.5771D0/
C
cc      USERW = DEXP( X-C - DEXP(X-C) )
      USERW1 = DEXP( X-C - DEXP(X-C) )
      RETURN
      E N D

C     PROGRAM 14.1  NGSMTH
      SUBROUTINE  NGSMTHF( Y,N,NOISEV,TAU2,BV,NOISEW,SIG2,BW,INITD,
     *                    TREND,SS,FF,NS,NFE,NPE,K )
C
C  ...  NON-GAUSSIAN SMOOTHING  ...
C
C     INPUTS:
C        NOISEV:   TYPE OF SYSTEM NOISE DENSITY (0,1,2,3)
C        NOISEW:   TYPE OF OBSER. NOISE DENSITY (0,1,2,3,4)
C        TAU2:     VARIANCE OR DISPERSION OF SYSTEM NOISE
C        SIG2:     VARIANCE OR DISPERSION OF OBSERVATION NOISE
C        BV:       SHAPE PARAMETER OF SYSTEM NOISE (FOR NOISEV = 2)
C        BW:       SHAPE PARAMETER OF OBSER. NOISE (FOR NOISEW = 2)
C     PARAMETERS:
C        MJ:       ADJUSTABLE DIMENSION OF Y, PS, FS, SS (MJ >= N)
C        K:        NUMBER OF INTERVALS + 1
C     @NFILTER.F:   5/14/85, 6/11/87, 5/19/88, 2/14/91
C     MODIFIED  2/16/93
C
cc      PARAMETER( MJ=500, K=201 )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      CHARACTER  TITLE*72
cc      DIMENSION  Y(MJ), P(K), F(K), S(K), T(K), Q(-K:K)
cc      DIMENSION  YM(10), ST(7), TREND(MJ,7), LOC(MJ)
cc      REAL*4    PS(K,MJ), FS(K,MJ), SS(K,MJ)
cxx      DIMENSION  Y(N), F(K), Q(-K:K)
cxx      DIMENSION  YM(2), ST(7), TREND(NPE,7), LOC(NPE)
cxx      REAL*4     SS(K,NPE)
cxx      REAL*8     SS(K,NPE)
C
      INTEGER N, NOISEV, NOISEW, INITD, NS, NFE, NPE, K
      DOUBLE PRECISION Y(N), TAU2, BV, SIG2, BW, TREND(NPE,7),
     1                 SS(K,NPE), FF
c local
      INTEGER I, J, LOC(NPE)
      DOUBLE PRECISION F(K), Q(-K:K), YM(2), ST(7), DX, XMIN, XMAX,
     1                 OUTMIN, OUTMAX, SSUM
C
cc      COMMON   /COMAAA/  Y
cc      COMMON   /C91214/  XMIN, XMAX, FIGMIN, FIGMAX, YM
cc      COMMON   /C91215/  NOISEV, NOISEW, INITD, ITF, ITH
cc      COMMON   /C91216/  B, OUTMIN, OUTMAX
cc      COMMON   /C91218/  SIG2, TAU2, BW, BV
cc      COMMON   /C91219/  N, KK, NS, NFE, NPE
cc      COMMON   /CMDATA/  TITLE
cc      COMMON    / DDD /  FF , AIC , SD
C     NAMELIST  /PARAM/  XMIN, XMAX, TAU20, SIG20, BC, IOPT, B,
C    *                   OUTMIN, OUTMAX, NS, NFE, NPE, FIGMIN, FIGMAX,
C    *                   NOISEV, NOISEW, ITF, ITH
C
cc      EXTERNAL  GAUSS
cc      EXTERNAL  PEARSN
cc      EXTERNAL  TWOEXP
cc      EXTERNAL  USERV
C
C  ...  READ DATA  ...
C 
cc      CALL  READTS( 1,Y,N )
      CALL  MOMENT( Y,N,YM(1),YM(2) )
C
      LOC(1) = 0
cc      KK = K
cc      CALL  DEFALT
      CALL  DEFALT( Y,N,XMIN,XMAX,OUTMIN,OUTMAX )
cc      READ( 5,* )  NOISEV, TAU2, BV
cc      READ( 5,* )  NOISEW, SIG2, BW
C
      DX = (XMAX-XMIN)/(K-1)
cc      CALL  IDIST( F,K,YM(1),YM(2),XMIN,DX )
      CALL  IDIST( F,K,YM(1),YM(2),XMIN,DX,INITD )
      CALL  NORMLZ( F,K,DX,SSUM )
cc      IF( NOISEV.EQ.0 )  CALL  TRANS( USERV ,K,DX,TAU2,BV,Q )
cc      IF( NOISEV.EQ.1 )  CALL  TRANS( GAUSS ,K,DX,TAU2,BV,Q )
cc      IF( NOISEV.EQ.2 )  CALL  TRANS( PEARSN,K,DX,TAU2,BV,Q )
cc      IF( NOISEV.EQ.3 )  CALL  TRANS( TWOEXP,K,DX,TAU2,BV,Q )
      IF( NOISEV.EQ.0 )  CALL  TRANS1( K,DX,TAU2,BV,Q )
      IF( NOISEV.EQ.1 )  CALL  TRANS2( K,DX,TAU2,BV,Q )
      IF( NOISEV.EQ.2 )  CALL  TRANS3( K,DX,TAU2,BV,Q )
      IF( NOISEV.EQ.3 )  CALL  TRANS4( K,DX,TAU2,BV,Q )
cc      CALL  NGSMTH( Y,P,F,S,T,N,K,DX,XMIN,Q,FF,PS,FS,SS,LOC )
      CALL  NGSMTH( NOISEW,SIG2,BW,Y,F,N,K,DX,XMIN,Q,FF,SS,LOC,
     *              OUTMIN,OUTMAX,NS,NFE,NPE )
C
      DO 30 I=1,NPE
      DO 10 J=1,K
cc   10 S(J) = SS(J,I)
cc      CALL  PINTVL( S,K,XMIN,DX,ST )
cxx   10 F(J) = DBLE(SS(J,I))
cxx   10 F(J) = SS(J,I)
      F(J) = SS(J,I)
   10 CONTINUE
      CALL  PINTVL( F,K,XMIN,DX,ST )
      DO 20 J=1,7
cxx   20 TREND(I,J) = ST(J) + DX*LOC(I)
      TREND(I,J) = ST(J) + DX*LOC(I)
   20 CONTINUE
   30 CONTINUE
C
cc      CALL  PRNGSM( NOISEV,NOISEW,TAU2,SIG2,FF,TREND,MJ,N )
C
cc      CALL  PTNGSM( N,NPE,NOISEV,NOISEW,TAU2,SIG2,FF,B,Y,TREND,MJ,
cc     *              FIGMIN,FIGMAX,SS,K,LOC )
      CALL  POST3D( SS,LOC,K,NPE )
      RETURN
      E N D
cc      SUBROUTINE  DEFALT
      SUBROUTINE  DEFALT( Y,N,XMIN,XMAX,OUTMIN,OUTMAX )
C
C  ...  THIS SUBROUTINE SETS DEFAULT VALUES OF PARAMETERS  ...
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(500)
cxx      DIMENSION  Y(N)
C
      INTEGER N
      DOUBLE PRECISION Y(N), XMIN, XMAX, OUTMIN, OUTMAX
c local
      DOUBLE PRECISION DY
C
cc      COMMON   /COMAAA/  Y
cc      COMMON   /C91214/  XMIN, XMAX, FIGMIN, FIGMAX, YM(10)
cc      COMMON   /C91215/  NOISEV, NOISEW, INITD, ITF, ITH
cc      COMMON   /C91216/  B, OUTMIN, OUTMAX
cc      COMMON   /C91219/  N, KK, NS, NFE, NPE
cc      NOISEV = 2
cc      NOISEW = 1
cc      INITD  = 1
cc      ITF    = 1
cc      ITH    = 1
cc      B = 1.00D0
      OUTMIN = -1.0D30
      OUTMAX =  1.0D30
cc      CALL  MAXMIN( Y,N,XMIN,XMAX,DY )
      CALL  MAXMINK( Y,N,XMIN,XMAX,DY )
cc      NS = 1
cc      NFE = N
cc      NPE = N
cc      FIGMIN = XMIN
cc      FIGMAX = XMAX
C
      RETURN
      E N D
cc      SUBROUTINE  NGSMTH( Y,P,F,S,T,N,K,DX,XMIN,Q,FF,PS,FS,SS,LOC )
      SUBROUTINE  NGSMTH( NOISEW,SIG2,BW,Y,F,N,K,DX,XMIN,Q,FF,SS,LOC,
     *                     OUTMIN,OUTMAX,NS,NFE,NPE )
C
C  ...  NON-GAUSSIAN SMOOTHER  ...
C
C     INPUTS:
C       Y(I):   TIME SERIES
C       P(I):   INITIAL DENSITY
C       N:      DATA LENGTH
C       K:      NUMBER OF INTERVALS IN STEP FUNCTION APPROXIMATION
C       DX:     WIDTH OF INTERVAL
C       XMIN:   MINIMUM OF THE INTERVAL
C       Q:      SYSTEM NOISE DENSITY
C     OUTPUTS:
C       FF:     LOG-LIKELIHOOD
C       LOC(I): LOCATION OF THE CENTER OF THE INTERVAL AT STEP I
C       SS:     SMOOTHED DENSITY
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  P(K), F(K), S(K), T(K), Y(N), Q(-K:K), LOC(N)
cc      REAL*4     FS(K,N), PS(K,N), SS(K,N)
cxx      DIMENSION  P(K), F(K), S(K), T(K), Y(N), Q(-K:K), LOC(NPE)
cxx      REAL*4     PS(K,NPE), SS(K,NPE)
cxx      DIMENSION     PS(K,NPE), SS(K,NPE)
C
      INTEGER NOISEW, N, K, NS, NFE, NPE, LOC(NPE)
      DOUBLE PRECISION SIG2, BW, Y(N), F(K), DX, XMIN, Q(-K:K), FF,
     1                 SS(K,NPE), OUTMIN, OUTMAX
c local
      INTEGER I, II, J
      DOUBLE PRECISION P(K), S(K), T(K), PS(K,NPE), PSUM, FINT, TSUM
C
cc      COMMON   /C91215/  NOISEV, NOISEW, INITD, ITF, ITH
cc      COMMON   /C91216/  B, OUTMIN, OUTMAX
cc      COMMON   /C91219/  NN, KK, NS, NFE, NPE
C
      FF = 0.0D0
C
      DO 200 II=NS,NPE
C
C  ...  CONVOLUTION (SYSTEM NOISE)  ...
C
      CALL  CONVOL( Q,F,K,P )
      CALL  NORMLZ( P,K,DX,PSUM )
C
C  ...  BAYES FORMULA  ...
C
      IF( Y(II).LE.OUTMIN .OR. Y(II).GE.OUTMAX .OR. II.GT.NFE ) THEN
      DO 110 I=1,K
cxx  110 F(I) = P(I)
      F(I) = P(I)
  110 CONTINUE
      ELSE
cc      CALL  BAYES( P,K,XMIN,DX,Y(II),F,LOC(II) )
      CALL  BAYES( NOISEW,SIG2,BW,P,K,XMIN,DX,Y(II),F,LOC(II) )
      CALL  NORMLZ( F,K,DX,FINT )
C
C  ...  LIKELIHOOD COMPUTATION  ...
C
      FF = FF + DLOG( FINT )
cc      IF( MOD(II,10).EQ.0)  WRITE(6,*) II,FF
      END IF
C
C  ...  SAVE FOR SMOOTHING  ...
C
      DO 130 I=1,K
      PS(I,II) = P(I)
cc  130 FS(I,II) = F(I)
cxx  130 SS(I,II) = F(I)
      SS(I,II) = F(I)
  130 CONTINUE
C
C  ...  SHIFT ORIGIN  ...
C
cc      CALL  SHIFT( F,K,T,II,N,LOC )
      CALL  SSHIFT( F,K,T,II,N,LOC )
C
  200 CONTINUE
C
C  ...  SMOOTHING  ...
C
cc      DO 190 J=NFE,NPE
cc      DO 190 I=1,K
cc  190 SS(I,J) = FS(I,J)
      DO 195 I=1,K
cc  195 S(I) = FS(I,NFE)
cxx  195 S(I) = SS(I,NFE)
      S(I) = SS(I,NFE)
  195 CONTINUE
C
      DO 300 II=NFE-1,NS,-1
cc      IF(MOD(II,10).EQ.0)  WRITE(6,*) II
      DO 210 I=1,K
      T(I) = 0.0D0
      P(I) = 0.0D0
cc  210 F(I) = FS(I,II)
cxx  210 F(I) = SS(I,II)
      F(I) = SS(I,II)
  210 CONTINUE
      DO 220 I=1,K
      J = I - (LOC(II+1)-LOC(II))
      IF( J.GE.1.AND.J.LE.K )  P(I) = PS(J,II+1)
cxx  220 IF( J.GE.1.AND.J.LE.K )  T(I) = S(J)
      IF( J.GE.1.AND.J.LE.K )  T(I) = S(J)
  220 CONTINUE
      DO 230 I=1,K
cxx  230 S(I) = T(I)
      S(I) = T(I)
  230 CONTINUE
C
cc      CALL  SCONVL( Q,S,P,F,K,T )
      CALL  SCONVLK( Q,S,P,F,K,T )
      CALL  NORMLZ( T,K,DX,TSUM )
C
      DO 240 I=1,K
      S(I) = T(I)
cxx  240 SS(I,II) = S(I)
      SS(I,II) = S(I)
  240 CONTINUE
  300 CONTINUE
C
      RETURN
      E N D
      SUBROUTINE  PINTVL( P,K,XMIN,DX,Y )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  P(K), Y(7), PROB(7), P1(401)
cxx      DIMENSION  P(K), Y(7), PROB(7), P1(K)
C
      INTEGER K
      DOUBLE PRECISION P(K), XMIN, DX, Y(7)
c local
      INTEGER I, II, J
      DOUBLE PRECISION PROB(7), P1(K), PP
C
      DATA  PROB /0.0013D0, 0.0227D0, 0.1587D0, 0.5000D0, 0.8413D0,
     *            0.9773D0,0.9987D0/
C
      P1(1) = 0.0
      DO 10 I=2,K
cxx   10 P1(I) = P1(I-1) + (P(I-1) + P(I))*DX/2.0
      P1(I) = P1(I-1) + (P(I-1) + P(I))*DX/2.0
   10 CONTINUE
C
      DO 31 J=1,7
cxx     DO 30 J=1,7
      PP = PROB(J)
cxx      DO 20 I=2,K
      DO 20 II=2,K
         I = II
      IF(P1(I-1).LE.PP .AND. P1(I).GT.PP)  GO TO 30
   20 CONTINUE
   30 Y(J) = XMIN + (I-2)*DX + DX*(PP - P1(I-1))/(P1(I) - P1(I-1))
   31 CONTINUE
C
      RETURN
      E N D
      SUBROUTINE  CONVOL( Q,S,K,P )
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  S(K), P(K), Q(-K:K)
C
      INTEGER K
      DOUBLE PRECISION Q(-K:K), S(K), P(K)
c local
      INTEGER I, J, J1, J2
      DOUBLE PRECISION SUM
C
      DO 20 I=1,K
      J1 = 1-I
      J2 = K-I
      SUM = 0.0
      DO 10 J=J1,J2
cxx   10 SUM = SUM + S(I+J)*Q(J)
      SUM = SUM + S(I+J)*Q(J)
   10 CONTINUE
cxx   20 P(I) = SUM
      P(I) = SUM
   20 CONTINUE
C
      RETURN
      E N D
cc      SUBROUTINE  SCONVL( Q,P,R,S,K,T )
      SUBROUTINE  SCONVLK( Q,P,R,S,K,T )
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  S(K), P(K), R(K), T(K), Q(-K:K)
C
      INTEGER K
      DOUBLE PRECISION Q(-K:K), P(K), R(K), S(K), T(K)
c local
      INTEGER I, J, J1, J2
      DOUBLE PRECISION SUM
C
      DO 20 I=1,K
      J1 = 1-I
      J2 = K-I
      SUM = 0.0D0
      DO 10 J=J1,J2
cxx   10 IF(P(I+J).GT.0.0D0)  SUM = SUM + P(I+J)/R(I+J)*Q(J)
      IF(P(I+J).GT.0.0D0)  SUM = SUM + P(I+J)/R(I+J)*Q(J)
   10 CONTINUE
cxx   20 T(I) = S(I)*SUM
      T(I) = S(I)*SUM
   20 CONTINUE
      RETURN
      E N D
      SUBROUTINE  TRANS1(K,DX,TAU2,BV,Q )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  Q(-K:K), PARAM(3)
C
      INTEGER K
      DOUBLE PRECISION DX, TAU2, BV, Q(-K:K)
c local
      INTEGER I, J
      DOUBLE PRECISION USERV, PARAM(3), X0, X, SUM
C
cc      EXTERNAL   FUNCT
C
      PARAM(1) = 0.0D0
      PARAM(2) = TAU2
      PARAM(3) = BV
C
      Q = 0
c
      DO 20 I=1-K,K-1
      X0 = -DX*I - DX/2
      SUM = (USERV(X0,PARAM) + USERV(X0+DX,PARAM))/2
      DO 10 J=1,49
      X = X0 + (DX*J)/50
cxx   10 SUM = SUM + USERV( X,PARAM )
      SUM = SUM + USERV( X,PARAM )
   10 CONTINUE
cxx   20 Q(I) = SUM*DX/50
      Q(I) = SUM*DX/50
   20 CONTINUE
C
      RETURN
      E N D
      SUBROUTINE  TRANS2( K,DX,TAU2,BV,Q )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  Q(-K:K), PARAM(3)
C
      INTEGER K
      DOUBLE PRECISION DX, TAU2, BV, Q(-K:K)
c local
      INTEGER I, J
      DOUBLE PRECISION GAUSS, PARAM(3), X0, X, SUM
C
cc      EXTERNAL   FUNCT
C
      PARAM(1) = 0.0D0
      PARAM(2) = TAU2
      PARAM(3) = BV
C
      Q = 0
c
      DO 20 I=1-K,K-1
      X0 = -DX*I - DX/2
      SUM = (GAUSS(X0,PARAM) + GAUSS(X0+DX,PARAM))/2
      DO 10 J=1,49
      X = X0 + (DX*J)/50
cxx   10 SUM = SUM + GAUSS( X,PARAM )
      SUM = SUM + GAUSS( X,PARAM )
   10 CONTINUE
cxx   20 Q(I) = SUM*DX/50
      Q(I) = SUM*DX/50
   20 CONTINUE
C
      RETURN
      E N D
      SUBROUTINE  TRANS3( K,DX,TAU2,BV,Q )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  Q(-K:K), PARAM(3)
C
      INTEGER K
      DOUBLE PRECISION DX, TAU2, BV, Q(-K:K)
c local
      INTEGER I, J
      DOUBLE PRECISION PEARSN, PARAM(3), X0, X, SUM
C
cc      EXTERNAL   FUNCT
C
      PARAM(1) = 0.0D0
      PARAM(2) = TAU2
      PARAM(3) = BV
C
      Q = 0
c
      DO 20 I=1-K,K-1
      X0 = -DX*I - DX/2
      SUM = (PEARSN(X0,PARAM) + PEARSN(X0+DX,PARAM))/2
      DO 10 J=1,49
      X = X0 + (DX*J)/50
cxx   10 SUM = SUM + PEARSN( X,PARAM )
      SUM = SUM + PEARSN( X,PARAM )
   10 CONTINUE
cxx   20 Q(I) = SUM*DX/50
      Q(I) = SUM*DX/50
   20 CONTINUE
C
      RETURN
      E N D
      SUBROUTINE  TRANS4( K,DX,TAU2,BV,Q )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  Q(-K:K), PARAM(3)
C
      INTEGER K
      DOUBLE PRECISION DX, TAU2, BV, Q(-K:K)
c local
      INTEGER I, J
      DOUBLE PRECISION TWOEXP, PARAM(3), X0, X, SUM
C
cc      EXTERNAL   FUNCT
C
      PARAM(1) = 0.0D0
      PARAM(2) = TAU2
      PARAM(3) = BV
C
      Q = 0
c
      DO 20 I=1-K,K-1
      X0 = -DX*I - DX/2
      SUM = (TWOEXP(X0,PARAM) + TWOEXP(X0+DX,PARAM))/2
      DO 10 J=1,49
      X = X0 + (DX*J)/50
cxx   10 SUM = SUM + TWOEXP( X,PARAM )
      SUM = SUM + TWOEXP( X,PARAM )
   10 CONTINUE
cxx   20 Q(I) = SUM*DX/50
      Q(I) = SUM*DX/50
   20 CONTINUE
C
      RETURN
      E N D

      SUBROUTINE  NORMLZ( P,K,DX,SUM )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  P(K)
      INTEGER K
      DOUBLE PRECISION P(K), DX, SUM
c local
      INTEGER I
C
      SUM = 0.0D0
      DO 10 I=1,K
cxx   10 SUM = SUM + P(I)
      SUM = SUM + P(I)
   10 CONTINUE
      SUM = SUM*DX
      DO 30 I=1,K
cxx   30 P(I) = P(I)/SUM
      P(I) = P(I)/SUM
   30 CONTINUE
C
      RETURN
      E N D
cc      SUBROUTINE  IDIST( P,K,P1,P2,XMIN,DX )
      SUBROUTINE  IDIST( P,K,P1,P2,XMIN,DX,INITD )
cxx      IMPLICIT REAL*8( A-H,O-Z )
cxx      DIMENSION  P(K), PARAM(3)
cc      COMMON   /C91215/  NOISEV, NOISEW, INITD, ITF, ITH
C
      INTEGER K, INITD 
      DOUBLE PRECISION P(K), P1, P2, XMIN, DX
c local
      INTEGER I
      DOUBLE PRECISION USERI, GAUSS, PARAM(3), X
C
      PARAM(1) = P1
      PARAM(2) = P2
C
      DO 10 I=1,K
      X = XMIN + DX*(I-1)
      IF( INITD.EQ.0 )  P(I) = USERI( X,PARAM )
      IF( INITD.EQ.1 )  P(I) = GAUSS( X,PARAM )
cc      IF( INITD.EQ.2 )  P(I) = UNIF ( X,PARAM )
cxx      IF( INITD.EQ.2 )  P(I) = UNIF ( X )
      IF( INITD.EQ.2 )  P(I) = 1.0D0
   10 CONTINUE
      RETURN
      END
cc      SUBROUTINE  BAYES( P,K,XMIN,DX,Y,F,LSHIFT )
      SUBROUTINE  BAYES( NOISEW,SIG2,BW,P,K,XMIN,DX,Y,F,LSHIFT )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  P(K), F(K), PARAM(3)
cc      COMMON   /C91215/  NOISEV, NOISEW, INITD, ITF, ITH
cc      COMMON   /C91218/  SIG2, TAU2, BW, BV
cc      EXTERNAL  USERW
cc      EXTERNAL  GAUSS
cc      EXTERNAL  PEARSN
cc      EXTERNAL  TWOEXP
cc      EXTERNAL  DBLEXP
C
      INTEGER NOISEW, K, LSHIFT
      DOUBLE PRECISION SIG2, BW, P(K), XMIN, DX, Y, F(K)
c local
      INTEGER I
      DOUBLE PRECISION USERW, GAUSS, PEARSN, TWOEXP, DBLEXP, PARAM(3)
C
      PARAM(2) = SIG2
      PARAM(3) = BW
C
      DO 10 I=1,K
      PARAM(1) = XMIN + DX*(I-1+LSHIFT)
      IF( NOISEW.EQ.0 )  F(I) = P(I)*USERW ( Y,PARAM )
      IF( NOISEW.EQ.1 )  F(I) = P(I)*GAUSS ( Y,PARAM )
      IF( NOISEW.EQ.2 )  F(I) = P(I)*PEARSN( Y,PARAM )
      IF( NOISEW.EQ.3 )  F(I) = P(I)*TWOEXP( Y,PARAM )
      IF( NOISEW.EQ.4 )  F(I) = P(I)*DBLEXP( Y,PARAM )
   10 CONTINUE
      RETURN
      E N D
      DOUBLE PRECISION FUNCTION  USERW( Y,PARAM )
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  PARAM(3)
C
      DOUBLE PRECISION Y, PARAM(3)
c local
      DOUBLE PRECISION C1, YMEAN, VAR
      DATA  C1  /2.506628275D0/
C
      YMEAN = 0.0D0
      VAR   = PARAM(1)
      USERW = DEXP( -(Y-YMEAN)**2/(2*VAR) )/(C1*DSQRT( VAR ))
      RETURN
      E N D
      DOUBLE PRECISION FUNCTION  USERV( X,PARAM )
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  PARAM(3)
C
      DOUBLE PRECISION X, PARAM(3)
c local
      DOUBLE PRECISION C1
      DATA  C1  /2.506628275D0/
C
      USERV = DEXP( -(X-PARAM(1))**2/(2*PARAM(2)) )
     *                /(C1*DSQRT( PARAM(2) ))
      RETURN
      E N D
      DOUBLE PRECISION FUNCTION  TWOEXP( X,PARAM )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  PARAM(2)
      DOUBLE PRECISION X, PARAM(2)
C
      TWOEXP = DEXP( -DABS(X-PARAM(1))*PARAM(2) )*PARAM(2)/2.0D0
      RETURN
C
      E N D
cc      SUBROUTINE  SHIFT( F,K,T,II,N,LOC )
      SUBROUTINE  SSHIFT( F,K,T,II,N,LOC )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cx      DIMENSION  F(K), T(K), LOC(*)
cxx      DIMENSION  F(K), T(K), LOC(N)
C
      INTEGER K, II, N, LOC(N)
      DOUBLE PRECISION F(K), T(K)
c local
      INTEGER I, IMAX, J
      DOUBLE PRECISION PMAX
C
C  ...  FIND THE POSTERIOR MODE AND SHIFT ORIGIN  ...
C
      PMAX = 0.0D0
c-----------
      IMAX = 1
c-----------
      DO 10  I=1,K
      IF( F(I).LE.PMAX )  GO TO 10
         PMAX = F(I)
         IMAX = I
   10 CONTINUE
      IF(II.LT.N)  LOC(II+1) = LOC(II) + IMAX - (K+1)/2
      DO 20 I=1,K
      J = I + IMAX - (K+1)/2
      T(I) = 0.0D0
cxx   20 IF(J.GE.1.AND.J.LE.K )  T(I) = F(J)
      IF(J.GE.1 .AND. J.LE.K )  T(I) = F(J)
   20 CONTINUE
      DO 30 I=1,K
cxx   30 F(I) = T(I)
      F(I) = T(I)
   30 CONTINUE
C
      RETURN
      E N D
      SUBROUTINE  POST3D( F,LOC,K,N )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      REAL*8     F(K,N)
cc      DIMENSION  YH(2000), FF(801), LOC(N)
cxx      REAL*8     FF(-K:2*K)
cxx      DIMENSION  LOC(N)
C
      INTEGER K, N, LOC(N)
      DOUBLE PRECISION F(K,N)
c local
      INTEGER I, II, I1, I2, J, NDIF, NN, N0
      DOUBLE PRECISION FF(-K:2*K)
C
cc      COMMON   /C91214/  X0, X1, F0, F1, YM(10)
cc      ANGLE = 70.0
C
cc      WX = 8.0
cc      WY = 2.0
cc      WZ =12.0
cc      CALL  DELX( X0,X1,DX )
cc      Z0 =  0.0
cc      Z1 =  N
cc      KK = 1
      NDIF = 1
cc      IF( N.GE.100 )  NDIF = 2
cc      IF( N.GE.200 )  NDIF = 4
cc      IF( N.GE.300 )  NDIF = 6
cc      IF( N.GE.500 )  NDIF = N/50
cc      DZ = NDIF*10
      NN = N/NDIF
      N0 = NDIF/2 + 1
cc      DO 10 I=1,K
cc   10 FF(I) = F(I,1)
cc      CALL  MAXMIN( FF,K,Y0,Y1,DY )
C
cc      CALL  PLOT3A( K,NN,WX,WY,WZ,ANGLE,X0,X1,DX,Y0,Y1,DY,Z0,Z1,DZ,
cc     *              KN,KK,YH )
c
      DO 100 J=N0,N,NDIF
cc      DO 80 I=1,K
      DO 80 I=-K,2*K
cxx   80 FF(I) = 0.0D0
      FF(I) = 0.0D0
   80 CONTINUE
      II = LOC(J)
      I1 = MAX0( 1,II )
      I2 = MIN0( K,K+II )
      DO 90 I=I1,I2
cxx   90 FF(I+II) = F(I,J)
      FF(I+II) = F(I,J)
   90 CONTINUE
cc      CALL  PLOT3B( FF,K,NN,WX,WY,WZ,ANGLE,Y0,Y1,KK,YH,ZS,ZC )
      DO 95 I=1,K
cxx   95 F(I,J) = FF(I)
      F(I,J) = FF(I)
   95 CONTINUE
  100 CONTINUE
cc      CALL  PLOT( -SNGL(ZC),-SNGL(ZS),-3 )
C
      RETURN
      E N D
      DOUBLE PRECISION FUNCTION  USERI( X,PARAM )
C
C  ...  User supplied density function  ...
C       (The following is an example of two-sided exponential dist.)
C
C     Inputs:
C        X:
C        PARAM(1):  mean
C        PARAM(2):  lambda
C     Output:
C        USERI:     density at X
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  PARAM(2)
C
      DOUBLE PRECISION X, PARAM(2)
c local
      DOUBLE PRECISION SIGMA
C
      SIGMA = DSQRT( PARAM(2) )
      USERI = SIGMA*DEXP( -SIGMA*DABS(X-PARAM(1)) )/2
      RETURN
      E N D
cc      SUBROUTINE  MAXMIN( X,N,XMIN0,XMAX0,DXL )
      SUBROUTINE  MAXMINK( X,N,XMIN0,XMAX0,DXL )
C
C  ...  This subroutine determines the minimum, the maximum and
C       the step width  ...
C
C     Inputs:
C        X(I):    data
C        N:       data length
C     Outputs:
C        XMIN0:   the minimum bound for the figure
C        XMAX0:   the maximum bound for the figure
C        DXL:     step width
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION X(N)
C
      INTEGER N
      DOUBLE PRECISION X(N), XMIN0, XMAX0, DXL
c local
      INTEGER I
      DOUBLE PRECISION XMIN, XMAX, DX, DIF
C
      XMIN = 1.0D30
      XMAX =-1.0D30
C
      DO 10 I=1,N
      IF( X(I) .LT. XMIN )   XMIN = X(I)
cxx   10 IF( X(I) .GT. XMAX )  XMAX = X(I)
      IF( X(I) .GT. XMAX )  XMAX = X(I)
   10 CONTINUE
      DX = XMAX-XMIN
      IF( DLOG10(DX) .GE. 0.0D0 )  DXL = INT( DLOG10(DX) )
      IF( DLOG10(DX) .LT. 0.0D0 )  DXL = INT( DLOG10(DX) )-1.0
      DXL = 10.0**DXL
      IF( DX/DXL.GT.6.0D0 )  DXL = DXL*2.0
      DIF = INT( DX/DXL )
      XMIN0 = INT( XMIN/DXL )*DXL
      XMAX0 = XMIN0 + DIF*DXL
      IF( XMIN0 .GT. XMIN )  XMIN0 = XMIN0 - DXL
   30 IF( XMAX0 .GE. XMAX )  GO TO 40
        XMAX0 = XMAX0 + DXL
        GO TO 30
   40 CONTINUE
C
      RETURN
      E N D

# PROGRAM 11.1
polreg <- function (y, order, plot = TRUE, ...) 
{
  n <- length(y)      # data length
  k <- order          # order of polinomial regression  
  k1 <- k + 1

  z <- .Fortran(C_polreg,
                as.double(y),
                as.integer(n),
                as.integer(k),
                a = double(k * k),
                sig2 = double(k1),
                aic = double(k1),
                imin = integer(1),
                data = double(n))

  a <- array(z$a, dim = c(k, k))
  coef <- list()
  for (i in 1:k)
    coef[[i]] <- a[1:i, i]
  aic <- z$aic
  imin <- z$imin
  daic <- aic - aic[imin + 1]

  polreg.out <- list(order.maice = imin, sigma2 = z$sig2, aic = aic, daic = daic,
                     coef = coef, trend = z$data)
  class(polreg.out) <- "polreg"

  if (plot) {
    rdata <- deparse(substitute(y))
    eval(parse(text=paste(rdata, "<- y")))
    eval(parse(text=paste("plot.polreg(polreg.out,", rdata, ", ...)")))
    invisible(polreg.out)
  } else polreg.out
}

plot.polreg <- function(x, rdata = NULL, ...)
{
  ts.atr <- tsp(rdata)
  trend <- x$trend
  if (is.null(ts.atr) == FALSE)
    trend <- ts(trend, start = ts.atr[1], frequency = ts.atr[3])

  old.par <- par(no.readonly = TRUE)

  morder <- x$order.maice
  aicmin <- format(round(x$aic[morder+1], 2), nsmall=2)
  ylim <- range(trend, rdata, na.rm = TRUE)

  if (is.null(rdata) == TRUE) {
    mtitle <- paste("Trend component\n minimum aic =",
                    aicmin, " at order", morder)
  } else {
    tsname <- deparse(substitute(rdata))
    mtitle <- paste(tsname, "and trend component\n minimum aic =", aicmin,
                    " at order", morder)
    plot(rdata, type = "l", ylim = ylim, xlab = "", ylab = "", main = "",
         xaxs = "i", xaxt = "n", ...)
    par(new = TRUE)
  }

  plot(trend, type = "l", ylim = ylim, col = 2, xlab = "", main = mtitle,
       ylab = "", xaxs = "i", cex.main = 1.0, ...) 
  par(old.par)
}

# PROGRAM 14.1  NGSMTH
ngsmth <- function(y, noisev = 2,tau2, bv = 1.0, noisew = 1, sigma2, bw = 1.0, 
                   initd = 1, k = 200, plot = TRUE, ...)
{
# y                     # original data
  n <- length(y)        # data length
# noisev                # type of system noise density (0,1,2,3)
                        # 1: Gaussian (normal) / 2: Pearson family
                        # 3: two-sides exponential
  if (noisev!=1 && noisev!=2 && noisev!=3)
    stop("'noisev' is numeric in {1,2,3}" )
# tau2                  # variance of dispersion of system noise
# bv                    # shape parameter of system noise (for noisev=2)
  if (noisev == 2)
    if ((bv == 0.5) || (bv < 0.5))
      stop("shape parameter of system noise is greater than 0.5")
# noisew                # type of observation noise density (0,1,2,3,4)
                        # 1: Gaussian (normal) / 2: Pearson family
                        # 3: two-sided exponential / 4:double exponential
  if (noisew!=1 && noisew!=2 && noisew!=3 && noisew!=4)
    stop("'noisew' is numeric in {1,2,3,4}" )
# sigma2                # variance of dispersion of observation noise
# bw                    # shape parameter of observation noise (for noisew=2)
  if (noisew == 2)
    if ((bw == 0.5) || (bw < 0.5))
      stop("shape parameter of observation noise is greater than 0.5")
# initd                 # type of density function
                        # 1: Gaussian (normal) / 2: uniform
                        # 3: two-sided exponential
  if (initd!=1 && initd!=2 && initd!=3)
    stop("'initd' is numeric in {1,2,3}")
  if (initd==3)
    initd <- 0
# k                     # number of intervals

  ns <- 1
  nfe <- n
  npe <- n
  k1 <- k+1

  z <- .Fortran(C_ngsmthf,
                as.double(y),
                as.integer(n),
                as.integer(noisev),
                as.double(tau2),
                as.double(bv),
                as.integer(noisew),
                as.double(sigma2),
                as.double(bw),
                as.integer(initd),
                trend = double(npe * 7),
                smt = double(k1 * npe),
                lkhood = double(1),
                as.integer(ns),
                as.integer(nfe),
                as.integer(npe),
                as.integer(k1))

  trend <- array(z$trend, dim = c(n, 7))
  smt <- array(z$smt, dim = c(k1, n))

  ngsmth.out <- list(llkhood = z$lkhood, trend = trend, smt = smt)
  class(ngsmth.out) <- "ngsmth"

  if (plot)
    plot.ngsmth(ngsmth.out, "trend",...)

  return(ngsmth.out)
}

print.ngsmth <- function(x, ...) {
  message(gettextf("\n log-likelihood\t%12.3f", x$llkhood), domain = NA)
}


plot.ngsmth <- function(x, type = c("trend", "smt"), theta = 0, phi = 15,
                        expand = 1, col = "lightblue", ticktype= "detail", ...)
{
  ntype <- length(type)

  if (ntype == 0 )
    return()
  type1 <- FALSE
  type2 <- FALSE
  for (i in 1:ntype) {
    if (type[i] == "trend")
      type1 <- TRUE
    if (type[i] == "smt")
      type2 <- TRUE
  }

  if (type1 == TRUE) {
    old.par <- par(no.readonly = TRUE)
    ylim <- c(floor(min(x$trend)), ceiling(max(x$trend)))
    for (i in 1:7) {
      if (i != 1)
        par(new = TRUE) 
      if (i == 4) {
        plot(x$trend[, i], type = "l", col = 2, ylim = ylim, xlab = "n",
             ylab = "trend  tn", xaxs = "i", yaxs = "i", ...)
      } else {
        plot(x$trend[, i], type = "l", ylim = ylim, xlab = "", ylab = "",
             xaxs = "i", yaxs = "i", ...)
      }
    }
    par(old.par)
  }

  if (type2 == TRUE) {
    if (type1 == TRUE)
      dev.new()

    k <- dim(x$smt)[1] - 1
    n <- dim(x$smt)[2]

    ndif <- 1
    if (n >= 100)
      ndif <- 2
    if (n >= 200)
      ndif <- 4
    if (n >= 300)
      ndif <- 6
    if (n >= 500)
      ndif <- as.integer(n / 50)
    n0 <- as.integer(ndif / 2) + 1
    nn <- as.integer((n-n0) / ndif) + 1
    ss <- array(0, dim = c(k+1, nn))
    jj <- 0
    for (j in n0:n) {
      if ((j-n0)%%ndif == 0) {
        ss[, jj] <- x$smt[, j]
        jj <- jj + 1
      }
    }

    xs <- floor(min(x$trend))
    xe <- ceiling(max(x$trend))
    xx <- seq(xs, xe, length = k+1)
    yy <- seq(1, n, length = nn)
    zlim <- c(0, ceiling(max(ss)))
    persp(xx, yy, ss, zlim = zlim, xlab = "tn", ylab = "n", zlab = "p(tn)",
          theta = theta, phi = phi, expand = expand, col = col,
          ticktype = ticktype, ...)
  }  
}

C     PROGRAM 13.1  TVVAR
      SUBROUTINE TVVARF(Y,N0,M,TAU20,IOPT,DELTA,TVVAR,NORDAT,Y2,N,TREND,
     *                  NOISE,TAUMAX,SIG2M,FFMAX,AIC)
C
C  ...  TIME-VARYING VARIANCE  ...
C
C     Inputs:
C        M:       Trend Order
C        IOPT:    Search Method
C        TAU20:   Initial Estimate of TAU2
C        DELTA:   Search Width
C     Parameters:
C        NMAX:    Adjustable dimension of Y
C        MJ:      Adjustable dimension of XF, VF, etc.
C        MAXM:    Adjustable dimension of A, B, C
C        NC:      Number of components
C     @TEST.FILTER2:  SEP.08,1990
C     MODIFIED  2/15/93
C
cc      PARAMETER( MJ1=3000,NMAX=MJ1/2,MJ=2,K=1,NDIM=NMAX )
cxxx      PARAMETER( K=1 )
      INTEGER, PARAMETER :: K=1
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(MJ1), Y2(NMAX)
cc      DIMENSION  F(MJ,MJ), G(MJ), H(MJ), Q(K,K)
cc      DIMENSION  XPS(MJ,NMAX), XFS(MJ,NMAX), XSS(MJ,NMAX)
cc      DIMENSION  VPS(MJ,MJ,NMAX), VFS(MJ,MJ,NMAX), VSS(MJ,MJ,NMAX)
cc      DIMENSION  XF(MJ), VF(MJ,MJ)
cxx      DIMENSION  Y(N0), Y2(N0/2)
cxx      DIMENSION  F(M,M), G(M,K), H(M), Q(K,K)
cxx      DIMENSION  XPS(M,N0/2), XFS(M,N0/2), XSS(M,N0/2)
cxx      DIMENSION  VPS(M,M,N0/2), VFS(M,M,N0/2), VSS(M,M,N0/2)
cxx      DIMENSION  XF(M), VF(M,M)
cxx      DIMENSION  TVVAR(N0/2), TREND(N0/2,3)
cxx      REAL*8     NORDAT(N0), NOISE(N0/2)
C
      INTEGER N0, M, IOPT, N
      DOUBLE PRECISION Y(N0), TAU20, DELTA, TVVAR(N0/2), NORDAT(N0),
     1                 Y2(N0/2), TREND(N0/2,3), NOISE(N0/2), TAUMAX,
     2                 SIG2M, FFMAX, AIC
c local
      INTEGER I, II, J, NS, NFE, NPE, NDIM
      DOUBLE PRECISION F(M,M), G(M,K), H(M), Q(K,K), XPS(M,N0/2),
     1                 XFS(M,N0/2), XSS(M,N0/2), VPS(M,M,N0/2),
     2                 VFS(M,M,N0/2), VSS(M,M,N0/2), XF(M), VF(M,M),
     3                 OUTMIN, OUTMAX, SIG2, YMIN, YMEAN, VAR, TAU2, R,
     4                 FF
C
      DATA  OUTMIN, OUTMAX /-1.0D30, 1.0D30/
C
cc      IDEV = 1
      SIG2 =1.0D0
cc      READ( 5,* )  M, IOPT
cc      IF( IOPT.EQ.1 )  READ( 5,* )  TAU20, DELTA
C
C  ...  Read Time Series  ...
C
cc      CALL READTS ( IDEV,Y,N0 )
      N = N0/2
      YMIN = 1.0D30
      DO 10 I=1,N
      J = 2*I
      Y2(I) = (Y(J-1)**2 + Y(J)**2)/2
cxx   10 IF( Y2(I).GT.0.0D0 .AND. Y2(I).LT.YMIN )  YMIN = Y2(I)
      IF( Y2(I).GT.0.0D0 .AND. Y2(I).LT.YMIN )  YMIN = Y2(I)
   10 CONTINUE
      DO 20 I=1,N
cxx   20 Y2(I) = DLOG( DMAX1(Y2(I),YMIN/2) )
      Y2(I) = DLOG( DMAX1(Y2(I),YMIN/2) )
   20 CONTINUE
C
      NS = 1
      NFE = N
      NPE = N
      NDIM = N
      CALL  MOMENT( Y2,N/10,YMEAN,VAR )      
C
      FFMAX  = -1.0D30
C
      DO 100  II=1,19
cc      IF( IOPT.NE.0 )  TAU2 = TAU20 + DELTA*(II-9)
      TAU2 = TAU20 + DELTA*(II-9)
      IF( IOPT.EQ.0 .AND. M.EQ.1 )  TAU2 = 2.0D0**(-II)
      IF( IOPT.EQ.0 .AND. M.GT.1 )  TAU2 = 2.0D0**(-II-5)
C
C  ...  Set Up State Space Model for Trend Estimation  ...
C
cc      CALL  SETTRN( M,MJ,F,G,H,R )
      CALL  SETTRN( M,F,G,H,R )
      R = (3.14159265D0)**2/6
C
C  ...  Set initial values  ...
C
cc      CALL  ISTATE( M,MJ,YMEAN,VAR,XF,VF )
      CALL  ISTATE( M,YMEAN,VAR,XF,VF )
C
C  ...  Kalman Filter  ...
C
      Q(1,1) = TAU2
cc      CALL  FILTER( Y2,XF,VF,F,G,H,Q,R,M,K,1,NS,NFE,NPE,MJ,NMAX,
      CALL  FILTER( Y2,XF,VF,F,G,H,Q,R,M,K,1,NS,NFE,NPE,
     *              NDIM,OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,SIG2 )
      IF( FF.GT.FFMAX )  THEN
         FFMAX  = FF
         TAUMAX = TAU2
         SIG2M  = SIG2
      END IF
cc  100 WRITE(6,600)  TAU2, SIG2, FF
  100 CONTINUE
      AIC = -2*FFMAX + 2*(M+2)
cc      WRITE(6,610)  TAUMAX, SIG2M, FFMAX, AIC
C
C  ...  Filtering with the best parameter  ...
C
cc      CALL  ISTATE( M,MJ,YMEAN,VAR,XF,VF )
      CALL  ISTATE( M,YMEAN,VAR,XF,VF )
      Q(1,1) = TAUMAX
cc      CALL  FILTER( Y2,XF,VF,F,G,H,Q,R,M,K,1,NS,NFE,NPE,MJ,NMAX,
      CALL  FILTER( Y2,XF,VF,F,G,H,Q,R,M,K,1,NS,NFE,NPE,
     *              NDIM,OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,SIG2 )
C
C  ... Fixed Interval Smoother  ...
C
cc      CALL  SMOOTH( F,M,MJ,NDIM,NS,NFE,NPE,VFS,VPS,XFS,XPS,
cc     *              VSS,XSS )
      CALL  SMOOTH( F,M,NDIM,NS,NFE,NPE,VFS,VPS,XFS,XPS,VSS,XSS )
C
C  ...  Plot and print data, trend and noise  ...
C
cc      CALL  PTTRND( Y2,XSS,VSS,N,MJ,M,TAUMAX,SIG2,FF,AIC )
cc      CALL  PRVAR( Y,M,TAUMAX,SIG2,FF,AIC,XSS,MJ,N,N0 )
cxx      CALL  PTTRND( Y2,XSS,VSS,N,MJ,M,TAUMAX,SIG2,FF,AIC,TREND,NOISE )
cxx      CALL  PRVAR( Y,M,TAUMAX,SIG2,FF,AIC,XSS,MJ,N,N0,TVVAR,NORDAT )
      CALL  PTTRND( Y2,XSS,VSS,N,M,SIG2,TREND,NOISE )
      CALL  PRVAR( Y,M,XSS,N,N0,TVVAR,NORDAT )
C
      RETURN
cxx  600 FORMAT( 1H ,5X,F15.10,F12.6,F13.5 )
cxx  610 FORMAT( 1H ,'TAUMAX =',F15.10,3X,'SIG2 =',F15.10,3X,
cxx     *            'FF =',F13.5,3X,'AIC =',F13.4 )
      E N D
cc      SUBROUTINE  PRVAR( Y,M,TAU2,SIG2,FF,AIC,XSS,MJ,N,N0 )
cxx      SUBROUTINE  PRVAR( Y,M,TAU2,SIG2,FF,AIC,XSS,MJ,N,N0,DATA,DATA1 )
      SUBROUTINE  PRVAR( Y,M,XSS,N,N0,DATA,DATA1 )
C
C  ...  Print out changing variance and normalized data  ...
C
C     Inputs:
C        Y:      Time Series
C        M:      Trend order
C        TAU2:   Variance of the system noise
C        SIG2:   Variance of the observational noise
C        FF:     Log-Likelihood of the mode
C        AIC:    AIC of the model
C        XSS:    Smoothed state
C        MJ:     Adjustable dimension of F and G
C        N:      Data LENGTH
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      CHARACTER  TITLE*72
cc      DIMENSION  XSS(MJ,*), Y(*), DATA(1500)
cx      DIMENSION  XSS(M,*), Y(*), DATA(N), DATA1(N0)
cxx      DIMENSION  XSS(M,N), Y(N0), DATA(N), DATA1(N0)
cc      COMMON  /CMDATA/  TITLE
C
      INTEGER M, N, N0
      DOUBLE PRECISION Y(N0), XSS(M,N), DATA(N), DATA1(N0)
c local
      INTEGER I, J
C
cc      WRITE(6,600)
cc      WRITE(6,610)  TITLE
cc      WRITE(6,620)  M, TAU2, SIG2, FF, AIC
      DO 10 I=1,N
cxx   10 DATA(I) = DEXP(XSS(1,I)+0.57721D0)
      DATA(I) = DEXP(XSS(1,I)+0.57721D0)
   10 CONTINUE
cc      WRITE(6,630)
cc      WRITE(6,640)  (DATA(I),I=1,N)
      DO 20 I=1,N0
      J = (I+1)/2
cc   20 Y(I) = Y(I)/DSQRT( DATA(J) )
cxx   20 DATA1(I) = Y(I)/DSQRT( DATA(J) )
      DATA1(I) = Y(I)/DSQRT( DATA(J) )
   20 CONTINUE
cc      WRITE(6,650)
cc      WRITE(6,640)  (Y(I),I=1,N0)
      RETURN
C
cxx  600 FORMAT( 1H1,'PROGRAM 12.3:  CHANGING VARIANCE ESTIMATION' )
cxx  610 FORMAT( 1H ,'DATA: ',A72 )
cxx  620 FORMAT( 1H ,'M =',I3,3X,'TAU2 =',D12.5,3X,'SIG2 =',D12.5,
cxx     *        3X,'LOG-L =',F12.4,3X,'AIC =',F12.4 )
cxx  630 FORMAT( 1H ,'***  TIME VARYING VARIANCE  ***' )
cc  640 FORMAT( 1H ,5D13.6 )
cxx  640 FORMAT( 5(f13.6,', '))
cxx  650 FORMAT( 1H ,'***  NORMALIZED DATA  ***' )
      E N D
cc      SUBROUTINE  PTTRND( Y,XSS,VSS,N,MJ,M,TAU2,SIG2,FF,AIC )
cxx      SUBROUTINE  PTTRND( Y,XSS,VSS,N,MJ,M,TAU2,SIG2,FF,AIC,TREND,DATA)
      SUBROUTINE  PTTRND( Y,XSS,VSS,N,M,SIG2,TREND,DATA)
C
C  ...  Plot Original time series, trend ans residuals  ...
C
C     Inputs:
C        Y(I):   Time Series
C        XSS:    Smoothed state
C        VSS:    Covariance matrices
C        N:      Data length
C        MJ:     Adjustable dimension of F and G
C        M:      Model order
C        TAU2:   Variance of the model
C        FF:     Log-Likelihood of the mode
C        AIC:    AIC of the model
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      CHARACTER  VNAME*8
cxx      DIMENSION  Y(N)
cc      DIMENSION  XSS(MJ,N), VSS(MJ,MJ,N)
cc      DIMENSION  DATA(1500), VNAME(20), VALUE(20)
cxx      DIMENSION  XSS(M,N), VSS(M,M,N)
cxx      DIMENSION  DATA(N), TREND(N,-1:1)
C
      INTEGER N, M
      DOUBLE PRECISION Y(N), XSS(M,N), VSS(M,M,N), SIG2, TREND(N,-1:1),
     1                 DATA(N)
c local
      INTEGER I, J
C
cc      VNAME(1) = 'M1    = '
cc      VNAME(2) = 'TAU2  = '
cc      VNAME(3) = 'SIG2  = '
cc      VNAME(4) = 'FF    = '
cc      VNAME(5) = 'AIC   = '
cc      VALUE(1) = M
cc      VALUE(2) = TAU2
cc      VALUE(3) = SIG2
cc      VALUE(4) = FF
cc      VALUE(5) = AIC
cc      CALL  PLOTS
C     call  plots( 1,0,0,1,0 )
C     call  form( 1 )
C     call  factor( 10.0 )
cc      CALL  HEADER( 'PROGRAM 10.2: TREND ESTIMATION ',38,5,
cc     *               VNAME,VALUE )
cc      WX = 15.0
cc      WY =  4.5
cc      DX =200.0
cc      FN = N
cc      IY = 10
C
C  ...  Plot original data  ...
C
cc      CALL  MAXMIN( Y,N,YMIN1,YMAX1,DY )
cc      CALL  NEWPEN( 2 )
cc      CALL  PLOT( 3.0,16.0-SNGL(WY),-3 )
cc      CALL  SYMBOL( 0.25,SNGL(WY)+0.25,0.25,'TIME SERIES',0.0,11 )
cc      CALL  AXISXY( 0.0D0,0.0D0,WX,WY,0.0D0,FN,YMIN1,YMAX1,DX,DY,
cc     *              0.2D0,1,IY,2 )
cc      CALL  PLOTY( Y,N,YMIN1,YMAX1,WX,WY,1,1 )
C
C  ...   PLOT  TREND  ...
C
cc      CALL  PLOT( 0.0,-SNGL(WY+1.0),-3 )
cc      CALL  NEWPEN( 2 )
cc      CALL  SYMBOL( 0.25,SNGL(WY)+0.25,0.25,'TREND',0.0,5 )
cc      CALL  AXISXY( 0.0D0,0.0D0,WX,WY,0.0D0,FN,YMIN1,YMAX1,DX,DY,
cc     *              0.2D0,1,IY,2)
C
      DO 120 J=-1,1
      DO 100 I=1,N
cc  100 DATA(I) = XSS(1,I) + DSQRT( VSS(1,1,I)*SIG2 )*J
cxx  100 TREND(I,J) = XSS(1,I) + DSQRT( VSS(1,1,I)*SIG2 )*J
      TREND(I,J) = XSS(1,I) + DSQRT( VSS(1,1,I)*SIG2 )*J
  100 CONTINUE
cc      CALL  NEWPEN( 1 )
cc      IF(J.EQ.0)  CALL  NEWPEN( 2 )
cc      CALL  PLOTY( DATA,N,YMIN1,YMAX1,WX,WY,1,1 )
  120 CONTINUE
C
C  ...  Plot residuals  ...
C
      DO 180 J=4,4,3
      DO 160 I=1,N
cxx  160 DATA(I) = Y(I) - XSS(1,I)
      DATA(I) = Y(I) - XSS(1,I)
  160 CONTINUE
cc      CALL  MAXMIN( DATA,N,DMIN,DMAX,DDY )
cc      YMIN3 = DY*INT(DMIN/DY)
cc      YMAX3 = DY*INT(DMAX/DY)
cc      IF( DMIN.LT.YMIN3 )  YMIN3 = YMIN3-DY
cc      IF( DMAX.GT.YMAX3 )  YMAX3 = YMAX3+DY
cc      WY3 = WY*(YMAX3-YMIN3)/(YMAX1-YMIN1)
C
cc      CALL  PLOT( 0.0,-SNGL(WY3+1.0),-3 )
cc      CALL  NEWPEN( 2 )
cc      CALL  SYMBOL( 0.25,SNGL(WY3)+0.25,0.25,'RESIDUAL',0.0,8 )
cc      CALL  AXISXY( 0.0D0,0.0D0,WX,WY3,0.0D0,FN,YMIN3,YMAX3,DX,DY,
cc     *              0.2D0,1,IY,2)
cc      CALL  NEWPEN( 1 )
cc      IF(J.EQ.4)  CALL  NEWPEN( 2 )
cc      CALL  PLOTY2( DATA,N,YMIN3,YMAX3,WX,WY3,1,1,0.0D0 )
  180 CONTINUE
C
cc      CALL  PLOTE
C     call  plot( 0.0,0.0,999 )
C
      RETURN
      E N D
cc      SUBROUTINE  SETTRN( M,MJ,F,G,H,R )
      SUBROUTINE  SETTRN( M,F,G,H,R )
C
C  ...  State space model for trend estimation  ...
C
C     Input:
C       M:   Order of the trend
C       MJ:  Adjustable Dimension of F
C     Outputs:
C       F:   M*M Transition Matrix
C       G,H: M Vector
C       R:   Observational Noise (=1)
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  F(MJ,MJ), G(MJ), H(MJ)
cxx      DIMENSION  F(M,M), G(M), H(M)
C
      INTEGER M
      DOUBLE PRECISION F(M,M), G(M), H(M), R
C
cc      DO 10  J=1,MJ
cxx      DO 10  J=1,M
cxx      G(J) = 0.0D0
cxx      H(J) = 0.0D0
cc      DO 10  I=1,MJ
cxx      DO 10  I=1,M
cxx   10 F(I,J) = 0.0D0
      F(1:M,1:M) = 0.0D0
      G(1:M) = 0.0D0
      H(1:M) = 0.0D0
C
      IF( M.EQ.1 )  THEN
        F(1,1) = 1.0D0
      END IF
      IF( M.EQ.2 )  THEN
        F(1,1) = 2.0D0
        F(1,2) =-1.0D0
        F(2,1) = 1.0D0
      END IF
      IF( M.EQ.3 )  THEN
        F(1,1) = 3.0D0
        F(1,2) =-3.0D0
        F(1,3) = 1.0D0
        F(2,1) = 1.0D0
        F(3,2) = 1.0D0
      END IF
      G(1) = 1.0D0
      H(1) = 1.0D0
      R    = 1.0D0
C
      RETURN
      E N D
cc      SUBROUTINE  FILTER( Y,XF,VF,F,G,H,Q,R,M,K,ISW,NS,NFE,NPE,MJ,NMAX,
      SUBROUTINE  FILTER( Y,XF,VF,F,G,H,Q,R,M,K,ISW,NS,NFE,NPE,
     *                    NDIM,OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,SIG2 )
C
C  ...  Kalman Filter (General Form, L=1)  ...
C
C     Inputs:
C        Y:      time series
C        NS:     Start position of filtering
C        NFE:    End position of filtering
C        NPE:    End position of prediction
C        XF:     Initial state vector
C        VF:     Initial covariance matrix
C        M:      Dimension of the state vector
C        K:      Dimension of the system noise
C        F:      M*M matrix
C        G:      M*K matrix
C        H:      M vector
C        Q:      K*K matrix, system noise covariance
C        R:      observation variance
C        ISW:    = 0;   R will be estimated
C                = 1;   R is given
C        MJ:     Adjustable dimension of XF, VF
C        NDIM:   Adjustable dimension of XFS, XPS, VFS, VPS
C                = 0   XF, XP, VF, VP are not stored
C                > 0   They are stored for smoothing
C        NMAX    Adjustable dimension of Y
C        OUTMIN: Lower limit for detecting outliers
C        OUTMAX: Upper limit for detecting outliers
C     Outputs:
C        VFS:    Covariance matrices of the filter
C        VPS:    Covariance matrices of the predictor
C        XFS:    Mean vectors of the filter
C        XPS:    Mean vectors of the predictor
C        FF:     Log likelihood
C        SIG2:   Estimated observational noise variance
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(NMAX)
cc      DIMENSION  F(MJ,MJ), G(MJ,K), H(MJ), Q(K,K)
cc      DIMENSION  XF(MJ), VF(MJ,MJ), XP(40), VP(40,40)
cc      DIMENSION  XFS(MJ,NDIM),    XPS(MJ,NDIM)
cc      DIMENSION  VFS(MJ,MJ,NDIM), VPS(MJ,MJ,NDIM)
cc      DIMENSION  WRK(40,40), VH(40), GAIN(40)
cxx      DIMENSION  Y(NPE)
cxx      DIMENSION  F(M,M), G(M,K), H(M), Q(K,K)
cxx      DIMENSION  XF(M), VF(M,M), XP(M), VP(M,M)
cxx      DIMENSION  XFS(M,NDIM),    XPS(M,NDIM)
cxx      DIMENSION  VFS(M,M,NDIM), VPS(M,M,NDIM)
cxx      DIMENSION  WRK(M,M), WRK1(M,K), VH(M), GAIN(M)
C
      INTEGER M, K, ISW, NS, NFE, NPE, NDIM
      DOUBLE PRECISION Y(NPE), XF(M), VF(M,M), F(M,M), G(M,K), H(M),
     1                 Q(K,K), R, OUTMIN, OUTMAX, VFS(M,M,NDIM),
     2                 VPS(M,M,NDIM), XFS(M,NDIM), XPS(M,NDIM), FF, SIG2
c local
      INTEGER I, II, J, JJ, NSUM
      DOUBLE PRECISION XP(M), VP(M,M), WRK(M,M), WRK1(M,K), VH(M),
     1                 GAIN(M), PI, SDET, SUM, PERR, PVAR
C
      DATA   PI  /3.1415926535D0/
C
      SIG2 = 0.0D0
      SDET = 0.0D0
      NSUM = 0
C
      DO 500  II=NS,NPE
C
C  ...  ONE STEP AHEAD PREDICTION  ...
C
      DO 20  I=1,M
      SUM = 0.0D0
      DO 10  J=1,M
cxx   10 SUM = SUM + F(I,J)*XF(J)
      SUM = SUM + F(I,J)*XF(J)
   10 CONTINUE
cxx   20 XP(I) = SUM
      XP(I) = SUM
   20 CONTINUE
C
cxx   DO 40  I=1,M
      DO 41 I=1,M
      DO 40  J=1,M
      SUM = 0.0D0
      DO 30  JJ=1,M
cxx   30 SUM = SUM + F(I,JJ)*VF(JJ,J)
      SUM = SUM + F(I,JJ)*VF(JJ,J)
   30 CONTINUE
cxx   40 WRK(I,J) = SUM
      WRK(I,J) = SUM
   40 CONTINUE
   41 CONTINUE
C
cxx      DO 60  I=1,M
      DO 61  I=1,M
      DO 60  J=1,M
      SUM = 0.0D0
      DO 50 JJ=1,M
cxx   50 SUM = SUM + WRK(I,JJ)*F(J,JJ)
      SUM = SUM + WRK(I,JJ)*F(J,JJ)
   50 CONTINUE
cxx   60 VP(I,J) = SUM
      VP(I,J) = SUM
   60 CONTINUE
   61 CONTINUE
C
cxx      DO 80  I=1,M
      DO 81  I=1,M
      DO 80  J=1,K
      SUM = 0.0D0
      DO 70 JJ=1,K
cxx   70 SUM = SUM + G(I,JJ)*Q(JJ,J)
      SUM = SUM + G(I,JJ)*Q(JJ,J)
   70 CONTINUE
cc   80 WRK(I,J) = SUM
cxx   80 WRK1(I,J) = SUM
      WRK1(I,J) = SUM
   80 CONTINUE
   81 CONTINUE
C
cxx      DO 100  I=1,M
      DO 101  I=1,M
      DO 100  J=1,M
      SUM = VP(I,J)
      DO 90  JJ=1,K
cc   90 SUM = SUM + WRK(I,JJ)*G(J,JJ)
cxx   90 SUM = SUM + WRK1(I,JJ)*G(J,JJ)
      SUM = SUM + WRK1(I,JJ)*G(J,JJ)
   90 CONTINUE
cxx  100 VP(I,J) = SUM
      VP(I,J) = SUM
  100 CONTINUE
  101 CONTINUE
C
C  ...  FILTERING  ...
C
      IF( Y(II).GT.OUTMIN.AND.Y(II).LT.OUTMAX.AND. II.LE.NFE ) THEN
C
      DO 210  I=1,M
      SUM = 0.0D0
      DO 200  J=1,M
cxx  200 SUM = SUM + VP(I,J)*H(J)
      SUM = SUM + VP(I,J)*H(J)
  200 CONTINUE
cxx  210 VH(I) = SUM
      VH(I) = SUM
  210 CONTINUE
C
      PERR = Y(II)
      PVAR = R
      DO 220  I=1,M
      PERR = PERR - H(I)*XP(I)
cxx  220 PVAR = PVAR + H(I)*VH(I)
      PVAR = PVAR + H(I)*VH(I)
  220 CONTINUE
C
      DO 250  I=1,M
cxx  250 GAIN(I) = VH(I)/PVAR
      GAIN(I) = VH(I)/PVAR
  250 CONTINUE
C
      DO 290  I=1,M
cxx  290 XF(I) = XP(I) + GAIN(I)*PERR
      XF(I) = XP(I) + GAIN(I)*PERR
  290 CONTINUE
C
cxx      DO 310  I=1,M
      DO 311  I=1,M
      DO 310  J=1,M
cxx  310 VF(I,J) = VP(I,J) - GAIN(I)*VH(J)
      VF(I,J) = VP(I,J) - GAIN(I)*VH(J)
  310 CONTINUE
  311 CONTINUE
C
      SIG2 = SIG2 + PERR**2/PVAR
      SDET = SDET + DLOG(PVAR)
      NSUM = NSUM + 1
C
C  ...  MISSING OBSERVATION  ...
C
      ELSE
cxx      DO 350  I=1,M
      DO 351  I=1,M
      XF(I) = XP(I)
      DO 350  J=1,M
cxx  350 VF(I,J) = VP(I,J)
      VF(I,J) = VP(I,J)
  350 CONTINUE
  351 CONTINUE
      END IF
C
C  ...  SAVE MEAN AND COVARIANCE  ...
C
      IF( NDIM.GT.1 )  THEN
cxx      DO 360  I=1,M
      DO 361  I=1,M
      XPS(I,II) = XP(I)
      XFS(I,II) = XF(I)
      DO 360  J=1,M
      VPS(I,J,II) = VP(I,J)
cxx  360 VFS(I,J,II) = VF(I,J)
      VFS(I,J,II) = VF(I,J)
  360 CONTINUE
  361 CONTINUE
      END IF
C
  500 CONTINUE
      SIG2 = SIG2/NSUM
      IF(ISW.EQ.0)  FF = -0.5D0*(NSUM*(DLOG(PI*2*SIG2) + 1) + SDET)
      IF(ISW.EQ.1)  FF = -0.5D0*(NSUM*(DLOG(PI*2) + SIG2) + SDET)
C
      RETURN
      E N D
cc      SUBROUTINE  ISTATE( M,MJ,XMEAN,XVAR,XF,VF )
      SUBROUTINE  ISTATE( M,XMEAN,XVAR,XF,VF )
C
C  ...  Initial state ...
C
C     Inputs:
C        M:     Dimension of the state vector
C        MJ:    Adjustable dimension of F
C        XMEAN: Mean value
C        XVAR:  Varaince
C     Outputs:
C         XF:   State vector, X(0|0)
C         VF:   State covarance matrix, V(0|0)
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  XF(MJ), VF(MJ,MJ)
cxx      DIMENSION  XF(M), VF(M,M)
C
      INTEGER M
      DOUBLE PRECISION XMEAN, XVAR, XF(M), VF(M,M)
c local
      INTEGER I
C
cc      DO 10  I=1,MJ
cc      DO 10  J=1,MJ
cxx      DO 10  I=1,M
cxx      DO 10  J=1,M
cxx   10 VF(I,J) = 0.0D0
      VF(1:M,1:M) = 0.0D0

C
      DO 20  I=1,M
cxx   20 XF(I) = XMEAN
      XF(I) = XMEAN
   20 CONTINUE
C
      DO 30  I=1,M
cxx   30 VF(I,I) = XVAR
      VF(I,I) = XVAR
   30 CONTINUE
C
      RETURN
      E N D

      SUBROUTINE ARMA(M,L,A,B,SIG2,K,KMAX,NF,G,COV,PAR,SP,
     * ROOTA,ROOTB,IER,JER)
C
C     PROGRAM 6.1  ARMA
C
C  ...  This program analyses time series via ARMA modeling  ...
C
C     Inputs:
C        IDEV:  Input devie (=5: CONSOLE)
C        M:     AR order
C        L:     MA order
C        SIG2:  Innovation variance
C        A(I):  AR coefficients
C        B(I):  MA coefficients
C        K:     Maximum lag of autocovariance function (K<MJ+1)
C     Parameter:
C        MJ:    Adjustable dimension of A, B, etc..
C        NF:    Number of frequencies in evaluating spectrum
C     Programmed by Y.I and G.K.
C
C     Outputs:
C         IER:  =1 : MATRIX WITH ZERO ROW IN DECOMPOSE
C               =2 : SINGULAR MATRIX IN DECOMPOSE.ZERO DIDIVIDE IN SOLVE
C               =3 : CONVERGENCE IN IMPRUV.MATRIX IS NEARLY SINGULAR
C         JER:  =1 : NON-CONVERGENCE AT POLYRT
C
cc      PARAMETER( NF=200,MJ=100,IDEV=1 )
cxx      IMPLICIT REAL*8( A-H,O-Z )
cc      DIMENSION  A(MJ), B(MJ), PAR(MJ), ROOTA(MJ,2), ROOTB(MJ,2)
cc      DIMENSION  G(0:MJ), COV(0:MJ), SP(0:NF)
cc      DIMENSION  WRK1(0:MJ), WRK2(0:MJ), WRK3(MJ)
cc      DATA  PAR/MJ*0.0D0/
cxx      DIMENSION  A(M), B(L), PAR(K), ROOTA(M,2), ROOTB(L,2)
cxx      DIMENSION  G(0:KMAX), COV(0:K), SP(0:NF)
cxx      DIMENSION  WRK1(0:K), WRK2(0:K), WRK3(K,K)
C
      INTEGER M, L, K, KMAX, NF, IER, JER
      DOUBLE PRECISION A(M), B(L), SIG2, G(0:KMAX), COV(0:K), PAR(K),
     1                 SP(0:NF), ROOTA(M,2), ROOTB(L,2)
c local
      INTEGER MAR, JER1, JER2
      DOUBLE PRECISION WRK1(0:K), WRK2(0:K), WRK3(K,K)
C
cc      WRITE( 6,* )  'K = ?'
cc      READ( 5,* )   K
cc      OPEN( IDEV,FILE='arma.dat' )
cc      READ(IDEV,*)  M, L
cc      READ(IDEV,*)  SIG2
cc      READ(IDEV,*)  (A(I),I=1,M)
cc      READ(IDEV,*)  (B(I),I=1,L)
cc      CLOSE( IDEV )
C
cc      KMAX = MAX(M,L,K)
      CALL  IMPULS( M,L,A,B,K,G )
cc      CALL  ARMCOV( M,L,A,B,SIG2,K,COV )
      CALL  ARMCOV( M,L,A,B,SIG2,K,COV,KMAX,IER )
      if( ier.ne.0 ) return
c------------
      PAR(1:K) = 0.0D0
c------------
      CALL  PARCOR( A,M,PAR )
      CALL  ARCOEF( PAR,M,A )
cc      IF( L.GT.0 )  CALL  ARYULE( COV,1000,K,WRK1,WRK2,PAR,WRK3,MAR )
      IF( L.GT.0 )  CALL  ARYULE( COV,0,K,WRK1,WRK2,PAR,WRK3,MAR )
      CALL  ARMASP( A,M,B,L,SIG2,NF,SP )
cc      CALL  CHROOT( A,M,ROOTA,MJ )
cc      CALL  CHROOT( B,L,ROOTB,MJ )
      CALL  CHROOT( A,M,ROOTA,M,JER1 )
      CALL  CHROOT( B,L,ROOTB,L,JER2 )
      JER = JER1
      IF( JER2 .NE. 0 ) JER = JER + JER2 + 1
cc      CALL  PRARMA( M,L,A,B,G,K,COV,K,PAR,SP,NF,ROOTA,ROOTB,MJ )
C      CALL  PTARMA( G,K,COV,K,PAR,SP,NF,ROOTA,M,ROOTB,L,MJ )
      RETURN
      E N D
cc      SUBROUTINE  CHROOT( A,M,ROOT,MJ )
      SUBROUTINE  CHROOT( A,M,ROOT,MJ,IER )
C
C  ...  Characteristic roots of the AR or MA operator  ...
C
C     Inputs:
C        A:     Vector of AR or MA coefficients
C        M:     Order of the model
C        MJ:    Adjustable dimension of ROOT
C     Output:
C        ROOT:  Characteristic roots (real part,imaginary part)
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  A(M), ROOT(MJ,2)
cc      DIMENSION  C(50), CW(50)
cxx      DIMENSION  C(M+1)
C
      INTEGER M, MJ, IER
      DOUBLE PRECISION A(M), ROOT(MJ,2)
c local
      INTEGER I, MMAX
      DOUBLE PRECISION C(M+1)
C
      IER = 0
C
      IF( M.EQ.0 )  RETURN
      DO 10  I=1,M
cxx   10 C(I) = -A(M-I+1)
      C(I) = -A(M-I+1)
   10 CONTINUE
      C(M+1) = 1.0D0
      MMAX = M
C
C  ... Characteristic roots of operator 1-A(1)*S- ... -A(M)*S**M=0  ...
C
cc      CALL  POLYRT( C,CW,MMAX,ROOT(1,1),ROOT(1,2),IER )
      CALL  POLYRT( C,MMAX,ROOT(1,1),ROOT(1,2),IER )
cc      IF( IER.NE.0 )   WRITE(6,600)
C
      RETURN
cxx  600 FORMAT( 1H0,'*****  NON-CONVERGENCE AT POLYRT  *****' )
      E N D
cc      SUBROUTINE  POLYRT( A,B,M,ROOTR,ROOTI,IER )
      SUBROUTINE  POLYRT( A,M,ROOTR,ROOTI,IER )
C
C  ...  This subroutine finds the roots of the equation
C            A(1) + A(2)*S + ... + A(M)*(S**M) = 0
C       by Newton-Raphson method
C
C     Inputs:
C        A:     Coefficients of the equation
C        B:     Working area
C        M:     Degree of the polynomial
C     Outputs:
C        ROOTR:   Real parts of the roots
C        ROOTI:   Imaginary parts of the roots
C        IER:     Error code to indicate non-convergence
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cc      DIMENSION A(1), B(1), ROOTR(1), ROOTI(1)
cxx      DIMENSION A(M+1), B(M+3), ROOTR(M), ROOTI(M)
C
      INTEGER M, IER 
      DOUBLE PRECISION A(M+1), ROOTR(M), ROOTI(M)
c local
      INTEGER I, IFIT, ISW, ICT, ITEM, IN, J, JSW, K, KX, KXX, KJI,
     1        K2, L
      DOUBLE PRECISION B(M+3), XPR, YPR, X, XO, Y, YO, UX, UY, U, V,
     1                 XT, YT, XT2, YT2, FI, SUM, DX, DY, TEM, ALPH
C
      IFIT = 0
      ISW = 0
      JSW = 0
      K = M
      IER = 0
      KX = K
      KXX = K+1
      KJI = KXX
      K2 = 1
c------------
      XPR = 0.0D0
      YPR = 0.0D0
c------------
      DO 10  I=1,KXX
      A(I) = A(I)/A(K+1)
      J = KXX - I+1
cxx   10 B(J) = A(I)
      B(J) = A(I)
   10 CONTINUE
   20 XO = 0.5D-02
      YO = 0.1D-01
      IN = 0
   30 X = XO
C
      XO = -10.D0*YO
      YO = -10.D0*X
      X = XO
      Y = YO
      ICT = 0
      IN = IN+1
      GO TO 50
C
   40 IFIT = 1
      XPR = X
      YPR = Y
C
   50 UX = 0.D0
      UY = 0.D0
      V  = 0.D0
      YT = 0.D0
      XT = 1.D0
      U = B(K+1)
      IF( U .EQ. 0.D0 )  GO TO 140
      DO 60  I=1,K
      L = K-I+1
      XT2 = X*XT - Y*YT
      YT2 = X*YT+Y*XT
      U = U + B(L)*XT2
      V = V + B(L)*YT2
      FI = I
      UX = UX + FI*XT*B(L)
      UY = UY - FI*YT*B(L)
      XT = XT2
      YT = YT2
   60 CONTINUE
      SUM = UX**2 + UY**2
      IF( SUM .EQ. 0.D0 )  GO TO 100
      DX = (V*UY - U*UX)/SUM
      X = X + DX
      DY = -(U*UY + V*UX)/SUM
      Y = Y + DY
      IF( DABS(DY)+DABS(DX) .LT. 1.0D-10 )  GO TO 80
C
      ICT = ICT+1
      IF( ICT .LT. 500 )  GO TO 50
      ISW = 1
      IF( IN .GE. 5 )  GO TO 70
      IF( IFIT .NE. 0 )  GO TO 80
   65 ISW = 0
      IFIT = 0
      GO TO 30
C
   70 IF( IFIT .EQ. 0 )  GO TO 300
      JSW = 1
   80 DO 90  L=1,KXX
      J = KJI-L+1
      TEM = A(J)
      A(J) = B(L)
cxx   90 B(L) = TEM
      B(L) = TEM
   90 CONTINUE
      ITEM = K
      K = KX
      KX = ITEM
  100 IF( IFIT .EQ. 0 )  GO TO 40
      IF( JSW .EQ. 1 )   GO TO 110
cc      IF( ISW-1 )  120,65,120
      IF( ISW-1 .LT. 0 )  GO TO 120
      IF( ISW-1 .EQ. 0 )  GO TO 65
      IF( ISW-1 .GT. 0 )  GO TO 120
  110 X = XPR
      Y = YPR
      ISW = 0
      JSW = 0
  120 IFIT = 0
      IF( X .EQ. 0.D0 )  GO TO 130
      IF( DABS( Y/X ) .LT. 1.0D-08 )  GO TO 150
  130 ALPH = 2*X
      SUM = X*X + Y*Y
      K = K-2
      GO TO 160
C
  140 X = 0.D0
      KX = KX-1
      KXX = KXX-1
  150 Y = 0.D0
      SUM = 0.D0
      ALPH = X
      K = K-1
  160 B(2) = B(2) + ALPH*B(1)
cxx  170 DO 180  L=2,K
      DO 180  L=2,K
cxx  180 B(L+1) = B(L+1) + ALPH*B(L) - SUM*B(L-1)
      B(L+1) = B(L+1) + ALPH*B(L) - SUM*B(L-1)
  180 CONTINUE
cc  190 ROOTI(K2) = Y
cc      ROOTR(K2) = X
  190 IF( K2.LE.M ) ROOTI(K2) = Y
      IF( K2.LE.M ) ROOTR(K2) = X
      K2 = K2+1
      IF( SUM .EQ. 0.D0 )  GO TO 200
      Y = -Y
      SUM = 0.D0
      GO TO 190
  200 IF (K .LE. 0 )  RETURN
      GO TO 20
C
  300 IER = 1
      RETURN
      E N D

C     PROGRAM 2.3  CRSCOR
      SUBROUTINE CRSCORF( Y,N,L,LAG,OUTMIN,OUTMAX,C,R,YMEAN )
C
C  ...  This program computes sample cross-covariance and
C       sample cross-correlation functions  ...
C
C     The following inputs are required in the subroutine READMD.
C        TITLE:   title of the data
C        N:       data length
C        L:       dimension of the observation
C        IFM:     = 1   ((Y(I,J),J=1,L),I=1,N)
C                 = 2   ((Y(I,J),I=1,N),J=1,L)
C        Y(I,J):  multi-variate time series
C     Parameters:
C        MJ:      adjustable dimension of Y; (MJ.GE.N)
C        MJ1:     adjustable dimension of Y, C, R; (MJ1.GE.L)
C        LAG:     maximum lag of the cross-covariance function
C        IDEV:    input device specification
C     MAR.24,1989:  modified  12/20/90, 7/18/92
C
cc      PARAMETER( MJ=1000,MJ1=7,LAG=50,IDEV=1,JDEV=6 )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(MJ,MJ1), OUTMIN(10), OUTMAX(10)
cc      DIMENSION  C(0:LAG,MJ1,MJ1), R(0:LAG,MJ1,MJ1), YMEAN(10)
cc      DATA  OUTMIN/10*-1.0D30/, OUTMAX/10*1.0D30/
cxx      DIMENSION  Y(N,L), OUTMIN(L), OUTMAX(L)
cxx      DIMENSION  C(0:LAG,L,L), R(0:LAG,L,L), YMEAN(L)
C
      INTEGER N, L, LAG
      DOUBLE PRECISION Y(N,L), OUTMIN(L), OUTMAX(L), C(0:LAG,L,L),
     1                 R(0:LAG,L,L), YMEAN(L)
C
C  ...  read in multivariate time series  ...
C
cc      CALL  READMD( IDEV,MJ,Y,N,L )
C
C  ...  cross-covariance function  ...
C
cc      CALL  CRSCOR( Y,N,L,LAG,MJ,MJ1,OUTMIN,OUTMAX,C,R,YMEAN )
      CALL  CRSCOR( Y,N,L,LAG,OUTMIN,OUTMAX,C,R,YMEAN )
C
C  ...  print out and plot cross-correlation function  ...
C
cc      CALL  PRMCOR( JDEV,C,R,L,LAG,MJ1 )
C
cc      STOP
      RETURN
      E N D

# PROGRAM 7.2 MARFIT
marfit <- function(y, lag = NULL)
{
  n <- dim(y)[1]              # data length
  l <- dim(y)[2]              # dimension of the observation
  if (is.null(lag))           # maximum lag of the cross-covariance function
    lag <- as.integer(2 * sqrt(n))

  z <- .Fortran(C_marfit,
             as.double(y),
             as.integer(n),
             as.integer(l),
             as.integer(lag),
             amin = double(l * l * lag),
             vmin = double(l * l),
             aic = double(lag + 1),
             mmin = integer(1))

  mmin <- z$mmin
  vmin <- array(z$vmin, c(l, l))
  amin <- array(z$amin, c(lag, l, l))
  arcoef <- array(, c(l, l, mmin))
  for (i in 1:mmin)
    arcoef[, , i] <- amin[i, , ]

  marfit.out <- list(maice.order = mmin, aic = z$aic, v = vmin, arcoef = arcoef)
  class(marfit.out) <- "maryule"
  return(marfit.out)
}

print.maryule <- function(x, ...)
{
  aic <- x$aic
  n <- length(aic)
  m <- dim(x$v)[1]
  morder <- x$maice.order
  aicmin <- aic[morder + 1] 

  message("\n Order     AIC")
  for (i in 1:n) 
    message(gettextf("%5i  %12.3f", i-1, aic[i]), domain = NA)

  message(gettextf("\n Minimum AIC = %12.3f   attained at m = %5i", aicmin,
          morder), domain = NA)

  message("\n Innovation covariance matrix")
  for (i in 1:m) {
    for (j in 1:m)
      message(gettextf(" %13.6e", x$v[i,j]), appendLF=FALSE, domain = NA)
    message("")
  }
}

# PROGRAM 7.3 MARLSQ
marlsq <- function(y, lag = NULL)
{
  n <- dim(y)[1]             # data length
  l <- dim(y)[2]             # dimension of the observation
  if (is.null(lag))          # maximum lag of the cross-covariance function
    lag <- as.integer(2 * sqrt(n))

  z <- .Fortran(C_marlsq,
                as.double(y),
                as.integer(n),
                as.integer(l),
                as.integer(lag),
                arcoef = double(l * l * lag),
                v = double(l * l),
                lmax = integer(1),
                aic = double(1))

  arcoef <- array(z$arcoef, c(l, l, lag))
  v <- array(z$v, c(l, l))
  lmax <- z$lmax
  a <- arcoef[, , 1:lmax]

  marlsq.out <- list(maice.order = lmax, aic = z$aic, v = v, arcoef = a)
  class(marlsq.out) <- "marlsq"
  return(marlsq.out)
}

print.marlsq <- function(x, ...)
{
  message(gettextf("\n Total AIC = %12.3f", x$aic), domain = NA)
  message(gettextf(" order of the MAICE model = %5i", x$maice.order),
          domain = NA)

  m <- dim(x$v)[1]
  message("\n Innovation covariance matrix")
  for (i in 1:m) {
    for (j in 1:m)
      message(gettextf(" %13.6e", x$v[i,j]), appendLF=FALSE, domain = NA)
    message("")
  }
}

# PROGRAM 6.1
armachar <- function(arcoef = NULL, macoef = NULL, v, lag = 50, nf = 200,
                     plot = TRUE, ...)
{
  if (is.null(arcoef)) {        # AR coefficients
    arorder <- 0
    arcoef <- 0.0
  } else {
    arorder <- length(arcoef)   # AR order
  }
  if (is.null(macoef)) {        # MA coefficients
    maorder <- 0
    macoef <- 0.0
  } else {
    maorder <- length(macoef)   # MA order
  }

 if (arorder == 0 && maorder == 0)
   stop(" Both AR coefficient and MA coefficient are NULL.")

# v                             # innovation variance
# lag                           # maximum lag of autocovariance function
  kmax <- max(arorder, maorder, lag)
# nf                            # number of frequencies in evaluating spectrum

  ar2 <- max(arorder*2, 2)
  ma2 <- max(maorder*2, 2)
	
  z <- .Fortran(C_arma,
                as.integer(arorder),
                as.integer(maorder),
                as.double(arcoef),
                as.double(macoef),
                as.double(v),
                as.integer(lag),
                as.integer(kmax),    
                as.integer(nf),
			    g = double(kmax+1),
				acov = double(lag+1),
				parcor = double(lag),
				spec = double(nf+1),
				roota = double(ar2),
				rootb = double(ma2),
				ier = integer(1),
				jer = integer(1))

  ier <- z$ier
  if (ier == 1)
    stop(" Matrix with zero row in decompose")
  if (ier == 2)
  stop(" Singular matrix in decompose.zero divide in solve")
  if (ier == 3)
    stop(" Convergence in impruv.matrix is nearly singular")

  impuls <- z$g
  impuls <- impuls[1:(lag+1)]

  if (arorder == 0) {
    roota <- NULL
  } else { # arorder != 0
    roota <- array(z$roota, dim = c(arorder, 2))
  }
  if (maorder == 0) {
    rootb <- NULL
  } else { # maorder != 0
    rootb <- array(z$rootb, dim = c(maorder, 2))
  }

  croot.ar <- NULL
  croot.ma <- NULL
  jer <- z$jer
  if (jer == 1) {
    warning(" AR : Non-convergence at polyrt\n")
  } else if (jer == 2) {
    warning(" MA : Non-convergence at polyrt\n")
  } else if (jer == 3) {
    warning(" Non-convergence at polyrt\n")
  }

  if (arorder != 0) {
    croot.ar <- list()
    for (i in 1:arorder) {
      re <- roota[i, 1]
      im <- roota[i, 2]
      amp <- sqrt(re**2 + im**2)
      atan <- atan2(im, re)
      croot.ar[[i]] <- list(re = re, im = im, amp = amp, atan = atan,
                            degree = atan * 57.29577951)
    }
  }

  if (maorder != 0) {
    croot.ma <- list()
    for (i in 1:maorder) {
      re <- rootb[i, 1]
      im <- rootb[i, 2]
      amp <- sqrt(re**2 + im**2)
      atan <- atan2(im,re)
      croot.ma[[i]] <- list(re = re, im = im, amp = amp, atan = atan,
                            degree = atan*57.29577951)
    }
  }

  armachar.out <- list(impuls = impuls, acov = z$acov, parcor = z$parcor,
                       spec = z$spec, croot.ar = croot.ar, croot.ma = croot.ma)
  class(armachar.out) <- "arma"

  if (plot) {
    plot.arma(armachar.out, ...)
    invisible(armachar.out)
  } else armachar.out

}

plot.arma <- function(x, ...)
{
  impuls <- x$impuls
  acov <- x$acov
  parcor <- x$parcor
  spec <- x$spec
  croot.ar <- x$croot.ar
  croot.ma <- x$croot.ma
  lag <- length(parcor)

  old.par <- par(no.readonly = TRUE)
  par(xaxs = "i")
  if (is.null(croot.ar) == TRUE && is.null(croot.ma) == TRUE) {
    ier <- 3
    par(mfrow = c(2, 2), xaxs = "i")
  } else {
    ier <- 0
    par(mfrow = c(3, 2), xaxs = "i")
  }

  x <- c(0:lag)
  ylim <- c(floor(min(impuls)), ceiling(max(impuls)))
  plot(x, impuls, type = "l", xlim = c(0, lag), ylim = ylim, xlab = "",
       ylab = "", ...)
  par(new = TRUE)
  plot(x, impuls, type = "h", xlim = c(0, lag), ylim = ylim, xlab = 'lag',
       ylab = 'impulse', ...)

  ylim <- c(floor(min(acov)), ceiling(max(acov)))
  plot(x, acov, type = "l", xlim = c(0, lag), ylim = ylim, xlab = "",
       ylab = "", ...)
  par(new = TRUE)
  plot(x, acov, type = "h", xlim = c(0, lag), ylim = ylim, xlab = 'lag',
       ylab = 'autocovariance', ...)

  plot(parcor, type = "l", xlim = c(0, lag), ylim = c(-1, 1), xlab = "",
       ylab = "", ...)
  par(new = TRUE)
  plot(parcor, type = "h", xlim = c(0, lag), ylim = c(-1, 1), xlab = 'lag',
       ylab = 'parcor', ...)

  k1 <- length(spec)
  k <- k1 - 1
  x <- rep(0, k1)
  for (i in 1:k1)
    x[i] <- (i - 1) / (2 * k)
  ylim <- c(floor(min(spec)), ceiling(max(spec)))
  plot(x, spec, type = "l", ylim = ylim, xlab = "frequency",
       ylab = "log spectrum", ...)

  if (ier == 0) {
    par(pty = "s")
    plot(x = c(-1.0, 1.0), y = c(0, 0), type = "l", xlim = c(-1.1, 1.1),
         ylim = c(-1.1, 1.1), xlab = "ARMA characteristic roots", ylab = "",
         axes = FALSE, ...)
    par(new = TRUE)
    plot(x = c(0, 0), y = c(-1.0, 1.0), type = "l", xlim = c(-1.1, 1.1),
         ylim = c(-1.1, 1.1), xlab = "", ylab = "", axes = FALSE, ...)
    symbols(x = 0, y = 0, circles = 1, xlim = c(-1.0, 1.0),
            ylim = c(-1.0, 1.0), inches = FALSE, add = TRUE, ...)

    if (is.null(croot.ar) == FALSE) {
      arorder <- length(croot.ar)
      for (i in 1:arorder) {     # characteristic roots of AR operator
        x1 <- croot.ar[[i]]$re
        y1 <- croot.ar[[i]]$im
        points(x1, y1, pch = 4, col = 6, cex = 0.8)
      }
    }

    if (is.null(croot.ma) == FALSE) {
      maorder <- length(croot.ma)
      for (i in 1:maorder) {     # characteristic roots of MA operator
        x2 <- croot.ma[[i]]$re
        y2 <- croot.ma[[i]]$im
        points(x2, y2, pch = 3, col = 4, cex = 0.8)
      }
    }

    par(xpd = T)
    legend(par()$usr[2], par()$usr[4], legend = c("AR", "MA"), col = c(6,4),
           pch = c(4, 3), cex = 0.8, bty="n")
  }

  par(old.par)
  invisible(x)
}

C------------------------------------------------- 
C     ARCOEF  ---  arfit, armaft, season, tvar
C     FOUGER  ---  spgrh, tvar
C     PARCOR  ---  arfit, armaft, season, tvar
C     REGRES  ---  lsqr, arfit, lsar1, lsar2, polreg
C     COMAIC  ---  lsqr, arfit, lsar1, lsar2, polreg
C     HUSHLD  ---  lsqr, arfit, marlsq, lsar1, lsar2 
C     RECOEF  ---  lsqr, arfit, lsar1, lsar2, polreg
C     REDUCT  ---  arfit, lsar1, lsar2
CC     REDUCT1  ---  polreg (polregf.f)
CC     REDUCT2  ---  lsqr (lsqrf.f)
C     SETXAR  ---  arfit, lsar1, lsar2
C     ARYULE  ---  arma, arfit
C     ARMASP  ---  arma, arfit, lsar1, tvspc
C     FOURIE  ---  period, arma, marspc, arfit, tvspc
C     MOMENT ---  trend, season, tvvar, ngsmth
C     SMOOTH  ---  tvvar, smooth
C     GINVRS  ---  tvvar, smooth, season
C     SETSEA  ---  simssm, ngsim
C     CHOLES  ---  simssm, ngsim
C     CRSCOR  ---  crscor, marfit
C     MEAN  ---  unicor, crscor, period, arfit, marfit
C     AUTCOR  --- unicor, arfit
C     AUTCOV  ---  unicor, period, arfit
C     UNICOR  ---  unicor, arfit
C     ERRACF  ---  unicor, arfit
C     WINDOW  ---  period, fftper
C     ARMCOV  ---  arma, armaft, season
C     DECOM  ---  arma, armaft, season
C     IMPULS  ---  arma, armaft, season
C     SOLVE  ---  arma, armaft, season
C     SMOTH1  ---  seasom, tvar
C
C     ID  ---  simssm, ngsim, season
C     INIT  ---  simssm, ngsim
C     GAUSS  ---  densty, klinfo, ngsmth, ngsim
C     PEARSN  ---  densty, ngsmth, ngsim
C     DBLEXP  ---  densty, ngsim, ngsmth
C     CAUCHY  ---  densty, klinfo
C-------------------------------------------------
C
      SUBROUTINE  ARCOEF( PAR,K,A )
C
C  ...  This subroutine computes AR coefficients from PARCOR  ...
C
C     Inputs:
C        PAR(I):   PARCOR
C        K:        Order of AR model
C     Output:
C        A(I):     AR coefficient
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cc      DIMENSION  PAR(K), A(K), AA(50)
cxx      DIMENSION  PAR(K), A(K), AA(K)
      INTEGER K
      DOUBLE PRECISION PAR(K), A(K)
c local
      INTEGER II, J
      DOUBLE PRECISION AA(K)
C
      DO 30  II=1,K
      A(II)  = PAR(II)
      AA(II) = PAR(II)
      IF( II-1.LE.0 )  GO TO 30
      DO 10  J=1,II-1
cxx   10 A(J) = AA(J) - PAR(II)*AA(II-J)
      A(J) = AA(J) - PAR(II)*AA(II-J)
   10 CONTINUE
      IF( II.LT.K )  THEN
        DO 20  J=1,II-1
cxx   20   AA(J) = A(J)
        AA(J) = A(J)
   20   CONTINUE
      END IF
   30 CONTINUE
      RETURN
      E N D
c
C
      SUBROUTINE FOUGER(G,LGP1,FC,FS,LF1)
C
C     FOURIER TRANSFORM (GOERTZEL METHOD)
C     THIS SUBROUTINE COMPUTES FOURIER TRANSFORM OF G(I),I=0,1,...,LG AT
C     FREQUENCIES K/(2*LF), K=0,1,...,LF AND RETURNS COSINE TRANSFORM IN
C     FC(K) AND SINE TRANSFORM IN FS(K).
C
C       INPUTS:
C          G(I):   ORIGINAL DATA (I=0,1,...,LG)
C          LG1:    = LG+1
C          LF1:    = LF+1
C
C       OUTPUTS:
C          FC(I):  COSINE TRANSFORM OF G  (I=0,1,...,LF)
C          FB(I):  SINE TRANSFORM OF G  (I=0,1,...,LF)
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION G(LGP1),FC(LF1),FS(LF1)
      INTEGER LGP1, LF1
      DOUBLE PRECISION G(LGP1),FC(LF1),FS(LF1)
c local
      INTEGER I, I2, K, LG, LG3, LG4, LF
      DOUBLE PRECISION PI, ALF, T, AK, TK, CK, SK, CK2, UM0, UM1, UM2
      LG=LGP1-1
      LF=LF1-1
C     REVERSAL OF G(I),I=1,...,LGP1 INTO G(LG3-I)   LG3=LGP1+1
      IF(LGP1.LE.1) GO TO 110
      LG3=LGP1+1
      LG4=LGP1/2
      DO 100 I=1,LG4
      I2=LG3-I
      T=G(I)
      G(I)=G(I2)
cxx  100 G(I2)=T
      G(I2)=T
  100 CONTINUE
  110 PI=3.1415926536D0
      ALF=LF
      T=PI/ALF
      DO 10 K=1,LF1
      AK=K-1
      TK=T*AK
      CK=DCOS(TK)
      SK=DSIN(TK)
      CK2=CK+CK
      UM1=0.0D0
      UM2=0.0D0
      IF(LG.EQ.0) GO TO 12
      DO 11 I=1,LG
      UM0=CK2*UM1-UM2+G(I)
      UM2=UM1
cxx   11 UM1=UM0
      UM1=UM0
   11 CONTINUE
   12 FC(K)=CK*UM1-UM2+G(LGP1)
      FS(K)=SK*UM1
   10 CONTINUE
      RETURN
      END
C
C
      SUBROUTINE  PARCOR( A,K,PAR )
C
C  ...  This subriutine computes PARCOR for AR coefficients  ...
C
C       Inputs:
C          A:   Vector of AR coefficients
C          K:   Order of the model
C
C       Output:
C          PAR: Vector of partial autocorrelations (PARCOR)
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cc      DIMENSION  A(K), PAR(K), G(50)
cxx      DIMENSION  A(K), PAR(K), G(K)
      INTEGER K
      DOUBLE PRECISION A(K), PAR(K)
c local
      INTEGER I, II, I2
      DOUBLE PRECISION G(K), S
C
      DO 10  I=1,K
cxx   10 PAR(I) = A(I)
      PAR(I) = A(I)
   10 CONTINUE
C
c-----
      IF( K .EQ. 1 )   RETURN                                           
c-----
      DO 40  II=K-1,1,-1
      S = 1.D0 - PAR(II+1)**2
      DO 20  I=1,II
cxx   20 G(I) = (PAR(I) + PAR(II+1)*PAR(II-I+1))/S
      G(I) = (PAR(I) + PAR(II+1)*PAR(II-I+1))/S
   20 CONTINUE
      I2 = (II+1)/2
      IF( MOD( II,2 ).EQ.1 )  G(I2) = PAR(I2)/(1.D0 - PAR(II+1))
      DO 30  I=1,II
cxx   30 PAR(I) = G(I)
      PAR(I) = G(I)
   30 CONTINUE
   40 CONTINUE
C
      RETURN
      E N D
C
C
cc      SUBROUTINE  REGRES( X,K,N,MJ1,MJ2,A,SIG2,AIC,IMIN )
      SUBROUTINE  REGRES( X,K,N,MJ1,A,SIG2,AIC,IMIN )
C
C  ...  Regression model fitting  ...
C  ...  Order of the model is selected by AIC  ...
C
C     Inputs:
C        X:      Householder reduced form (upper triangular matrix)
C        K:      Maximum number of regressors
C        N:      Number of data
C        MJ1:    Adjustable dimension of X
C        MJ2:    Adjustable dimension of A, ISG2 and AIC
C     Outputs:
C        A(I,M): Regression coefficients of the model with order M
C        SIG2:   Residual variances
C        AIC:    AIC's
C        IMIN:   MAICE order
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  X(MJ1,1), A(MJ2,MJ2), SIG2(0:MJ2), AIC(0:MJ2)
cxx      DIMENSION  X(MJ1,K+1), A(K,K), SIG2(0:K), AIC(0:K)
      INTEGER K, N, MJ1, IMIN
      DOUBLE PRECISION X(MJ1,K+1), A(K,K), SIG2(0:K), AIC(0:K)
c local
      INTEGER M
      DOUBLE PRECISION AICM
C
      A(1:K, 1:K) = 0.0D0
C
      CALL  COMAIC( X,N,K,MJ1,SIG2,AIC )
C
      IMIN = 0
      AICM = AIC(0)
      DO 10  M=1,K
      IF( AIC(M).LT.AICM )  THEN
         IMIN = M
         AICM = AIC(M)
      END IF
      CALL  RECOEF( X,M,K,MJ1,A(1,M) )
   10 CONTINUE
C
      RETURN
      E N D
C
C
      SUBROUTINE  COMAIC( X,N,K,MJ1,SIG2,AIC )
C
C  ...  This subroutine computes residual variances and AIC's  ...
C
C     Inputs:
C        X(I,J):  Householder reduced form
C        N:       Data length
C        K:       Highest order
C        MJ1:     Adjustable dimension of X
C     Outputs:
C        SIG2(I): residual variance of the model with order I
C        AIC(I):  AIC of the model with order I
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cc      DIMENSION  X(MJ1,1), AIC(0:K), SIG2(0:K)
cxx      DIMENSION  X(MJ1,K+1), AIC(0:K), SIG2(0:K)
      INTEGER N, K, MJ1
      DOUBLE PRECISION X(MJ1,K+1), SIG2(0:K), AIC(0:K)
c local
      INTEGER I
      DOUBLE PRECISION PI2, PVAR
      DATA  PI2/6.28318531D0/
C
      PVAR = 0.0D0
      DO 10 I=K,0,-1
      PVAR = PVAR + X(I+1,K+1)**2
      SIG2(I) = PVAR / N
cxx   10 AIC(I)  = N*DLOG( PI2*SIG2(I) ) + N + 2*(I+1)
      AIC(I)  = N*DLOG( PI2*SIG2(I) ) + N + 2*(I+1)
   10 CONTINUE
C
      RETURN
      E N D
C
C
cc      SUBROUTINE  HUSHLD( X,D,MJ1,N,K )
      SUBROUTINE  HUSHLD( X,MJ1,N,K )
C
C  ...  Householder transformation  ...
C
C     Inputs:
C        X(I,J):  Original matrix
C        D(I):    Working area
C        MJ1:     Adjustable dimension of X
C        N:       Number of rows of X
C        K:       Number of columns of X
C     Output:
C        X(I,J):  Householder reduced form (upper triangular form)
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cc      DIMENSION  X(MJ1,1), D(MJ1)
cxx      DIMENSION  X(MJ1,K), D(MJ1)
      INTEGER MJ1, N, K
      DOUBLE PRECISION X(MJ1,K)
c local
      INTEGER I, II, J
      DOUBLE PRECISION D(MJ1), TOL, H, G, F, S
C
      TOL = 1.0D-60
C
cxx      DO 100  II=1,K
      DO 101  II=1,K
        H = 0.0D0
        DO 10  I=II,N
          D(I) = X(I,II)
cxx   10     H = H + D(I)**2
          H = H + D(I)**2
   10 CONTINUE
        IF( H .GT. TOL )  GO TO 20
        G = 0.0D0
        GO TO 100
   20   G = DSQRT( H )
        F = X(II,II)
        IF( F .GE. 0.0D0 )   G = -G
        D(II) = F - G
        H = H - F*G
C
        DO 30  I=II+1,N
cxx   30   X(I,II) = 0.0D0
        X(I,II) = 0.0D0
   30   CONTINUE
        DO 60  J=II+1,K
          S = 0.0D0
          DO 40  I=II,N
cxx   40     S = S + D(I)*X(I,J)
            S = S + D(I)*X(I,J)
   40     CONTINUE
          S = S/H
          DO 50  I=II,N
cxx   50     X(I,J) = X(I,J) - D(I)*S
            X(I,J) = X(I,J) - D(I)*S
   50     CONTINUE
   60   CONTINUE
  100 X(II,II) = G
  101 CONTINUE
C
      RETURN
      E N D
C
C
      SUBROUTINE  RECOEF( X,M,K,MJ,A )
C
C  ...  Regression coefficients  ...
C
C     Inputs:
C        X(I,J):  Householder reduced form
C        M:       Number of actually used regressors
C        K:       Heighest order
C        MJ:      Adjustable dimension of X
C     Output:
C        A(I):    Vector of regression coefficients
C
cxx      IMPLICIT REAL*8 (A-H,O-Z)
cc      DIMENSION  X(MJ,1), A(1)
cxx      DIMENSION  X(MJ,K+1), A(M)
      INTEGER M, K, MJ
      DOUBLE PRECISION X(MJ,K+1), A(M)
c local
      INTEGER I, J
      DOUBLE PRECISION SUM
C
      A(M) = X(M,K+1)/X(M,M)
c-----
      IF( M .EQ. 1 ) RETURN
c-----
      DO 20 I=M-1,1,-1
      SUM = X(I,K+1)
      DO 10 J=I+1,M
cxx   10 SUM  = SUM - A(J)*X(I,J)
      SUM  = SUM - A(J)*X(I,J)
   10 CONTINUE
cxx   20 A(I) = SUM/X(I,I)
      A(I) = SUM/X(I,I)
   20 CONTINUE
C
      RETURN
      E N D
C
C
      SUBROUTINE  REDUCT( SETX,Z,NMK,N0,K,MJ1,X )
C
C  ...  Successive Householder reduction  ...
C
C     Inputs:
C        SETX:    Name of the subroutine for making X(I,J)
C        Z(I):    Data vector
C        D(I):    Working area
C        NMK:     Number of actually used observations
C        N0:      Time point of the previous set ofobservations
C        K:       Heighest order of the model
C        MJ1:     Adjustable dimension of X
C     Output:
C        X(I,J):  data matrix
C
cxx      IMPLICIT  REAL*8( A-H,O-Z )
cc      DIMENSION  X(MJ1,1) , D(1), Z(1)
cx      DIMENSION  X(MJ1,1) , Z(1)
cxx      DIMENSION  X(MJ1,K+1) , Z(N0+NMK+K)
      INTEGER NMK, N0, K, MJ1
      DOUBLE PRECISION Z(N0+NMK+K), X(MJ1,K+1)
c local
      INTEGER L, LK, K1, N1, N2
C
      L = MIN0( NMK,MJ1 )
      K1 = K + 1
      N1 = L
C
cdd      CALL  SETX( Z,N0,L,K,MJ1,0,X )
      CALL SETXAR( Z,N0,L,K,MJ1,0,X )
cc      CALL  HUSHLD( X,D,MJ1,L,K1 )
      CALL  HUSHLD( X,MJ1,L,K1 )
      IF( N1 .GE. NMK )  RETURN
C
   10 L = MIN0( NMK-N1,MJ1-K1 )
C
      LK = L + K1
      N2 = N0 + N1
      CALL  SETX( Z,N2,L,K,MJ1,1,X )
cc      CALL  HUSHLD( X,D,MJ1,LK,K1 )
      CALL  HUSHLD( X,MJ1,LK,K1 )
      N1 = N1 + L
      IF( N1.LT.NMK )  GO TO 10
C
      RETURN
C
      E N D
C
C
      SUBROUTINE  SETXAR( Z,N0,L,K,MJ1,JSW,X )
C
C  ...  Data matrix for AR model  ...
C
C     Inputs:
C        Z(I):    Time series
C        N0:      Origin of the current observations
C        L:       Number of current observations
C        K:       Number of regressors
C        MJ1:     Adjustable dimension of X
C        JSW=0:   Make initial data matrix
C           =1:   Apend L*(K+1) data matrix below the triangular one
C     Output:
C        X(I,J):  Data matrix
C
cc      REAL*8  X(MJ1,1), Z(1)
cxx      REAL*8  X(MJ1,K+1), Z(N0+L+K)
      INTEGER N0, L, K, MJ1, JSW
      DOUBLE PRECISION Z(N0+L+K), X(MJ1,K+1)
c local
      INTEGER I, II, I0, J, JJ
C
      I0 = 0
      IF( JSW .EQ. 1 )     I0 = K+1
cxx      DO 10  I=1,L
      DO 11  I=1,L
        II = I + I0
        JJ = N0 + K + I
        X(II,K+1) = Z(JJ)
      DO 10  J=1,K
        JJ = JJ - 1
cxx   10 X(II,J) = Z(JJ)
      X(II,J) = Z(JJ)
   10 CONTINUE
   11 CONTINUE
C
      RETURN
      E N D
C
C
      SUBROUTINE ARYULE( C,N,MAXM,SIG2,AIC,PARCOR,A,MAR )
C
C  ...  Yule-Walker method  ...
C
C     Inputs:
C        C(I):    Autocovariance function
C        N:       Data length
C        MAXM:    Highest AR order
C     Outputs:
C        SIG2(I): Innovation variance
C        AIC(I):  AIC
C        PARCOR(I):  PARCOR
C        AMIN:     AR coefficients of the best model
C        MAR:      Selected order of the model
C
cxx      IMPLICIT REAL*8 (A-H,O-Z)
cxx      DIMENSION  C(0:MAXM), SIG2(0:MAXM), AIC(0:MAXM)
cxx      DIMENSION  PARCOR(MAXM), A(MAXM,MAXM)
      INTEGER N, MAXM, MAR
      DOUBLE PRECISION C(0:MAXM), SIG2(0:MAXM), AIC(0:MAXM),
     1                 PARCOR(MAXM), A(MAXM,MAXM)
c local
      INTEGER I, J, M
      DOUBLE PRECISION CONST, AICMIN, SUM
C
      CONST = N*(DLOG(2*3.1415926535D0) + 1)
C
      SIG2(0) = C(0)
      AIC(0) = CONST + N*DLOG(SIG2(0)) + 2
      AICMIN = AIC(0)
      MAR = 0
C
      DO 50 M=1,MAXM
      SUM  = C(M)
      IF (M.GE.2) THEN
      DO 10 I=1,M-1
cxx   10 SUM  = SUM - A(I,M-1)*C(M-I)
      SUM  = SUM - A(I,M-1)*C(M-I)
   10 CONTINUE
      END IF
      A(M,M) = SUM /SIG2(M-1)
      IF (M.GE.2) THEN
      DO 20 J=1,M-1
cxx   20 A(J,M) = A(J,M-1)-A(M,M)*A(M-J,M-1)
      A(J,M) = A(J,M-1)-A(M,M)*A(M-J,M-1)
   20 CONTINUE
      END IF
      SIG2(M) = SIG2(M-1)*(1.0D0-A(M,M)**2)
      AIC(M) = CONST + N*DLOG(SIG2(M)) + 2*(M+1)
      PARCOR(M) = A(M,M)
      IF( AIC(M).LT.AICMIN )  THEN
         AICMIN = AIC(M)
         MAR = M
      END IF
   50 CONTINUE
      RETURN
      E N D
C
C
      SUBROUTINE ARMASP( A,M,B,L,SIG2,NF,SP )
C
C  ...  Logarithm of the power spectrum of the ARMA model  ...
C
C     Inputs:
C        M:     AR order
C        L:     MA order
C        A(I):  AR coefficient
C        B(I):  MA coefficient
C        SIG2:  Innovation variance
C        NF:    Number of frequencies
C     Output:
C        SP(I): Power spectrum (in log scale)
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION A(M), B(L)
cc      DIMENSION SP(0:NF), H(0:500), FR(0:500), FI(0:500)
cxx      DIMENSION SP(0:NF), H(0:M+L), FR(0:NF), FI(0:NF)
      INTEGER M, L, NF
      DOUBLE PRECISION A(M), B(L), SIG2, SP(0:NF)
c local
      INTEGER I
      DOUBLE PRECISION H(0:M+L), FR(0:NF), FI(0:NF)
C
      H(0) = 1.0D0
      DO 10 I=1,M
cxx   10 H(I) = -A(I)
      H(I) = -A(I)
   10 CONTINUE
C
      CALL  FOURIE( H,M+1,NF+1,FR,FI )
C
      DO 20 I=0,NF
cxx   20 SP(I) = SIG2/( FR(I)**2 + FI(I)**2 )
      SP(I) = SIG2/( FR(I)**2 + FI(I)**2 )
   20 CONTINUE
C
      IF (L .EQ. 0) GO TO 41
      H(0) = 1.0D0
      DO 30 I=1,L
cxx   30 H(I) = -B(I)
      H(I) = -B(I)
   30 CONTINUE
      CALL  FOURIE( H,L+1,NF+1,FR,FI )
      DO 40 I=0,NF
cxx   40 SP(I) = SP(I)*( FR(I)**2 + FI(I)**2 )
      SP(I) = SP(I)*( FR(I)**2 + FI(I)**2 )
   40 CONTINUE
   41 CONTINUE
C
      DO 50 I=0,NF
cxx   50 SP(I) = DLOG10( SP(I) )
      SP(I) = DLOG10( SP(I) )
   50 CONTINUE
C
      RETURN
      E N D
C
C
      SUBROUTINE FOURIE( X,N,M,FC,FS )
C
C  ...  Discrete Fourier transformation by Goertzel method  ...
C
C     Inputs:
C        X(I):   data (I=1,N)
C        N:      data length
C        M:      number of Fourier components
C        FC(J):  Fourier cosine transform (J=1,M)
C        FS(J):  Fourier sine transform   (J=1,M)
C
cxx      IMPLICIT REAL*8 (A-H,O-Z)
cxx      DIMENSION  X(N), FC(M), FS(M)
      INTEGER N, M
      DOUBLE PRECISION X(N), FC(M), FS(M)
c local
      INTEGER I, J
      DOUBLE PRECISION PI, W, CI, SI, T0, T1, T2
      DATA  PI/3.14159265358979D0/
C
      W = PI/(M-1)
      DO 20 I=1,M
      CI = DCOS(W*(I-1))
      SI = DSIN(W*(I-1))
      T1 = 0.0
      T2 = 0.0
      DO 10 J=N,2,-1
        T0 = 2*CI*T1 - T2 + X(J)
        T2 = T1
cxx   10   T1 = T0
      T1 = T0
   10 CONTINUE
      FC(I) = CI*T1 - T2 + X(1)
cxx   20 FS(I) = SI*T1
      FS(I) = SI*T1
   20 CONTINUE
C
      RETURN
      E N D
      SUBROUTINE  MOMENT( Y,N,YMEAN,VAR )
C
C  ...  Mean and variance of the data  ...
C
C     Inputs:
C       Y(I):   data
C       N:      data length
C     Outputs:
C       YMEAN:  mean
C       VAR:    variance
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  Y(N)
      INTEGER N
      DOUBLE PRECISION Y(N), YMEAN, VAR, SUM
c local
      INTEGER I
C
      SUM = 0.0D0
      DO 10 I=1,N
cxx   10 SUM = SUM + Y(I)
      SUM = SUM + Y(I)
   10 CONTINUE
      YMEAN = SUM/N
      SUM = 0.0D0
      DO 20 I=1,N
cxx   20 SUM = SUM + (Y(I)-YMEAN)**2
      SUM = SUM + (Y(I)-YMEAN)**2
   20 CONTINUE
      VAR = SUM/N
C
      RETURN
      E N D
cc      SUBROUTINE  SMOOTH( F,M,MJ,NDIM,NS,NFE,NPE,VFS,VPS,XFS,XPS,
cc     *                    VSS,XSS )
      SUBROUTINE  SMOOTH( F,M,NDIM,NS,NFE,NPE,VFS,VPS,XFS,XPS,VSS,XSS )
C
C  ...  Fixed Interval Smoother (General Form)  ...
C
C     Inputs:
C        NS:     Start position of filtering
C        NFE:    End position of filtering
C        NPE:    End position of prediction
C        M:      Dimension of the state vector
C        F:      M*M matrix
C        MJ:     Adjustable dimension of XF, VF
C        NDIM:   Adjustable dimension of XFS, XPS, VFS, VPS
C        NMAX    Adjustable dimension of Y
C        VFS:    Covariance matrices of the filter
C        VPS:    Covariance matrices of the predictor
C        XFS:    Mean vectors of the filter
C        XPS:    Mean vectors of the predictor
C     Outputs:
C        VSS:    Covariance matrices of the smoother
C        XSS:    Mean vectors of the smoother
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  F(MJ,MJ)
cc      DIMENSION  XS(40), VS(40,40), VP(40,40)
cc      DIMENSION  XFS(MJ,NDIM), XPS(MJ,NDIM), XSS(MJ,NDIM)
cc      DIMENSION  VFS(MJ,MJ,NDIM), VPS(MJ,MJ,NDIM), VSS(MJ,MJ,NDIM)
cc      DIMENSION  WRK(40,40), SGAIN(40,40)
cxx      DIMENSION  F(M,M)
cxx      DIMENSION  XS(M), VS(M,M), VP(M,M)
cxx      DIMENSION  XFS(M,NDIM), XPS(M,NDIM), XSS(M,NDIM)
cxx      DIMENSION  VFS(M,M,NDIM), VPS(M,M,NDIM), VSS(M,M,NDIM)
cxx      DIMENSION  WRK(M,M), SGAIN(M,M)
      INTEGER M, NDIM, NS, NFE, NPE
      DOUBLE PRECISION F(M,M), VFS(M,M,NDIM), VPS(M,M,NDIM),
     1                 XFS(M,NDIM), XPS(M,NDIM), VSS(M,M,NDIM),
     2                 XSS(M,NDIM)
c local
      INTEGER I, II, J, IJ, NZERO
      DOUBLE PRECISION XS(M), VS(M,M), VP(M,M), WRK(M,M), SGAIN(M,M),
     1                 VDET, SUM
C
C
C  ...  SMOOTHING  ...
C
cxx      DO 10 II=NFE,NPE
cxx      DO 10  I=1,M
      DO 12 II=NFE,NPE
      DO 11  I=1,M
      XSS(I,II)   = XFS(I,II)
      DO 10  J=1,M
cxx   10 VSS(I,J,II) = VFS(I,J,II)
      VSS(I,J,II) = VFS(I,J,II)
   10 CONTINUE
   11 CONTINUE
   12 CONTINUE
cxx      DO 20  I=1,M
      DO 21  I=1,M
      XS(I)   = XFS(I,NFE)
      DO 20  J=1,M
cxx   20 VS(I,J) = VFS(I,J,NFE)
      VS(I,J) = VFS(I,J,NFE)
   20 CONTINUE
   21 CONTINUE
C
      DO 500  II=NFE-1,NS,-1
C
      NZERO = 0
      DO 100 I=1,M
cxx  100 IF( VFS(I,I,II).GT.1.0D-12 )  NZERO = NZERO + 
      IF( VFS(I,I,II).GT.1.0D-12 )  NZERO = NZERO + 1
  100 CONTINUE
C
      IF( NZERO.EQ.0 )  THEN
cxx         DO 110  I=1,M
         DO 111  I=1,M
         XS(I)     = XFS(I,II)
         XSS(I,II) = XFS(I,II)
         DO 110  J=1,M
         VS(I,J)     = VFS(I,J,II)
cxx  110    VSS(I,J,II) = VFS(I,J,II)
         VSS(I,J,II) = VFS(I,J,II)
  110    CONTINUE
  111    CONTINUE
C
      ELSE
cxx      DO 410  I=1,M
      DO 411  I=1,M
      DO 410  J=1,M
cxx  410 VP(I,J) = VPS(I,J,II+1)
      VP(I,J) = VPS(I,J,II+1)
  410 CONTINUE
  411 CONTINUE
C
cc      CALL  GINVRS( VP,VDET,M,40 )
      CALL  GINVRS( VP,VDET,M )
C
cxx      DO 425  I=1,M
      DO 426  I=1,M
      DO 425  J=1,M
      SUM = 0.0D0
      DO 420  IJ=1,M
cxx  420 SUM = SUM + VFS(I,IJ,II)*F(J,IJ)
      SUM = SUM + VFS(I,IJ,II)*F(J,IJ)
  420 CONTINUE
cxx  425 WRK(I,J) = SUM
      WRK(I,J) = SUM
  425 CONTINUE
  426 CONTINUE
C
cxx      DO 440  I=1,M
      DO 441  I=1,M
      DO 440  J=1,M
      SUM = 0.0D0
      DO 430 IJ=1,M
cxx  430 SUM = SUM + WRK(I,IJ)*VP(IJ,J)
      SUM = SUM + WRK(I,IJ)*VP(IJ,J)
  430 CONTINUE
cxx  440 SGAIN(I,J) = SUM
      SGAIN(I,J) = SUM
  440 CONTINUE
  441 CONTINUE
C
cxx      DO 450  I=1,M
      DO 451  I=1,M
      XS(I) = XFS(I,II)
      DO 450  J=1,M
      WRK(I,J) = 0.0D0
cxx  450 VS (I,J) = VFS(I,J,II)
      VS (I,J) = VFS(I,J,II)
  450 CONTINUE
  451 CONTINUE
C
cxx      DO 460  J=1,M
      DO 461  J=1,M
      DO 460  I=1,M
cxx  460 XS(I) = XS(I) + SGAIN(I,J)*(XSS(J,II+1) - XPS(J,II+1))
      XS(I) = XS(I) + SGAIN(I,J)*(XSS(J,II+1) - XPS(J,II+1))
  460 CONTINUE
  461 CONTINUE
C
cxx      DO 470  J=1,M
cxx      DO 470 IJ=1,M
      DO 472  J=1,M
      DO 471 IJ=1,M
      DO 470  I=1,M
cxx  470 WRK(I,J)=WRK(I,J) + SGAIN(I,IJ)*(VSS(IJ,J,II+1)-VPS(IJ,J,II+1))
      WRK(I,J)=WRK(I,J) + SGAIN(I,IJ)*(VSS(IJ,J,II+1)-VPS(IJ,J,II+1))
  470 CONTINUE
  471 CONTINUE
  472 CONTINUE
C
cxx      DO 480  J=1,M
cxx      DO 480 IJ=1,M
      DO 482  J=1,M
      DO 481 IJ=1,M
      DO 480  I=1,M
cxx  480 VS(I,J) = VS(I,J) + WRK(I,IJ)*SGAIN(J,IJ)
      VS(I,J) = VS(I,J) + WRK(I,IJ)*SGAIN(J,IJ)
  480 CONTINUE
  481 CONTINUE
  482 CONTINUE
      DO 485 I=1,M
cxx  485 IF( VS(I,I).LT.0.0D0 )  VS(I,I) = 0.0D0
      IF( VS(I,I).LT.0.0D0 )  VS(I,I) = 0.0D0
  485 CONTINUE
C
cxx      DO 490  I=1,M
      DO 491  I=1,M
      XSS(I,II) = XS(I)
      DO 490  J=1,M
cxx  490 VSS(I,J,II) = VS(I,J)
      VSS(I,J,II) = VS(I,J)
  490 CONTINUE
  491 CONTINUE
      END IF
C
  500 CONTINUE
C
      RETURN
      E N D
cc      SUBROUTINE  GINVRS( A,DET,M,MJ )
      SUBROUTINE  GINVRS( A,DET,M )
C
C  ...  Generalized inverse of a square matrix A  ...
C
C     Inputs:
C        A:     M*M matrix
C        M:     Dimension of A
C        MJ:    Adjustable dimension of A
C     Outputs:
C        A:     Generalize inverse of A
C        DET:   Determinant of A
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  A(MJ,MJ), IND(50)
cxx      DIMENSION  A(M,M), IND(M+1)
      INTEGER M, IND(M+1)
      DOUBLE PRECISION A(M,M), DET
c local
      INTEGER I, I0, IMAX, J, L, LMAX, MS
      DOUBLE PRECISION EPS, AMAX, SUM
C
      EPS = 1.0D-10
C
      DO 10  I=1,M
cxx   10 IND(I) = I
       IND(I) = I
   10 CONTINUE
cc------------
      I0 = 0
      LMAX = 0
cc------------
      DO 60  L=1,M
      AMAX = 0.0D0
      DO 20  I=L,M
      IF( A(IND(I),IND(I)).GT.AMAX )  THEN
        AMAX = A(IND(I),IND(I))
        I0 = I
      END IF
   20 CONTINUE
      IF( AMAX.GT.EPS*A(IND(1),IND(1)) )  THEN
         IMAX = IND(I0)
         DO 30  I=I0,L+1,-1
cxx   30    IND(I) = IND(I-1)
         IND(I) = IND(I-1)
   30    CONTINUE
         IND(L) = IMAX
         LMAX   = L
cxx         DO 40  I=L+1,M
         DO 41  I=L+1,M
         A(IND(I),IMAX) = -A(IND(I),IMAX)/A(IMAX,IMAX)
         DO 40  J=L+1,M
cxx   40    A(IND(I),IND(J)) = A(IND(I),IND(J))
         A(IND(I),IND(J)) = A(IND(I),IND(J))
     *                    + A(IND(I),IMAX)*A(IMAX,IND(J))
   40    CONTINUE
   41    CONTINUE
      ELSE
cxx         DO 50  I=L,M
         DO 51  I=L,M
         DO 50  J=L,M
cxx   50    A(IND(I),IND(J)) = 0.0D0
         A(IND(I),IND(J)) = 0.0D0
   50    CONTINUE
   51    CONTINUE
         GO TO 70
      END IF
   60 CONTINUE
C
   70 DET = 1.0D0
      DO 80  I=1,M
C     DET = DET*A(IND(I),IND(I))
C     IF( A(IND(I),IND(I)).GT.EPS*AMAX )  THEN
      IF( A(IND(I),IND(I)).GT.0.0D0 )  THEN
        A(IND(I),IND(I)) = 1.0D0/A(IND(I),IND(I))
      ELSE
        A(IND(I),IND(I)) = 0.0D0
      END IF
   80 CONTINUE
C
      MS = MIN0( M-1,LMAX )
      DO 200  L=MS,1,-1
      DO 100 J=L+1,M
      SUM = 0.0D0
      DO 90  I=L+1,M
cxx   90 SUM = SUM + A(IND(I),IND(L))*A(IND(I),IND(J))
      SUM = SUM + A(IND(I),IND(L))*A(IND(I),IND(J))
   90 CONTINUE
cxx  100 A(IND(L),IND(J)) = SUM
      A(IND(L),IND(J)) = SUM
  100 CONTINUE
      SUM = A(IND(L),IND(L))
      DO 110  I=L+1,M
cxx  110 SUM = SUM + A(IND(L),IND(I))*A(IND(I),IND(L))
      SUM = SUM + A(IND(L),IND(I))*A(IND(I),IND(L))
  110 CONTINUE
      A(IND(L),IND(L)) = SUM
      DO 120  I=L+1,M
cxx  120 A(IND(I),IND(L)) = A(IND(L),IND(I))
      A(IND(I),IND(L)) = A(IND(L),IND(I))
  120 CONTINUE
      IMAX = IND(L)
      DO 130 I=L+1,M
      IF( IMAX.GT.IND(I) )  THEN
         IND(I-1) = IND(I)
         IND(I)   = IMAX
      END IF
  130 CONTINUE
  200 CONTINUE
C
      RETURN
      E N D

cc      SUBROUTINE  CRSCOR( Y,N,ID,LAG,MJ,MJ1,OUTMIN,OUTMAX,C,R,YMEAN )
       SUBROUTINE  CRSCOR( Y,N,ID,LAG,OUTMIN,OUTMAX,C,R,YMEAN )
C
C ... cross correlation function computation ...
C
C     Inputs:
C        Y(I,J):   multi-variate time series
C        N:        data length
C        ID:       dimension of the observation
C        LAG:      maximum lag of cross-covariance
C        MJ:       adjustable dimension (MJ.GE.N)
C        MJ1:      adjustable dimension (MJ1.GE.ID)
C        OUTMIN:   bound for outliers in low side
C        OUTMAX:   bound for outliers in high side
C     Outputs:
C        C:        sample cross-covariance function
C        R:        sample cross-correlation function
C        YMEAN:    sample mean vector
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(MJ,MJ1), OUTMIN(MJ1), OUTMAX(MJ1)
cc      DIMENSION  C(0:LAG,MJ1,MJ1), R(0:LAG,MJ1,MJ1)
cc      DIMENSION  YMEAN(MJ1), NSUM(10)
cxx      DIMENSION  Y(N,ID), OUTMIN(ID), OUTMAX(ID)
cxx      DIMENSION  C(0:LAG,ID,ID), R(0:LAG,ID,ID)
cxx      DIMENSION  YMEAN(ID), NSUM(ID)
      INTEGER N, ID, LAG, NSUM(ID)
      DOUBLE PRECISION Y(N,ID), OUTMIN(ID), OUTMAX(ID), C(0:LAG,ID,ID),
     1                 R(0:LAG,ID,ID), YMEAN(ID), SUM
c local
      INTEGER I, II, J, L, NNN
C
      DO 10 J=1,ID
cxx   10 CALL  MEAN( Y(1,J),N,OUTMIN(J),OUTMAX(J),NSUM(J),YMEAN(J) )
      CALL  MEAN( Y(1,J),N,OUTMIN(J),OUTMAX(J),NSUM(J),YMEAN(J) )
   10 CONTINUE
C
cxx      DO 30 I=1,ID
cxx      DO 30 J=1,ID
      DO 32 I=1,ID
      DO 31 J=1,ID
cc      WRITE(6,*)  I,J
      DO 30 L=0,LAG
      SUM = 0.0D0
      NNN = 0
      DO 20 II=L+1,N
      IF( Y(II,I).GT.OUTMIN(I).AND.Y(II,I).LT.OUTMAX(I) )  THEN
        IF( Y(II-L,J).GT.OUTMIN(J).AND.Y(II-L,J).LT.OUTMAX(J) )  THEN
          SUM = SUM + (Y(II,I)-YMEAN(I))*(Y(II-L,J)-YMEAN(J))
          NNN = NNN + 1
        END IF
      END IF
   20 CONTINUE
cxx   30 C(L,I,J)=SUM/DSQRT( DBLE( NSUM(I)*NSUM(J) ) )
      C(L,I,J)=SUM/DSQRT( DBLE( NSUM(I)*NSUM(J) ) )
   30 CONTINUE
   31 CONTINUE
   32 CONTINUE
C
cxx      DO 40 I=1,ID
cxx      DO 40 J=1,ID
      DO 42 I=1,ID
      DO 41 J=1,ID
      DO 40 L=0,LAG
cxx   40 R(L,I,J) = C(L,I,J)/DSQRT(C(0,I,I)*C(0,J,J))
      R(L,I,J) = C(L,I,J)/DSQRT(C(0,I,I)*C(0,J,J))
   40 CONTINUE
   41 CONTINUE
   42 CONTINUE
      RETURN
C
      E N D

      SUBROUTINE  MEAN( Y,N,OUTMIN,OUTMAX,NSUM,YMEAN )
C
C  ...  This subroutine computes sample mean  ...
C
C     Inputs:
C        Y(I):    time series
C        N:       data length
C        OUTMIN:  bound for outliers in low side
C        OUTMAX:  bound for outliers in high side
C     Outputs:
C        NSUM:    number of non-outlier observations
C        YMEAN:   sample mean
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION Y(N)
      INTEGER N, NSUM
      DOUBLE PRECISION Y(N), OUTMIN, OUTMAX, YMEAN, YSUM
c local
      INTEGER I
C
      NSUM = 0
      YSUM = 0.0D0
      DO 10 I=1,N
      IF( Y(I) .GT.OUTMIN .AND. Y(I).LT.OUTMAX ) THEN
         NSUM = NSUM + 1
         YSUM = YSUM + Y(I)
      END IF
   10 CONTINUE
      YMEAN = YSUM/NSUM
C
      RETURN
      E N D
      SUBROUTINE AUTCOR( C,MAXLAG,R )
C
C  ...  This subroutine compute sample autocorrelation  ...
C
C     Inputs:
C        C(I):    sample autocovariance
C        MAXLAG:  maximum lag of autocovariance
C     Output:
C        R(I):    sample autocorrelation
C
cxx      IMPLICIT REAL*8 (A-H,O-Z)
cxx      DIMENSION  C(0:MAXLAG), R(0:MAXLAG)
      INTEGER MAXLAG
      DOUBLE PRECISION C(0:MAXLAG), R(0:MAXLAG)
c local
      INTEGER LAG
C
      DO 10 LAG=0,MAXLAG
cxx   10 R(LAG) = C(LAG)/C(0)
      R(LAG) = C(LAG)/C(0)
   10 CONTINUE
      RETURN
      E N D
cc      SUBROUTINE AUTCOV( Y,N,MAXLAG,OUTMIN,OUTMAX,C )
      SUBROUTINE AUTCOV( Y,N,MAXLAG,OUTMIN,OUTMAX,C,YMEAN )
C
C  ...  This subroutine computes sample autocovariance  ...
C
C     Inputs:
C        Y(I):    time series
C        N:       data length
C        MAXLAG:  maximum lag of autocovariance
C        OUTMIN:  bound for outliers in low side
C        OUTMAX:  bound for outliers in high side
C     OUTPUT:
C        C(I):    autocovariance function
C
cxx      IMPLICIT REAL*8( A-H,O-Z )
cxx      DIMENSION Y(N), C(0:MAXLAG )
      INTEGER N, MAXLAG
      DOUBLE PRECISION Y(N), OUTMIN, OUTMAX, C(0:MAXLAG ), YMEAN, SUM
c local
      INTEGER I, LAG, NSUM
C
C  ...  sample mean  ...
C
      CALL  MEAN( Y,N,OUTMIN,OUTMAX,NSUM,YMEAN )
C
C  ...  sample autocovariance  ...
C
      DO 20 LAG = 0,MAXLAG
      SUM = 0.0D0
      DO 10 I=LAG+1,N
      IF( Y(I).GT.OUTMIN .AND. Y(I).LT.OUTMAX )   THEN
         IF( Y(I-LAG).GT.OUTMIN .AND. Y(I-LAG).LT.OUTMAX ) THEN
         SUM = SUM + ( Y(I)-YMEAN)*( Y(I-LAG) - YMEAN )
         END IF
      END IF
   10 CONTINUE
cxx   20 C(LAG) = SUM/NSUM
      C(LAG) = SUM/NSUM
   20 CONTINUE
C
      RETURN
      E N D
cc      SUBROUTINE  UNICOR( Y,N,MAXLAG,OUTMIN,OUTMAX,COV )
      SUBROUTINE  UNICOR( Y,N,MAXLAG,OUTMIN,OUTMAX,COV,YMEAN )
C
C  ...  This subroutine computes sample autocovariance function,
C       sample autocorrelationfunction and their error bounds  ...
C
C     Inputs:
C        Y(I):    time series
C        N:       data length
C        MAXLAG:  maximum lag of autocovariance
C        OUTMIN:  bound for outliers in low side
C        OUTMAX:  bound for outliers in high side
C     Output:
C        COV(I,J):  I=0,...,MAXLAG
C                   J = 1   sample autocovariance
C                     = 2   sample autocorrelation
C                     = 3   error bound for sample autocovariance
C                     = 4   error bound for sample autocorrelation
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  Y(N), COV(0:MAXLAG,4)
      INTEGER N, MAXLAG
      DOUBLE PRECISION Y(N), OUTMIN, OUTMAX, COV(0:MAXLAG,4), YMEAN
C
C  ...  sample autocovariance function  ...
C
cc      CALL  AUTCOV( Y,N,MAXLAG,OUTMIN,OUTMAX,COV )
      CALL  AUTCOV( Y,N,MAXLAG,OUTMIN,OUTMAX,COV,YMEAN )
C
C  ...  sample autocorrelation function  ...
C
      CALL  AUTCOR( COV,MAXLAG,COV(0,2) )
C
C  ...  error bounds  ...
C
      CALL  ERRACF( COV,N,MAXLAG,COV(0,3),COV(0,4) )
C
      RETURN
      E N D
      SUBROUTINE ERRACF( C,N,MAXLAG,CERR,RERR )
C
C  ...  This subroutine computes error bounds for sample autocovariance
C       and autocorrelation function  ...
C
C     Inputs:
C        C(I):     sample autocovariance
C        N:        data length
C        MAXLAG:   maximum lag of autocovariance
C     Outputs:
C        CERR(I):  error bound for I-th autocovariance
C        RERR(I):  error bound for I-th autocorrelation
C
cxx      IMPLICIT REAL*8 (A-H,O-Z)
cxx      DIMENSION  C(0:MAXLAG), CERR(0:MAXLAG), RERR(0:MAXLAG)
      INTEGER N, MAXLAG
      DOUBLE PRECISION C(0:MAXLAG), CERR(0:MAXLAG), RERR(0:MAXLAG), SUM
c local
      INTEGER LAG
C
      SUM = C(0)**2
      CERR(0)   = DSQRT( 2*SUM/N )
      DO 10 LAG=1,MAXLAG
      IF(LAG.GE.2)  SUM = SUM + 2*C(LAG-1)**2
cxx   10 CERR(LAG) = DSQRT( SUM/N )
      CERR(LAG) = DSQRT( SUM/N )
   10 CONTINUE
C
      RERR(0) = 0.0D0
      DO 20 LAG=1,MAXLAG
cxx   20 RERR(LAG) = CERR(LAG)/C(0)
      RERR(LAG) = CERR(LAG)/C(0)
   20 CONTINUE
      RETURN
      E N D
C
C
cc      SUBROUTINE  WINDOW( PE,NP,IWINDW,SPE )
      SUBROUTINE  WINDOW( PE,NP,IWINDW,SPE,IFG )
C
C  ...  Smoothing by spectral window and log-transformation  ...
C
C     Inputs:
C        PE(I):    raw specrum
C        NP:       number of frequencies
C        IWINDW:   window type (0: box-car, 1: Hanning, 2: Hamming)
C     Outputs:
C        SPE(I):   logarithm of smoothed periodogram
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  PE(0:NP), SPE(0:NP)
cxx      DIMENSION  W(0:1,2)
      INTEGER NP, IWINDW, IFG
      DOUBLE PRECISIONPE(0:NP), SPE(0:NP), W(0:1,2), PMIN
c local
      INTEGER I
      DATA  W/0.5D0, 0.25D0, 0.54D0, 0.23D0/
C
      IF( IWINDW.EQ.0 )  THEN
        PMIN = 1.0D30
        DO 10 I=0,NP
cxx   10   IF( PE(I).GT.0.0D0 .AND. PE(I).LT.PMIN )  PMIN = PE(I)
        IF( PE(I).GT.0.0D0 .AND. PE(I).LT.PMIN )  PMIN = PE(I)
   10   CONTINUE
        DO 20 I=0,NP
cxx   20   SPE(I) = DMAX1( PE(I),PMIN )
        SPE(I) = DMAX1( PE(I),PMIN )
   20   CONTINUE
      ELSE
        SPE(0) = W(0,IWINDW)*PE(0) + W(1,IWINDW)*PE(1)*2
        SPE(NP)= W(0,IWINDW)*PE(NP)+ W(1,IWINDW)*PE(NP-1)*2
        DO 30  I=1,NP-1
cxx   30   SPE(I) = W(0,IWINDW)*PE(I) + W(1,IWINDW)*(PE(I-1) + PE(I+1))
        SPE(I) = W(0,IWINDW)*PE(I) + W(1,IWINDW)*(PE(I-1) + PE(I+1))
   30   CONTINUE
      END IF
c---------- 2013/07/03
      IF ( MINVAL(SPE) .GT. 0 ) THEN
      IFG = 0
      DO 50 I=0,NP
c   50 SPE(I) = DLOG10( SPE(I) )
       SPE(I) = DLOG10( SPE(I) )
   50 CONTINUE
      ELSE
         IFG = -1
      END IF
C
      RETURN
      E N D
C
C
      SUBROUTINE  SETSEA( M1,M2,M3,IPER,AR,TAU1,TAU2,TAU3,SIG2,
cc     *                    F,G,H,Q,R,M,K,MJ )
     *                    F,G,H,Q,R,M,K )
C
C  ...  SET STATE SPACE MODEL FOR SEASONAL ADJUSTMENT  ...
C
C     INPUTS:
C        M1,M2,M3:   MODEL ORDERS
C        IPER:       NUMBER OF SEASONS IN ONE PERIOD
C        AR(I):      AR COEFFICIENTS
C        TAU1-TAU3:  SYSTEM NOISE VARIANCES
C        MJ:         ADJUSTABLE DIMENSION
C     OUTPUTS:
C        F,G,H:      MATRICES OF STATE SPACE MODEL
C        Q,R:        SYSTEM NOISE AND OBS NOISE VARIANCES
C        M:          STATE DIMENSION
C        K:          DIMENSION OF SYSTEM NOISE
C     MODIFIED  2/16/93
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  F(MJ,MJ), G(MJ,MJ), H(MJ), Q(MJ,MJ), AR(*), R(1,1)
cxx      DIMENSION  F(M,M), G(M,K), H(M), Q(K,K), AR(M3), R(1,1)
      INTEGER M1, M2, M3, IPER, M, K 
      DOUBLE PRECISION AR(M3), TAU1, TAU2, TAU3, SIG2, F(M,M), G(M,K),
     1                 H(M), Q(K,K), R(1,1)
c local
      INTEGER I, L1, L12, M12, ID
C
cc      M = M1 + M2*(IPER-1) + M3
cc      K = ID(M1) + ID(M2) + ID(M3)
cxx      DO 10 I=1,M
cxx      H(I) = 0.0D0
cxx      DO 10 J=1,M
cxx   10 F(I,J) = 0.0D0
cxx      DO 20 I=1,M
cxx      DO 20 J=1,K
cxx   20 G(I,J) = 0.0D0
cxx      DO 30 I=1,K
cc   30 Q(I,I) = 0.0D0
cxx      DO 30 J=1,K
cxx   30 Q(I,J) = 0.0D0
      H(1:M) = 0.0D0
      F(1:M,1:M) = 0.0D0
      G(1:M,1:K) = 0.0D0
      Q(1:K,1:K) = 0.0D0
C
      IF( M1.GT.0 )  THEN
        IF( M1.EQ.1 )  F(1,1) = 1.0D0
        IF( M1.EQ.2 )  THEN
          F(1,1) = 2.0D0
          F(1,2) =-1.0D0
          F(2,1) = 1.0D0
        END IF
        G(1,1) = 1.0D0
        H(1)   = 1.0D0
        Q(1,1) = TAU1
      END IF
C
C  ...  SEASONAL COMPONENT  ...
C
      IF( M2.GT.0 )  THEN
        L1 = ID(M1) + 1
        DO 40 I=1,IPER-1
cxx   40   F(M1+1,M1+I) = -1.0D0
        F(M1+1,M1+I) = -1.0D0
   40   CONTINUE
        DO 50 I=2,IPER-1
cxx   50   F(M1+I,M1+I-1) = 1.0D0
        F(M1+I,M1+I-1) = 1.0D0
   50   CONTINUE
        G(M1+1,L1) = 1.0D0
        H(M1+1)    = 1.0D0
        Q(L1,L1)   = TAU2
      END IF
C
C  ...  AR COMPONENT  ...
C
      IF( M3.GT.0 )  THEN
        M12 = M1 + M2*(IPER-1)
        L12 = ID(M1) + ID(M2) + 1
        DO 60 I=1,M3
cxx   60   F(M12+1,M12+I) = AR(I)
        F(M12+1,M12+I) = AR(I)
   60   CONTINUE
        DO 70 I=2,M3
cxx   70   F(M12+I,M12+I-1) = 1.0D0
        F(M12+I,M12+I-1) = 1.0D0
   70   CONTINUE
        G(M12+1,L12) = 1.0D0
        H(M12+1)     = 1.0D0
        Q(L12,L12)   = TAU3
      END IF
C
      R(1,1) = SIG2
C
      RETURN
      E N D
      SUBROUTINE  CHOLES ( X,MJ,N,Y,NJ )
C
C  ...  CHOLESKY DECOMPOSITION  ...
C
C     INPUTS:
C        X(I,J):   SYMMETRIC MATRIX
C        N:        DIMENSION OF Z
C        MJ:       ADJUSTABLE DIMENSION OF X
C        NJ:       ADJUSTABLE DIMENSION OF Y
C     OUTPUT:
C        Y(I,J):   LOWER TRIANGULAR MATRIX; X = Y*Y'
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  X(MJ,MJ), Y(NJ,NJ)
      INTEGER MJ, N, NJ
      DOUBLE PRECISION X(MJ,MJ), Y(NJ,NJ), SUM1, SUM2
c local
      INTEGER I, J, K
C
cxx      DO 10 I=1,N
cxx      DO 10 J=1,N
cxx   10 Y(I,J) = 0.0D0
      Y(1:N,1:N) = 0.0D0
C
      DO 100 J = 1,N
      SUM1 = X(J,J)
      DO 20 K=1,J-1
cxx   20 SUM1 = SUM1 - Y(J,K)**2
      SUM1 = SUM1 - Y(J,K)**2
   20 CONTINUE
      IF( SUM1.GT.0.0D0 ) Y(J,J) = DSQRT( SUM1 )
      IF( SUM1.EQ.0.0D0 ) Y(J,J) = 0.0D0
      DO 40 I=J+1,N
      SUM2 = 0.0D0
      DO 30 K=1,J-1
cxx   30 SUM2 = SUM2 + Y(I,K)*Y(J,K)
      SUM2 = SUM2 + Y(I,K)*Y(J,K)
   30 CONTINUE
cxx   40 Y(I,J) = ( X(I,J) - SUM2 ) / Y(J,J)
      Y(I,J) = ( X(I,J) - SUM2 ) / Y(J,J)
   40 CONTINUE
  100 CONTINUE
C
      RETURN
      E N D
cc      SUBROUTINE ARMCOV( M,L,A,B,SIG2,K,COV )
      SUBROUTINE ARMCOV( M,L,A,B,SIG2,K,COV,KMAX,IER )
C
C ...  Autocovariance Function of ARMA model  ...
C
C     Inputs:
C        M:     AR order
C        L:     MA order
C        A(I):  AR coefficient
C        B(I):  MA coefficient
C        SIG2:  innovation variance
C        K:     Required maximum lag of autocovariance
C     Output:
C        COV(I):  Autocovariance function
C     Y.I.
cxx      IMPLICIT REAL*8( A-H,O-Z )
cc      DIMENSION  A(*), B(*), COV(0:K), G(0:100), X(30,30)
cc      DIMENSION  Z(100), UL(30,30), IPS(100)
cxx      DIMENSION  A(M), B(L), COV(0:K), G(0:KMAX), X(M+1,M+1)
cxx      DIMENSION  Z(M+1), UL(M+1,M+1), IPS(M+1)
      INTEGER M, L, K, KMAX, IER, IPS(M+1)
      DOUBLE PRECISION A(M), B(L), SIG2, COV(0:K)
c local
      INTEGER I, J
      DOUBLE PRECISION G(0:KMAX), X(M+1,M+1), Z(M+1), UL(M+1,M+1), SUM
C
cc      KMAX = MAX(M,L,K)
      CALL  IMPULS( M,L,A,B,KMAX,G )
C
cxx      DO 10 I=1,M+1
cxx      DO 10 J=1,M+1
cxx   10 X(I,J) = 0.0D0
      X(1:M+1,1:M+1) = 0.0D0
      DO 20 I=1,M+1
cxx   20 X(I,I) = 1.0D0
      X(I,I) = 1.0D0
   20 CONTINUE
c
      IF( M.GT.0 ) THEN

cxx      DO 30 I=1,M
      DO 31 I=1,M
      DO 30 J=2,M-I+2
cxx   30 X(I,J) = X(I,J) - A(I+J-2)
      X(I,J) = X(I,J) - A(I+J-2)
   30 CONTINUE
   31 CONTINUE
cxx      DO 40 I=2,M+1
      DO 41 I=2,M+1
      DO 40 J=1,I-1
cxx   40 X(I,J) = X(I,J) - A(I-J)
      X(I,J) = X(I,J) - A(I-J)
   40 CONTINUE
   41 CONTINUE

      END IF
c
C
cc      CALL  DECOM( M+1,X,30,UL,IPS )
      CALL  DECOM( M+1,X,UL,IPS,IER )
      if( ier.ne.0 ) return
C
      SUM = 1.0D0
c
      IF( L.GT.0 ) THEN

      DO 50 J=1,L
cxx   50 SUM = SUM - B(J)*G(J)
      SUM = SUM - B(J)*G(J)
   50 CONTINUE

      END IF
c
      Z(1)= SIG2*SUM
c
      IF( M.GT.0 ) THEN

      DO 70 I=2,M+1
      SUM = 0.0D0
      DO 60 J=I-1,L
cc   60 SUM = SUM - B(J)*G(J-I+1)
cxx   60 IF( L.GT. 0) SUM = SUM - B(J)*G(J-I+1)
      IF( L.GT. 0) SUM = SUM - B(J)*G(J-I+1)
   60 CONTINUE
cxx   70 Z(I) = SIG2*SUM
      Z(I) = SIG2*SUM
   70 CONTINUE

      END IF
c
C
cc      CALL  SOLVE( M+1,UL,30,Z,COV,IPS)
      CALL  SOLVE( M+1,UL,Z,COV,IPS )
C
      DO 100 J=M+1,K
      SUM = 0.0D0
      DO 80 I=1,M
cc   80 SUM = SUM + A(I)*COV(J-I)
cxx   80 IF( M.GT.0 ) SUM = SUM + A(I)*COV(J-I)
      IF( M.GT.0 ) SUM = SUM + A(I)*COV(J-I)
   80 CONTINUE
      DO 90 I=J,L
cc   90 SUM = SUM - B(I)*G(I-J)*SIG2
cxx   90 IF( L.GT.0 ) SUM = SUM - B(I)*G(I-J)*SIG2
      IF( L.GT.0 ) SUM = SUM - B(I)*G(I-J)*SIG2
   90 CONTINUE
cxx  100 COV(J) = SUM
      COV(J) = SUM
  100 CONTINUE
C
      RETURN
      E N D
C
cc      SUBROUTINE DECOM( N,A,MJ,UL,IPS )
      SUBROUTINE DECOM( N,A,UL,IPS,IER )
C
C  ...  UL decomposition:  A = L*U  ...
C
C     Inputs:
C        N:      Dimension of the matrix A
C        A(I,J): N*N positive definite matrix
C        MJ:     Adjustable dimension of A and UL
C     Outputs:
C        UL(I,J):  L-I and U
C        IPS:    Index vector
C     Y.I.
C        IER:    Error code
C
cxx      IMPLICIT REAL*8(A-H,O-Z )
cc      DIMENSION A(MJ,*),UL(MJ,*),SCALES(100),IPS(100)
cxx      DIMENSION A(N,N),UL(N,N),SCALES(N),IPS(N)
      INTEGER N, IPS(N), IER
      DOUBLE PRECISION A(N,N), UL(N,N), SCALES(N), RNORM, BIG, SIZE,
     1                 PIVOT, TM
c local
      INTEGER I, J, K, INDEX
C
      IER = 0
C
      DO 20 I=1,N
      IPS(I) = I
      RNORM = 0.0D0
      DO 10 J=1,N
      UL(I,J) = A(I,J)
      IF(RNORM.LT.ABS(UL(I,J)) ) RNORM = ABS( UL(I,J) )
   10 CONTINUE
      IF( RNORM .NE. 0.0D0 )  THEN
          SCALES(I) = 1/RNORM
      ELSE
          SCALES(I) = 0.0D0
cc          CALL  SING(0)         
          IER = 1
      END IF
   20 CONTINUE
      if( ier.ne.0 ) return
C
cc-------------
      INDEX = 0
cc-------------
      DO 60 K=1,N-1
      BIG = 0.0D0
      DO 30 I=K,N
      SIZE = ABS( UL(IPS(I),K) )*SCALES( IPS(I) )
      IF( BIG.LT.SIZE ) THEN
          BIG = SIZE
          INDEX = I
      END IF
   30 CONTINUE
      IF( BIG.EQ. 0.0D0 )  THEN
cc          CALL  SING(1)
          IER = 2
          GO TO 60
      END IF
      IF( INDEX.NE.K ) THEN
      J = IPS(K)
      IPS(K) = IPS(INDEX)
      IPS(INDEX) = J
      END IF
C
      PIVOT = UL(IPS(K),K)
      DO 50 I=K+1,N
      TM = UL( IPS(I),K)/PIVOT
      UL( IPS(I),K) = TM
      IF( TM.NE. 0.0D0 )  THEN
      DO 40 J = K+1,N
cxx   40 UL( IPS(I),J ) = UL( IPS(I),J)-TM*UL( IPS(K),J)
      UL( IPS(I),J ) = UL( IPS(I),J)-TM*UL( IPS(K),J)
   40 CONTINUE
C     WRITE(6,*) (UL(IPS(I),J),J=1,N)
      END IF
   50 CONTINUE
   60 CONTINUE
      if( ier.ne.0 ) return
C
cc      IF( UL(IPS(N),N) .EQ. 0.0D0 )   CALL  SING(2)
      IF( UL(IPS(N),N) .EQ. 0.0D0 )   IER = 3
      RETURN
      E N D
C
      SUBROUTINE IMPULS( M,L,A,B,K,G )
C
C ...  Impulse Response Function  ...
C
C     Inputs:
C        M:     AR order
C        L:     MA order
C        A(I):  AR coefficient
C        B(I):  MA coefficient
C        K:     Required maximum lag of impulse respose
C     Output:
C        G(I):  Impulse response function
C     Y.I.
cxx      IMPLICIT REAL*8( A-H,O-Z )
cc      DIMENSION A(*), B(*), G(0:K)
cxx      DIMENSION A(M), B(L), G(0:K)
      INTEGER M, L, K
      DOUBLE PRECISION A(M), B(L), G(0:K), SUM
c local
      INTEGER I, J
C
      G(0) = 1.0
      DO  20 I=1,K
      SUM = 0.0D0
      IF(I.LE.L) SUM = -B(I)
      DO  10 J=1,I
cxx   10 IF(J.LE.M) SUM = SUM + A(J)*G(I-J)
      IF(J.LE.M) SUM = SUM + A(J)*G(I-J)
   10 CONTINUE
cxx   20 G(I) = SUM
      G(I) = SUM
   20 CONTINUE
C
      RETURN
      E N D
C
cc      SUBROUTINE  SOLVE( N,UL,MJ,B,X,IPS )
      SUBROUTINE  SOLVE( N,UL,B,X,IPS )
C
C  ...  Solve Ax=b using UL obtained by DECOM  ...
C
C     Inputs:
C        N:     Dimension of UL and B
C        UL:    LU decomposition of A
C        MJ:    Adjustable dimension of A
C        B:
C        IPS:   index vector
C     Output:
C        X:     Solution
C     Y.I.
cxx      IMPLICIT REAL*8( A-H,O-Z )
cc      DIMENSION UL(MJ,*),B(*),X(*),IPS(100)
cxx      DIMENSION UL(N,N),B(N),X(N),IPS(N)
      INTEGER N, IPS(N)
      DOUBLE PRECISION UL(N,N), B(N),X(N), SUM
c local
      INTEGER I, J
C
      DO 20 I=1,N
      SUM = 0.0D0
      DO 10 J=1,I-1
cxx   10 SUM = SUM + UL(IPS(I),J)*X(J)
      SUM = SUM + UL(IPS(I),J)*X(J)
   10 CONTINUE
cxx   20 X(I) = B(IPS(I)) - SUM
      X(I) = B(IPS(I)) - SUM
   20 CONTINUE
C
      DO 40 I=N,1,-1
      SUM = 0.0D0
      DO 30 J=I+1,N
cxx   30 SUM = SUM + UL(IPS(I),J)*X(J)
      SUM = SUM + UL(IPS(I),J)*X(J)
   30 CONTINUE
cxx   40 X(I) = ( X(I)-SUM )/UL(IPS(I),I)
      X(I) = ( X(I)-SUM )/UL(IPS(I),I)
   40 CONTINUE
      RETURN
cxx  600 FORMAT(1H ,'N=',I10,/,(5X,'IPS=',I10 ) )
      E N D
C
C
cc      SUBROUTINE  SMOTH1( A,M,MMAX,NC,NS,N,NE,MJ,
      SUBROUTINE  SMOTH1( A,M,MMAX,NC,NS,N,NE,NMAX,MJ,
     *                    VFS,VPS,VSS,XFS,XPS,XSS )
C
C  ...  Fixed Interval Smoother (General Form)  ...
C
C     Inputs:
C        A:      Parameter of matrix F
C        M:      Dimension of the state vector
C        MMAX:   Adjustable dimension of A
C        NC:     Number of components
C        N:      Data length
C        NS:     Start position of filtering
C        NE:     End position of filtering
C        MJ:     Adjustable dimension of XF, VF
C        VFS:    Covariance matrices of the filter
C        VPS:    Covariance matrices of the predictor
C        XFS:    Mean vectors of the filter
C        XPS:    Mean vectors of the predictor
C     Outputs:
C        VSS:    Covariance matrices of the smoother
C        XSS:    Mean vectors of the smoother
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  A(MMAX,NC), M(NC)
cc      DIMENSION  XS(40), VS(40,40), VP(40,40)
cc      DIMENSION  XFS(MJ,N), XPS(MJ,N), XSS(MJ,N)
cc      DIMENSION  VFS(MJ,MJ,N), VPS(MJ,MJ,N), VSS(MJ,MJ,N)
cc      DIMENSION  WRK(40,40), SGAIN(40,40)
cc      DIMENSION  I0(10)
cxx      DIMENSION  XS(MJ), VS(MJ,MJ), VP(MJ,MJ)
cxx      DIMENSION  XFS(MJ,NMAX), XPS(MJ,NMAX), XSS(MJ,NMAX)
cxx      DIMENSION  VFS(MJ,MJ,NMAX), VPS(MJ,MJ,NMAX), VSS(MJ,MJ,NMAX)
cxx      DIMENSION  WRK(MJ,MJ), SGAIN(MJ,MJ)
cxx      DIMENSION  I0(NC)
      INTEGER MMAX, NC, N S, N, NE, NMAX, MJ, M(NC), I0(NC)
      DOUBLE PRECISION A(MMAX,NC), VFS(MJ,MJ,NMAX), VPS(MJ,MJ,NMAX),
     1                 VSS(MJ,MJ,NMAX), XFS(MJ,NMAX), XPS(MJ,NMAX),
     2                 XSS(MJ,NMAX)
c local
      INTEGER I, II, IJ, J, L, MM, NSS, NZERO
      DOUBLE PRECISION XS(MJ), VS(MJ,MJ), VP(MJ,MJ), WRK(MJ,MJ),
     1                 SGAIN(MJ,MJ), SUM, VDET
C
      I0(1) = 0
      DO 10 I=2,NC
cxx   10 I0(I) = I0(I-1) + M(I-1)
      I0(I) = I0(I-1) + M(I-1)
   10 CONTINUE
      MM = I0(NC) + M(NC)
C
C  ...  SMOOTHING  ...
C
      NSS = MIN0( N,NE )
cxx      DO 20  I=1,MM
      DO 21  I=1,MM
      XS(I)      = XFS(I,NSS)
      XSS(I,NSS) = XFS(I,NSS)
      DO 20  J=1,MM
      VS(I,J)      = VFS(I,J,NSS)
cxx   20 VSS(I,J,NSS) = VFS(I,J,NSS)
      VSS(I,J,NSS) = VFS(I,J,NSS)
   20 CONTINUE
   21 CONTINUE
cxx      DO 30 II=NSS+1,NE
cxx      DO 30 I=1,MM
      DO 32 II=NSS+1,NE
      DO 31 I=1,MM
      XSS(I,II) = XFS(I,II)
      DO 30 J=1,MM
cxx   30 VSS(I,J,II) = VFS(I,J,II)
      VSS(I,J,II) = VFS(I,J,II)
   30 CONTINUE
   31 CONTINUE
   32 CONTINUE
C
      DO 500  II=NSS-1,NS,-1
C
      NZERO = 0
      DO 100 I=1,MM
cxx  100 IF( VFS(I,I,II).GT.1.0D-12 )  NZERO = NZERO + 1
      IF( VFS(I,I,II).GT.1.0D-12 )  NZERO = NZERO + 1
  100 CONTINUE
C
      IF( NZERO.EQ.0 )  THEN
cxx         DO 110  I=1,MM
         DO 111  I=1,MM
         XS(I)     = XFS(I,II)
         XSS(I,II) = XFS(I,II)
         DO 110  J=1,MM
         VS(I,J)     = VFS(I,J,II)
cxx  110    VSS(I,J,II) = VFS(I,J,II)
         VSS(I,J,II) = VFS(I,J,II)
  110 CONTINUE
  111 CONTINUE
C
      ELSE
cxx      DO 410  I=1,MM
      DO 411  I=1,MM
      DO 410  J=1,MM
cxx  410 VP(I,J) = VPS(I,J,II+1)
      VP(I,J) = VPS(I,J,II+1)
  410 CONTINUE
  411 CONTINUE
C
cc      CALL  GINVRS( VP,VDET,MM,40 )
      CALL  GINVRS( VP,VDET,MM )
C
cxx      DO 420  I=1,MM
cxx      DO 420  L=1,NC
      DO 422  I=1,MM
      DO 421  L=1,NC
      WRK(I,I0(L)+M(L)) = VFS(I,I0(L)+1,II)*A(M(L),L)
      DO 420  J=1,M(L)-1
cxx  420 WRK(I,I0(L)+J) = VFS(I,I0(L)+1,II)*A(J,L) + VFS(I,I0(L)+J+1,II)
      WRK(I,I0(L)+J) = VFS(I,I0(L)+1,II)*A(J,L) + VFS(I,I0(L)+J+1,II)
  420 CONTINUE
  421 CONTINUE
  422 CONTINUE
C
cxx      DO 440  I=1,MM
      DO 441  I=1,MM
      DO 440  J=1,MM
      SUM = 0.0D0
      DO 430 IJ=1,MM
cxx  430 SUM = SUM + WRK(I,IJ)*VP(IJ,J)
      SUM = SUM + WRK(I,IJ)*VP(IJ,J)
  430 CONTINUE
cxx  440 SGAIN(I,J) = SUM
      SGAIN(I,J) = SUM
  440 CONTINUE
  441 CONTINUE
C
cxx      DO 450  I=1,MM
      DO 451  I=1,MM
      XS(I) = XFS(I,II)
      DO 450  J=1,MM
      WRK(I,J) = 0.0D0
cxx  450 VS (I,J) = VFS(I,J,II)
      VS (I,J) = VFS(I,J,II)
  450 CONTINUE
  451 CONTINUE
C
cxx      DO 460  J=1,MM
      DO 461  J=1,MM
      DO 460  I=1,MM
cxx  460 XS(I) = XS(I) + SGAIN(I,J)*(XSS(J,II+1) - XPS(J,II+1))
      XS(I) = XS(I) + SGAIN(I,J)*(XSS(J,II+1) - XPS(J,II+1))
  460 CONTINUE
  461 CONTINUE
C
cxx      DO 470  J=1,MM
cxx      DO 470 IJ=1,MM
      DO 472  J=1,MM
      DO 471 IJ=1,MM
      DO 470  I=1,MM
cxx  470 WRK(I,J) = WRK(I,J) + SGAIN(I,IJ)*(VSS(IJ,J,II+1)-VPS(IJ,J,II+1))
      WRK(I,J) = WRK(I,J) + SGAIN(I,IJ)*(VSS(IJ,J,II+1)-VPS(IJ,J,II+1))
  470 CONTINUE
  471 CONTINUE
  472 CONTINUE
C
cxx      DO 480  J=1,MM
cxx      DO 480 IJ=1,MM
      DO 482  J=1,MM
      DO 481 IJ=1,MM
      DO 480  I=1,MM
cxx  480 VS(I,J) = VS(I,J) + WRK(I,IJ)*SGAIN(J,IJ)
      VS(I,J) = VS(I,J) + WRK(I,IJ)*SGAIN(J,IJ)
  480 CONTINUE
  481 CONTINUE
  482 CONTINUE
      DO 485 I=1,MM
cxx  485 IF( VS(I,I).LT.0.0D0 )  VS(I,I) = 0.0D0
      IF( VS(I,I).LT.0.0D0 )  VS(I,I) = 0.0D0
  485 CONTINUE
C
cxx      DO 490  I=1,MM
      DO 491  I=1,MM
      XSS(I,II) = XS(I)
      DO 490  J=1,MM
cxx  490 VSS(I,J,II) = VS(I,J)
      VSS(I,J,II) = VS(I,J)
  490 CONTINUE
  491 CONTINUE
      END IF
C
  500 CONTINUE
C
      RETURN
      E N D
C------------------------------------------------------------------
C
      FUNCTION  ID( K )                                                 
C                                                                       
C  ...  ID = 1:    IF K > 0                                             
C       ID = 0:    OTHERWISE                                            
C                                                                       
      INTEGER K, ID
      ID = 0                                                            
      IF( K .GT. 0 )  ID = 1                                            
      RETURN                                                            
      E N D                                                             

      SUBROUTINE INIT(IX)
      INTEGER IX, v(8)
C
      if ( IX .ge. 0 ) then
          call init_genrand64(IX)
      else
         call date_and_time( values=v )
         call init_genrand64( sum(v) )
      endif
      RETURN
      END

      DOUBLE PRECISION FUNCTION  GAUSS( X,PARAM )
C
C  ...  Gaussian (normal) distribution  ...
C
C     Inputs:
C        X:
C        PARAM(1):  mean
C        PARAM(2):  variance
C     Output:
C        GAUSS:     density at X
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  PARAM(2)
      DOUBLE PRECISION X, PARAM(2), C1
      DATA  C1  /2.506628275D0/
C
      GAUSS = DEXP( -(X-PARAM(1))**2/(2*PARAM(2)) )/(C1*DSQRT(PARAM(2)))
      RETURN
      E N D
      DOUBLE PRECISION FUNCTION  PEARSN( X,PARAM )
C
C  ...  Pearson family of  distributions  ...
C
C     Inputs:
C        X:
C        PARAM(1):  location parameter, mu
C        PARAM(2):  dispersion parameter, tau square
C        PARAM(3):  shape parameter
C     Output:
C        PEARSN:    density at X
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  PARAM(3)
      DOUBLE PRECISION X, PARAM(3), PI, dgammafn
      DATA  PI/3.1415926535D0/
C
CXX      PEARSN = DGAMMA(PARAM(3))/DGAMMA(PARAM(3)-0.5D0)
      PEARSN = dgammafn(PARAM(3))/dgammafn(PARAM(3)-0.5D0)
     *                  /DSQRT(PI)*PARAM(2)**(PARAM(3)-0.5D0)
     *                  /((X-PARAM(1))**2 + PARAM(2))**PARAM(3)
      RETURN
C
      END

      DOUBLE PRECISION FUNCTION  DBLEXP( X,PARAM )
C
C  ...  double exponential distribution  f(x) = exp(x - exp(x))  ...
C
C     Inputs:
C        X:
C     Output:
C        DBLEXP:    density at X
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      dimension PARAM(3)
      DOUBLE PRECISION X, PARAM(3)
C
cxx 2018/07/02      DBLEXP = DEXP( X-DEXP(X) )
      DBLEXP = DEXP( (X-PARAM(1))-DEXP(X-PARAM(1)) )
      RETURN
C
      E N D

      DOUBLE PRECISION FUNCTION  CAUCHY( X,PARAM )
C
C  ...  Cauchy distribution  ...
C
C     Inputs:
C        X:
C        PARAM(1):  location parameter, mu
C        PARAM(2):  dispersion parameter, tau square
C     Output:
C        CAUCHY:    density at X
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  PARAM(2)
      DOUBLE PRECISION X, PARAM(2), PI
      DATA  PI /3.1415926535D0/
C
      CAUCHY = DSQRT( PARAM(2) )/(PARAM(2) + (X-PARAM(1))**2)/PI
      RETURN
C
      E N D

# PROGRAM 2.1
crscor <- function (y, lag = NULL, outmin = NULL, outmax = NULL, plot = TRUE, ...)
{
  n <- dim(y)[1]
  id <- dim(y)[2]
  if (is.null(lag))
    lag <- as.integer(2 * sqrt(n))    # maximum lag
  lag1 <- lag + 1

  if (is.null(outmin)) {
    outmin <- rep(-1.0e+30, id)
  } else {
    if (length(outmin) != id)
      stop("specify the values of 'outmin' for each dimension")
  }
  if (is.null(outmax)) {
    outmax <- rep(1.0e+30, id)
  } else {
    if (length(outmax) != id)
      stop ("specify the values of 'outmax' for each dimension")
  }

  y[is.na(y)] <- outmin

  z <- .Fortran(C_crscorf,
                as.double(y),
                as.integer(n),
                as.integer(id),
                as.integer(lag),
                as.double(outmin),
                as.double(outmax),
                cov = double(lag1 * id * id),
                cor = double(lag1 * id * id),
                mean = double(id))

  cov <- array(z$cov, c(lag1, id, id))
  cor <- array(z$cor, c(lag1, id, id))

  crscor.out <- list(cov = cov, cor = cor, mean = z$mean)
  class(crscor.out) <- "crscor"

  if (plot) {
    plot.crscor(crscor.out, colnames(y), ...)
    invisible(crscor.out)
  } else crscor.out
}

plot.crscor <- function(x, vnames, ...) {
  cor <- x$cor
  lag <- dim(cor)[1] - 1
  id <- dim(cor)[2]

  old.par <- par(no.readonly = TRUE)
    new.mar <- old.par$mar
    new.mar[3:4] <- new.mar[3:4] * 0.8
    new.mgp <- old.par$mgp
    new.mgp[1] <- new.mgp[1] * 0.8
    
    
  par(mfcol = c(id, id), xaxs = "i", yaxs = "i", oma = c(0, 1, 1, 0),
      mar = new.mar, mgp = new.mgp, cex.main = 0.9)

  for (j in 1:id) {
    for (i in 1:id) {
      xlab <- "";  ylab <- ""
      if (j == 1)
        ylab <- vnames[i]
      if (i == id)
        xlab <- "Lag"
      plot((0:lag), cor[, i, j], type = "l", ylim = c(-1, 1),
           xlab = xlab, ylab = ylab)
      if (i == 1)
        title(main = vnames[j], line = 1)

      par(new = TRUE)
      abline(h = 0)
    }
  }
  par(old.par)
}

C     PROGRAM  11.1  POLREG
      SUBROUTINE POLREG(Y,N,K,A,SIG2,AIC,IMIN,DATA)
C
C  ...  Polynomial regression model  ...
C
C     INPUT:
C        K:       Order of polynomial regression
C     PARATERS:
C        MJ:      Adjustable dimension of Y (MJ.GE.N)
C        MJ1:     Adjustable dimension of X and D
C        MJ2:     Adjustable dimension of A, SIG2 and AIC
C        IDEV:    Input device
C        LAG:     Number of sine and cosine terms
C     @TEST.PN51:
C
cc      PARAMETER( MJ=1000,MJ1=200,MJ2=21,ISW=2,IDEV=1 )
cxxx      PARAMETER( MJ1=200 )
      INTEGER, PARAMETER :: MJ1=200
cxx      IMPLICIT   REAL*8 (A-H,O-Z)
cc      DIMENSION  Y(MJ), AIC(0:MJ2)
cc      DIMENSION  X(MJ1,MJ2), D(MJ1), A(MJ2,MJ2), SIG2(0:MJ2)
cxx      DIMENSION  Y(N), AIC(0:K)
cxx      DIMENSION  X(MJ1,K+1), A(K,K), SIG2(0:K), DATA(N)
C
      INTEGER N, K, IMIN
      DOUBLE PRECISION Y(N), A(K,K), SIG2(0:K), AIC(0:K), DATA(N)
c local
      INTEGER I,J
      DOUBLE PRECISION X(MJ1,K+1), SUM, XX
C
      EXTERNAL   SETXPL
C
cc      READ( 5,* )  K
cc      CALL  READTS( IDEV,Y,N )
C
cc      CALL  REDUCT( SETXPL,Y,D,N,0,K+1,MJ1,X )
cx      CALL  REDUCT( SETXPL,Y,N, 0,K1,MJ1,X )
      CALL  REDUCT1( SETXPL,Y,N, 0,K,MJ1,X )
C
cc      CALL  REGRES( X,K+1,N,MJ1,MJ2,A,SIG2,AIC,IMIN )
      CALL  REGRES( X,K,N,MJ1,A,SIG2,AIC,IMIN )
C
cc      CALL  PRPOL( N,K+1,MJ2,A,SIG2,AIC )
C      CALL  PTPOL( Y,N,A(1,IMIN),IMIN )
c-----    PTPOL   ----
      DO 20 I=1,N
      SUM = 0.0D0
      XX  = 1.0D0
      DO 10 J=1,IMIN
      SUM = SUM + A(J,IMIN)*XX
cxx   10 XX = XX*I
      XX = XX*I
   10 CONTINUE
cxx   20 DATA(I) = SUM
      DATA(I) = SUM
   20 CONTINUE
c--------------------
cc      STOP
      RETURN
      E N D
cc      SUBROUTINE  REDUCT( SETX,Z,D,NMK,N0,K,MJ1,X )
      SUBROUTINE  REDUCT1( SETX,Z,NMK,N0,K,MJ1,X )
C
C  ...  Successive Householder reduction  ...
C
C     Inputs:
C        SETX:    Name of the subroutine for making X(I,J)
C        Z(I):    Data vector
C        D(I):    Working area
C        NMK:     Number of actually used observations
C        N0:      Time point of the previous set ofobservations
C        K:       Heighest order of the model
C        MJ1:     Adjustable dimension of X
C     Output:
C        X(I,J):  data matrix
C
cxx      IMPLICIT  REAL*8( A-H,O-Z )
cc      DIMENSION  X(MJ1,1) , D(1), Z(1)
cx      DIMENSION  X(MJ1,1) , Z(1)
cxx      DIMENSION  X(MJ1,K+1) , Z(N0+NMK)
      INTEGER NMK, N0, K, MJ1
      DOUBLE PRECISION Z(N0+NMK), X(MJ1,K+1)
c local
      INTEGER K1, L, LK, N1, N2
C
      L = MIN0( NMK,MJ1 )
      K1 = K + 1
      N1 = L
C
      CALL  SETX( Z,N0,L,K,MJ1,0,X )
cc      CALL  HUSHLD( X,D,MJ1,L,K1 )
      CALL  HUSHLD( X,MJ1,L,K1 )
      IF( N1 .GE. NMK )  RETURN
C
   10 L = MIN0( NMK-N1,MJ1-K1 )
C
      LK = L + K1
      N2 = N0 + N1
      CALL  SETX( Z,N2,L,K,MJ1,1,X )
cc      CALL  HUSHLD( X,D,MJ1,LK,K1 )
      CALL  HUSHLD( X,MJ1,LK,K1 )
      N1 = N1 + L
      IF( N1.LT.NMK )  GO TO 10
C
      RETURN
C
      E N D
C
C
      SUBROUTINE  SETXPL( Z,N0,L,K,MJ1,JSW,X )
C
C  ...  Data matrix for polynomial regression  ...
C
C     Inputs:
C        Z(I):    Data vector
C        N0:      Origin of the current observations
C        L:       Number of current observations
C        K:       Number of regressors
C                 (Order of polynomial = K-1)
C        MJ1:     Adjustable dimension of X
C        JSW=0:   Make initial data matrix
C           =1:   Apend L*(K+1) data matrix below the triangular one
C     Output:
C        X(I,J):  Data matrix
C
cc      REAL*8  X(MJ1,1), Z(1)
cxx      REAL*8  X(MJ1,K+1), Z(N0+L)
C
      INTEGER N0, L, K, MJ1, JSW
      DOUBLE PRECISION Z(N0+L), X(MJ1,K+1)
c local
      INTEGER I, II, I0, J
      DOUBLE PRECISION SUM
C
      I0 = 0
      IF( JSW .EQ. 1 )     I0 = K+1
cxx      DO 10  I=1,L
      DO 11  I=1,L
        II = I + I0
        X(II,K+1) = Z(N0+I)
        X(II,1) = 1.0D0
        SUM = 1.0D0
      DO 10  J=1,K-1
        SUM = SUM*(N0+I)
cxx   10 X(II,J+1) = SUM
      X(II,J+1) = SUM
   10 CONTINUE
   11 CONTINUE
C
      RETURN
C
      E N D

C     PROGRAM  3.1  PERIOD
cx      SUBROUTINE PERIODF(Y,N,IWINDW,OUTMIN,OUTMAX,PE,SPE,NP)
ccb      SUBROUTINE PERIODF(Y,N,IWINDW,OUTMIN,OUTMAX,PE,SPE,NP,IFG)
ccxx      SUBROUTINE PERIODF(Y,N,NP,IWINDW,OUTMIN,OUTMAX,PE,SPE,IFG)
      SUBROUTINE PERIODF(Y,N,NP,IWINDW,LAG,OUTMIN,OUTMAX,PE,SPE,IFG)
C
C  ...  This program computes periodogram  ...
C
C     The following inputs are required in READTS.
C        TITLE:   title of the data
C        N:       data length
C        Y(I):    time series
C     Parameters:
C        NMAX:    adjustable dimension of Y (NMAX.GE.N)
C        MJ:      adjustable dimension of COV
C        NF:      adjustable dimension of PE and SPE
C        IDEV:    input device
C        JDEV:    output device
C        IWINDW:  window type (0: box-car, 1: Hanning, 2: Hamming)
C     @TEST.PN31:  5/16/89, 12/5/90, 12/21/90, 8/5/91
C
cc      PARAMETER( NMAX=200,MJ=NMAX,NF=512,IDEV=1,JDEV=6,IWINDW=1 )
ccb      PARAMETER( NF=512 )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  Y(NMAX), PE(0:NF), SPE(0:NF), COV(0:MJ)
ccb      DIMENSION  Y(N), PE(0:NF), SPE(0:NF), COV(0:1024)
cxx      DIMENSION  Y(N), PE(0:NP), SPE(0:NP), COV(0:1024)
C
      INTEGER N, NP, IWINDW, LAG, IFG
      DOUBLE PRECISION Y(N), OUTMIN, OUTMAX, PE(0:NP), SPE(0:NP)
c local
ccxx      DOUBLE PRECISION COV(0:1024)
      DOUBLE PRECISION COV(0:LAG)
C
cc      DATA  OUTMIN/-1.0D30/, OUTMAX/ 1.0D30/
C
cc      CALL  READTS( IDEV,Y,N )
C
cc      IF( IWINDW.EQ.0 )  LAG = MIN0( N-1,MJ )
ccxx      IF( IWINDW.EQ.0 )  LAG = N-1
cxx      IF( IWINDW.GT.0 )  LAG = 2*DSQRT( DBLE(N) )
ccxx      IF( IWINDW.GT.0 )  LAG = INT(2*DSQRT( DBLE(N) ))
ccxx      IF( IWINDW.EQ.0 )  NP  = (LAG+1)/2
ccxx      IF( IWINDW.GT.0 )  NP  = LAG
C
      CALL  PERIOD( Y,N,LAG,OUTMIN,OUTMAX,NP,0,COV,PE )
C
cx      CALL  WINDOW( PE,NP,IWINDW,SPE )
      CALL  WINDOW( PE,NP,IWINDW,SPE,IFG )
C
cc      CALL  PRPER( JEV,PE,SPE,N,NP,IWINDW )
C
cc      STOP
      RETURN
      E N D
      SUBROUTINE  PERIOD( Y,N,LAG,OUTMIN,OUTMAX,NN,ISW,COV,P )
C
C ... Periodogram computation by Fourier transf. of autocovariance  ...
C
C     Inputs:
C        Y(I):    time series
C        N:       data length
C        LAG:     maximum lag of autocovariance
C        NN:      number of frequencies
C        OUTMIN:  lower bound for detecting outliers
C        OUTMAX:  upper bound for detecting outliers
C        ISW = 0: autocovariance is not given
C            = 1:                   given
C        COV:     autocovariance (if ISW=1)
C     Outputs:
C        COV(I):  autocovariance (if ISW=0)
C        P(I):    periodogram (raw spectrum)
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
ccb      DIMENSION  Y(N), COV(0:1024), P(0:512), FC(513), FS(513)
cxx      DIMENSION  Y(N), COV(0:1024), P(0:NN), FC(NN+1), FS(NN+1)
C
      INTEGER N, LAG, NN, ISW
ccxx      DOUBLE PRECISION Y(N), OUTMIN, OUTMAX, COV(0:1024), P(0:NN)
      DOUBLE PRECISION Y(N), OUTMIN, OUTMAX, COV(0:LAG), P(0:NN)
c local
      INTEGER I
      DOUBLE PRECISION FC(NN+1), FS(NN+1), YMEAN
C
ccxx      IF( LAG.GE.1023 )  LAG = 1023
      IF( ISW.EQ.0 )  CALL  AUTCOV( Y,N,LAG,OUTMIN,OUTMAX,COV,YMEAN )
C
C  ...  Fourier transformation by Goertzel method  ...
C
      CALL  FOURIE( COV(0),LAG+1,NN+1,FC,FS )
C
      DO 10 I=0,NN
cxx   10 P(I) =  2.0D0*FC(I+1) - COV(0)
      P(I) =  2.0D0*FC(I+1) - COV(0)
   10 CONTINUE
C
      RETURN
      E N D

C     PROGRAM 13.2  TVAR
      SUBROUTINE TVAR(Y, N, M, K, NOBS, IOPT, NOUT, LOUT, TAU20, DELTA,
     *                TAUMAX, SIG2, FF, AIC, TAR, PAR )
C
C  ...  Time Varying AR model  ...
C
C     Inputs:
C        M:       AR Order (M =< 10)
C        K:       Trend Order
C        NOBS:    Local Stationary Span (N/NOBS =< NDIM)
C        IOPT:    Search Method
C        NOUT:    Number of Outliers
C          LOUT(I):  Position of I-th Outlier
C     Parameters:
C        NMAX:    Adjustable dimension of Y
C        MJ:      Adjustable dimension of XF, VF, etc.
C        NDIM:    Adjustable dimension OF VFS, VSS, etc.
C     @TEST.FILTER2:  SEP.08,1990
C
cc      PARAMETER( NMAX=3000,MJ=20,NDIM=200 )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  Y(N)
cc      DIMENSION  A(2,MJ), MM(MJ), LOUT(10), LLOUT(NDIM)
cc      DIMENSION  XPS(MJ,NDIM), XFS(MJ,NDIM), XSS(MJ,NDIM)
cc      DIMENSION  VPS(MJ,MJ,NDIM), VFS(MJ,MJ,NDIM), VSS(MJ,MJ,NDIM)
cc      DIMENSION  XF(MJ), VF(MJ,MJ)
cxx      DIMENSION  LOUT(NOUT), TAR(M,N/NOBS), PAR(M,N/NOBS)
cxx      DIMENSION  A(K,M), MM(M), LLOUT(N/NOBS)
cxx      DIMENSION  XPS(M*K,N/NOBS), XFS(M*K,N/NOBS), XSS(M*K,N/NOBS)
cxx      DIMENSION  VPS(M*K,M*K,N/NOBS), VFS(M*K,M*K,N/NOBS)
cxx      DIMENSION  VSS(M*K,M*K,N/NOBS)
cxx      DIMENSION  XF(M*K), VF(M*K,M*K)
C
      INTEGER N, M, K, NOBS, IOPT, NOUT, LOUT(NOUT)
      DOUBLE PRECISION Y(N), TAU20, DELTA, TAUMAX, SIG2, FF, AIC,
     1                 TAR(M,N/NOBS), PAR(M,N/NOBS)
c local
      INTEGER I, II, INUM, J, MJ, NDIM, NN, MM(M), LLOUT(N/NOBS)
      DOUBLE PRECISION A(K,M), XPS(M*K,N/NOBS), XFS(M*K,N/NOBS),
     1                 XSS(M*K,N/NOBS), VPS(M*K,M*K,N/NOBS),
     2                 VFS(M*K,M*K,N/NOBS), VSS(M*K,M*K,N/NOBS),
     3                 XF(M*K), VF(M*K,M*K), OUTMIN, OUTMAX, FMAX, TAU2,
     4                 FLK, SIG2M
C
      DATA  OUTMIN, OUTMAX /-1.0D30, 1.0D30/
cc      DATA  LLOUT /NDIM*0/
C
      MJ = M*K
      NDIM = N/NOBS
cxx      DO 5 I=1,NDIM
cxx    5 LLOUT(I) = 0
      LLOUT(1:NDIM) = 0
C
cc      READ( 5,* )  M, K, NOBS, IOPT, NOUT
cc      IF( IOPT.EQ.1 )  READ(5,*)  TAU20, DELTA
      IF( NOUT.GT.0 )  THEN
cc        READ(5,*)  (LOUT(I),I=1,NOUT)
        DO 10 I=1,NOUT
        J = LOUT(I)/NOBS
        IF( J*NOBS-LOUT(I).GT.NOBS/2 )  J = J+1
cxx   10   LLOUT(J) = 1
        LLOUT(J) = 1
   10   CONTINUE
      END IF
      INUM = 19
      IF( IOPT.EQ.0 )  INUM = 9
C
C  ...  Read Time Series  ...
C
cc      CALL  READTS( 1,Y,N )
C
      FMAX  = -1.0D30
C
      CALL  SETCAR( M,K,A,MM )
      DO 100  II=1,INUM
      IF( IOPT.NE.0 )  TAU2 = TAU20 + DELTA*(II-9)
      IF( IOPT.EQ.0 .AND. K.EQ.1 )  TAU2 =10.0D0**(-II)
      IF( IOPT.EQ.0 .AND. K.GT.1 )  TAU2 =10.0D0**(-II-1)
      CALL  ISTCAR( M,K,MJ,XF,VF )
C
C  ...  Log-Likelihood Computation  ...
C
      CALL  FILTR2( Y,XF,VF,TAU2,M,K,N,NOBS,MJ,1,LLOUT,
     *              OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FLK,SIG2 )
      IF( FLK.GT.FMAX )  THEN
         FMAX  = FLK
         TAUMAX = TAU2
         SIG2M  = SIG2
      END IF
cc  100 WRITE(6,600)  TAU2, SIG2, FLK
  100 CONTINUE
      AIC = -2*FMAX + 2*(M+2)
cc      WRITE(6,*)  M, K
cc      WRITE(6,610)  TAUMAX, SIG2M, FMAX, AIC
cc      WRITE(6,*)  FF,SIG2
C
C  ... Fixed Interval Smoother  ...
C
      CALL  ISTCAR( M,K,MJ,XF,VF )
      CALL  FILTR2( Y,XF,VF,TAUMAX,M,K,N,NOBS,MJ,NDIM,LLOUT,
     *              OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,SIG2 )
      NN = N/NOBS
cc      CALL  SMOTH1( A,MM,K,M,1,NN,NN,MJ,VFS,VPS,VSS,XFS,XPS,XSS )
      CALL  SMOTH1( A,MM,K,M,1,NN,NN,NDIM,MJ,
     *        VFS,VPS,VSS,XFS,XPS,XSS )
C
C  ...  Plot PARCOR and print AR Coefficients  ...
C
cc      CALL  PTCAR( XSS,XPS,N,NOBS,MJ,M,K,TAUMAX,SIG2,FF,AIC )
      CALL  PTCAR( XSS,N,NOBS,MJ,M,K,TAR,PAR )
cc      CALL  PRCAR( XSS,N,NOBS,MJ,M,K,TAUMAX,SIG2,FF,AIC )
C
      RETURN
cxx  600 FORMAT( 1H ,5X,F15.10,F12.6,F13.5 )
cxx  610 FORMAT( 1H ,'TAUMAX =',F15.10,3X,'SIG2 =',F15.10,3X,
cxx     *            'FF =',F13.5,3X,'AIC =',F13.4 )
      E N D
      SUBROUTINE  SETCAR( M,K,A,MM )
C
C  ...  State space model for trend estimation  ...
C
C     Input:
C       M:   AR Order
C       K:   Trend Order
C     Outputs:
C       A:   Parameter of the Transition Matrix F
C       MM:  Dimension of the Component Model
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION  A(2,*), MM(*)
cxx      DIMENSION  A(K,M), MM(M)
C
      INTEGER M, K, MM(M)
      DOUBLE PRECISION A(K,M)
C
      IF( K.EQ.1 )  THEN
cxx        DO 10 I=1,M
cxx   10   A(1,I) = 1.0D0
         A(1,1:M) = 1.0D0
      END IF
      IF( K.EQ.2 )  THEN
cxx        DO 20 I=1,M
cxx        A(1,I) = 2.0D0
cxx   20   A(2,I) =-1.0D0
        A(1,1:M) = 2.0D0
        A(2,1:M) =-1.0D0
      END IF
cxx      DO 30 I=1,M
cxx   30 MM(I) = K
      MM(1:M) = K
C
      RETURN
      E N D
      SUBROUTINE  FILTR2( Y,XF,VF,Q,M,K,N,NOBS,MJ,NDIM,LLOUT,
     *                    OUTMIN,OUTMAX,VFS,VPS,XFS,XPS,FF,SIG2 )
C
C  ...  Kalman Filter (for Time Varying AR model)  ...
C
C     Inputs:
C        Y:      time series
C        XF:     Initial state vector
C        VF:     Initial covariance matrix
C        Q:      K*K matrix, system noise covariance
C        M:      Dimension of the state vector
C        K:      Dimension of the system noise
C        N:      Data length
C        NOBS:   Local Stationary Span ( N/NOBS =< NDIM )
C        MJ:     Adjustable dimension of XF, VF
C        NDIM:   Adjustable dimension of VPS, VFS etc.
C        OUTMIN: Lower limit for detecting outliers
C        OUTMAX: Upper limit for detecting outliers
C     Outputs:
C        VFS:    Covariance matrices of the filter
C        VPS:    Covariance matrices of the predictor
C        XFS:    Mean vectors of the filter
C        XPS:    Mean vectors of the predictor
C        FF:     Log likelihood
C        SIG2:   Estimated observational noise variance
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cx      DIMENSION  Y(N), LLOUT(NDIM)
cc      DIMENSION  XF(MJ), VF(MJ,MJ), XP(40), VP(40,40)
cxx      DIMENSION  Y(N), LLOUT(N/NOBS)
cxx      DIMENSION  XF(MJ), VF(MJ,MJ), XP(MJ), VP(MJ,MJ)
cxx      DIMENSION  XFS(MJ,NDIM),    XPS(MJ,NDIM)
cxx      DIMENSION  VFS(MJ,MJ,NDIM), VPS(MJ,MJ,NDIM)
cc      DIMENSION  VH(40), GAIN(40)
cxx      DIMENSION  VH(MJ), GAIN(MJ)
C
      INTEGER M, K, N, NOBS, MJ, NDIM, LLOUT(N/NOBS)
      DOUBLE PRECISION Y(N), XF(MJ), VF(MJ,MJ), Q, OUTMIN, OUTMAX,
     1                 VFS(MJ,MJ,NDIM), VPS(MJ,MJ,NDIM), XFS(MJ,NDIM),
     2                 XPS(MJ,NDIM), FF, SIG2
c local
      INTEGER I, II, I1, I2, J, JJ, J2, MK, NS, NE, NSUM
      DOUBLE PRECISION XP(MJ), VP(MJ,MJ), VH(MJ), GAIN(MJ), PI, SDET,
     1                 SUM, PERR, PVAR
C
      DATA   PI  /3.1415926535D0/
C
      MK = M*K
      SIG2 = 0.0D0
      SDET = 0.0D0
      NSUM = 0
      NS = M/NOBS+1
      NE = N/NOBS
C
      DO 500  II=NS,NE
C
C  ...  ONE STEP AHEAD PREDICTION  ...
C
      IF( K.EQ.1 )  THEN
cxx         DO 10 J=1,M
         DO 11 J=1,M
         XP(J) = XF(J)
         DO 10 I=1,M
cxx   10    VP(I,J) = VF(I,J)
         VP(I,J) = VF(I,J)
   10    CONTINUE
   11    CONTINUE
      END IF
      IF( K.EQ.2 )  THEN
cxx         DO 20 J=1,M
         DO 21 J=1,M
         J2 = 2*J
         XP(J2-1) = 2*XF(J2-1) + XF(J2)
         XP(J2)   =  -XF(J2-1)
         DO 20 I=1,M
         I2 = 2*I
         VP(I2-1,J2-1) = 4*VF(I2-1,J2-1) + 2*VF(I2,J2-1)
     *                 + 2*VF(I2-1,J2)   + VF(I2,J2)
         VP(I2-1,J2)   =-2*VF(I2-1,J2-1) - VF(I2,J2-1)
         VP(I2,J2-1)   =-2*VF(I2-1,J2-1) - VF(I2-1,J2)
cxx   20    VP(I2,J2)     =   VF(I2-1,J2-1)
         VP(I2,J2)     =   VF(I2-1,J2-1)
   20    CONTINUE
   21    CONTINUE
      END IF
C
      DO 30 I=1,M
cxx   30 VP(K*(I-1)+1,K*(I-1)+1) = VP(K*(I-1)+1,K*(I-1)+1) + Q
      VP(K*(I-1)+1,K*(I-1)+1) = VP(K*(I-1)+1,K*(I-1)+1) + Q
   30 CONTINUE
      IF( LLOUT(II).EQ.1 )  THEN
        DO 35 I=1,MK
cxx   35   VP(I,I) = 1.0D3
        VP(I,I) = 1.0D3
   35   CONTINUE
      END IF
C
C  ...  SAVE MEAN AND COVARIANCE  ...
C
      IF( NDIM.GT.1 )  THEN
cxx      DO 40  I=1,MK
      DO 41  I=1,MK
      XPS(I,II) = XP(I)
      DO 40  J=1,MK
cxx   40 VPS(I,J,II) = VP(I,J)
      VPS(I,J,II) = VP(I,J)
   40 CONTINUE
   41 CONTINUE
      END IF
C
C
C  ...  FILTERING  ...
C
      DO 400  JJ=1,NOBS
      I1 = NOBS*(II-1) + JJ
      IF( I1.LE.M )  GO TO 400
      IF( Y(I1).GT.OUTMIN.AND.Y(I1).LT.OUTMAX.AND. I1.LE.N ) THEN
C
      DO 210  I=1,MK
      SUM = 0.0D0
      DO 200  J=1,M
cxx  200 SUM = SUM + VP(I,K*(J-1)+1)*Y(I1-J)
      SUM = SUM + VP(I,K*(J-1)+1)*Y(I1-J)
  200 CONTINUE
cxx  210 VH(I) = SUM
      VH(I) = SUM
  210 CONTINUE
C
      PERR = Y(I1)
      PVAR = 1.0D0
      DO 220  I=1,M
      PERR = PERR - Y(I1-I)*XP(K*(I-1)+1)
cxx  220 PVAR = PVAR + Y(I1-I)*VH(K*(I-1)+1)
      PVAR = PVAR + Y(I1-I)*VH(K*(I-1)+1)
  220 CONTINUE
C
      DO 250  I=1,MK
cxx  250 GAIN(I) = VH(I)/PVAR
      GAIN(I) = VH(I)/PVAR
  250 CONTINUE
C
      DO 290  I=1,MK
cxx  290 XF(I) = XP(I) + GAIN(I)*PERR
      XF(I) = XP(I) + GAIN(I)*PERR
  290 CONTINUE
C
cxx      DO 310  I=1,MK
      DO 311  I=1,MK
      DO 310  J=1,MK
cxx  310 VF(I,J) = VP(I,J) - GAIN(I)*VH(J)
      VF(I,J) = VP(I,J) - GAIN(I)*VH(J)
  310 CONTINUE
  311 CONTINUE
C
      IF( I1.GT.MJ )  THEN
c------------------   
         if( PVAR.LE.0 ) GO TO 600
c------------------
        SIG2 = SIG2 + PERR**2/PVAR
        SDET = SDET + DLOG(PVAR)
        NSUM = NSUM + 1
      END IF
C
C  ...  MISSING OBSERVATION  ...
C
      ELSE
cxx      DO 350  I=1,MK
      DO 351  I=1,MK
      XF(I) = XP(I)
      DO 350  J=1,MK
cxx  350 VF(I,J) = VP(I,J)
      VF(I,J) = VP(I,J)
  350 CONTINUE
  351 CONTINUE
      END IF
      IF( JJ.NE.NOBS )  THEN
cxx        DO 360 J=1,MK
        DO 361 J=1,MK
        XP(J) = XF(J)
        DO 360 I=1,MK
cxx  360   VP(I,J) = VF(I,J)
        VP(I,J) = VF(I,J)
  360   CONTINUE
  361   CONTINUE
      END IF
  400 CONTINUE
C
C  ...  SAVE MEAN AND COVARIANCE  ...
C
      IF( NDIM.GT.1 )  THEN
cxx      DO 370  I=1,MK
      DO 371  I=1,MK
      XFS(I,II) = XF(I)
      DO 370  J=1,MK
cxx  370 VFS(I,J,II) = VF(I,J)
      VFS(I,J,II) = VF(I,J)
  370 CONTINUE
  371 CONTINUE
      END IF
C
  500 CONTINUE
      SIG2 = SIG2/NSUM
      FF = -0.5D0*(NSUM*(DLOG(PI*2*SIG2) + 1) + SDET)
C
cxx      DO 510 II=1,NS-1
cxx      DO 510 J=1,MK
      DO 512 II=1,NS-1
      DO 511 J=1,MK
      XPS(J,II) = XPS(J,NS)
      XFS(J,II) = XFS(J,NS)
      DO 510 I=1,MK
      VPS(I,J,II) = VPS(I,J,NS)
cxx  510 VFS(I,J,II) = VFS(I,J,NS)
      VFS(I,J,II) = VFS(I,J,NS)
  510 CONTINUE
  511 CONTINUE
  512 CONTINUE
C
      RETURN
c------------------   
  600 FF = -1.0D30
      RETURN
c------------------
      E N D
      SUBROUTINE  ISTCAR( M,K,MJ,XF,VF )
C
C  ...  Initial State (for Time-Varying AR Model)  ...
C
C     Inputs:
C        M:     AR order
C        K:     Trend order
C        MJ:    Adjustable dimension of F
C     Outputs:
C         XF:   State vector, X(0|0)
C         VF:   State covarance matrix, V(0|0)
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      DIMENSION  XF(MJ), VF(MJ,MJ)
C
      INTEGER M, K, MJ
      DOUBLE PRECISION XF(MJ), VF(MJ,MJ)
c local
      INTEGER I, MK
C
      MK = M*K
cxx      DO 10  J=1,MK
cxx      XF(J) = 0.0D0
cxx      DO 10  I=1,MK
cxx   10 VF(I,J) = 0.0D0
      XF(1:MK) = 0.0D0
      VF(1:MK,1:MK) = 0.0D0
C
      DO 20  I=1,MK
cxx   20 VF(I,I) = 1.0D2
      VF(I,I) = 1.0D2
   20 CONTINUE
C
      RETURN
      E N D
C
C
cc      SUBROUTINE  PTCAR( XSS,WRK,N,NOBS,MJ,M,K,TAU2,SIG2,FF,AIC )
      SUBROUTINE  PTCAR( XSS,N,NOBS,MJ,M,K,TAR,WRK )
C
C  ...  Plot Original Time Varying PARCOR  ...
C
C     Inputs:
C        XSS:    Smoothed state
C        N:      Data length
C        NOBS:   Local Stationary Span
C        MJ:     Adjustable dimension of F and G
C        M,K:    AR and Trend Orders
C        TAU2, SIG2: Variances of the State Space Model
C        FF:     Log-Likelihood of the model
C        AIC:    AIC of the Model
C     MODIFIED  2/15/93
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      CHARACTER  VNAME*8
cc      DIMENSION  XSS(MJ,N), WRK(MJ,N), AR(10), PAR(10)
cxx      DIMENSION  XSS(MJ,N/NOBS), WRK(M,N/NOBS)
cxx      DIMENSION  AR(M), PAR(M), TAR(M,N/NOBS)
cc      DIMENSION  DATA(400), VNAME(20), VALUE(20)
C
      INTEGER N, NOBS, MJ, M, K
      DOUBLE PRECISION XSS(MJ,N/NOBS), TAR(M,N/NOBS), WRK(M,N/NOBS)
c local
      INTEGER I, II, NN
      DOUBLE PRECISION AR(M), PAR(M), FN
C
cc      VNAME(1) = 'M     = '
cc      VNAME(2) = 'K     = '
cc      VNAME(3) = 'NOBS  = '
cc      VNAME(4) = 'TAU2  = '
cc      VNAME(5) = 'SIG2  = '
cc      VNAME(6) = 'FF    = '
cc      VNAME(7) = 'AIC   = '
cc      VALUE(1) = M
cc      VALUE(2) = K
cc      VALUE(3) = NOBS
cc      VALUE(4) = TAU2
cc      VALUE(5) = SIG2
cc      VALUE(6) = FF
cc      VALUE(7) = AIC
cc      CALL  PLOTS
C     call  plots( 1,0,0,1,0 )
C     call  form( 1 )
C     call  factor( 10.0 )
cc      CALL  HEADER( 'PROGRAM 12.1: TVCAR MODEL ',33,7,
cc     *               VNAME,VALUE )
cc      WX = 12.0
cc      WY =  3.0
cc      DX =500.0
      FN = N
      NN = N/NOBS
cc      IY = 10
cc      YMIN1 = -1.0D0
cc      YMAX1 =  1.0D0
cc      DY = 1.0D0
      DO 40 II=1,NN
      DO 10 I=1,M
cxx   10 AR(I) = XSS(K*(I-1)+1,II)
      AR(I) = XSS(K*(I-1)+1,II)
   10 CONTINUE
      CALL  PARCOR( AR,M,PAR )
      DO 20 I=1,M
      IF( PAR(I).GT. 0.95D0 )  PAR(I) = 0.95D0
cxx   20 IF( PAR(I).LT.-0.95D0 )  PAR(I) =-0.95D0
      IF( PAR(I).LT.-0.95D0 )  PAR(I) =-0.95D0
   20 CONTINUE
      CALL  ARCOEF( PAR,M,AR )
      DO 30 I=1,M
      XSS(K*(I-1)+1,II) = AR(I)
c--------------------------------
      TAR(I,II) = AR(I)
c--------------------------------
cxx   30 WRK(I,II) = PAR(I)
      WRK(I,II) = PAR(I)
   30 CONTINUE
   40 CONTINUE
C
C  ...   PLOT  PARCOR  ...
C
cc      DO 200 II=1,M
cc      IF( II.EQ.9 )  CALL  PLOTI
C     if( ii.eq.9 )  call  plot( 0.0,0.0,777 )
cc      IF( MOD(II,8).EQ.1 )  CALL  PLOT( 2.0,17.0-SNGL(WY)-1.0,-3 )
cc      IF( II.EQ.5 )  CALL  PLOT( SNGL(WX)+2.0,3*SNGL(WY+1.0),-3 )
cc      IF( MOD(II,4).NE.1 )  CALL  PLOT( 0.0,-SNGL(WY+1.0),-3 )
C     CALL  SYMBOL( 0.25,SNGL(WY)+0.25,0.25,'TREND',0.0,5 )
cc      CALL  AXISXY( 0.0D0,0.0D0,WX,WY,0.0D0,FN,YMIN1,YMAX1,DX,DY,
cc     *              0.2D0,1,IY,2)
C
cc      DO 100 I=1,NN
cc  100 DATA(I) = WRK(II,I)
cc      CALL  NEWPEN( 1 )
cc      CALL  PLOTY( DATA,NN,YMIN1,YMAX1,WX,WY,1,1 )
cc  200 CONTINUE
C
cc      CALL  PLOTE
C     call  plot( 0.0,0.0,999 )
      RETURN
      E N D

C     PROGRAM 15.1  SIMSSM
      SUBROUTINE SIMSSMF( M1,M2,M3,M,K,N,INI,SIG2,PERIOD,TAU1,TAU2,TAU3,
     &                    AR,X,Y )
C
C  ...  SIMULATION BY GAUSSIAN STATE SPACE MODEL  ...
C
C     INPUTS:
C        M1:   TREND ORDER (0,1,2)
C        M2:   SEASONAL ORDER (0 OR 1)
C        M3:   AR ORDER (0,...,20)
C        N:    SIMULATION INTERVAL
C        NS:   Y(NS+1),...,Y(N) ARE ACTUALLY USED
C        INI:  INITIAL VALUE FOR RANDOM NUMBER GENERATOR
C     PARAMETERS:
C        MJ:   MAXIMUM STATE DIMENSION
C        NN:   MAXIMUM DATA LENGTH
C     MODIFED  2/16/93
C
cc      PARAMETER( MJ=20,NN=1000 )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cxx      INTEGER    PERIOD
cc      DIMENSION  X(MJ), F(MJ,MJ), G(MJ,MJ), H(MJ)
cc      DIMENSION  Q(MJ,MJ), Y(NN), AR(20), R(1,1)
cc      INI = 1992092521
cxx      DIMENSION  X(M), F(M,M), G(M,K), H(M)
cxx      DIMENSION  Q(K,K), Y(N), AR(M3), R(1,1)
C
      INTEGER M1, M2, M3, M, K, N, INI, PERIOD
      DOUBLE PRECISION SIG2, TAU1, TAU2, TAU3, AR(M3), X(M), Y(N)
c local
      INTEGER INI0
      DOUBLE PRECISION F(M,M), G(M,K), H(M), Q(K,K), R(1,1)
C
cc      MM = 0
C
cc      READ( 5,* )  M1, M2, M3, N, NS, INI
cc      READ( 5,* )  SIG2
cc      IF( M1.GT.0 )  THEN
cc         READ( 5,* )  TAU1
cc         READ( 5,* )  (X(I),I=1,M1)
cc         MM = M1
cc      END IF
cc      IF( M2.GT.0 )  THEN
cc         READ( 5,* )  TAU2, PERIOD
cc         READ( 5,* )  (X(MM+I),I=1,PERIOD-1)
cc         MM = MM + PERIOD-1
cc      END IF
cc      IF( M3.GT.0 )  THEN
cc         READ( 5,* )  TAU3, (AR(I),I=1,M3)
cc         READ( 5,* )  (X(MM+I),I=1,M3)
cc      END IF
      INI0  = INI
C
C  ...  SET STATE SPACE MODEL  ...
C
      CALL  SETSEA( M1,M2,M3,PERIOD,AR,TAU1,TAU2,TAU3,SIG2,F,G,H,
cc     *              Q,R,M,K,MJ )
     *              Q,R,M,K )
C
C  ...  SIMULATION  ...
C
cc      CALL  SIMSSM( F,G,H,Q,R,X,N,M,1,K,INI,MJ,NN,Y )
      CALL  SIMSSM( F,G,H,Q,R,X,N,M,1,K,INI,N,Y )
C
C  ...  PLOT AND PRINT OUT SIMULATED TIME SERIES  ...
C
C      CALL  PTSIM( Y,N,NS,K,M1,M2,M3,INI0 )
cc      CALL  PRSIM( Y,N,NS,M1,M2,M3,INI0 )
C
cc      STOP
      RETURN
      E N D
cc      SUBROUTINE SIMSSM( F,G,H,Q,R,X,N,M,L,K,IX,MJ,NN,Y )
      SUBROUTINE SIMSSM( F,G,H,Q,R,X,N,M,L,K,IX,NN,Y )
C
C  ...  SIMULATION BY STATE SPACE MODEL  ...
C
C     INPUTS:
C        F,G,H:   COEFFICIENT MATRICES OF STATE SPACE MODEL
C        Q,R:     SYSTEM NOISE AND OBS. NOISE VARIANCES
C        X:       INITIAL STATE VECTOR
C        M,L,K:   DIMENSIONS OF THE STATE, SYSTEM NOISE AND OBS.
C        IX:      INITAL SEED FOR RANDOM NUMBER GENERATOR
C        MJ:      ADJUSTABLE DIMENSION
C        NN:      NUMBER OF DATA
C     OUTPUT:
C        Y(I):    SIMULATED TIME SERIES
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION X(MJ), F(MJ,MJ), G(MJ,MJ), H(L,MJ)
cc      DIMENSION Q(MJ,MJ), R(L,L), SQ(10,10), SR(10,10)
cc      DIMENSION XT(40), T(20), V(20), W(10), Y(NN,L)
cxx      DIMENSION X(M), F(M,M), G(M,K), H(L,M)
cxx      DIMENSION Q(K,K), R(L,L), SQ(K,K), SR(L,L)
cxx      DIMENSION XT(M), T(L), V(K), W(L), Y(NN,L)
C
      INTEGER N, M, L, K, IX, NN
      DOUBLE PRECISION F(M,M), G(M,K), H(L,M), Q(K,K), R(L,L), X(M),
     1                 Y(NN,L)
c local
      INTEGER I, II, J
      DOUBLE PRECISION SQ(K,K), SR(L,L), XT(M), T(L), V(K), W(L)
C
C  ...  CHOLESKY DECOMPOSITION OF Q AND R  ...
C
cc      CALL  CHOLES( Q,MJ,K,SQ,10 )
cc      CALL  CHOLES( R,L,L,SR,10 )
      CALL  CHOLES( Q,K,K,SQ,K )
      CALL  CHOLES( R,L,L,SR,L )
C
C----- Initialize Mersenne Twister with given seed value
      call init(IX)
c-----
      DO 100 II= 1,N
C
C  ...  X = F*X + V  ...
C
ccxx      CALL  WHITE( K,SQ,IX,V )
      CALL  WHITE( K,SQ,V )
C
cxx      DO 10 I=1,M
cxx   10 XT(I) = 0.0D0
      XT(1:M) = 0.0D0
cxx      DO 20 J=1,M
      DO 21 J=1,M
      DO 20 I=1,M
cxx   20 XT(I) = XT(I) + F(I,J)*X(J)
      XT(I) = XT(I) + F(I,J)*X(J)
   20 CONTINUE
   21 CONTINUE
cxx      DO 30 J=1,K
      DO 31 J=1,K
      DO 30 I=1,M
cxx   30 XT(I) = XT(I) + G(I,J)*V(J)
      XT(I) = XT(I) + G(I,J)*V(J)
   30 CONTINUE
   31 CONTINUE
      DO 40 I=1,M
cxx   40 X(I) = XT(I)
      X(I) = XT(I)
   40 CONTINUE
C
C  ...  T = H*X + W  ...
C
ccxx      CALL  WHITE( L,SR,IX,W )
      CALL  WHITE( L,SR,W )
cxx      DO 50 I=1,L
cxx   50 T(I) = 0.0D0
      T(1:L) = 0.0D0
cxx      DO 60 I=1,L
      DO 61 I=1,L
      DO 60 J=1,M
cxx   60 T(I) = T(I) + H(I,J)*X(J)
      T(I) = T(I) + H(I,J)*X(J)
   60 CONTINUE
   61 CONTINUE
      DO 80 I=1,L
cxx   80 Y(II,I) = T(I) + W(I)
      Y(II,I) = T(I) + W(I)
   80 CONTINUE
  100 CONTINUE
C
      RETURN
      E N D
ccxx      SUBROUTINE  WHITE( K,Q,IX,V )
      SUBROUTINE  WHITE( K,Q,V )
C
C  ...  GENERATE K-DIMENSIONAL GAUSSIAN RANDOM NUMBER  ...
C
C     INPUTS:
C        K:   DIMENSION OF THE RANDOM NUMBER
C        Q:   COVARIANCE
C        IX:  INITIAL SEED FOR RANDOM NUMBER GENERATOR
C     OUTPUT:
C        V:   K-DIMENSIONAL RANDOM NUMBER WITH COVARIANCE Q
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      DIMENSION Q(K,K),W(100),V(K)
cxx      DIMENSION Q(K,K),W(K),V(K)
C
      INTEGER K
      DOUBLE PRECISION Q(K,K), V(K)
c local
      INTEGER I, IC, J
      DOUBLE PRECISION RGAUSS, W(K), V2, S, SUM
C
      IC = 0
      DO 10 I=1,K
cc   10 W(I) = RGAUSS( IX )
cxx   10 W(I) = RGAUSS( IX,IC,V2,S )
ccxx      W(I) = RGAUSS( IX,IC,V2,S )
      W(I) = RGAUSS( IC,V2,S )
   10 CONTINUE
      DO 30 I=1,K
      SUM = 0.0D0
      DO 20 J=1,K
cxx   20 SUM = SUM + Q(J,I)*W(J)
      SUM = SUM + Q(J,I)*W(J)
   20 CONTINUE
cxx   30 V(I) = SUM
      V(I) = SUM
   30 CONTINUE
      RETURN
      E N D
cc      DOUBLE PRECISION FUNCTION  RGAUSS( IX )
ccxx      DOUBLE PRECISION FUNCTION  RGAUSS( IX,ICOUNT,V2,S )
      DOUBLE PRECISION FUNCTION  RGAUSS( ICOUNT,V2,S )
C
C  ...  GAUSSIAN RANDON NUMBER GENERATOR (MARSAGLIA'S ALGORITHM)  ...
C
cxx      IMPLICIT REAL*8( A-H,O-Z)
      INTEGER ICOUNT
      DOUBLE PRECISION V2, S
c local
cxx      DOUBLE PRECISION RUNI, U1, U2, V1
      DOUBLE PRECISION U1, U2, V1, random

cc      DATA  ICOUNT /0/
C
cxx      IF( IX.EQ.0 )  IX = 1990103011
      IF( ICOUNT.EQ.0 )  THEN
cxx   10 U1 = RUNI(IX)
cxx      U2 = RUNI(IX)
   10 U1 = random()
      U2 = random()
      V1 = 2*U1-1
      V2 = 2*U2-1
      S = V1**2 + V2**2
      IF(S.GE.1) GO TO 10
      RGAUSS = V1*SQRT( -2*DLOG(S)/S )
      ICOUNT = 1
C
      ELSE
        RGAUSS = V2*SQRT( -2*DLOG(S)/S)
        ICOUNT = 0
      END IF
      RETURN
      E N D

C     PROGRAM  5.1  LSQR
cxx      SUBROUTINE LSQRF(Y,N,K,AIC,SIG2,IMIN,A,DATA)
cxxx      SUBROUTINE LSQRF(Y,N,K,MJ1,AIC,SIG2,IMIN,A,DATA)
      SUBROUTINE LSQR(Y,N,K,PERIOD,MJ1,AIC,SIG2,IMIN,A,DATA)
C
C  ...  The least squares method via Householder transformation  ...
C
C     The following inputs are required in the subroutine  READTS:
C        TITLE:   title of the data
C        FORMAT:  reading format of the data
C        Y(I):    time series
C     PARATERS:
C        MJ:      Adjustable dimension of Y (MJ.GE.N)
C        MJ1:     Adjustable dimension of X and D
C        MJ2:     Adjustable dimension of A, SIG2 and AIC
C        IDEV:    Input device
C        LAG:     Number of sine and cosine terms
C     @TEST.PN51:
C
cc      PARAMETER (MJ=1000,MJ1=100,MJ2=22,ISW=2,IDEV=1,LAG=10,K=LAG*2+1)
cxx      PARAMETER (MJ1=100)
cxx      IMPLICIT   REAL*8 (A-H,O-Z)
cc      CHARACTER  TITLE*72
cc      DIMENSION  Y(MJ), AIC(0:MJ2)
cc      DIMENSION  X(MJ1,MJ2), D(MJ1), A(MJ2,MJ2), SIG2(0:MJ2)
cc      COMMON     /CMDATA/  TITLE
cxx      DIMENSION  Y(N), DATA(N), AIC(0:K)
cxx      DIMENSION  X(MJ1,K+1), A(K,K), SIG2(0:K)
C
      INTEGER N, K, PERIOD, MJ1, IMIN
      DOUBLE PRECISION Y(N), AIC(0:K), SIG2(0:K), A(K,K), DATA(N)
c local
      DOUBLE PRECISION X(MJ1,K+1)
C
      EXTERNAL   SETXTP
C
cc      MJ2=K
C
cc      CALL  READTS( IDEV,Y,N )
C
cc      CALL  REDUCT( SETXTP,Y,D,N,0,K,MJ1,X )
cx      CALL  REDUCT( SETXTP,Y,N,0,K,MJ1,X )
cxxx      CALL  REDUCT1( SETXTP,Y,N,0,K,MJ1,X )
      CALL  REDUCT2( SETXTP,Y,N,0,K,PERIOD,MJ1,X )
C
cc      CALL  REGRES( X,K,N,MJ1,MJ2,A,SIG2,AIC,IMIN )
      CALL  REGRES( X,K,N,MJ1,A,SIG2,AIC,IMIN )
C
cc      CALL  PRREG( N,K,MJ2,A,SIG2,AIC )
cxx      CALL  PTTPL( Y,N,A(1,IMIN),IMIN,DATA )
cxxx      CALL  PTTPL( N,A(1,IMIN),IMIN,DATA )
      CALL  PTTPL( N,A(1,IMIN),IMIN,PERIOD,DATA )
C
cc      STOP
      RETURN
      E N D
C
cc      SUBROUTINE  REDUCT( SETX,Z,D,NMK,N0,K,MJ1,X )
      SUBROUTINE  REDUCT2( SETX,Z,NMK,N0,K,PERIOD,MJ1,X )
C
C  ...  Successive Householder reduction  ...
C
C     Inputs:
C        SETX:    Name of the subroutine for making X(I,J)
C        Z(I):    Data vector
C        D(I):    Working area
C        NMK:     Number of actually used observations
C        N0:      Time point of the previous set ofobservations
C        K:       Heighest order of the model
C          PERIOD:       Period of one cycle
C        MJ1:     Adjustable dimension of X
C     Output:
C        X(I,J):  data matrix
C
cxx      IMPLICIT  REAL*8( A-H,O-Z )
cc      DIMENSION  X(MJ1,1) , D(1), Z(1)
cx      DIMENSION  X(MJ1,1) , Z(1)
cxx      DIMENSION  X(MJ1,K+1) , Z(N0+NMK)
      INTEGER NMK, N0, K, MJ1, PERIOD
      DOUBLE PRECISION Z(N0+NMK), X(MJ1,K+1)
c local
      INTEGER K1, L, LK, N1, N2
C
      L = MIN0( NMK,MJ1 )
      K1 = K + 1
      N1 = L
C
cxxx      CALL  SETX( Z,N0,L,K,MJ1,0,X )
      CALL  SETX( Z,N0,L,K,PERIOD,MJ1,0,X )
cc      CALL  HUSHLD( X,D,MJ1,L,K1 )
      CALL  HUSHLD( X,MJ1,L,K1 )
      IF( N1 .GE. NMK )  RETURN
C
   10 L = MIN0( NMK-N1,MJ1-K1 )
C
      LK = L + K1
      N2 = N0 + N1
cxxx      CALL  SETX( Z,N2,L,K,MJ1,1,X )
      CALL  SETX( Z,N2,L,K,PERIOD,MJ1,1,X )
cc      CALL  HUSHLD( X,D,MJ1,LK,K1 )
      CALL  HUSHLD( X,MJ1,LK,K1 )
      N1 = N1 + L
      IF( N1.LT.NMK )  GO TO 10
C
      RETURN
C
      E N D
C
cxxx      SUBROUTINE  SETXTP( Z,N0,L,K,MJ1,JSW,X )
      SUBROUTINE  SETXTP( Z,N0,L,K,PERIOD,MJ1,JSW,X )
C
C  ...  Data matrix for trigonometric polynomial regression  ...
C
C     Inputs:
C        Z(I):    Data vector
C        N0:      Origin of the current observations
C        L:       Number of current observations
C        K:       Number of regressors
C          PERIOD:       Period of one cycle
C        MJ1:     Adjustable dimension of X
C        JSW=0:   Make initial data matrix
C           =1:   Apend L*(K+1) data matrix below the triangular one
C     Output:
C        X(I,J):  Data matrix
C
cc      REAL*8  X(MJ1,1), Z(1), W
cxx      REAL*8  X(MJ1,K+1), Z(N0+L), W
      INTEGER N0, L, K, MJ1, JSW, PERIOD
      DOUBLE PRECISION Z(N0+L), X(MJ1,K+1)
c local
      INTEGER I, II, I0, J
      DOUBLE PRECISION W
C
cxxx      W = 2*3.1415926536D0/365.0D0
      W = 2*3.1415926536D0/DBLE(period)
      I0 = 0
      IF( JSW .EQ. 1 )     I0 = K+1
cxx      DO 10  I=1,L
      DO 11  I=1,L
        II = I + I0
        X(II,K+1) = Z(N0+I)
        X(II,1) = 1.0D0
      DO 10  J=1,(K-1)/2
      X(II,J*2)   = DSIN( W*(I+N0)*J )
cxx   10 X(II,J*2+1) = DCOS( W*(I+N0)*J )
      X(II,J*2+1) = DCOS( W*(I+N0)*J )
   10 CONTINUE
   11 CONTINUE
C
      RETURN
C
      E N D

cc      SUBROUTINE PTTPL( Y,N,A,M )
cxx      SUBROUTINE PTTPL( Y,N,A,M,DATA )
cxxx      SUBROUTINE PTTPL( N,A,M,DATA )
      SUBROUTINE PTTPL( N,A,M,PERIOD,DATA )
C
C  ...  This subroutine draws fitted trigonometric polynomial  ...
C
C     Inputs:
C        Y(I):   Original data
C        N:      Data length
C        A(I):   Regression coefficients
C        M:      Order of regression model
C          PERIOD:       Period of one cycle
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      CHARACTER  VNAME*8
cc      DIMENSION  Y(N), A(M), DATA(1000)
cxx      DIMENSION  Y(N), A(M), DATA(N)
cc      DIMENSION  VNAME(5), VALUE(5)
C
      INTEGER N, M, PERIOD
      DOUBLE PRECISION A(M), DATA(N)
c local
      INTEGER I, J
      DOUBLE PRECISION W, SUM
C
cc      WX = 20.0
cc      WY = 6.0
cc      IPOS = 1
cc      CALL  PLOTS
C     call  plots( 1,0,0,1,0 )
C     call  form( 1 )
C     call  factor( 10.0 )
cc      CALL  HEADER( 'TRIGONOMETRIC REGRESSION',24,0,VNAME,VALUE )
C
cc      CALL  DELX( 0.0,DBLE(N),DX )
cc      CALL  MAXMIN( Y,N,YMIN,YMAX,DY )
cc      CALL  AXISXY( 3.0D0,15.0-WY,WX,WY,0.0D0,DBLE(N),YMIN,YMAX,
cc     *              DX,DY,0.2D0,1,10,2 )
cc      CALL  SYMBOL( 0.0,SNGL(WY)+0.1,0.2,'ORIGINAL AND TREND',0.0,18 )
cc      CALL  NEWPEN( 1 )
cc      CALL  PLOTY ( Y,N,YMIN,YMAX,WX,WY,IPOS,1 )
C
cxxx      W = 2*3.1415926536D0/365.0D0
      W = 2*3.1415926536D0/DBLE(period)
      DO 20 I=1,N
      SUM = A(1)
      DO 10 J=1,10
      IF( 2*J.LE.M )   SUM = SUM + A(2*J)*DSIN(W*I*J)
cxx   10 IF( 2*J+1.LE.M ) SUM = SUM + A(2*J+1)*DCOS(W*I*J)
      IF( 2*J+1.LE.M ) SUM = SUM + A(2*J+1)*DCOS(W*I*J)
   10 CONTINUE
cxx   20 DATA(I) = SUM
      DATA(I) = SUM
   20 CONTINUE
cc      CALL  NEWPEN( 2 )
cc      CALL  PLOTY( DATA,N,YMIN,YMAX,WX,WY,IPOS,1 )
cc      CALL  PLOTE
C     call  plot( 0.0,0.0,999 )
C
      RETURN
      E N D

# PROGRAM 7.1 ARFIT
arfit <- function(y, lag = NULL, method = 1, plot = TRUE, ...)
{
  n <- length(y)             # data length
  if (is.null(lag)) {
    lag <- as.integer(2 * sqrt(n))    # highest order of AR model
  } else if (lag < 1) {
    stop("lag is not a positive integer.\n")
  }

# method                     # estimation procedure
  nf <- 200                  # number of frequencies for computing spectrum
# mj2                        # Adjustable dimension in Fortran
  if (method == 2) {
    mj2 <- n
    if (n > 50000)
      mj2 <- 2 * lag
  } else {
    mj2 <- 1
  }

  yy <- y
  ier <- 0
  if (anyNA(yy) == TRUE) {
    if (method == 1) {
      outmax <- 1.0e+30
      yy[is.na(yy)] <- outmax + 1
    } else {
      stop("Cannot handle the data with missing values.\n")
    }
  }

  z <- .Fortran(C_arfit,
                as.double(yy),
                as.integer(n),
                as.integer(lag),
                as.integer(nf),
                as.integer(mj2),
                as.integer(method),
			    sig = double(lag+1),
			    aic = double(lag+1),
		        mar = integer(1),
			    a = double(lag*lag),
			    par = double(lag),
			    sp = double(nf+1))

  mmin <- z$mar
  a <- array(z$a, dim = c(lag, lag))
  arcoef <- list()
  for (i in 1:lag)
    arcoef[[i]] <- a[1:i, i]

  arfit.out <- list(sigma2 = z$sig, maice.order = mmin, aic = z$aic,
                    arcoef = arcoef, parcor = z$par, spec = z$sp,
                    tsname = deparse(substitute(y)))

  class(arfit.out) <- "arfit"

  if (plot) {
    plot.arfit(arfit.out, ...)
    invisible(arfit.out)
  } else arfit.out

}

plot.arfit <- function(x, ...)
{
  lag <- length(x$aic) - 1
  morder <- x$maice.order
  aicmin <- x$aic[morder + 1]

  old.par <- par(no.readonly = TRUE)
  par(mfrow=c(2, 2), xaxs = "i", yaxs = "i")

  ylim <- c(-1, 1)
  xmax <- lag
  mtitle <- c(paste(x$tsname), "Parcor")
  plot(x$parcor, type = "l", xlim = c(0, xmax), ylim = ylim,
       main = mtitle, xlab = "", ylab = "", ...)
  par(new = TRUE)
  plot(x$parcor, type = "h", xlim = c(0, xmax), ylim = ylim, xlab = "lag",
       ylab = "", ...)

  daic <- x$aic - aicmin
  xx <- c(0:lag)
  ymax <- min(max(daic), 50)
  mtitle <- paste("\nminimum AIC = ", format(round(aicmin, 2), nsmall = 2),
                  "(at order", morder, ")")
  ylabel <- paste("aic - aicmin (Truncated at ", ymax, ")")
  if (ymax < quantile(daic, probs = 0.1))
    ymax <- as.integer(quantile(daic, probs = 0.1))
  plot(xx, daic, type = "l", ylim = c(0, ymax), main = mtitle, xlab = "Lag",
       ylab = ylabel, ...)
  par(new = TRUE)
  plot(xx, daic, type = "h", ylim = c(0, ymax), xlab = "", ylab = "", ...)

  nf1 <- length(x$spec)
  xx <- rep(0, nf1)
  for (i in 1:nf1)
    xx[i] <- (i - 1) / (2 * (nf1-1))
  ylim <- c(floor(min(x$spec)), ceiling(max(x$spec)))
  plot(xx, x$spec, type = "l", ylim = ylim, main = "\nPower spectrum",
       xlab = "f", ylab = "log p(f)", ...)

  par(old.par)
}

# PROGRAM 13.1  TVVAR
tvvar <- function(y, trend.order, tau2.ini = NULL, delta, plot = TRUE, ...)
{
  n <- length(y)             # length of data
  m <- trend.order           # trend order
# tau2.ini                   # initial variance of systen noise
# delta                      # delta for computing variance of system noise
  iopt <- 1                  # search method
  if (is.null(tau2.ini)) {
    iopt  <- 0
    tau2.ini <- 0
    delta <- 0
  }

  n2 <- n / 2
  
  z <- .Fortran(C_tvvarf,
             as.double(y),
             as.integer(n),
             as.integer(m),
             as.double(tau2.ini),
             as.integer(iopt),
             as.double(delta),
             tvvar = double(n2),
             nordat = double(n),
             y1 = double(n2),
             n1 = integer(1),
             trend = double(n2 * 3),
             noise = double(n2),
             taumax = double(1),
             sig2m = double(1),
             ffmax = double(1),
             aic = double(1))

  trend <- array(z$trend, dim = c(z$n1, 3))

  tvvar.out <- list(tvv = z$tvvar, nordata = z$nordat, sm = z$y1, trend = trend,
                    noise = z$noise, tau2 = z$taumax, sigma2 = z$sig2m,
                    llkhood = z$ffmax, aic = z$aic,
                    tsname = deparse(substitute(y)))
  class(tvvar.out) <- "tvvar"

  if (plot)
    plot.tvvar(tvvar.out, ...)
  return(tvvar.out)
}

print.tvvar <- function(x, ...)
{
  message(gettextf("\n tau2\t\t%12.5e", x$tau2), domain = NA)
  message(gettextf(" sigma2\t\t%12.5e", x$sigma2), domain = NA)
  message(gettextf(" log-likelihood\t%12.3f", x$llkhood), domain = NA)
  message(gettextf(" aic\t\t%12.3f\n", x$aic), domain = NA)
}

plot.tvvar <- function(x, ...)
{
    old.par <- par(no.readonly = TRUE)

    new.mar <- old.par$mar
    new.mar[1] <- 3.1
    new.mar[3] <- 1.1
    new.mgp <- old.par$mgp
    new.mgp[1] <- 2.25
    new.mgp[2] <- 0.75

    par(oma = c(0, 0, 2, 0), mfrow=c(4, 1), xaxs = "i", mar = new.mar,
        mgp = new.mgp)

    ylim <- c(floor(min(x$trend, x$sm)), ceiling(max(x$trend, x$sm)))
    plot(x$sm, type = "l", ylim = ylim, xlab = "", main = "",
         ylab = "transformed data", mar = new.mar)
    title(xlab = "m", mgp = c(2, 0.75, 0))

    plot(x$trend[, 1], type = "l", ylim = ylim, xlab = "", ylab = "trend t(m)") 
    par(new = TRUE) 
    plot(x$trend[, 2], type = "l", col=2, ylim = ylim, xlab = "", ylab = "")
    par(new = TRUE)
    plot(x$trend[, 3], type = "l", ylim = ylim, xlab = "", ylab = "") 
    title(xlab = "m", mgp = c(2, 0.75, 0))

    plot(x$noise, type = "h", xlab = "", ylab = "residual")
    par(new = TRUE)
    abline(h = 0)
    title(xlab = "m", mgp = c(2, 0.75, 0))

    ylim <- c(floor(min(x$nordat)), ceiling(max(x$nordat)))
    plot(x$nordat, type = 'h', ylim = ylim, xlab = "", ylab = "normalized data")
    abline(h=0)
    title(xlab = "m", mgp = c(2, 0.75, 0))

    mtext(paste(x$tsname), side = 3, line = 0, outer = TRUE)

    par(old.par)
}

# PROGRAM 13.2  TVAR
tvar <- function (y, trend.order = 2, ar.order = 2, span, outlier = NULL,
                  tau2.ini = NULL, delta, plot = TRUE)     
{
# y                     # original data
  n <- length(y)        # data length
# ar.order              # AR order
# trend.order           # Trend order
  if (trend.order != 1 && trend.order != 2)
    stop("'trend.order' is 1 or 2" )
# span                  # local stationary span
# outlier               # position of i-th outlier
  if (is.null(outlier)) {
    nout <- 0           # number of outliers
    outlier <- 0
  } else {
    nout <- length(outlier)
  }
# tau2.ini              # initial variance of systen noise
# delta                 # delta for computing variance of system noise
  iopt <- 1             # search method
  if (is.null(tau2.ini)) {
    iopt  <- 0
    tau2.ini <- 0
    delta <- 0
  }

  nn <- n/span
  
  z <- .Fortran(C_tvar,
                as.double(y),
                as.integer(n),
                as.integer(ar.order),
                as.integer(trend.order),
                as.integer(span),
                as.integer(iopt),
                as.integer(nout),
                as.integer(outlier),
                as.double(tau2.ini),
                as.double(delta),
                tau2 = double(1),
                sigma2 = double(1),
                lkhood = double(1),
                aic = double(1),
                arcoef = double(nn * ar.order),
                parcor = double(nn * ar.order))

  arcoef <- array(z$arcoef, dim = c(ar.order, nn))
  parcor <- array(z$parcor, dim = c(ar.order, nn))

  tvar.out <- list(arcoef = arcoef, sigma2 = z$sigma2, tau2 = z$tau2,
                   llkhood = z$lkhood, aic = z$aic, parcor = parcor)
  class(tvar.out) <- "tvvar"

  if (plot)
    plotParcor(parcor, span)

  return(tvar.out)
}

plotParcor <- function(parcor, span)
{
  ar.order <- dim(parcor)[1]
  n <- dim(parcor)[2]

  x <- span
  for (i in 2:n)
    x <- c(x, i*span)

  if (ar.order > 3)
    nc <- 3
  old.par <- par(no.readonly = TRUE)
  par(mfrow = c(nc, 1))

  for (i in 1:ar.order ) {
    if ((i%%3 == 1) & i > 1)
      par(ask = TRUE)
      plot(x, parcor[i, ], type = "l", xlab = "", ylim = c(-1.0, 1.0),
           ylab = paste("parcor[", i, ", ]"), xaxs = "i", yaxs = "i")
  }

  par(old.par)
}


# PROGRAM 13.3  TVSPC
tvspc <- function (arcoef, sigma2, var = NULL, span = 20, nf = 200)
{
# arcoef                        # Time varying AR coefficient
  n <- dim(arcoef)[2]           # number of points
  ar.order <- dim(arcoef)[1]    # AR order

  if (is.null(var)) {
    ivar <- 0
    var <- rep(0, n * span)     # time varying variance
  } else {
    ivar <- 1                   # =1: for variance correction
  }
# span                          # local stationary span
# nf                            # number of frequencies

  nf1 <- nf + 1
  
  z <- .Fortran(C_tvspc,
                 as.integer(n),
                 as.integer(ar.order),
                 as.integer(span),
                 as.integer(nf),
                 as.integer(ivar),
                 as.double(sigma2),
                 as.double(arcoef),
                 as.double(var),
                 spec = double(nf1 * n))

  spectra <- array(z$spec, dim = c(nf1, n))
  yy <- seq(0, n*span, length = n)

  out <- list(y = yy, z = spectra)
  class(out) <- c("tvspc")
  return(out)

}

# PROGRAM 13.3  TVSPC
plot.tvspc <- function(x, tvv = NULL, dx = 2, dy = 0.25, ...) {

  zoff <- FALSE

  if (dx < 0 || dx > 10)
    stop("0 <= dx <= 10")
  if (dy > 1 || dy < 0.05)
    stop("0.05 <= dy <= 1")

  nf1 <- dim(x$z)[1]
  nf <- nf1 - 1
  if (zoff == FALSE) {
    xoff <- 0
  } else {
    xoff <- nf/20
  }

  nrep <- dim(x$z)[2]
  zmax <- max(x$y)

  span <- zmax / nrep
  xw <- (nf + dx * nrep)
  xmax <- max(xw, (2.5*nf)) * 1.1

  z <- x$z
  if (is.null(tvv) == FALSE) {
    if ((length(tvv) /10) != nrep)
      stop("the length of time varying variance is invalid.")
    for (i in 1:nrep)
      z[, i] <- z[, i] + log10(tvv[i * 10])
  }
  yoff <- 1
  ymin <- trunc(min(z)) - yoff
  ymax0 <- ceiling(max(z))

  yw <- ymax0 + dy * nrep - ymin
  ymax <- yw * 1.1

  iexp <- 0
  zmax0 <- zmax
  while (zmax0 > 10) {
    zmax0 <- zmax0 / 10
    iexp <- iexp + 1
  }
  dz <- 10 ** iexp
  if (zmax0 > 8) {
    dz <- dz * 2
  } else if (zmax0 < 3) {
    while (zmax0 < 3) {
      dz <- dz / 2
      zmax0 <- zmax0 * 2
    }
  }

  x <- c(0:nf)
  y <- z[, 1]
  zz <- y

  old.par <- par(no.readonly = TRUE)
  par(bty = "n", xaxs = "i", yaxs = "i")
  new.mgp <- c(1.8, 0.5, 0)

# plot the spectrum
  plot(x, zz, type="l", xlim = c(0, xmax), ylim = c(ymin, ymax), xaxt = "n",
       yaxt = "n", xlab = "", ylab = "", ...)
  title(xlab = "f", adj = nf / 2 / xmax, mgp = new.mgp)
  title(ylab = "log p(f)", adj = (ymax0 - ymin) / 2 / ymax, , mgp = new.mgp)

  for (i in 2:nrep){
    par(new=T)
    x <- x + dx
    for (j in 1:nf1)
      zz[j] <- z[j,i] + dy*(i-1)
    for (j in 1:(nf-dx+1))
      zz[j] <- max(zz[j], y[j+dx])
    y <- zz
    plot(x, zz, type="l", xlim = c(0, xmax), ylim = c(ymin, ymax), xaxt = 'n',
         yaxt = "n", xlab = "", ylab = "", ...)
  }
  xend <- x[nf1]
  yend <- dy * nrep

# draw axes
  xlab <- c("0", "","","","", "0.5")
  axis(1, at = seq(0, nf, length = 6), labels = paste(xlab), tck = -0.01,
       cex.axis = 0.8, mgp = new.mgp)

  ylab <- c(ymin:ymax0)
  axis(2, at = ymin:ymax0, cex.axis = 0.8, mgp = new.mgp)

  zlab <- seq(0, zmax, by = dz)
  nz <- length(zlab)
  zlab.max <- zlab[nz]
  maxp <- zlab.max / span
  zlab <- as.character(zlab)
  segments(x0 = nf + xoff, y0 = ymin, x1 = xend + xoff, y1 = ymin + yend)

  ddx <- (dx * maxp) / (nz - 1)
  ddy <- (dy * maxp)  / (nz - 1)

  for (i in 1:nz) {
    xx <- nf + ddx * (i-1) + xoff
    yy <- ymin + ddy *(i-1)
    if (i != 1)
      text(xx + 5, yy, zlab[i], cex=0.8, pos=4)
    segments(xx, yy, xx + 5, yy, xpd = TRUE) 
  }
  text(xend + xoff + 5 , max(yend, yy+ddy/3), "n", cex = 0.8, pos = 4)

  par(bty = old.par$bty, xaxs = old.par$xaxs, yaxs = old.par$yaxs)
}

C     PROGRAM  8.1  LSAR1
      SUBROUTINE LSAR1( Y,N,LAG,NS0,NB,NF0,NNS,NN0,NN1,
c     * IIF,MS,SDS,AICS,MP,SDP,AICP,AS,MFS,SIG2S,NNF )
     * IIF,MS,SDS,AICS,MP,SDP,AICP,AS,MFS,SIG2S,NNF,IER )
C
C  ...  Decomposition of time interval to stationary subintevals  ...
C
C     Inputs:
C        LAG:     Highest order od AR model
C        NS:      Basic local span
C     The following inputs are required in the subroutine READTS.
C        TITLE:   Caption of the data set
C        N:       Data length
C        Y(I):    Time series, (i=1,...,N)
C     Parameters:
C        IDEV:    Input device for time series
C        NMAX:    Adjustable dimension of Y (NMAX.GE.N)
C        KMAX,MJ2:  Adjustable dimensions
C        NF:      Number of frequencies for computing spectrum
C
cc      PARAMETER( NMAX=3000,KMAX=20,MJ2=KMAX+1,MJ1=100,NF=100,IDEV=1)
cx      PARAMETER( MJ1=100 )
cxx      IMPLICIT   REAL*8(A-H,O-Z)
cc      DIMENSION  Y(NMAX), X(MJ1,MJ2), D(MJ1), U(MJ2,MJ2)
cc      DIMENSION  AS(KMAX), A(KMAX)
cx      DIMENSION  Y(N), X(MJ1,LAG+1), U(2*(LAG+1),LAG+1)
cxx      DIMENSION  Y(N), X(3*(LAG+1),LAG+1), U(LAG+1,LAG+1)
cxx      DIMENSION  AS(LAG,NB), A(LAG)
c
cxx      DIMENSION  NNS(NB), NN0(NB), NN1(NB), IIF(NB)
cxx      DIMENSION  MS(NB), SDS(NB), AICS(NB), MP(NB), SDP(NB), AICP(NB)
cxx      DIMENSION  MFS(NB), SIG2S(NB), NNF(NB)
C
      INTEGER N, LAG, NS0, NB, NF0, NNS(NB), NN0(NB), NN1(NB), IIF(NB),
     1        MS(NB), MP(NB), MFS(NB), NNF(NB), IER
      DOUBLE PRECISION Y(N), SDS(NB), AICS(NB), SDP(NB), AICP(NB),
     1                 AS(LAG,NB), SIG2S(NB)
c local
      INTEGER I, II, IF, L, LK, MF, MJ1, NF, NS, NBLOCK
      DOUBLE PRECISION X(3*(LAG+1),LAG+1), U(LAG+1,LAG+1), A(LAG),
     1                 SIG2, AICF
c
      EXTERNAL   SETXAR
C
C     ISW = 0
cc      READ( 5,* )  LAG, NS
      NF = NF0
      NS = NS0
      MJ1 = 3*(LAG+1)
C
C  ...  Read time series  ...
C
cc      CALL  READTS( IDEV,Y,N )
C
      IF = 0
      IIF(1) = 0
      AICF = 0
      IER = 0
C
      NBLOCK = N/NS
      DO 100 II=1,NBLOCK
C
      L = NS*(II-1)
cc      WRITE(6,600) L
      IF( II.EQ.NBLOCK )  NS = N - NS*(II-1) - LAG
      LK  = L + LAG
c
      NNS(II) = NS
      NN0(II) = LK + 1
      NN1(II) = LK + NS
C
C  ...  Locally stationary time series  ...
C
cc      CALL  LOCAL( SETXAR,Y,X,U,D,LAG,L,NS,LAG,IF,MJ1,MJ2,A,MF,SIG2)
cx      CALL  LOCAL( SETXAR,Y,X,U,LAG,NF,L,NS,LAG,IF,MJ1,
cx     *  A,MF,SIG2,MS(II),SDS(II),AICS(II),MP(II),SDP(II),AICP(II),AICF)
      CALL  LOCAL( SETXAR,Y,N,X,U,LAG,NF,L,NS,LAG,IF,MJ1,A,MF,SIG2,
     *  MS(II),SDS(II),AICS(II),MP(II),SDP(II),AICP(II),AICF, IER)
c 2023/01/11
      IF (IER .NE. 0) RETURN
      IIF(II) = IF
C
      NNF(II) = NF
C
cc      IF( IF .EQ. 2 )     LK0 = LK + 1
      IF( IF.EQ.2 .AND. II.GT.1 )  THEN
C         CALL  ARMASP( AS,MFS,B,0,SIG2S,NF,SP )
C         CALL  PTLSP( Y,N,SP,NF,LK0S,LK2S )
      END IF
cc      MFS = MF
cc      SIG2S = SIG2
      MFS(II) = MF
      SIG2S(II) = SIG2
cc      LK0S = LK0
cc      LK2S = LK + NS
      DO 20 I=1,MF
cc   20 AS(I) = A(I)
cxx   20 AS(I,II) = A(I)
      AS(I,II) = A(I)
   20 CONTINUE
C
  100 CONTINUE
C      CALL  ARMASP( AS,MFS,B,0,SIG2S,NF,SP )
C      CALL  PTLSP( Y,N,SP,NF,LK0S,LK2S )
C      CALL  PLOTE
C      call  plot( 0.0,0.0,999 )
C
cc      STOP
      RETURN
cxx  600 FORMAT( 1H ,'L =',I5 )
      E N D
cc      SUBROUTINE  LOCAL( SETX,Z,X,U,D,LAG,N0,NS,K,IF,MJ1,MJ2,
cc     *                   A,MF,SDF )
cx      SUBROUTINE  LOCAL( SETX,Z,X,U,LAG,NF,N0,NS,K,IF,MJ1,
cx     *   A,MF,SDF,MS,SDS,AICS,MP,SDP,AICP,AICF)
      SUBROUTINE  LOCAL( SETX,Z,N,X,U,LAG,NF,N0,NS,K,IF,MJ1,
     *   A,MF,SDF,MS,SDS,AICS,MP,SDP,AICP,AICF,IER)
C
C  ...  Locally stationary AR model  ...
C
C     Inputs:
C        SETX:    Name of the subroutine for making X(I,J)
C        Z(I):    Data vector
C        D,U:     Working area
C        LAG:     Highest order of the model
C        N0:      Time point of the previous set ofobservations
C        NS:      Number of new observations
C        MJ1,MJ2: Adjustable dimension
C     Output:
C        A(I):    AR coefficients of the current model
C        MF:      Order of the current model
C        SDF:     Innovation variance of the current model
C
cxx      IMPLICIT  REAL*8 (A-H,O-Z)
cx      DIMENSION  Z(1)
cc      DIMENSION  X(MJ1,1), U(MJ2,1), D(1), A(1)
cx      DIMENSION  X(MJ1,K+1), U(2*(LAG+1),LAG+1), A(LAG)
cc      DIMENSION  B(20), AA(20,20), AIC(0:20), SIG2(0:20)
cxx      DIMENSION  Z(N)
cxx      DIMENSION  X(MJ1,K+1), U(LAG+1,LAG+1), A(LAG)
cxx      DIMENSION  B(LAG), AA(LAG,LAG), AIC(0:LAG), SIG2(0:LAG)
C
      INTEGER N, LAG, NF, N0, NS, K, IF, MJ1, MF, MS, MP, IER
      DOUBLE PRECISION Z(N), X(MJ1,K+1), U(LAG+1,LAG+1), A(LAG), SDF,
     1                 SDS, AICS, SDP, AICP, AICF
c local
      INTEGER I, K1, K2, L, MJ2, NN0, NN1, NP
      DOUBLE PRECISION B(LAG), AA(LAG,LAG), AIC(0:LAG), SIG2(0:LAG),
     1                 AIC0
C
      EXTERNAL   SETX
C
      K1 = K + 1
      K2 = K1*2
C
cx      MJ2 = K2
      MJ2 = K1
C
      NN0 = N0 + LAG + 1
      NN1 = N0 + LAG + NS
cc      WRITE(6,600)  NN0, NN1
C
cc      CALL  REDUCT( SETX,Z,D,NS,N0,K,MJ1,X )
cc      CALL  REGRES( X,K,NS,MJ1,20,AA,SIG2,AIC,MS )
c 2023/01/11
      L = MIN0(NS, MJ1)
      if (L .GE. K+1) THEN

      CALL  REDUCT( SETX,Z,NS,N0,K,MJ1,X )
      CALL  REGRES( X,K,NS,MJ1,AA,SIG2,AIC,MS )
C
      SDS = SIG2(MS)
      DO 10 I=1,MS
cxx   10 B(I) = AA(I,MS)
      B(I) = AA(I,MS)
   10 CONTINUE
      IF( IF.EQ.0 )  THEN
      CALL  COPY( X,K1,0,0,MJ1,MJ2,U )
      AICS = AIC(MS)
      AIC0 = AIC(MS)
cc      WRITE(6,610)  NS, MS, SDS, AICS
c-----
      AICP = 0.0d0
      SDP = 0.0d0
c-----
      ELSE
C
      AICS = AIC(MS) + AICF
      AIC0 = AIC(MS)
cc      WRITE(6,620)  NF, NS, MS, SDS, AICS
      CALL  COPY( X,K1,0,K2,MJ1,MJ1,X )
      CALL  COPY( U,K1,0,K1,MJ2,MJ1,X )
cc      CALL  HUSHLD( X,D,MJ1,K2,K1 )
      CALL  HUSHLD( X,MJ1,K2,K1 )
      NP = NF + NS
cc      CALL  REGRES( X,K,NP,MJ1,20,AA,SIG2,AIC,MP )
      CALL  REGRES( X,K,NP,MJ1,AA,SIG2,AIC,MP )
C
      AICP = AIC(MP)
      SDP  = SIG2(MP)
      DO 20 I=1,MP
cxx   20 A(I) = AA(I,MP)
      A(I) = AA(I,MP)
   20 CONTINUE
cc      WRITE(6,630)  NP, MP, SDP, AICP
      IF( AICS.GE.AICP )  GO TO 40
cc      WRITE(6,640)
      CALL  COPY( X,K1,K2,0,MJ1,MJ2,U )
      END IF
C
      IF = 2
      NF = NS
      MF = MS
      AICF = AIC0
      DO 30  I=1,MF
cxx   30 A(I) = B(I)
      A(I) = B(I)
   30 CONTINUE
      SDF = SDS
C
      GO TO 50
   40 IF = 1
      CALL  COPY( X,K1,0,0,MJ1,MJ2,U )
cc      WRITE(6,650)
      SDF = SDP
      MF = MP
      AICF = AICP
      NF = NF + NS
   50 CONTINUE
C
      ELSE
         IER = -1
      END IF
      RETURN
cxx  600 FORMAT( 1H0,'<<< NEW DATA (N =',I4,' ---',I4,')  >>>' )
cxx  610 FORMAT( 1H ,'  INITIAL MODEL: NS =',I4,/,10X,'MS =',I2,3X,
cxx     1  'SDS =',D13.6,3X,'AICS =',F12.3 )
cxx  620 FORMAT( 1H ,'  SWITCHED MODEL: (NF =',I4,', NS =',I4,1H),
cxx     2  /,10X,'MS =',I2,3X,'SDS =',D13.6,3X,'AICS =',F12.3 )
cxx  630 FORMAT( 1H ,'  POOLED MODEL:   (NP =',I4,1H),
cxx     3  /,10X,'MP =',I2,3X,'SDP =',D13.6,3X,'AICP =',F12.3 )
cxx  640 FORMAT( 1H ,30X,'***  SWITCHED MODEL ACCEPTED  ***' )
cxx  650 FORMAT( 1H ,30X,'***   POOLED MODEL ACCEPTED   ***' )
      E N D
      SUBROUTINE  COPY( X,K,II,JJ,MJ1,MJ2,Y )
C
C  ...  Make a copy of X on Y
C
cxx      IMPLICIT  REAL*8( A-H,O-Z )
cc      DIMENSION  X(MJ1,1) , Y(MJ2,1)
cxx      DIMENSION  X(MJ1,K) , Y(MJ2,K)
C
      INTEGER K, II, JJ, MJ1, MJ2 
      DOUBLE PRECISION X(MJ1,K) , Y(MJ2,K)
c local
      INTEGER I, I1, I2, J
C
cxx      DO 10  I=1,K
      DO 11  I=1,K
      I1 = I + II
      I2 = I + JJ
      DO 10  J=1,K
cxx   10 Y(I2,J) = X(I1,J)
      Y(I2,J) = X(I1,J)
   10 CONTINUE
   11 CONTINUE
C
      RETURN
      E N D

# PROGRAM 10.1  ARMAFT
armafit <- function (y, ar.order, ar = NULL, ma.order, ma = NULL)
{
  n <- length(y)             # length of data
  m <- ar.order
  l <- ma.order
  mlmax <- max(m, l+1)
  if (is.null(ar) || is.null(ma)) {
    iparam <- 0
    ar <- rep(0, m)
    ma <- rep(0, l)
  } else {
    iparam <- 1
  }

  yy <- y
  if (anyNA(yy) == TRUE) {
    outmax <- 1.0e+30
    yy[is.na(yy)] <- outmax + 1
  }

  z <- .Fortran(C_armaft,
                as.double(yy),
                as.integer(n),
                as.integer(m),
                as.integer(l),
                as.integer(mlmax),
                as.integer(iparam),
                as.double(ar),
                as.double(ma),
                sig2 = double(1),
                flk = double(1),
                aic = double(1),
                ar = double(m),
                ma = double(l),
                ier = integer(1))

  ier <- z$ier
  if (ier == 1) {
    stop(" Matrix with zero row in decompose" )
  } else if (ier == 2) {
    stop(" Singular matrix in decompose.zero divide in solve" )
  } else if (ier == 3) {
    stop(" Convergence in impruv.matrix is nearly singular" )
  } else if (ier == -1) {
    stop(gettextf(" The model of the order (%d , %d) cannot be estimated", m,l), domain = NA )
  }

  armafit.out <- list(sigma2 = z$sig2, llkhood = z$flk, aic = z$aic,
                      arcoef = z$ar, macoef = z$ma)
  class(armafit.out) <- "armafit"

  return(armafit.out)
}

print.armafit <- function(x, ...)
{
  message(gettextf("\n sigma2\t\t%12.5e", x$sigma2), domain = NA)
  message(gettextf(" log-likelihood\t%12.3f", x$llkhood), domain = NA)
  message(gettextf(" aic\t\t%12.3f", x$aic), domain = NA)

  m <- length(x$arcoef)
  if (m > 0) {
    message("\n AR coefficients")
    for (i in 1:m)
      message(gettextf(" %12.3f\t", x$arcoef[i]), appendLF=FALSE, domain = NA)
    message("")
  }

  l <- length(x$macoef)
  if (l > 0) {
    message("\n MA coefficients")
    for (i in 1:l)
      message(gettextf(" %12.3f\t", x$macoef[i]), appendLF=FALSE, domain = NA)
    message("")
  }
}

#------------------------------------------------------------------------------

armafit2 <- function (y, ar.order, ma.order)
{
  n <- length(y)             # length of data
  mmax <- ar.order
  lmax <- ma.order
  mlmax <- max(mmax, lmax, 1) + 1

  yy <- y
  if (anyNA(yy) == TRUE) {
    outmax <- 1.0e+30
    yy[is.na(yy)] <- outmax + 1
  }

  mmax1 = mmax + 1;
  lmax1 = lmax + 1;
	
  z <- .Fortran(C_armaft2,
                as.double(yy),
                as.integer(n),
                as.integer(mmax),
                as.integer(lmax),
                as.integer(mlmax),
                sig2 = double(mmax1*lmax1),
                flk = double(mmax1*lmax1),
                aic = double(mmax1*lmax1),
                ar = double(mmax*mmax1*lmax1),
                ma = double(lmax*mmax1*lmax1),
                ier = integer(3))

  ier <- z$ier
  if (ier[1] == 1)
    stop(" Matrix with zero row in decompose" )
  if (ier[1] == 2)
    stop(" Singular matrix in decompose.zero divide in solve" )
  if (ier[1] == 3)
    stop(" Convergence in impruv.matrix is nearly singular" )
  if (ier[1] == -1) {
    m <- ier[2]
    l <- ier[3]
    stop(gettextf(" The model of the order (%d , %d) cannot be estimated", m,l), domain = NA )
  }

  sig2 <- array(z$sig2, c(mmax1, lmax1))
  flk <- array(z$flk, c(mmax1, lmax1))
  aic <- array(z$aic, c(mmax1, lmax1))
  ar <- array(z$ar, c(mmax, mmax1, lmax1))
  ma <- array(z$ma, c(lmax, mmax1, lmax1))
  ar <- aperm(ar, c(2, 3, 1))
  ma <- aperm(ma, c(2, 3, 1))

  mm1 <- mmax + 1
  ll1 <- lmax + 1
  coef <- list()
  length(coef) <- mm1
  for (i in 1:mm1) {
    coef[[i]] <- list()
    length(coef[[i]]) <- ll1
    for (j in 1:ll1) {
      if (i == 1) {
        arcoef <- NULL
      } else {
        arcoef <- ar[i, j, 1:(i-1)]
      }   
      if (j == 1) {
        macoef <- NULL
      } else {
        macoef <- ma[i, j, 1:(j-1)]
      }
      coef[[i]][[j]] <- list(ar = arcoef, ma = macoef)
    }
 }

  aic.min <- min(aic, na.rm=T)
  morder <- which(aic == aic.min, arr.ind=TRUE)
  m <- morder[1]
  l <- morder[2]
  m1 <- m - 1
  l1 <- l - 1

  armafit2.out <- list(aicmin = aic.min, maice.order = c(m1, l1), sigma2 = sig2,
                     llkhood = flk, aic = aic, coef = coef)

  class(armafit2.out) <- "armafit2"
  return(armafit2.out)
}

print.armafit2 <- function(x, ...)
{
  mmax <- dim(x$sigma2)[1] - 1
  lmax <- dim(x$sigma2)[2] - 1

  sigma2 <- x$sigma2
  llkhood <- x$llkhood
  aic <- x$aic
  rownames(sigma2) <- c(0:mmax)
  colnames(sigma2) <- c(0:lmax)
  rownames(llkhood) <- c(0:mmax)
  colnames(llkhood) <- c(0:lmax)
  rownames(aic) <- c(0:mmax)
  colnames(aic) <- c(0:lmax)

  m <- x$maice.order[1]
  l <- x$maice.order[2]
  m1 <- m + 1
  l1 <- l + 1

  message(gettextf("\n minimum AIC %12.3f at order( %d, %d )", x$aicmin, m, l),
          domain = NA)
  if (m > 0) {
    message(" AR coefficients")
    arcoef <- x$coef[[m1]][[l1]]$ar
    for (i in 1:m) {
      message(gettextf(" %12.3f\t", arcoef[i]), appendLF=FALSE, domain = NA)
      if ((i %% 5 == 0) || i == m)
        message("")
    }
  }
  if (l > 0) {
    message(" MA coefficients")
    macoef <- x$coef[[m1]][[l1]]$ma
    for (i in 1:l) {
      message(gettextf(" %12.3f\t", macoef[i]), appendLF=FALSE, domain = NA)
      if ((i %% 5 == 0) || i == l)
        message("")
    }
  }
  message(gettextf("\n sigma2(m,l) (0 <= m <= %d, 0 <= l <= %d)", mmax, lmax), domain = NA)
  message(paste0(capture.output(sigma2), collapse = "\n"))
  message(gettextf("\n llkhood(m,l) (0 <= m <= %d, 0 <= l <= %d)", mmax, lmax), domain = NA)
  message(paste0(capture.output(llkhood), collapse = "\n"))
  message(gettextf("\n aic(m,l) (0 <= m <= %d, 0 <= l <= %d)", mmax, lmax), domain = NA)
  message(paste0(capture.output(aic), collapse = "\n"))
  message("")
}

*     particle_smoother_nonlinear.f         JUl.12,2019   For nonlinear smoothing
*     particle_smoother.f       for R function  JUl.12,2019
*     Monte-S.f   8/13/2002     Small memory version   
*     monte.f     12/06/2001    
C     @MONTE.JCGS.FIG32:                    JUN.  ,1995
C     @NFILTER.SIM4:    FORWARD SMOOTHING   NOV.20,1992
C     @NFILTER.SIM3:    SMOOTHING           NOV.19,1992
C     @NFILTER.SIM2:    MONTE CARLO METHOD  NOV.02,1992
C
      SUBROUTINE PFILTERN(Y,N,M,LAG,SIG2,TAU2,XMIN,XMAX,IX,T,FF)
C
cc      PARAMETER( NMAX=500,IDEV=1,MM=1000000, LAG=20 )
cc      IMPLICIT REAL*8(A-H,O-Z)
cc      real*4  PS, PST
cc      DIMENSION  Y(NMAX), T(NMAX,8), PS(MM,0:LAG), PST(MM,0:LAG)
cc      CHARACTER   DATE*8, TIME2*10
      INTEGER N, M, LAG, IX
      DOUBLE PRECISION Y(N), SIG2, TAU2, XMIN, XMAX, T(N,8), FF
c local
cdebug      INTEGER IST, L
      INTEGER L
ccc      DOUBLE PRECISION PS(M,0:LAG), PST(M,0:LAG)
      DOUBLE PRECISION, allocatable :: PS(:,:), PST(:,:)
      allocate(PS(M,0:LAG))
      allocate(PST(M,0:LAG))
cc      COMMON  /INIT/ IX
ccdebug      common  /COMIST/  IST
cc      IX = 1992110113
ccdebug      IST = 0
C
cc      MODEL = 1
*           M is the number of particles. 1000 - 100000
*           LAG:  Fixed lag smoother LAG=0 => filter
*           IX:  initial seed for random number generator
cc      M = 1000000
cc      M = 100000
      L = 1
cc      INID = 0
*      
cc      IX = 2019071117
C      IX = 2002081119
*      IX = 1997102221
cc      IX0= IX
*
*     sig2  observation noise variance
*     tau2: system noise variance
cc      SIG2 = 10.0D0
cc      TAU2 =  1.0D0
*
*      READTS reads in data: change this subroutine
*
cc      CALL  READTS( IDEV,Y,N )
cc      WRITE(6,600)  MODEL, M, L, IX, INID
cc  600 FORMAT( 1H ,'MODEL =',I2,3X,'M =',I6,3X,'L =',I3,3X,'IX =',I12,
cc     *            3X,'INID =',I3 )
C
cc      CALL  DATE_AND_TIME( DATE,TIME2 )     
cc      WRITE(6,650)  DATE, TIME2      
cc  650 FORMAT( 'Date',2X,A8,3X,'Time:',2X,A10 )  

cc      CALL  FILTER( Y,N,M,L,T,PS,PST,MODEL,SIG2,TAU2,LAG,NMAX,FF,MM,
cc     *              INID,IX )
      CALL  FILTERNL( Y,N,M,L,T,PS,PST,SIG2,TAU2,LAG,XMIN,XMAX,IX,FF)
cc      CALL  DATE_AND_TIME( DATE,TIME2 )     
cc      WRITE(6,650)  DATE, TIME2      
C
cc      open( 6,file='results.dat' )
cc      CALL  PTDATA( Y,T,N,M,L,MODEL,SIG2,TAU2,IX0,LAG,NMAX,FF )
cc      close( 6 )
C
cc      STOP
      deallocate(PS, PST)
      return
      E N D
cc      SUBROUTINE  FILTER( Y,N,M,L,T,PS,PST,MODEL,SIG2,TAU2,LAG,NMAX,FF,
cc     *                    MM, INID )
      SUBROUTINE  FILTERNL(Y,N,M,L,T,PS,PST,SIG2,TAU2,LAG,XMIN,XMAX,IX,
     *                     FF)
cc      IMPLICIT REAL*8(A-H,O-Z)
cc      real*4  PS, PST
cc      DIMENSION  Y(N), F(1000000), P(1000000), S(1000000)
cc      DIMENSION  PS(MM,0:LAG), T(NMAX,8), PST(MM,0:LAG)
cc      DIMENSION  TT(7)
      INTEGER N, M, L, LAG, IX
      DOUBLE PRECISION Y(N), T(N,8), PS(M,0:LAG), PST(M,0:LAG), SIG2,
     *                 TAU2, XMIN, XMAX, FF
c local
      INTEGER I, II, IJ, I0, J, JJ
      DOUBLE PRECISION TAU, F(M), P(M), S(M), TT(7), RNOR, RR, SUM,
ccx     *           RUNIFT
     *                 GAUS2, random 
cc      COMMON  /INIT/ IX
c----- Initialize Mersenne Twister with given seed value
      call init(IX)
c-----
*
      TAU = DSQRT( TAU2 )
      FF = 0.0D0
*       Specify initial distribution : you can change these
      DO 10 I=1,M
cc   10 F(I) = RNOR( 1.0D0 )*DSQRT(5.0D0)
      F(I) = RNOR( 1.0D0 )*DSQRT(5.0D0)
   10 CONTINUE
*
*    Monte Carlo filtering   N is data length
*
      DO 100 II=1,N
*
*     Prediction
*
      I0 = min0( ii,lag )

      DO 20 I=1,M
      P(I) = F(I)/2 + 25*F(I)/(1+F(I)**2) + 8*DCOS(1.2D0*II)
cc     *       + RNOR( 1.0D0 )*TAU
     *       + RNOR( 1.0D0 )*TAU
   20 CONTINUE
C
*     Filtering
*
*   GAUS2:  Gaussian density, change this if you want other distribution
      SUM = 0.0D0
      DO 30 I=1,M*L
      S(I) = GAUS2( Y(II)-P(I)**2/20.0D0,SIG2 )
cc   30 SUM = SUM + S(I)
      SUM = SUM + S(I)
   30 CONTINUE
*
*     FF: log-likelihood
*
      FF = FF + DLOG( SUM/M )
cc   35 CONTINUE
*
*     Re-sampling
*
      S(1) = S(1)/SUM
      DO 40 I=2,M
cc   40 S(I) = S(I-1) + S(I)/SUM
      S(I) = S(I-1) + S(I)/SUM
   40 CONTINUE
C  
      JJ = 1
      DO 50 I=1,M
ccx      RR = (I-1+RUNIFT(IX))/M
      RR = (I-1+random())/M
      DO 60 J=JJ,M*L
      IF( S(J).GE.RR )  GO TO 55
   60 CONTINUE
cc   55 IF( J.GE.M )  J = M
   55 CONTINUE
      IF( J.GE.M )  J = M
      F(I) = P(J)
      JJ = J
      I0 = min0( ii,lag )
      DO 56 IJ=1,I0
cc   56 PST(I,IJ) = PS(J,IJ-1)
      PST(I,IJ) = PS(J,IJ-1)
   56 CONTINUE
   50 CONTINUE
*
cc      DO 65 IJ=1,I0
      DO 66 IJ=1,I0
      DO 65 I=1,M
cc   65 PS(I,IJ) = PST(I,IJ)
      PS(I,IJ) = PST(I,IJ)
   65 CONTINUE
   66 CONTINUE
C
      DO 70 I=1,M
cc   70 PS(I,0) = F(I)
      PS(I,0) = F(I)
   70 CONTINUE
cc   90 CONTINUE

      IF( II.LE.LAG )  GO TO 100
      do 320 I=1,8
cc  320 T(II-LAG,I) = 0.0d0
      T(II-LAG,I) = 0.0d0
  320 CONTINUE

cc      CALL  DENSTY( PS(1,LAG),M,TT )
      CALL  DENSTY1( PS(1,LAG),M,TT,XMIN,XMAX )

      DO 310 I=1,7
cc  310 T(II-LAG,I) = TT(I)
      T(II-LAG,I) = TT(I)
  310 CONTINUE

  100 CONTINUE
*
*     Compute Posterior Distributions
*     T(II,J), J=4: MEdian, other: 1,2,3 sigma interval
*              J=8  MEAN
*
cc      WRITE(6,660)  TAU2, SIG2, FF
cc  660 FORMAT( 1H ,F15.8,F10.4,F13.4 )
      do 200 II=0,LAG-1
cc      CALL  DENSTY( PS(1,II),M,TT )
      CALL  DENSTY1( PS(1,II),M,TT,XMIN,XMAX )
      DO 210 I=1,7
cc  210 T(N-II,I) = TT(I)
      T(N-II,I) = TT(I)
  210 CONTINUE
*      DO 110 I=1,M
*  110 SUM = SUM + PS(I,II)
*      T(II,8) = SUM/M
  200 CONTINUE
C
      RETURN
      E N D

C     PROGRAM 4.1  DENSTY
      SUBROUTINE DENSTYF( MODEL, PARAM, XMIN, XMAX, K, F ) 
C
C  ...  This program draws probability density function  ...
C
C     The following inputs are required in the main program:
C        MODEL:   function number (1: GAUSS, 2: CAUCHY, etc.)
C        XMIN:    lower bound of the interval
C        XMAX:    upper bound of the interval
C        PARAM:   parameter vector
C     @TEST.PN41: DEC.26,1990, 8/6/91
C
cc      PARAMETER( K=201 )
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      CHARACTER*24   TITLE1(0:7)
cc      CHARACTER*72   TITLE
cxx      DIMENSION  F(K), PARAM(3), NP(0:7)
cc      COMMON  /CMDATA/  TITLE
C
      INTEGER MODEL, K
      DOUBLE PRECISION PARAM(3), XMIN, XMAX, F(K)
c local
      INTEGER NP(0:7)
      DOUBLE PRECISION USERF, GAUSS, CAUCHY, PEARSN, EXPNTL, CHISQR,
     1                 DBLEXP, UNIFRM
C
      EXTERNAL  USERF
      EXTERNAL  GAUSS
      EXTERNAL  CAUCHY
      EXTERNAL  PEARSN
      EXTERNAL  EXPNTL
      EXTERNAL  CHISQR
      EXTERNAL  DBLEXP
      EXTERNAL  UNIFRM
C
cc     DATA TITLE1/'USER SUPPLIED FUNCTION  ','NORMAL DISTRIBUTION     '
cc    *          ,'CAUCHY DISTRIBUTION     ','PEARSON DISTRIBUTION    '
cc    *          ,'EXPONENTIAL DISTRIBUTION','CHI-SQUARE DISTRIBUTION '
cc    *          ,'DOUBLE EXPONENTIAL DIST.','UNIFORM DISTRIBUTION    '/
      DATA  NP/0,2,2,3,1,1,0,2/
C
cc      WRITE(6,*)  'INPUT MODEL NUMBER ?'
cc      READ(5,*)   MODEL
cc      WRITE(6,*)  'INPUT XMIN AND XMAX ?'
cc      READ(5,*)   XMIN, XMAX
cc      IF( MODEL.EQ.0 )  THEN
cc         WRITE(6,*)  'INPUT THE NUMBER OF PARAMETERS'
cc         READ(5,*)   NP(0)
cc      END IF
cc      WRITE(6,*)  'INPUT PARAM(I),I=1,',NP(MODEL),' ?'
cc      READ(5,*)   (PARAM(I),I=1,NP(MODEL))
cc      TITLE = TITLE1(MODEL)
cc      DO 10 I=1,6
cc   10 TITLE = TITLE//'        '
C
      IF(MODEL.EQ.1)  CALL  DENSTY( GAUSS ,F,K,PARAM,XMIN,XMAX )
      IF(MODEL.EQ.2)  CALL  DENSTY( CAUCHY,F,K,PARAM,XMIN,XMAX )
      IF(MODEL.EQ.3)  CALL  DENSTY( PEARSN,F,K,PARAM,XMIN,XMAX )
      IF(MODEL.EQ.4)  CALL  DENSTY( EXPNTL,F,K,PARAM,XMIN,XMAX )
      IF(MODEL.EQ.5)  CALL  DENSTY( CHISQR,F,K,PARAM,XMIN,XMAX )
      IF(MODEL.EQ.6)  CALL  DENSTY( DBLEXP,F,K,PARAM,XMIN,XMAX )
      IF(MODEL.EQ.7)  CALL  DENSTY( UNIFRM,F,K,PARAM,XMIN,XMAX )
      IF(MODEL.EQ.0)  CALL  DENSTY( USERF ,F,K,PARAM,XMIN,XMAX )
C      CALL  PTDENS( F,K,XMIN,XMAX,PARAM,NP(MODEL) )
cc      CALL  PRDENS( F,K )
cc      STOP
      RETURN
cxx  600 FORMAT( 1H ,10F8.4 )
      E N D
      SUBROUTINE  DENSTY( DIST,P,K,PARAM,XMIN,XMAX )
C
C  ...  This subroutine evaluates values of density  ...
C               DIST(X), X=XMIN,...,XMAX
C     Inputs:
C        DIST:    name of function
C        PARAM:   parameters of the density
C        XMIN:    minimum of X
C        XMAX:    maximum of X
C        K:       number of location, I-th location is XMIN + I*DX
C                 where  DX = (I-1)*(XMAX-XMIN)/(K-1)
C     OUTPUT:
C        P(I):    density of DIST at I-th location
C
cxx      IMPLICIT REAL*8( A-H,O-Z )
cx      DIMENSION  P(K), PARAM(*)
cxx      DIMENSION  P(K), PARAM(3)
C
      INTEGER K
      DOUBLE PRECISION P(K), PARAM(3), XMIN, XMAX
c local
      INTEGER I
      DOUBLE PRECISION DX, X, DIST
C
      EXTERNAL  DIST
C
      DX = (XMAX-XMIN)/(K-1)
      DO 10 I=1,K
      X = XMIN + DX*(I-1)
cxx   10 P(I) = DIST( X,PARAM )
      P(I) = DIST( X,PARAM )
   10 CONTINUE
      RETURN
      E N D
      DOUBLE PRECISION FUNCTION  EXPNTL( X,PARAM )
C
C  ...  Exponential  distribution  ...
C
C     Inputs:
C        X:
C        PARAM(1):  lambda
C     Output:
C        EXPNTL:    density at X
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  PARAM(1)
      DOUBLE PRECISION X, PARAM(1)
C
      EXPNTL = 0.0D0
      IF( X.GE.0.0D0 )  EXPNTL = PARAM(1)*DEXP( -PARAM(1)*X )
cxx      IF( X.LT.0.0D0 )  EXPNTL = 0.0D0
      RETURN
      E N D
      DOUBLE PRECISION FUNCTION  CHISQR( X,PARAM )
C
C  ...  Chi-square  distributions  ...
C
C     Inputs:
C        X:
C        PARAM(1):  degree of freedoms, k
C     Output:
C        CHISQR:    density at X
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cx      DIMENSION  PARAM(*)
cxx      DIMENSION  PARAM(1)
      DOUBLE PRECISION X, PARAM(1), dgammafn
C
      CHISQR = 0.0D0
      IF( X.GT.0.0D0 ) CHISQR = DEXP( -X/2 )*(X/2)**(PARAM(1)/2-1.D0)
CXX     *                           /(2*DGAMMA(PARAM(1)/2))
     *                           /(2*dgammafn(PARAM(1)/2))
cxx      IF( X.LE.0.0D0 ) CHISQR = 0.0D0
      RETURN
      E N D
      DOUBLE PRECISION FUNCTION  UNIFRM( X,PARAM )
C
C  ...  Uniform distribution on [a,b]  ...
C
C     Inputs:
C        X:
C        PARAM(1):  a
C        PARAM(2):  b
C     Output:
C        UNIFRM:    density at X
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cxx      DIMENSION  PARAM(2)
      DOUBLE PRECISION X, PARAM(2)
C
      IF( X.GT.PARAM(1) .AND. X.LE.PARAM(2) )  THEN
         UNIFRM = 1.0D0/(PARAM(2)-PARAM(1))
      ELSE
         UNIFRM = 0.0D0
      END IF
      RETURN
      E N D
      DOUBLE PRECISION FUNCTION  USERF( X,PARAM )
C
C  ...  User supplied density function  ...
C       (The following is an example of two-sided exponential dist.)
C
C     Inputs:
C        X:
C        PARAM(1):  lambda
C     Output:
C        USERF:     density at X
C
cxx      IMPLICIT  REAL*8(A-H,O-Z)
cx      DIMENSION  PARAM(2)
cxx      DIMENSION  PARAM(1)
      DOUBLE PRECISION X, PARAM(1)
C
      IF( X.GE.0.0D0 )  THEN
         USERF = PARAM(1)*DEXP( -PARAM(1)*X )/2
      ELSE
         USERF = PARAM(1)*DEXP(  PARAM(1)*X )/2
      END IF
      RETURN
      E N D

# PROGRAM 5.5 LSQR
lsqr <- function(y, lag = NULL, period = 365, plot = TRUE, ...)
{
  n <- length(y)                 # length of data
  if (is.null(lag))
    lag <- as.integer(sqrt(n))   # number of sine and cosine terms
  if (as.integer(period/2) < lag) {
    lag <- as.integer(period/2)
    warning(gettextf("lag is corrected to %d. \n
            Maximum number of lag should be less than or equal to floor(period/2).", lag), domain = NA)
  }
  k <- 2 * lag + 1

  mj <- n
  if (n > 50000) mj <- 2 * k

  z <- .Fortran(C_lsqr,
                as.double(y),
                as.integer(n),
                as.integer(k),
                as.integer(period),
                as.integer(mj),
                aic = double(k + 1),
                sig2 = double(k + 1),
                imin = integer(1),
                reg = double(k * k),
                data = double(n))

  a <- array(z$reg, dim = c(k, k))
  reg <- list()
  for (i in 1:k)
    reg[[i]] <- a[1:i, i]

  lsqr.out <- list(aic = z$aic, sigma2 = z$sig2, maice.order = z$imin,
                   regress = reg, tripoly = z$data)
  class(lsqr.out) <- "lsqr"

  if (plot) {
    rdata <- deparse(substitute(y))
    eval(parse(text=paste(rdata, "<- y")))
    eval(parse(text=paste("plot.lsqr(lsqr.out,", rdata, ", ...)")))
  }

  lsqr.out
}

print.lsqr <- function(x, ...)
{
  aic <- x$aic
  nl <- length(aic)
  sig2 <- x$sigma2
  morder <- x$maice.order
  aicmin <- aic[morder+1] 

  message("\n Order     Sigma2            AIC         AIC-AICMIN")
  for (i in 1:nl) 
    message(gettextf("%5i  %13.6e   %12.3f   %12.3f", i-1, sig2[i], aic[i],
            aic[i] - aicmin), domain = NA)

  message(gettextf("\n Minimum AIC = %12.3f   attained at m = %5i", aicmin,
          morder), domain = NA)
  message(gettextf(" Sigma2 = %13.6e\n", sig2[morder+1]), domain = NA)
}

plot.lsqr <- function(x, rdata = NULL, ...)
{
  ts.atr <- tsp(rdata)
  data <- x$tripoly
  if (is.null(ts.atr) == FALSE)
    data <- ts(data, start = ts.atr[1], frequency = ts.atr[3])
  imin <- x$maice.order

  if (is.null(rdata) == FALSE) {
    ylim <- c(floor(min(data, rdata)), ceiling(max(data, rdata)))
    tsname <- deparse(substitute(rdata))
    mtitle <-
      paste(tsname, "\nand regression curve of the model with order", imin)
    plot(rdata, type = "l", ylim = ylim, xlab = "", ylab = "", xaxs = "i", ...)
    par(new = TRUE) 
  } else {
    ylim <- c(floor(min(data)), ceiling(max(data)))
    mtitle <-
      paste("Rregression curve of the model with order",imin)
  }
  plot(data, type = "l", ylim = ylim, col=2, xlab = "", ylab = "",
       main = mtitle, xaxs = "i", cex.main = 1.0, lwd = 2, ...)

}

C     PROGRAM  4.4   BOXCOX
      SUBROUTINE BOXCOXF(Y, N, AICZT, FFZT, AICZ, FFZ, ZMEAN, ZVAR, ZZ)
C
C  ...  Box-Cox transformation
C
C     The following inputs are required in READTS
C        TITLE:   title of the data set
C        N:       data length
C        Y(I):    data
C     12/27/90 Y.I. 8/7/91 G.K.
C
cc      PARAMETER(MJ=1000)
cxx      IMPLICIT REAL*8(A-H,O-Z )
cc      CHARACTER  TITLE*72
cc      DIMENSION  Y(MJ), Z(MJ)
cc      COMMON  /CMDATA/  TITLE
cxx      DIMENSION Y(N), Z(N), ZZ(N)
cxx      DIMENSION AICZT(21), FFZT(21), AICZ(21), FFZ(21)
cxx      DIMENSION ZMEAN(21), ZVAR(21)
C
      INTEGER N
      DOUBLE PRECISION Y(N), AICZT(21), FFZT(21), AICZ(21), FFZ(21),
     1                 ZMEAN(21), ZVAR(21), ZZ(N)
c local
      INTEGER I, II, J
      DOUBLE PRECISION Z(N), YMEAN, YVAR, FFY, AICY, A, ZJACOB, AICM
C
cc      CALL  READTS( 1,Y,N )
      CALL  GAUSSM( Y,N,YMEAN,YVAR,FFY,AICY )
cc      WRITE(6,600)
cc      WRITE(6,610)  TITLE
cc      WRITE(6,620)
      I = 0
      AICM=AICZT(1)
      DO 200 II=10,-10,-1
         I = I+1
      A = II/10.0D0
      CALL  BOXCOX( Y,N,A,Z,ZJACOB )
C
cc      CALL  GAUSSM( Z,N,ZMEAN,ZVAR,FFZ,AICZ )
      CALL  GAUSSM( Z,N,ZMEAN(I),ZVAR(I),FFZ(I),AICZ(I) )
C
cc      FFZT = FFZ + ZJACOB
cc      AICZT = AICZ-2*ZJACOB
      FFZT(I) = FFZ(I) + ZJACOB
      AICZT(I) = AICZ(I)-2*ZJACOB
cc      WRITE(6,630)  A, AICZT, FFZT, AICZ, FFZ, ZMEAN, ZVAR
c-----
      IF( I.EQ.1 ) AICM=AICZT(1)
      IF( AICZT(I).LE.AICM ) THEN
         DO 100 J=1,N
cxx  100    ZZ(J) = Z(J)
         ZZ(J) = Z(J)
  100    CONTINUE
         AICM = AICZT(I)
      END IF
c-----
  200 CONTINUE
cc      STOP
      RETURN
cxx  600 FORMAT( 1H ,'PROGRAM 4.4:   BOX-COX TRANSFORMATION' )
cxx  610 FORMAT( 1H ,A72 )
cxx  620 FORMAT( 1H ,'LAMBDA',5X,'AIC''',8X,'LL''',7X,'AIC',9X,'LL',
cxx     *            9X,'MEAN',9X,'VARIANCE' )
cxx  630 FORMAT( 1H ,F5.2,4F11.2,2D15.6 )
      E N D
      SUBROUTINE BOXCOX( Y,N,A,Z,ZJACOB )
C
C  ...  Box-Cox transformation:  Z = (Y**A - 1)/A  ...
C
C     Inputs:
C        Y(I):   data
C        N:      data length
C        A:      lambda
C     Outputs:
C        Z(I):   transformed data
C        ZJACOB: log(Jacobian), log |dz/dy|
C
cxx      IMPLICIT REAL*8(A-H,O-Z )
cxx      DIMENSION  Y(N), Z(N)
C
      INTEGER N
      DOUBLE PRECISION Y(N), A, Z(N), ZJACOB
c local
      INTEGER I
      DOUBLE PRECISION SUM
C
      SUM = 0.0D0
      DO 10 I=1,N
      IF( A .NE. 0.0D0 ) THEN
         Z(I) = (Y(I)**A - 1)/A
         SUM = SUM + (A-1)*DLOG( DABS( Y(I) ) )
      ELSE
         Z(I) = DLOG( Y(I) )
         SUM = SUM - DLOG( DABS( Y(I) ) )
      END IF
   10 CONTINUE
      ZJACOB = SUM
C
      RETURN
      E N D
      SUBROUTINE GAUSSM( Y,N,YMEAN,YVAR,FF,AIC )
C
C  ...  This subroutine fits Gaussian distribution model  ...
C
C     Inputs:
C        Y(I):   data
C        N:      data length
C     Outputs:
C        YMEAN:  mean
C        YVAR:   variance
C        FF:     log-likelihood
C        AIC:    AIC = -2FF + 4
C
cxx      IMPLICIT REAL*8( A-H,O-Z )
cxx      DIMENSION Y(N)
C
      INTEGER N
      DOUBLE PRECISION Y(N), YMEAN, YVAR, FF, AIC
c local
      INTEGER I
      DOUBLE PRECISION PI, SUM
C
      DATA  PI/3.1415926535D0/
C
      SUM = 0.0D0
      DO 10 I=1,N
cxx   10 SUM = SUM + Y(I)
      SUM = SUM + Y(I)
   10 CONTINUE
      YMEAN = SUM/N
C
      SUM = 0.0D0
      DO 20 I=1,N
cxx   20 SUM = SUM + (Y(I)-YMEAN)**2
      SUM = SUM + (Y(I)-YMEAN)**2
   20 CONTINUE
      YVAR = SUM/N
      FF  = -0.5D0*N*(DLOG( 2*PI*YVAR ) + 1)
      AIC = -2*FF + 4
C
      RETURN
      E N D

pfilter <- function(y, m = 10000, model = 0, lag = 20, initd = 0, sigma2, tau2,
                    alpha = 0.99, bigtau2 = NULL, init.sigma2 = 1,
                    xrange = NULL, seed = NULL, plot = TRUE,  ...)
{
# y                     # original data
  n <- length(y)        # data length
# m                     # number of particles
# model                 # model (0,1,2)
  if (model < 0 || model > 2)
    stop("'model' must be 0, 1 or 2.")
# lag                   # fixed lag smoother
# initd                 # initial distribution (0,1,2,3)
  if (initd < 0 || initd > 3)
    stop("'model' must be 0, 1, 2 or 3.")
# sigma2                # observation noise variance
  if (sigma2 < 0 || sigma2 == 0)
    stop("'sigma2' must be greater than zero.")
# tau2                  # system noise variance
  if (tau2 < 0)
    stop("'tau2' must be greater than or equal to zero.")
# alpha                 # mixture weight (model = 2)
# xrange                # lower and upper bounds of the distribution's range
  if (is.null(xrange) == TRUE) {
    r2 <- floor((max(y) - min(y)) /  2)
    xmin <- floor(min(y) - r2)
    xmax <- ceiling(max(y) + r2)
  } else {
    xmin <- xrange[1]
    xmax <- xrange[2]
    if (xmin == xmax || xmin > xmax)
      stop("xrange[2] must be greater than xrange[1].")
  }
# bigtau2               # vairance of the second component (model = 2)
  if (is.null(bigtau2) == TRUE) {
    bigtau2 <- (xmax - xmin) / 2
  } else if (bigtau2 < 0) {
    stop("'bigtau2' must be greater than or equal to zero.")
  }
# init.sigma2           # variance of initial state distribution (initd = 0)
  if (init.sigma2 < 0)
    stop("'init.sigma2' must be greater than or equal to zero.")
# seed :     for random number generator
  ix <- seed
  if (is.null(ix))
    ix <- -1

  z <- .Fortran(C_pfilter,
                as.double(y),
                as.integer(n),
                as.integer(m),
                as.integer(model),
                as.integer(lag),
                as.integer(initd),
                as.double(sigma2),
                as.double(tau2),
                as.double(alpha),
                as.double(bigtau2),
                as.double(init.sigma2),
                as.double(xmin),
                as.double(xmax),
                as.integer(ix),
                t = double(n * 8),
                ff = double(1))

  t <- array(z$t, dim = c(n, 8))
  tdist <- t[, 7:1]
  ff <- z$ff
  message(gettextf("\n log-likelihood\t%12.3f", ff), domain = NA)

  pfilter.out <- list(llkhood = ff, smooth.dist = tdist)
  class(pfilter.out) <- "pfilter"

  if (plot) {
    plot.pfilter(pfilter.out, ...)
    invisible(pfilter.out)
  } else {
    return(pfilter.out)
  }
}


pfilterNL <- function(y, m = 10000, lag = 20, sigma2, tau2, xrange = NULL,
                      seed = NULL, plot = TRUE,  ...)
{
# y                     # original data
  n <- length(y)        # data length
# m                     # number of particles
# lag                   # fixed lag smoother
# sigma2                # observation noise variance
  if (sigma2 < 0 || sigma2 == 0)
    stop("'sigma2' must be greater than zero.")
# tau2                  # system noise variance
  if (tau2 < 0)
    stop("'tau2' must be greater than or equal to zero.")
# xrange                # lower and upper bounds of the distribution's range
  if (is.null(xrange) == TRUE) {
    r2 <- floor((max(y) - min(y)) /  2)
    xmin <- floor(min(y) - r2)
    xmax <- ceiling(max(y) + r2)
  } else {
    xmin <- xrange[1]
    xmax <- xrange[2]
    if (xmin == xmax || xmin > xmax)
      stop("xrange[2] must be greater than xrange[1].")
  }
# seed :     for random number generator
  ix <- seed
  if (is.null(ix))
    ix <- -1

  z <- .Fortran(C_pfiltern,
             as.double(y),
             as.integer(n),
             as.integer(m),
             as.integer(lag),
             as.double(sigma2),
             as.double(tau2),
             as.double(xmin),
             as.double(xmax),
             as.integer(ix),
             t = double(n * 8),
             ff = double(1))

  t <- array(z$t, dim = c(n, 8))
  tdist <- t[, 7:1]
  ff <- z$ff
  message(gettextf("\n log-likelihood\t%12.3f", ff), domain = NA)

  pfilterNL.out <- list(llkhood = ff, smooth.dist = tdist)
  class(pfilterNL.out) <- "pfilter"

  if (plot) {
    plot.pfilter(pfilterNL.out, ...)
    invisible(pfilterNL.out)
  } else {
    return(pfilterNL.out)
  }
}


plot.pfilter <- function(x, ...)
{
  t <- x$smooth.dist
  mint <- floor(min(t) - 0.2)
  maxt <- ceiling(max(t) + 0.2)
#  minmaxt <- max(abs(mint), abs(maxt))
#  ylim <- c(-minmaxt, minmaxt)
  ylim <- c(mint, maxt)
  n <- dim(t)[1]
  xx <- seq(1,n, length=n)
#  gcol <- c("gray90", "gray80", "gray70", "gray70", "gray80", "gray90")
  gcol <- c("white", "gray85", "gray75", "gray75", "gray85", "white")
  mtitle <- "Marginal smoothed distribution of trend"

  plot(t[, 1], type = 'l', ylim = ylim, main = mtitle, xlab = "time",
       ylab = "x", xaxs = 'i', yaxs = 'i', ...)
  for(i in 2:7) {
    par(new = TRUE)
    plot(t[, i], type = 'l', ylim = ylim, xlab = "", ylab = "", xaxs = 'i',
         yaxs = 'i', ...)
    y1 <- t[xx,i-1]
    y2 <- t[xx,i]
    polygon( c(xx,rev(xx)), c(y1,rev(y2)), col=gcol[i-1])
  }
  par(new = TRUE)
  plot(t[, 4], type = 'l', ylim = ylim, xlab = "", ylab = "", xaxs = 'i',
       yaxs = 'i', lwd = 2, ...)
}

# PROGRAM 4.4 BOXCOX
boxcox <- function(y, plot = TRUE, ...)
{
  n <- length(y)             # length of data
  for (i in 1:n)
    if ((y[i] == 0) || (y[i] < 0))
      stop("Log-transformation cannot be applied to zeros and nagative numbers")

  z <- .Fortran(C_boxcoxf,
                as.double(y),
                as.integer(n),
                aiczt = double(21),
                ffzt = double(21),
                aicz = double(21),
                ffz = double(21),
                mean = double(21),
                var = double(21),
                zz = double(n))

  boxcox.out <- list(y, mean = z$mean, var = z$var, aic = z$aicz,
                     llkhood = z$ffz, z = z$zz, aic.z = z$aiczt,
                     llkhood.z = z$ffzt)

  class(boxcox.out) <- "boxcox"

  if (plot) {
    rdata <- deparse(substitute(y))
    eval(parse(text=paste(rdata, "<- y")))
    eval(parse(text=paste("plot.boxcox(boxcox.out,", rdata, ", ...)")))
  }

  return(boxcox.out)
}

print.boxcox <- function(x, ...)
{
  message("\n lambda    aic'         LL'         aic          LL        mean",
          appendLF=FALSE)
  message("           variance")

  aicm <- x$aic.z[1]
  am <- 1
  for (i in 1:21) {
    a <- (-i + 11) / 10
    message(gettextf("%6.2f %11.2f %11.2f %11.2f %11.2f   %e   %e", a,
            x$aic.z[i], x$llkhood.z[i], x$aic[i], x$llkhood[i], x$mean[i],
            x$var[i]), domain = NA)
    if (x$aic.z[i] < aicm) {
      aicm <- x$aic.z[i]
      am <- a
    } 
  }
  message(gettextf("\n lambda = %6.2f\t AIC' minimum = %11.2f\n", am, aicm),
          domain = NA)
}

plot.boxcox <- function(x, rdata = NULL, ...)
{
  ts.atr <- tsp(rdata)
  old.par <- par(no.readonly = TRUE)
  par(xaxs = "i")

  if (is.null(rdata) == FALSE) {
    tsname <- deparse(substitute(rdata))
    mtitle <- paste(tsname)
    par(mfcol = c(2, 1))
    plot(rdata, type = "l", main = mtitle, xlab = "", ylab = "y", ...)
  }

  tdata <- x$z
  if (is.null(ts.atr) == FALSE)
    tdata <- ts(tdata, start = ts.atr[1], frequency = ts.atr[3])
  plot(tdata, type = "l", main = "Transformed data", xlab = "",
       ylab = "log y", ...)
  par(old.par)
}

# PROGRAM 2.1
unicor <- function (y, lag = NULL, minmax = c(-1.0e+30, 1.0e+30), plot = TRUE, ...)
{
  n <- length(y)
  if (is.null(lag))
    lag <- as.integer(2 * sqrt(n))    # maximum lag
  lag1 <- lag + 1
  outmin <- minmax[1]
  outmax <- minmax[2]

  z <- .Fortran(C_unicorf,
                as.double(y),
                as.integer(n),
                as.integer(lag),
                as.double(outmin),
                as.double(outmax),
                cov = double(lag1 * 4),
                mean = double(1))

  cov <- matrix(z$cov, lag1, 4)
  acov <- cov[, 1]
  acor <- cov[, 2]
  cerr <- cov[, 3]
  rerr <- cov[, 4]

  unicor.out <- list(acov = acov, acor = acor, acov.err = cerr, acor.err = rerr,
                     mean = z$mean, tsname = deparse(substitute(y)))
  class(unicor.out) <- "unicor"

  if (plot) {
    plot.unicor(unicor.out, ...)
    invisible(unicor.out)
  } else unicor.out
}

plot.unicor <- function(x, ...) {
  acor <- x$acor
  lag <- length(acor)[1] - 1

  old.par <- par(no.readonly = TRUE)
  mtitle <- paste(x$tsname)
  plot(c(0:lag), acor, type = "h", ylim = c(-1, 1), main = mtitle,
       ylab = "Autocorrelation", xlab = "Lag", xaxs = "i", ...)
  par(new = TRUE)
  plot(c(0:lag), acor, type = "l", ylim = c(-1, 1), ylab = "", xlab = "",
       xaxs = "i", ...)
  par(new = TRUE)
  abline(h = 0)
  par(old.par)
}

\name{HAKUSAN}
\docType{data}
\alias{HAKUSAN}
\title{Ship's Navigation Data}
\description{
  A multivariate time series of a ship's yaw rate, rolling, pitching and rudder
  angles which were recorded every second while navigating across the Pacific
  Ocean.
}
\usage{
data(HAKUSAN)
}
\format{
  A data frame with 1000 observations on the following 4 variables.
  \tabular{rll}{
  [, 1] \tab YawRate \tab yaw rate\cr
  [, 2] \tab Rolling \tab rolling\cr
  [, 3] \tab Pitching\tab pitching\cr
  [, 4] \tab Rudder  \tab rudder angle
  }
}
\source{
  The data were offered by Prof. K. Ohtsu of Tokyo University of Marine Science
  and Technology.
}
\examples{
data(HAKUSAN)
HAKUSAN234 <- HAKUSAN[, c(2,3,4)]

## put histograms on the diagonal
panel.hist <- function(x, ...)
{
    usr <- par("usr")
    par(usr = c(usr[1:2], 0, 1.3))
    nB <- 20; nB1 <- nB + 1
    xmin <- min(x)
    xmax <- max(x)
    w <- (xmax - xmin) / nB
    breaks <- xmin
    b <- xmin
    for (i in 1:nB) {
      b <- b + w
      breaks <- c(breaks, b)
    }
    h <- hist(x, breaks = breaks, plot = FALSE)
    y <- h$counts; y <- y / max(y)
    rect(breaks[1:nB], 0, breaks[2:nB1], y, ...)

}

par(xaxs = "i", yaxs = "i", xaxt = "n", yaxt = "n")
pairs(HAKUSAN234, diag.panel = panel.hist, pch = 20, cex.labels = 1.5, 
      label.pos = 0.9, lower.panel = NULL)
}
\keyword{datasets}

\name{pdfunc}
\alias{pdfunc}

\title{Probability Density Function}
\description{
  Evaluate probability density function for normal distribution, Cauchy
 distribution, Pearson distribution, exponential distribution, Chi-square
 distributions, double exponential distribution and uniform distribution.}
\usage{
pdfunc(model = "norm", mean = 0, sigma2 = 1, mu = 0, tau2 = 1, shape,
       lambda = 1, side = 1, df, xmin = 0, xmax = 1, plot = TRUE, \dots)
}
\arguments{
  \item{model}{a character string indicating the model type of probability
    density function: either \code{"norm"}, \code{"Cauchy"}, \code{"Pearson"},
    \code{"exp"}, \code{"Chi2"}, \code{"dexp"} or \code{"unif"}.}
  \item{mean}{mean. (valid for \code{"norm"})}
  \item{sigma2}{variance. (valid for \code{"norm"})}
  \item{mu}{location parameter \eqn{\mu}. (valid for \code{"Cauchy"} and
    \code{"Pearson"})}
  \item{tau2}{dispersion parameter \eqn{\tau^2}. (valid for \code{"Cauchy"} and
    \code{"Pearson"})}
  \item{shape}{shape parameter (> 0.5). (valid for \code{"Pearson"})}
  \item{lambda}{lambda \eqn{\lambda}. (valid for \code{"exp"})}
  \item{side}{1: exponential, 2: two-sided exponential.
    (valid for \code{"exp"})}
  \item{df}{degree of freedoms \eqn{k}.  (valid for \code{"Chi2"})}
  \item{xmin}{lower bound of the interval.}
  \item{xmax}{upper bound of the interval.}
  \item{plot}{logical. If \code{TRUE} (default), probability density function is
              plotted.}
  \item{\dots}{graphical arguments passed to the \code{plot} method.}
}
\value{
  An object of class \code{"pdfunc"} which has a \code{plot} method. This is a
  list with the following components:
  \item{density}{values of density function.}
  \item{interval}{lower and upper bound of interval.}
  \item{param}{parameters of model.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# normal distribution
pdfunc(model = "norm", xmin = -4, xmax = 4) 

# Cauchy distribution
pdfunc(model = "Cauchy", xmin = -4, xmax = 4) 

# Pearson distribution
pdfunc(model = "Pearson", shape = 2, xmin = -4, xmax = 4) 

# exponential distribution
pdfunc(model = "exp", xmin = 0, xmax = 8) 

pdfunc(model = "exp", xmin = -4, xmax = 4)

# Chi-square distribution
pdfunc(model = "Chi2", df = 3, xmin = 0, xmax = 8) 

# double exponential distribution
pdfunc(model = "dexp", xmin = -4, xmax = 2) 

# uniform distribution
pdfunc(model = "unif", xmin = 0, xmax = 1) 
}

\keyword{ts}

\name{lsar}
\alias{lsar}

\title{Decomposition of Time Interval to Stationary Subintervals}
\description{
  Decompose time series to stationary subintervals and estimate local spectrum.
}
\usage{
lsar(y, max.arorder = 20, ns0, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{max.arorder}{highest order of AR model.}
  \item{ns0}{length of basic local span.}
  \item{plot}{logical. If \code{TRUE} (default), local spectra are plotted.}
  \item{\dots}{graphical arguments passed to the \code{plot} method.}
}
\value{
  An object of class \code{"lsar"} which has a \code{plot} method. This is a
  list with the following components:
  \item{model}{1: pooled model is accepted.\cr 2: switched model is accepted.} 
  \item{ns}{number of observations of local span.}
  \item{span}{start points and end points of local spans.}
  \item{nf}{number of frequencies in computing local power spectrum.}
  \item{ms}{order of switched model.}
  \item{sds}{innovation variance of switched model.}
  \item{aics}{AIC of switched model.}
  \item{mp}{order of pooled model.}
  \item{sdp}{innovation variance of pooled model.}
  \item{aics}{AIC of pooled model.}
  \item{spec}{local spectrum.}
  \item{tsname}{the name of the univariate time series \code{y}.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# seismic data
data(MYE1F)
lsar(MYE1F, max.arorder = 10, ns0 = 100)
}

\keyword{ts}

\name{WHARD}
\docType{data}
\alias{WHARD}
\title{Wholesale Hardware Data}
\description{
The monthly record of wholesale hardware data. (January 1967 - November 1979)
}
\usage{
data(WHARD)
}
\format{
A time series of 155 observations.
}
\source{
The data were obtained from the United States Bureau of Labor Statistics (BLS).
}
\keyword{datasets}

\name{pfilterNL}
\alias{pfilterNL}

\title{Particle Filtering and Smoothing for Nonlinear State-Space Model}
\description{
  Trend estimation by particle filter and smoother via nonlinear state-space model.
}
\usage{
pfilterNL(y, m = 10000, lag = 20, sigma2, tau2, xrange = NULL, seed = NULL,
          plot = TRUE, \dots)
}
\arguments{
  \item{y}{univariate time series.}
  \item{m}{number of particles.}
  \item{lag}{lag length for fixed-lag smoothing.}
  \item{sigma2}{observation noise variance.}
  \item{tau2}{system noise variance.}
  \item{xrange}{specify the lower and upper bounds of the distribution's range.} 
  \item{seed}{arbitrary positive integer to generate a sequence of uniform
    random numbers. The default seed is based on the current time.}
  \item{plot}{logical. If \code{TRUE} (default), marginal smoothed distribution
    is plotted.}
  \item{\dots}{graphical arguments passed to the \code{plot} method.}
}
\details{
  This function performs particle filtering and smoothing for the following nonlinear state-space model;

  \tabular{ll}{
    \eqn{x_n = \frac{1}{2} x_{n-1} + \frac{25 x_{n-1}}{x_{n-1}^2 + 1} + 8cos(1.2n) + v_n,}{%
         x(n) = x(n-1) / 2 + (25x(n-1)) / (x(n-1)**2 + 1) + 8cos(1.2n) + v(n),} \tab (system model) \cr
    \eqn{y_n = \frac{x_n^2}{10} + w_n,}{y(n) = x(n)**2 / 10 + w(n),} \tab (observation model)
  }

where \eqn{y_n}{y(n)} is a time series, \eqn{x_n}{x(n)} is the state vector.
The system noise \eqn{v_n}{v(n)} and the observation noise \eqn{w_n}{w(n)} are assumed to be white noises
 which follow a Gaussian distribution and \eqn{v_0}{v(0)} ~ \eqn{N(0, 5)}.


The algorithm of the particle filtering and smoothing are presented in Kitagawa (2020).
For more details, please refer to Kitagawa (1996) and Doucet et al. (2001).
}
\seealso{
\code{\link{pfilter}} performs particle filtering and smoothing for linear
 non-Gaussian state-space model.
}
\value{
  An object of class \code{"pfilter"} which has a \code{plot} method. This is a
  list with the following components:
  \item{llkhood}{log-likelihood.}
  \item{smooth.dist}{marginal smoothed distribution of the trend \eqn{T(i,j)} 
      \eqn{(i = 1,...,n, j = 1,...,7)}, where \eqn{n} is the length of \code{y}.
    \tabular{rll}{
      \tab j = 4: \tab 50\% point \cr
      \tab j = 3, 5: \tab 1-sigma points (15.87\% and 84.14\% points) \cr
      \tab j = 2, 6: \tab 2-sigma points (2.27\% and 97.73\% points) \cr
      \tab j = 1, 7: \tab 3-sigma points (0.13\% and 99.87\% points)}}
}
\references{
  Kitagawa, G. (1996)
  \emph{Monte Carlo filter and smoother for non-Gaussian nonlinear state space models},
 J. of Comp. and Graph. Statist., 5, 1-25.

  Doucet, A., de Freitas, N. and Gordon, N. (2001)
  \emph{Sequential Monte Carlo Methods in Practice}, Springer, New York.

  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}. Chapman & Hall/CRC.
}
\examples{
data(NLmodel)
x <- NLmodel[, 2]
pfilterNL(x, m = 100000, lag = 20 , sigma2 = 10.0, tau2 = 1.0,
          xrange = c(-20, 20), seed = 2019071117)
}

\keyword{ts}

\name{ngsim}
\alias{ngsim}

\title{Simulation by Non-Gaussian State Space Model}
\description{
  Simulation by non-Gaussian state space model.
}
\usage{
ngsim(n = 200, trend = NULL, seasonal.order = 0, seasonal = NULL, arcoef = NULL, 
      ar = NULL, noisew = 1, wminmax = NULL, paramw = NULL, noisev = 1,
      vminmax = NULL, paramv = NULL, seed = NULL, plot = TRUE, \dots)
}
\arguments{
  \item{n}{number of data generated by simulation.}
  \item{trend}{initial values of trend component of length \eqn{m1}, where
    \eqn{m1} is trend order (1, 2). If \code{NULL} (default), trend order is 0.}
  \item{seasonal.order}{order of seasonal component model (0, 1, 2).} 
  \item{seasonal}{if \code{seasonal.order} > 0, initial values of seasonal component
    of length \eqn{p-1}, where \eqn{p} is period of one season.}
  \item{arcoef}{AR coefficients.}
  \item{ar}{initial values of AR component.}
  \item{noisew}{type of the observational noise.
    \tabular{rl}{
      -1 : \tab Cauchy random number \cr
      -2 : \tab exponential distribution \cr
      -3 : \tab double exponential distribution \cr
       0 : \tab double exponential distribution (+ Euler's constant) \cr
       1 : \tab normal distribution (generated by inverse function) \cr
       2 : \tab Pearson distribution (generated by inverse function) \cr
       3 : \tab double exponential distribution (generated by inverse function)}
  }
  \item{wminmax}{lower and upper bound of observational noise.}
  \item{paramw}{parameter of the observational noise density.
    \tabular{rl}{
      noisew = 1 : \tab variance\cr
      noisew = 2 : \tab dispersion parameter (tau square) and shape parameter}
  }
  \item{noisev}{type of the system noise.
    \tabular{rl}{
      -1 : \tab Cauchy random number \cr
      -2 : \tab exponential distribution \cr
      -3 : \tab double exponential distribution \cr
       0 : \tab double exponential distribution (+ Euler's constant) \cr
       1 : \tab normal distribution (generated by inverse function) \cr
       2 : \tab Pearson distribution (generated by inverse function) \cr
       3 : \tab double exponential distribution (generated by inverse function)}
  }
  \item{vminmax}{lower and upper bound of system noise.}
  \item{paramv}{parameter of the system noise density.
    \tabular{rl}{
    noisev = 1 : \tab variance\cr
    noisev = 2 : \tab dispersion parameter (tau square) and shape parameter}
  }
  \item{seed}{arbitrary positive integer to generate a sequence of uniform
    random numbers. The default seed is based on the current time.}
  \item{plot}{logical. If \code{TRUE} (default), simulated data are plotted.}
  \item{\dots}{graphical arguments passed to \code{\link{plot.simulate}}.}
}
\value{
  An object of class \code{"simulate"}, giving simulated data of non-Gaussian
  state space model.
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
ar1 <- ngsim(n = 400, arcoef = 0.95, noisew = 1, paramw = 1, noisev = 1,
             paramv = 1, seed = 555)
plot(ar1, use = c(201, 400))

ar2 <- ngsim(n = 400, arcoef = c(1.3, -0.8), noisew = 1, paramw = 1, noisev = 1,
             paramv = 1, seed = 555)
plot(ar2, use = c(201, 400))


}

\keyword{ts}

\name{lsar.chgpt}
\alias{lsar.chgpt}

\title{Estimation of the Change Point}
\description{
  Precisely estimate a change point of subinterval for locally stationary AR
  model.
}
\usage{
lsar.chgpt(y, max.arorder = 20, subinterval, candidate, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{max.arorder}{highest order of AR model.}
  \item{subinterval}{a vector of the form \code{c(n0, ne)} which gives a start
    and end point of time interval used for model fitting.}
  \item{candidate}{a vector of the form \code{c(n1, n2)} which gives minimum
    and maximum of the candidate for change point.

    \code{n0+2k} < \code{n1} < \code{n2+k} < \code{ne},
    ( \code{k} is \code{max.arorder} )}
  \item{plot}{logical. If \code{TRUE} (default), \code{y[n0:ne]} and
    \code{aic} are plotted.}
  \item{\dots}{graphical arguments passed to the \code{plot} method.}
}
\value{
  An object of class \code{"chgpt"} which has a \code{plot} method. This is a
  list with the following components:
  \item{aic}{AICs of the AR models fitted on \code{[n1, n2]}.}
  \item{aicmin}{minimum AIC.}
  \item{change.point}{estimated change point.}
  \item{subint}{information about the original sub-interval.} 
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# seismic data
data(MYE1F)
lsar.chgpt(MYE1F, max.arorder = 10, subinterval = c(200, 1000),
           candidate = c(400, 800))

lsar.chgpt(MYE1F, max.arorder = 10, subinterval = c(600, 1400),
           candidate = c(800, 1200))
}

\keyword{ts}

\name{Sunspot}
\docType{data}
\alias{Sunspot}
\title{Sunspot Number Data}
\description{
  Yearly numbers of sunspots from to 1749 to 1979.
}
\usage{
data(Sunspot)
}
\format{
  A time series of 231 observations; yearly from 1749 to 1979.
}
\details{
  Sunspot is a part of the dataset \code{\link{sunspot.year}} from 1700 to 1988.
  Value "0" is converted into "0.1" for log transformation.
}
\keyword{datasets}

\name{plot.season}
\alias{plot.season}
\title{Plot Trend, Seasonal and AR Components}
\description{
  Plot trend component, seasonal component, AR component and noise returned by
  \code{\link{season}}.
}
\usage{
\method{plot}{season}(x, rdata = NULL, \dots)
}
\arguments{
  \item{x}{an object of class \code{"season"}.}
  \item{rdata}{original data, if necessary.}
  \item{\dots}{further graphical parameters may also be supplied as arguments.}
}
\keyword{ts}

\name{crscor}
\alias{crscor}

\title{Cross-Covariance and Cross-Correlation}
\description{
  Compute cross-covariance and cross-correlation functions of the multivariate
  time series.
}
\usage{
crscor(y, lag = NULL, outmin = NULL, outmax = NULL, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a multivariate time series.}
  \item{lag}{maximum lag. Default is \eqn{2 \sqrt{n}}{2*sqrt(n)}, where \eqn{n}
    is the length of the time series \code{y}.}
  \item{outmin}{bound for outliers in low side. A default value is -1.0e+30 for
    each dimension.}
  \item{outmax}{bound for outliers in high side. A default value is 1.0e+30 for
    each dimension.}
  \item{plot}{logical. If \code{TRUE} (default), cross-correlations are plotted.}
  \item{\dots}{graphical arguments passed to the \code{plot} method.}
}
\value{
  An object of class \code{"crscor"} which has a \code{plot} method. This is a
  list with the following components:
  \item{cov}{cross-covariances.}
  \item{cor}{cross-correlations.}
  \item{mean}{mean vector.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# Yaw rate, rolling, pitching and rudder angle of a ship
data(HAKUSAN)
y <- as.matrix(HAKUSAN[, 2:4])   # Rolling, Pitching, Rudder
crscor(y, lag = 50)

# The groundwater level and the atmospheric pressure
data(Haibara)
crscor(Haibara, lag = 50)
}

\keyword{ts}

\name{tvar}
\alias{tvar}

\title{Time Varying Coefficients AR Model}
\description{
  Estimate time varying coefficients AR model.
}
\usage{
tvar(y, trend.order = 2, ar.order = 2, span, outlier = NULL, tau2.ini = NULL,
     delta, plot = TRUE)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{trend.order}{trend order (1 or 2).}
  \item{ar.order}{AR order.}
  \item{span}{local stationary span.}
  \item{outlier}{positions of outliers.}
  \item{tau2.ini}{initial estimate of variance of the system noise \eqn{\tau^2}.
    If \code{tau2.ini} = \code{NULL}, the most suitable value is chosen in
    \eqn{\tau^2 = 2^{-k}}.}
  \item{delta}{search width.} 
  \item{plot}{logical. If \code{TRUE} (default), PARCOR is plotted.}
}
\details{
  The time-varying coefficients AR model is given by

    \deqn{y_t = a_{1,t}y_{t-1} + \ldots + a_{p,t}y_{t-p} + u_t}{%
          y(t) = a(1,t)y(t-1) + ... + a(p,t)y(t-p) + u(t)}

    where \eqn{a_{i,t}}{a(i,t)} is \eqn{i}-lag AR coefficient at time \eqn{t}
    and \eqn{u_t}{u(t)} is a zero mean white noise.

  The time-varying spectrum can be plotted using AR coefficient \code{arcoef}
  and variance of the observational noise \code{sigma2} by \code{\link{tvspc}}.
}
\value{
  \item{arcoef}{time varying AR coefficients.}
  \item{sigma2}{variance of the observational noise \eqn{\sigma^2}.}
  \item{tau2}{variance of the system noise \eqn{\tau^2}.}
  \item{llkhood}{log-likelihood of the model.}
  \item{aic}{AIC.}
  \item{parcor}{PARCOR.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.

  Kitagawa, G. and Gersch, W. (1996)
  \emph{Smoothness Priors Analysis of Time Series}.
  Lecture Notes in Statistics, No.116, Springer-Verlag.

  Kitagawa, G. and Gersch, W. (1985)
  \emph{A smoothness priors time varying AR coefficient modeling of
  nonstationary time series}. IEEE trans. on Automatic Control, AC-30, 48-56.
}

\seealso{
\code{\link{tvspc}}, \code{\link{plot.tvspc}}
}

\examples{
# seismic data
data(MYE1F)
z <- tvar(MYE1F, trend.order = 2, ar.order = 8, span = 20,
          outlier = c(630, 1026), tau2.ini = 6.6e-06, delta = 1.0e-06)
z

spec <- tvspc(z$arcoef, z$sigma2)
plot(spec)
}

\keyword{ts}

\name{plot.lsqr}
\alias{plot.lsqr}
\title{Plot Fitted Trigonometric Polynomial}
\description{
  Plot original data and fitted trigonometric polynomial returned by
  \code{\link{lsqr}}.
}
\usage{
\method{plot}{lsqr}(x, rdata = NULL, \dots)
}
\arguments{
  \item{x}{an object of class \code{"lsqr"}.}
  \item{rdata}{original data, if necessary.}
  \item{\dots}{further graphical parameters may also be supplied as arguments.}
}
\keyword{ts}

\name{NLmodel}
\docType{data}
\alias{NLmodel}
\title{The Nonlinear State-Space Model Data}
\description{The series generated by the nonlinear state-space model.}
\usage{
data(NLmodel)
}
\format{
A matrix with 100 rows and 2 columns.
\tabular{rl}{
  [, 1] \tab \eqn{x_n}{x(n)} \cr
  [, 2] \tab \eqn{y_n}{y(n)}
}
}
\details{
The system model \eqn{x_n}{x(n)} and the observation model \eqn{y_n}{y(n)} are
 generated by following state-space model:
\deqn{x_n = \frac{1}{2} x_{n-1} + \frac{25x_{n-1}}{x_{n-1}^2+1} + 8cos(1.2n) + v_n}{%
x(n) = 1/2 * x(n-1) + 25x(n-1) / (x(n-1)**2 + 1) + 8cos(1.2n) + v(n)}
\deqn{y_n = \frac{x_n^2}{10} + w_n,}{y(n) = x(n)**2 / 10 + w(n),}
where \eqn{v_n}{v(n)} ~ \eqn{N(0, 1)}, \eqn{w_n}{w(n)} ~ \eqn{N(0, 10)},
 \eqn{v_0}{v(0)} ~ \eqn{N(0, 5)}.
}
\keyword{datasets}

\name{plot.boxcox}
\alias{plot.boxcox}
\title{Plot Box-Cox Transformed Data}
\description{
  Plot original data and transformed data with minimum AIC.
}
\usage{
\method{plot}{boxcox}(x, rdata = NULL, \dots)
}
\arguments{
  \item{x}{an object of class \code{"boxcox"}.}
  \item{rdata}{original data, if necessary.}
  \item{\dots}{further graphical parameters may also be supplied as arguments.}
}
\keyword{ts}

\name{tvvar}
\alias{tvvar}
\alias{print.tvvar}
\alias{plot.tvvar}

\title{Time Varying Variance}
\description{
  Estimate time-varying variance.
}
\usage{
tvvar(y, trend.order, tau2.ini = NULL, delta, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{trend.order}{trend order.}
  \item{tau2.ini}{initial estimate of variance of the system noise \eqn{\tau^2}.
    If \code{tau2.ini} = \code{NULL}, the most suitable value is chosen in
    \eqn{\tau^2 = 2^{-k}}.}
  \item{delta}{search width.}
  \item{plot}{logical. If \code{TRUE} (default), transformed data, trend and
    residuals are plotted.}
  \item{\dots}{graphical arguments passed to the \code{plot} method.}
}
\details{
  Assuming that \eqn{\sigma_{2m-1}^2 = \sigma_{2m}^2}{%
  \sigma_(2m-1)^2 = \sigma_(2m)^2}, we define a transformed time series
  \eqn{s_1,\dots,s_{N/2}}{s_1,\dots,s_(N/2)} by 

  \deqn{s_m = y_{2m-1}^2 + y_{2m}^2,}{s_m = y_(2m-1)^2 + y_(2m)^2,}

  where \eqn{y_n} is a Gaussian white noise with mean \eqn{0} and variance
  \eqn{\sigma_n^2}. \eqn{s_m} is distributed as a \eqn{\chi^2} distribution with
  \eqn{2} degrees of freedom, so the probability density function of \eqn{s_m}
  is given by

  \deqn{f(s) = \frac{1}{2\sigma^2} e^{-s/2\sigma^2}.}{%
        f(s) = e^{-s/(2\sigma^2)} / (2\sigma^2).}
 
  By further transformation

  \deqn{z_m = \log \left( \frac{s_m}{2} \right),}{z_m = log{(s_m)/2},}

  the probability density function of \eqn{z_m} is given by

  \deqn{g(z) = \frac{1}{\sigma^2} \exp{ \left\{ z-\frac{e^z}{\sigma^2} \right\} } = \exp{ \left\{ (z-\log\sigma^2) - e^{(z-\log\sigma^2)} \right\} }.}{%
        g(z) = exp{z - e^z/\sigma^2} / \sigma^2 = exp{(z-log(\sigma^2)) - e^(z-log(\sigma^2))}.}

  Therefore, the transformed time series is given by

  \deqn{z_m = \log \sigma^2 + w_m,}{z_m = log(\sigma^2) + w_m,}

  where \eqn{w_m} is a double exponential distribution with probability density
  function 

  \deqn{h(w) = \exp{\{w-e^w\}}.}{h(w) = exp{w-e^w}.}

  In the space state model 

  \deqn{z_m = t_m + w_m}

  by identifying trend components of \eqn{z_m}, the log variance of original
  time series \eqn{y_n} is obtained.
}
\value{
  An object of class \code{"tvvar"} which has a \code{plot} method. This is a
  list with the following components:
  \item{tvv}{time varying variance.}
  \item{nordata}{normalized data.}
  \item{sm}{transformed data.}
  \item{trend}{trend.}
  \item{noise}{residuals.}
  \item{tau2}{variance of the system noise.}
  \item{sigma2}{variance of the observational noise.}
  \item{llkhood}{log-likelihood of the model.}
  \item{aic}{AIC.}
  \item{tsname}{the name of the univariate time series \code{y}.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.

  Kitagawa, G. and Gersch, W. (1996)
  \emph{Smoothness Priors Analysis of Time Series}. Lecture Notes in Statistics,
  No.116, Springer-Verlag.

  Kitagawa, G. and Gersch, W. (1985)
  \emph{A smoothness priors time varying AR coefficient modeling of
  nonstationary time series}. IEEE trans. on Automatic Control, AC-30, 48-56.
}
\examples{
# seismic data
data(MYE1F)
tvvar(MYE1F, trend.order = 2, tau2.ini = 6.6e-06, delta = 1.0e-06)
}

\keyword{ts}

\name{season}
\alias{season}

\title{Seasonal Adjustment}
\description{
  Seasonal adjustment by state space modeling. 
}
\usage{
season(y, trend.order = 1, seasonal.order = 1, ar.order = 0, trade = FALSE,
       period = NULL, tau2.ini = NULL, filter = c(1, length(y)),
       predict = length(y), arcoef.ini = NULL, log = FALSE, log.base = "e",
       minmax = c(-1.0e+30, 1.0e+30), plot = TRUE, ...)
}
\arguments{
  \item{y}{a univariate time series with or without the tsp attribute.}
  \item{trend.order}{trend order (0, 1, 2 or 3).}
  \item{seasonal.order}{seasonal order (0, 1 or 2).}
  \item{ar.order}{AR order (0, 1, 2, 3, 4 or 5).}
  \item{trade}{logical; if \code{TRUE}, the model including trading day effect
               component is considered.}
  \item{period}{If the tsp attribute of \code{y} is \code{NULL}, valid number of
                seasons in one period in the case that \code{seasonal.order} > 0 
                and/or \code{trade} = \code{TRUE}.
    \tabular{rl}{
      4 : \tab quarterly data\cr
      12 : \tab monthly data\cr
      5 : \tab daily data (5 days a week)\cr
      7 : \tab daily data (7 days a week)\cr
      24 : \tab hourly data
    }}
  \item{tau2.ini}{initial estimate of variance of the system noise \eqn{\tau^2} 
    less than 1.}
  \item{filter}{a numerical vector of the form \code{c(x1,x2)} which gives start
    and end position of filtering.}
  \item{predict}{the end position of prediction (\eqn{\geq}{>=} \code{x2}).}
  \item{arcoef.ini}{initial estimate of AR coefficients (for \code{ar.order} >
    0).}
  \item{log}{logical. If \code{TRUE}, the data \code{y} is log-transformed.}
  \item{log.base}{the letter "e" (default) or "10" specifying the base of
    logarithmic transformation. Valid only if log = TRUE.}
  \item{minmax}{lower and upper limits of observations.}
  \item{plot}{logical. If \code{TRUE} (default), trend, seasonal, AR and noise 
    components are plotted.}
  \item{\dots}{graphical arguments passed to \code{\link{plot.season}}.}
}
\value{
  An object of class \code{"season"}, which is a list with the following
   components:
  \item{tau2}{variance of the system noise.}
  \item{sigma2}{variance of the observational noise.}
  \item{llkhood}{log-likelihood of the model.}
  \item{aic}{AIC of the model.}
  \item{trend}{trend component (for \code{trend.order} > 0).}
  \item{seasonal}{seasonal component (for \code{seasonal.order} > 0).}
  \item{arcoef}{AR coefficients (for \code{ar.order} > 0).}
  \item{ar}{AR component (for \code{ar.order} > 0).}
  \item{day.effect}{trading day effect (for \code{trade} = \code{TRUE}).}
  \item{noise}{noise component.}
  \item{cov}{covariance matrix of smoother.}
}
\note{
  For time series with the tsp attribute, set \code{frequency} to
  \code{period}.
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# BLSALLFOOD data
data(BLSALLFOOD)
season(BLSALLFOOD, trend.order = 2, seasonal.order = 1, ar.order = 2)

season(BLSALLFOOD, trend.order = 2, seasonal.order = 1, ar.order = 2,
       filter = c(1, 132))

# Wholesale hardware data
data(WHARD)
season(WHARD, trend.order = 2, seasonal.order = 1, ar.order = 0, trade = TRUE,
       log = TRUE)

season(WHARD, trend.order = 2, seasonal.order = 1, ar.order = 0, trade = TRUE,
       filter = c(1, 132), log = TRUE)
}

\keyword{ts}

\name{marspc}
\alias{marspc}

\title{Cross Spectra and Power Contribution}
\description{
  Compute cross spectra, coherency and power contribution.
}
\usage{
marspc(arcoef, v, plot = TRUE, \dots)
}
\arguments{
  \item{arcoef}{AR coefficient matrices.}
  \item{v}{innovation variance matrix.}
  \item{plot}{logical. If \code{TRUE} (default), cross spectra, coherency and
    power contribution are plotted.}
  \item{\dots}{graphical arguments passed to the \code{plot} method.}
}
\value{
  An object of class \code{"marspc"} which has a \code{plot} method. This is a
  list with the following components:
  \item{spec}{cross spectra.}
  \item{amp}{amplitude spectra.}
  \item{phase}{phase spectra.}
  \item{coh}{simple coherency.}
  \item{power}{decomposition of power spectra.}
  \item{rpower}{relative power contribution.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# Yaw rate, rolling, pitching and rudder angle of a ship
data(HAKUSAN)
yy <- as.matrix(HAKUSAN[, c(1,2,4)])
nc <- dim(yy)[1]
n <- seq(1, nc, by = 2) 
y <- yy[n, ]
z <- marfit(y, lag = 20)

marspc(z$arcoef, v = z$v)
}

\keyword{ts}

\name{BLSALLFOOD}
\docType{data}
\alias{BLSALLFOOD}
\title{BLSALLFOOD Data}
\description{
The monthly time series of the number of workers engaged in food industries in
the United States (January 1967 - December 1979).
}
\usage{
data(BLSALLFOOD)
}
\format{
A time series of 156 observations.
}
\source{
The data were obtained from the United States Bureau of Labor Statistics (BLS).
}
\keyword{datasets}

\name{plot.ngsmth}
\alias{plot.ngsmth}
\title{Plot Smoothed Density Function}
\description{
  Plot the smoothed density function returned by \code{\link{ngsmth}}.
}
\usage{
\method{plot}{ngsmth}(x, type = c("trend", "smt"), theta = 0, phi = 15,
          expand = 1, col = "lightblue", ticktype= "detail", \dots)
}
\arguments{
  \item{x}{an object of class \code{"ngsmth"}.}
  \item{type}{plotted values, either or both of "trend" and "smt".}
  \item{theta, phi, expand, col, ticktype}{graphical parameters in perspective
        plot \code{\link[graphics]{persp}}.}
  \item{\dots}{further graphical parameters may also be supplied as arguments.}
}
\keyword{ts}

\name{Nikkei225}
\docType{data}
\alias{Nikkei225}
\title{Nikkei225}
\description{
A daily closing values of the Japanese stock price index, Nikkei225, quoted from
 January 4, 1988, to December 30, 1993.
}
\usage{
data(Nikkei225)
}
\format{
A time series of 1480 observations.
}
\source{
\url{https://indexes.nikkei.co.jp/nkave/archives/data}
}
\keyword{datasets}

\name{tsmooth}
\alias{tsmooth}

\title{Prediction and Interpolation of Time Series}
\description{
Predict and interpolate time series based on state space model by Kalman filter.
}
\usage{
tsmooth(y, f, g, h, q, r, x0 = NULL, v0 = NULL, filter.end = NULL,
        predict.end = NULL, minmax = c(-1.0e+30, 1.0e+30), missed = NULL,
        np = NULL, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series \eqn{y_n}{y(n)}.}
  \item{f}{state transition matrix \eqn{F_n}{F(n)}.}
  \item{g}{matrix \eqn{G_n}{G(n)}.}
  \item{h}{matrix \eqn{H_n}{H(n)}.}
  \item{q}{system noise variance \eqn{Q_n}{Q(n)}.}
  \item{r}{observational noise variance \eqn{R}.}
  \item{x0}{initial state vector \eqn{X(0\mid0)}{X(0|0)}.}
  \item{v0}{initial state covariance matrix \eqn{V(0\mid0)}{V(0|0)}.}
  \item{filter.end}{end point of filtering.}
  \item{predict.end}{end point of prediction.}
  \item{minmax}{lower and upper limits of observations.}
  \item{missed}{start position of missed intervals.}
  \item{np}{number of missed observations.}
  \item{plot}{logical. If \code{TRUE} (default), mean vectors of the smoother
    and estimation error are plotted.}
  \item{\dots}{graphical arguments passed to \code{\link{plot.smooth}}.}
}
\details{
  The linear Gaussian state space model is

    \deqn{x_n = F_n x_{n-1} + G_n v_n,}{x(n) = F(n)x(n-1) + G(n)v(n),}
    \deqn{y_n = H_n x_n + w_n,}{y(n) = H(n)x(n) + w(n),}

  where \eqn{y_n}{y(n)} is a univariate time series, \eqn{x_n}{x(n)} is an
    \eqn{m}-dimensional state vector.
  
  \eqn{F_n}{F(n)}, \eqn{G_n}{G(n)} and \eqn{H_n}{H(n)} are \eqn{m \times m}{m * m},
  \eqn{m \times k}{m * k} matrices and a vector of length \eqn{m} , respectively.
  \eqn{Q_n}{Q(n)} is \eqn{k \times k}{k * k} matrix and \eqn{R_n}{R(n)} is a scalar.
  \eqn{v_n}{v(n)} is system noise and \eqn{w_n}{w(n)} is observation noise,
  where we assume that \eqn{E(v_n, w_n) = 0}{E(v(n), w(n)) = 0},
  \eqn{v_n \sim N(0, Q_n)}{v(n) ~ N(0, Q(n))} and
  \eqn{w_n \sim N(0, R_n)}{w(n) ~ N(0, R(n))}. User should give all the matrices
  of a state space model and its parameters. In current version, \eqn{F_n}{F(n)},
  \eqn{G_n}{G(n)}, \eqn{H_n}{H(n)}, \eqn{Q_n}{Q(n)}, \eqn{R_n}{R(n)} should be
  time invariant.
}
\value{
  An object of class \code{"smooth"}, which is a list with the following
  components:
  \item{mean.smooth}{mean vectors of the smoother.}
  \item{cov.smooth}{variance of the smoother.}
  \item{esterr}{estimation error.}
  \item{llkhood}{log-likelihood.}
  \item{aic}{AIC.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.

  Kitagawa, G. and Gersch, W. (1996)
  \emph{Smoothness Priors Analysis of Time Series}.
  Lecture Notes in Statistics, No.116, Springer-Verlag.
}
\examples{
## Example of prediction (AR model)
data(BLSALLFOOD)
BLS120 <- BLSALLFOOD[1:120]
z1 <- arfit(BLS120, plot = FALSE)
tau2 <- z1$sigma2

# m = maice.order, k=1
m1 <- z1$maice.order
arcoef <- z1$arcoef[[m1]]
f <- matrix(0.0e0, m1, m1)
f[1, ] <- arcoef
if (m1 != 1)
  for (i in 2:m1) f[i, i-1] <- 1
g <- c(1, rep(0.0e0, m1-1))
h <- c(1, rep(0.0e0, m1-1))
q <- tau2[m1+1]
r <- 0.0e0
x0 <- rep(0.0e0, m1)
v0 <- NULL

s1 <- tsmooth(BLS120, f, g, h, q, r, x0, v0, filter.end = 120, predict.end = 156)
s1

plot(s1, BLSALLFOOD)

## Example of interpolation of missing values (AR model)
z2 <- arfit(BLSALLFOOD, plot = FALSE)
tau2 <- z2$sigma2

# m = maice.order, k=1
m2 <- z2$maice.order
arcoef <- z2$arcoef[[m2]]
f <- matrix(0.0e0, m2, m2)
f[1, ] <- arcoef
if (m2 != 1)
  for (i in 2:m2) f[i, i-1] <- 1
g <- c(1, rep(0.0e0, m2-1))
h <- c(1, rep(0.0e0, m2-1))
q <- tau2[m2+1]
r <- 0.0e0
x0 <- rep(0.0e0, m2)
v0 <- NULL

tsmooth(BLSALLFOOD, f, g, h, q, r, x0, v0, missed = c(41, 101), np = c(30, 20))
}

\keyword{ts}

\name{armachar}
\alias{armachar}

\title{Calculate Characteristics of Scalar ARMA Model}
\description{
  Calculate impulse response function, autocovariance function, autocorrelation
  function and characteristic roots of given scalar ARMA model.
}
\usage{
armachar(arcoef = NULL, macoef = NULL, v, lag = 50, nf = 200, plot = TRUE, \dots)
}
\arguments{
  \item{arcoef}{AR coefficients.}
  \item{macoef}{MA coefficients.}
  \item{v}{innovation variance.}
  \item{lag}{maximum lag of autocovariance function.}
  \item{nf}{number of frequencies in evaluating spectrum.}
  \item{plot}{logical. If \code{TRUE} (default), impulse response function,
   autocovariance, power spectrum, PARCOR and characteristic roots are plotted.}
  \item{\dots}{graphical arguments passed to the \code{plot} method.}
}
\details{
  The ARMA model is given by

  \deqn{y_t - a_1y_{t-1} - \dots - a_py_{t-p} = u_t - b_1u_{t-1} - \dots - b_qu_{t-q},}{%
        y(t) - a(1)y(t-1) - \dots - a(p)y(t-p) = u(t) - b(1)u(t-1) - \dots - b(q)u(t-q),}

  where \eqn{p} is AR order, \eqn{q} is MA order and \eqn{u_t}{u(t)} is a zero
  mean white noise.

  Characteristic roots of AR / MA operator is a list with the following components:
  \itemize{
    \item re: real part \eqn{R}
    \item im: imaginary part \eqn{I}
    \item amp: \eqn{\sqrt{R^2+I^2}}{sqrt(R^2+I^2)}
    \item atan: \eqn{\arctan(I/R)}{atan(I/R)}
    \item degree
  }
}
\value{
  An object of class \code{"arma"} which has a \code{plot} method. This is a
  list with components:
  \item{impuls}{impulse response function.}
  \item{acov}{autocovariance function.}
  \item{parcor}{PARCOR.}
  \item{spec}{power spectrum.}
  \item{croot.ar}{characteristic roots of AR operator. See Details.}
  \item{croot.ma}{characteristic roots of MA operator. See Details.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# AR model : y(n) = a(1)*y(n-1) + a(2)*y(n-2) + v(n)
a <- c(0.9 * sqrt(3), -0.81)
armachar(arcoef = a, v = 1.0, lag = 20)

# MA model : y(n) = v(n) - b(1)*v(n-1) - b(2)*v(n-2)
b <- c(0.9 * sqrt(2), -0.81)
armachar(macoef = b, v = 1.0, lag = 20)

# ARMA model :  y(n) = a(1)*y(n-1) + a(2)*y(n-2)
#                      + v(n) - b(1)*v(n-1) - b(2)*v(n-2)
armachar(arcoef = a, macoef = b, v = 1.0, lag = 20)
}

\keyword{ts}

\name{lsqr}
\alias{lsqr}

\title{The Least Squares Method via Householder Transformation}
\description{
  Compute regression coefficients of the model with minimum AIC by the least
  squares method via Householder transformation.
}
\usage{
lsqr(y, lag = NULL, period = 365, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{lag}{number of sine and cosine components. Default is \eqn{\sqrt{n}}{sqrt(n)},
    where \eqn{n} is the length of the time series \code{y}.}
  \item{period}{period of one cycle.}
  \item{plot}{logical. If \code{TRUE} (default), original data and fitted
    trigonometric polynomial are plotted.}
  \item{\dots}{graphical arguments passed to \code{\link{plot.lsqr}}.}
}
\value{
  An object of class \code{"lsqr"}, which is a list with the following
   components:
  \item{aic}{
    AIC's of the model with order \eqn{0,\dots,k ( = 2}\code{lag}\eqn{ + 1)}.}
  \item{sigma2}{residual variance of the model with order \eqn{0,\dots,k}.}
  \item{maice.order}{order of minimum AIC.}
  \item{regress}{regression coefficients of the model.}
  \item{tripoly}{trigonometric polynomial.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# The daily maximum temperatures in Tokyo
data(Temperature)
lsqr(Temperature, lag = 10)
}

\keyword{ts}

\name{plot.trend}
\alias{plot.trend}
\title{Plot Trend and Residuals}
\description{
  Plot trend component and residuals returned by \code{\link{trend}}.
}
\usage{
\method{plot}{trend}(x, rdata = NULL, \dots)
}
\arguments{
  \item{x}{an object of class \code{"trend"}.}
  \item{rdata}{original data, if necessary.}
  \item{\dots}{further graphical parameters may also be supplied as arguments.}
}
\keyword{ts}

\name{TSSS-package}
\alias{TSSS-package}
\alias{TSSS}
\docType{package}

\title{
  Time Series Analysis with State Space Model
}
\description{
  R functions for statistical analysis, modeling and simulation of time series
  with state space model.
}
\details{
  This package provides functions for statistical analysis, modeling and
  simulation of time series. These functions are developed based on source code
  of "FORTRAN 77 Programming for Time Series Analysis".

  After that, the revised edition "Introduction to Time Series Analysis (in Japanese)" 
  and the translation version "Introduction to Time Series Modeling" are published.

  Currently the revised edition "Introduction to Time Series Modeling with Applications in R"
  is published, in which calculations of most of the modeling or methods are explained using
  this package.
}
\references{
  Kitagawa, G. and Gersch, W. (1996)
  \emph{Smoothness Priors Analysis of Time Series}. Lecture Notes in Statistics,
  No.116, Springer-Verlag.

  Kitagawa, G. (2010)
  \emph{Introduction to Time Series Modeling}. Chapman & Hall/CRC.

  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}. Chapman & Hall/CRC.

  Kitagawa, G. (1993)
  \emph{FORTRAN 77 Programming for Time Series Analysis}.
  The Iwanami Computer Science Series, Iwanami Publishing Company (in Japanese).

  Kitagawa, G. (2005)
  \emph{Introduction to Time Series Analysis}.
  Iwanami Publishing Company (in Japanese).

  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with R}.
  Iwanami Publishing Company (in Japanese).
}

\keyword{package}

\name{plot.spg}
\alias{plot.spg}
\title{Plot Smoothed Periodogram}
\description{
  Plot smoothed periodogram or logarithm of smoothed periodogram.
}
\usage{
\method{plot}{spg}(x, type = "vl", \dots)
}
\arguments{
  \item{x}{an object of class \code{"spg"} as returned by \code{\link{period}} 
    and \code{\link{fftper}}.}
  \item{type}{type of plot. ("l": lines, "vl" : vertical lines)}
  \item{\dots}{further graphical parameters may also be supplied as arguments.}
}
\keyword{ts}

\name{tvspc}
\alias{tvspc}

\title{Evolutionary Power Spectra by Time Varying AR Model}
\description{
  Estimate evolutionary power spectra by time varying AR model.
}
\usage{
tvspc(arcoef, sigma2, var = NULL, span = 20, nf = 200)
}
\arguments{
  \item{arcoef}{time varying AR coefficients.}
  \item{sigma2}{variance of the observational noise.}
  \item{var}{time varying variance.}
  \item{span}{local stationary span.}
  \item{nf}{number of frequencies in evaluating power spectrum.}
}
\value{
  return an object of class \code{"tvspc"} giving power spectra, which has a
  \code{plot} method (\code{\link{plot.tvspc}}).
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.

  Kitagawa, G. and Gersch, W. (1996)
  \emph{Smoothness Priors Analysis of Time Series}. Lecture Notes in Statistics,
  No.116, Springer-Verlag.

  Kitagawa, G. and Gersch, W. (1985)
  \emph{A smoothness priors time varying AR coefficient modeling 
  of nonstationary time series}. IEEE trans. on Automatic Control, AC-30, 48-56.
}
\examples{
# seismic data
data(MYE1F)
z <- tvar(MYE1F, trend.order = 2, ar.order = 8, span = 20,
          outlier = c(630, 1026), tau2.ini = 6.6e-06, delta = 1.0e-06)
spec <- tvspc(z$arcoef, z$sigma2)
plot(spec)
}

\keyword{ts}

\name{Rainfall}
\alias{Rainfall}
\docType{data}
\title{Rainfall Data}
\description{
Number of rainy days in two years (1975-1976) at Tokyo, Japan.
}
\usage{
data(Rainfall)
}
\format{
Integer-valued time series of 366 observations.
}
\source{
The data were obtained from Tokyo District Meteorological Observatory.
\url{https://www.data.jma.go.jp/obd/stats/etrn/}
}
\keyword{datasets}

\name{PfilterSample}
\docType{data}
\alias{PfilterSample}
\title{Sample Data for Particle Filter and Smoother}
\description{
An artificially generated sample data with shifting mean value.
}
\usage{
data(PfilterSample)
}
\format{
A time series of 400 observations.
}
\details{
This data generated by the following models;
  \tabular{rrlr}{
    \eqn{y_n} ~ \eqn{N(\mu_n, 1)}, \tab \eqn{\mu_n} \tab  = 0, \tab 1 <= n <= 100 \cr
    \tab \tab  = 1, \tab 101 <= n <= 200 \cr
    \tab \tab  = -1, \tab 201 <= n <= 300 \cr
    \tab \tab  = 0, \tab 301 <= n <= 400 
  }
}
\keyword{datasets}

\name{ngsmth}
\alias{ngsmth}

\title{Non-Gaussian Smoothing}
\description{
  Trend estimation by non-Gaussian smoothing.
}
\usage{
ngsmth(y, noisev = 2, tau2, bv = 1.0, noisew = 1, sigma2, bw = 1.0, 
       initd = 1, k = 200, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{noisev}{type of system noise density.
    \tabular{rl}{
      1 : \tab Gaussian (normal) \cr
      2 : \tab Pearson family \cr
      3 : \tab two-sided exponential}
  }
  \item{tau2}{variance or dispersion of system noise.}
  \item{bv}{shape parameter of system noise (for \code{noisev} = 2).}
  \item{noisew}{type of observation noise density
    \tabular{rl}{
      1 : \tab Gaussian (normal) \cr
      2 : \tab Pearson family \cr
      3 : \tab two-sided exponential \cr
      4 : \tab double exponential}
  }
  \item{sigma2}{variance or dispersion of observation noise.}
  \item{bw}{shape parameter of observation noise (for \code{noisew} = 2).}
  \item{initd}{type of density function.
    \tabular{rl}{
      1 : \tab Gaussian (normal) \cr
      2 : \tab uniform \cr
      3 : \tab two-sided exponential}
  }
  \item{k}{number of intervals in numerical integration.}
  \item{plot}{logical. If \code{TRUE} (default), trend is plotted.}
  \item{\dots}{graphical arguments passed to \code{\link{plot.ngsmth}}.}
}
\details{
  Consider a one-dimensional state space model
    \deqn{x_n = x_{n-1} + v_n,}{x(n) = x(n-1) + v(n),}
    \deqn{y_n = x_n + w_n,}{y(n) = x(n) + w(n),}
  where the observation noise \eqn{w_n}{w(n)} is assumed to be Gaussian
  distributed and the system noise \eqn{v_n}{v(n)} is assumed to be distributed
  as the Pearson system
    \deqn{q(v_n) = c/(\tau^2 + v_n^2)^b}{q(v(n)) = c/{\tau^2 + v(n)^2}^b}
  with \eqn{\frac{1}{2} < b < \infty}{1/2 < b < \infty} and
  \eqn{c = \tau^{2b-1}\Gamma(b) \bigm/ \Gamma(\frac{1}{2})\Gamma(b-\frac{1}{2})}{%
  c = \tau^(2b-1) \Gamma(b) / {\Gamma(1/2) \Gamma(b-1/2)}}.

  This broad family of distributions includes the Cauchy distribution
  (\eqn{b = 1}) and \eqn{t}-distribution (\eqn{b = (k+1)/2}).
}
\value{
  An object of class \code{"ngsmth"}, which is a list with the following
   components:
  \item{llkhood}{log-likelihood.}
  \item{trend}{trend.}
  \item{smt}{smoothed density.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.

  Kitagawa, G. and Gersch, W. (1996)
  \emph{Smoothness Priors Analysis of Time Series}. Lecture Notes in Statistics,
  No.116, Springer-Verlag.
}
\examples{
## test data
data(PfilterSample)
par(mar = c(3, 3, 1, 1) + 0.1)

# system noise density : Gaussian (normal)
s1 <- ngsmth(PfilterSample, noisev = 1, tau2 = 1.4e-02, noisew = 1, sigma2 = 1.048)
s1
plot(s1, "smt", theta = 25, phi = 30, expand = 0.25, col = "white")

# system noise density : Pearson family
s2 <- ngsmth(PfilterSample, noisev = 2, tau2 = 2.11e-10, bv = 0.6, noisew = 1,
             sigma2 = 1.042)
s2
plot(s2, "smt", theta = 25, phi = 30, expand = 0.25, col = "white")

## seismic data
data(MYE1F)
n <- length(MYE1F)
yy <- rep(0, n)
for (i in 2:n) yy[i] <- MYE1F[i] - 0.5 * MYE1F[i-1]
m <- seq(1, n, by = 2)
y <- yy[m]
z <- tvvar(y, trend.order = 2, tau2.ini = 4.909e-02, delta = 1.0e-06)

# system noise density : Gaussian (normal)
s3 <- ngsmth(z$sm, noisev = 1, tau2 = z$tau2, noisew = 2, sigma2 = pi*pi/6,
             k = 190)
s3
plot(s3, "smt", phi = 50, expand = 0.5, col = 8)
}

\keyword{ts}

\name{pfilter}
\alias{pfilter}
\alias{plot.pfilter}

\title{Particle Filtering and Smoothing}
\description{
  Trend estimation by particle filter and smoother.
}
\usage{
pfilter(y, m = 10000, model = 0, lag = 20, initd = 0, sigma2, tau2,
        alpha = 0.99, bigtau2 = NULL, init.sigma2 = 1, xrange = NULL,
        seed = NULL, plot = TRUE, \dots)
}
\arguments{
  \item{y}{univariate time series.}
  \item{m}{number of particles.}
  \item{model}{model for the system noise.
    \tabular{rl}{
      0: \tab normal distribution \cr
      1: \tab Cauchy distribution \cr
      2: \tab Gaussian mixture distribution \cr
           \tab \eqn{\alpha} \eqn{N}(0, \eqn{\tau^2}) + (1 - \eqn{\alpha}) \eqn{N}(0, \eqn{T^2}), \cr
           \tab where \eqn{N} is the normal density.}}
  \item{lag}{lag length for fixed-lag smoothing.}
  \item{initd}{type of initial state distribution.
    \tabular{rl}{
      0: \tab normal distribution \cr
      1: \tab uniform distribution \cr
      2: \tab Cauchy distribution \cr
      3: \tab fixed point (default value = 0)}}
  \item{sigma2}{observation noise variance \eqn{\sigma^2}.}
  \item{tau2}{system noise variance \eqn{\tau^2} for \code{model} = 0 or dispersion parameter
    for \code{model} = 1.}
  \item{alpha}{mixture weight \eqn{\alpha}. (valid for \code{model} = 2)}
  \item{bigtau2}{variance of the second component \eqn{T^2}. (valid for \code{model} = 2)}
  \item{init.sigma2}{variance for \code{initd} = 0 or dispersion parameter of
    initial state distribution for \code{initd} = 2.}
  \item{xrange}{specify the lower and upper bounds of the distribution's range.} 
  \item{seed}{arbitrary positive integer to generate a sequence of uniform
    random numbers. The default seed is based on the current time.}
  \item{plot}{logical. If \code{TRUE} (default), marginal smoothed distribution
              is plotted.}
  \item{\dots}{graphical arguments passed to the \code{plot} method.}
}
\details{
This function performs particle filtering and smoothing for the first order trend model;

  \tabular{ll}{
    \eqn{x_n = x_{n-1} + v_n,}{x(n) = x(n-1) + v(n),} \tab (system model) \cr 
    \eqn{y_n = x_n + w_n,}{y(n) = x(n) + w(n),} \tab (observation model)} 

where \eqn{y_n}{y(n)} is a time series, \eqn{x_n}{x(n)} is the state vector.
The system noise \eqn{v_n}{v(n)} and the observation noise \eqn{w_n}{w(n)} are assumed to be white noises
 which follow a Gaussian distribution or a Cauchy distribution, and non-Gaussian distribution, respectively.


The algorithm of the particle filter and smoother are presented in Kitagawa (2020).
For more details, please refer to Kitagawa (1996) and Doucet et al. (2001).
}
\seealso{
\code{\link{pfilterNL}} performs particle filtering and smoothing for nonlinear
 non-Gaussian state-space model.
}
\value{
  An object of class \code{"pfilter"} which has a \code{plot} method. This is a
  list with the following components:
  \item{llkhood}{log-likelihood.}
  \item{smooth.dist}{marginal smoothed distribution of the trend \eqn{T(i,j)} 
      \eqn{(i = 1,...,n, j = 1,...,7)}, where \eqn{n} is the length of \code{y}.
    \tabular{rll}{
      \tab j = 4: \tab 50\% point \cr
      \tab j = 3, 5: \tab 1-sigma points (15.87\% and 84.14\% points) \cr
      \tab j = 2, 6: \tab 2-sigma points (2.27\% and 97.73\% points) \cr
      \tab j = 1, 7: \tab 3-sigma points (0.13\% and 99.87\% points)}}
}
\references{
  Kitagawa, G. (1996)
  \emph{Monte Carlo filter and smoother for non-Gaussian nonlinear state space models},
 J. of Comp. and Graph. Statist., 5, 1-25.

  Doucet, A., de Freitas, N. and Gordon, N. (2001)
  \emph{Sequential Monte Carlo Methods in Practice}, Springer, New York.

  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}. Chapman & Hall/CRC.
}
\examples{
data(PfilterSample)
y <- PfilterSample

\dontrun{
pfilter(y, m = 100000, model = 0, lag = 20, initd = 0, sigma2 = 1.048,
        tau2 = 1.4e-2, xrange = c(-4, 4), seed = 2019071117)

pfilter(y, m = 100000, model = 1, lag = 20 , initd = 0, sigma2 = 1.045,
        tau2 = 3.53e-5, xrange = c(-4, 4), seed = 2019071117)

pfilter(y, m = 100000, model = 2, lag = 20 , initd = 0, sigma2 = 1.03,
        tau2 = 0.00013, alpha = 0.991, xrange = c(-4, 4), seed = 2019071117)
}
}

\keyword{ts}

\name{armafit}
\alias{armafit}

\title{Scalar ARMA Model Fitting}
\description{
Fit a scalar ARMA model by maximum likelihood method.
}
\usage{
armafit(y, ar.order, ar = NULL, ma.order, ma = NULL)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{ar.order}{AR order.}
  \item{ar}{initial AR coefficients. If \code{NULL} (default), use default
    initial values.}
  \item{ma.order}{MA order.}
  \item{ma}{initial MA coefficients. If \code{NULL} (default), use default
    initial values.}
}
\value{
  \item{sigma2}{innovation variance.}
  \item{llkhood}{log-likelihood of the model.}
  \item{aic}{AIC of the model.}
  \item{arcoef}{AR coefficients.}
  \item{macoef}{MA coefficients.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# Sunspot number data
data(Sunspot)
y <- log10(Sunspot)
z <- armafit(y, ar.order = 3, ma.order = 3)
z

armachar(arcoef = z$arcoef, macoef = z$macoef, v = z$sigma2, lag = 20)
}

\keyword{ts}

\name{simssm}
\alias{simssm}

\title{Simulation by Gaussian State Space Model}
\description{
  Simulate time series by Gaussian State Space Model.
}
\usage{
simssm(n = 200, trend = NULL, seasonal.order = 0, seasonal = NULL,
       arcoef = NULL,  ar = NULL, tau1 = NULL, tau2 = NULL, tau3 = NULL,
       sigma2 = 1.0, seed = NULL, plot = TRUE, \dots)
}
\arguments{
  \item{n}{the number of data generated by simulation.}
  \item{trend}{initial values of trend component of length \eqn{m1}, where
    \eqn{m1} is trend order (1, 2). If \code{NULL} (default), trend order is 0.}
  \item{seasonal.order}{order of seasonal component model (0, 1, 2).} 
  \item{seasonal}{if \code{seasonal.order} > 0, initial values of seasonal component
    of length \eqn{p-1}, where \eqn{p} is period of one season.}
  \item{arcoef}{AR coefficients.} 
  \item{ar}{initial values of AR component.}
  \item{tau1}{variance of trend component model.}
  \item{tau2}{variance of AR component model.}
  \item{tau3}{variance of seasonal component model.}
  \item{sigma2}{variance of the observation noise.}
  \item{seed}{arbitrary positive integer to generate a sequence of uniform
    random numbers. The default seed is based on the current time.}
  \item{plot}{logical. If \code{TRUE} (default), simulated data are plotted.}
  \item{\dots}{graphical arguments passed to \code{\link{plot.simulate}}.}
}
\value{
  An object of class \code{"simulate"}, giving simulated data of Gaussian state
  space model.
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# BLSALLFOOD data
data(BLSALLFOOD)
m1 <- 2; m2 <- 1; m3 <- 2
z <- season(BLSALLFOOD, trend.order = m1, seasonal.order = m2, ar.order = m3)

nl <- length(BLSALLFOOD)
trend <- z$trend[m1:1]
arcoef <- z$arcoef
period <- 12
seasonal <- z$seasonal[(period-1):1]
ar <- z$ar[m3:1]
tau1 <- z$tau2[1]
tau2 <- z$tau2[2]
tau3 <- z$tau2[3]
simssm(n = nl, trend, seasonal.order = m2, seasonal, arcoef, ar, tau1, tau2, tau3,
        sigma2 = z$sigma2, seed = 333)
}

\keyword{ts}

\name{plot.simulate}
\alias{plot.simulate}
\title{Plot Simulated Data Generated by State Space Model}
\description{
  Plot simulated data of Gaussian / non-Gaussian generated by state space model.
}
\usage{
\method{plot}{simulate}(x, use = NULL, \dots)
}
\arguments{
  \item{x}{an object of class \code{"simulate"} as returned by 
           \code{\link{simssm}} and \code{\link{ngsim}}.}
  \item{use}{start and end time \code{c(x1, x2)} to be plotted actually.}
  \item{\dots}{further graphical parameters may also be supplied as arguments.}
}
\keyword{ts}

\name{klinfo}
\alias{klinfo}

\title{Kullback-Leibler Information}
\description{
  Compute Kullback-Leibler information.}
\usage{
klinfo(distg = 1, paramg = c(0, 1), distf = 1, paramf, xmax = 10)
}
\arguments{
  \item{distg}{function for the true density (1 or 2).
    \tabular{rl}{
      1 : \tab Gaussian (normal) distribution\cr
      \tab paramg(1): mean\cr
      \tab paramg(2): variance\cr
      2 : \tab Cauchy distribution\cr
      \tab paramg(1): \eqn{\mu} (location parameter)\cr
      \tab paramg(2): \eqn{\tau^2} (dispersion parameter)}
  }
  \item{paramg}{parameter vector of true density.}
  \item{distf}{function for the model density (1 or 2).
    \tabular{rl}{
      1 : \tab Gaussian (normal) distribution\cr
      \tab paramf(1): mean\cr
      \tab paramf(2): variance\cr
      2 : \tab Cauchy distribution\cr
      \tab paramf(1): \eqn{\mu} (location parameter)\cr
      \tab paramf(2): \eqn{\tau^2} (dispersion parameter)}
  }
  \item{paramf}{parameter vector of the model density.}
  \item{xmax}{upper limit of integration. lower limit xmin = -\code{xmax}.}
}
\value{
  \item{nint}{number of function evaluation.}
  \item{dx}{delta.}
  \item{KLI}{Kullback-Leibler information, \eqn{I(g;f)}.}
  \item{gint}{integration of \eqn{g(y)} over [-\code{xmax}, \code{xmax}].}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# g:Gauss, f:Gauss
klinfo(distg = 1, paramg = c(0, 1), distf = 1, paramf = c(0.1, 1.5), xmax = 8)

# g:Gauss, f:Cauchy
klinfo(distg = 1, paramg = c(0, 1), distf = 2, paramf = c(0, 1), xmax = 8)
}

\keyword{ts}

\name{armafit2}
\alias{armafit2}

\title{Scalar ARMA Model Fitting}
\description{
Estimate all ARMA models within the user-specified maximum order by
 maximum likelihood method.
}
\usage{
armafit2(y, ar.order, ma.order)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{ar.order}{maximum AR order.}
  \item{ma.order}{maximum MA order.}
}
\value{
  \item{aicmin}{minimum AIC.}
  \item{maice.order}{AR and MA orders of minimum AIC model.}
  \item{sigma2}{innovation variance of all models.}
  \item{llkhood}{log-likelihood of all models.}
  \item{aic}{AIC of all models.}
  \item{coef}{AR and MA coefficients of all models.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# Sunspot number data
data(Sunspot)
y <- log10(Sunspot)
armafit2(y, ar.order = 5, ma.order = 5)
}

\keyword{ts}

\name{arfit}
\alias{arfit}

\title{Univariate AR Model Fitting}
\description{
  Fit a univariate AR model by the Yule-Walker method, the least squares (Householder)
  method or the PARCOR method.
}
\usage{
arfit(y, lag = NULL, method = 1, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{lag}{highest order of AR model. Default is \eqn{2 \sqrt{n}}{2*sqrt(n)},
    where \eqn{n} is the length of the time series \code{y}.}
  \item{method}{estimation procedure.
    \tabular{rl}{
      1 : \tab Yule-Walker method\cr
      2 : \tab Least squares (Householder) method\cr
      3 : \tab PARCOR method (Partial autoregression)\cr
      4 : \tab PARCOR method (PARCOR)\cr
      5 : \tab PARCOR method (Burg's algorithm)}
  }
  \item{plot}{logical. If \code{TRUE} (default), PARCOR, AIC and power spectrum
    are plotted.}
  \item{\dots}{graphical arguments passed to the \code{plot} method.}
}
\value{
  An object of class \code{"arfit"} which has a \code{plot} method. This is a
  list with the following components:
  \item{sigma2}{innovation variance.}
  \item{maice.order}{order of minimum AIC.}
  \item{aic}{AICs of the estimated AR models.}
  \item{arcoef}{AR coefficients of the estimated AR models.}
  \item{parcor}{PARCOR.}
  \item{spec}{power spectrum (in log scale) of the AIC best AR model.}
  \item{tsname}{the name of the univariate time series \code{y}.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# Sunspot number data
data(Sunspot)
arfit(log10(Sunspot), lag = 20, method = 1)

# BLSALLFOOD data
data(BLSALLFOOD)
arfit(BLSALLFOOD)
}

\keyword{ts}

\name{trend}
\alias{trend}

\title{Trend Estimation}
\description{
  Estimate the trend by state space model.
}
\usage{
trend(y, trend.order = 1, tau2.ini = NULL, delta, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{trend.order}{trend order.}
  \item{tau2.ini}{initial estimate of variance of the system noise \eqn{\tau^2}.
    If \code{tau2.ini} = \code{NULL}, the most suitable value is chosen in
    \eqn{\tau^2 = 2^{-k}}.}
  \item{delta}{search width (for \code{tau2.ini} is specified (not \code{NULL}))
    .}
  \item{plot}{logical. If \code{TRUE} (default), trend component and residuals
    are plotted.}
  \item{\dots}{graphical arguments passed to \code{\link{plot.trend}}.}}
\details{
  The trend model can be represented by a state space model

    \deqn{x_n = Fx_{n-1} + Gv_n,}{x(n) = F * x(n-1) + G * v(n),}
    \deqn{y_n = Hx_n + w_n,}{y(n) = H * x(n) + w(n),}

  where \eqn{F}, \eqn{G} and \eqn{H} are matrices with appropriate dimensions.
  We assume that \eqn{v_n}{v(n)} and \eqn{w_n}{w(n)} are white noises that have
  the normal distributions \eqn{N(0,\tau^2)} and \eqn{N(0, \sigma^2)},
  respectively.
}
\value{
  An object of class \code{"trend"}, which is a list with the following
  components:
  \item{trend}{trend component.}
  \item{residual}{residuals.}
  \item{tau2}{variance of the system noise \eqn{\tau^2}.}
  \item{sigma2}{variance of the observational noise \eqn{\sigma^2}.}
  \item{llkhood}{log-likelihood of the model.}
  \item{aic}{AIC.}
  \item{cov}{covariance matrix of smoother.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# The daily maximum temperatures for Tokyo
data(Temperature)
trend(Temperature, trend.order = 1, tau2.ini = 0.223, delta = 0.001)

trend(Temperature, trend.order = 2)
}

\keyword{ts}

\name{plot.tvspc}
\alias{plot.tvspc}

\title{Plot Evolutionary Power Spectra Obtained by Time Varying AR Model}
 
\description{
  Plot evolutionary power spectra obtained by time varying AR model returned
  by \code{\link{tvspc}}.
}
\usage{
\method{plot}{tvspc}(x, tvv = NULL, dx = 2, dy = 0.25, \dots)
}
\arguments{
  \item{x}{an object of class \code{"tvspc"}.}
  \item{tvv}{time varying variance as returned by \code{\link{tvvar}}.}
  \item{dx}{step width for the X axis.}
  \item{dy}{step width for the Y axis.}
  \item{\dots}{further graphical parameters may also be supplied as arguments.}
}
\examples{
# seismic data
data(MYE1F)
v <- tvvar(MYE1F, trend.order = 2, tau2.ini = 6.6e-06, delta = 1.0e-06,
           plot = FALSE )

z <- tvar(v$nordata, trend.order = 2, ar.order = 8, span = 20,
          outlier = c(630, 1026), tau2.ini = 6.6e-06, delta = 1.0e-06,
          plot = FALSE)

spec <- tvspc(z$arcoef, z$sigma2, span = 20, nf = 400)
plot(spec, tvv = v$tvv, dx = 2, dy = 0.10)
}

\keyword{ts}

\name{marlsq}
\alias{marlsq}

\title{Least Squares Method for Multivariate AR Model}
\description{
  Fit a multivariate AR model by least squares method.
}
\usage{
marlsq(y, lag = NULL)
}
\arguments{
  \item{y}{a multivariate time series.}
  \item{lag}{highest AR order. Default is \eqn{2 \sqrt{n}}{2*sqrt(n)}, where
    \eqn{n} is the length of the time series \code{y}.}
}
\value{
  An object of class \code{"marlsq"}, which is a list with the following
   components:
  \item{maice.order}{order of the MAICE model.}
  \item{aic}{AIC of the MAR model with minimum AIC orders.}
  \item{v}{innovation covariance matrix.}
  \item{arcoef}{AR coefficient matrices.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# Yaw rate, rolling, pitching and rudder angle of a ship
data(HAKUSAN)
y <- as.matrix(HAKUSAN[, c(1,2,4)])   # Yaw rate, Rolling, Rudder angle
z <- marlsq(y)
z

marspc(z$arcoef, v = z$v)
}

\keyword{ts}

\name{Haibara}
\docType{data}
\alias{Haibara}
\title{Haibara Data}
\description{
A bivariate time series of the groundwater level and the atmospheric pressure
 that were observed at 10-minuite intervals at the Haibara observatory of the
 Tokai region, Japan.
}
\usage{
data(Haibara)
}
\format{
A data frame with 400 observations on the following 2 variables.
  \tabular{rl}{
  [, 1] \tab Groundwater level\cr
  [, 2] \tab Atmospheric pressure
  }
}
\source{
The data were offered by Dr. M. Takahashi and Dr. N. Matsumoto of National
 Institute of Advanced Industrial Science and Technology.
}
\examples{
data(Haibara)

## put histograms on the diagonal
panel.hist <- function(x, ...)
{
    usr <- par("usr")
    par(usr = c(usr[1:2], 0, 1.3))
    nB <- 15; nB1 <- nB + 1
    xmin <- min(x, na.rm = TRUE)
    xmax <- max(x, na.rm = TRUE)
    w <- (xmax - xmin) / nB
    breaks <- xmin
    b <- xmin
    for (i in 1:nB) {
      b <- b + w
      breaks <- c(breaks, b)
    }
    h <- hist(x, breaks = breaks, plot = FALSE)
    y <- h$counts
    y <- y / max(y)
    rect(breaks[1:nB], 0, breaks[2:nB1], y, ...)
}

par(xaxs = "i", yaxs = "i", xaxt = "n", yaxt = "n")
pairs(Haibara, diag.panel = panel.hist, pch = 20, cex.labels = 1.5,
      label.pos = 0.9, lower.panel = NULL)
}
\keyword{datasets}

\name{Temperature}
\docType{data}
\alias{Temperature}
\title{Temperatures Data}
\description{
The daily maximum temperatures in Tokyo (from 1979-01-01 to 1980-04-30).
}
\usage{
data(Temperature)
}
\format{
A time series of 486 observations.
}
\source{
The data were obtained from Tokyo District Meteorological Observatory.
\url{https://www.data.jma.go.jp/obd/stats/etrn/}
}
\keyword{datasets}
 

\name{boxcox}
\alias{boxcox}

\title{Box-Cox Transformation}
\description{
  Compute Box-Cox transformation and find an optimal lambda with minimum AIC.}
\usage{
boxcox(y, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{plot}{logical. If \code{TRUE} (default), original data and transformed
  data with minimum AIC are plotted.}
  \item{\dots}{graphical arguments passed to \code{\link{plot.boxcox}}.}
}
\value{
  An object of class \code{"boxcox"}, which is a list with the following
   components:
  \item{mean}{mean of original data.}
  \item{var}{variance of original data.}
  \item{aic}{AIC of the model with respect to the original data.}
  \item{llkhood}{log-likelihood of the model with respect to the original data.}
  \item{z}{transformed data with the AIC best lambda.}
  \item{aic.z}{AIC of the model with respect to the transformed data.}
  \item{llkhood.z}{log-likelihood of the model with respect to the transformed
                   data.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# Sunspot number data
data(Sunspot)
boxcox(Sunspot)

# Wholesale hardware data
data(WHARD)
boxcox(WHARD)
}

\keyword{ts}

\name{MYE1F}
\docType{data}
\alias{MYE1F}
\title{Seismic Data}
\description{
The time series of East-West components of seismic waves, recorded every 0.02
seconds.
}
\usage{
data(MYE1F)
}
\format{
A time series of 2600 observations.
}
\source{
Takanami, T. (1991), "ISM data 43-3-01: Seismograms of foreshocks of 1982
 Urakawa-Oki earthquake", Ann. Inst. Statist. Math., 43, 605.
}
\keyword{datasets}

\name{period}
\alias{period}

\title{Compute a Periodogram}
\description{
  Compute a periodogram of the univariate time series.}
\usage{
period(y, window = 1, lag = NULL, minmax = c(-1.0e+30, 1.0e+30),
       plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{window}{smoothing window type. (0: box-car, 1: Hanning, 2: Hamming)}
  \item{lag}{maximum lag of autocovariance. If \code{NULL} (default), \cr
    \code{window} = 0 : \code{lag} = \eqn{n} - 1, \cr
    \code{window} > 0 : \code{lag} = 2 \eqn{\sqrt{n}}{sqrt(n)}, \cr
    where \eqn{n} is the length of data.}
  \item{minmax}{bound for outliers in low side and high side.}
  \item{plot}{logical. If \code{TRUE} (default), smoothed periodogram is plotted.}
  \item{\dots}{graphical arguments passed to \code{\link{plot.spg}}.}
}
\details{
  \tabular{rll}{
    Hanning Window : \tab \eqn{W_0} = 0.5 \tab \eqn{W_1} = 0.25 \cr
    Hamming Window : \tab \eqn{W_0} = 0.54 \tab \eqn{W_1} = 0.23}
}
\value{
  An object of class \code{"spg"}, which is a list with the following components:
  \item{period}{periodogram(or raw spectrum).}
  \item{smoothed.period}{smoothed log-periodogram. Smoothed periodogram is given
    if there is a negative value in the smoothed periodogram.}
  \item{log.scale}{if \code{TRUE} "smooth the periodogram on log scale.}
  \item{tsname}{the name of the univariate time series \code{y}.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
## BLSALLFOOD data
data(BLSALLFOOD)
period(BLSALLFOOD)

## seismic Data
data(MYE1F)

# smoothed periodogram 
period(MYE1F)

# periodogram
period(MYE1F, window = 0)

# raw spectrum
period(MYE1F, window = 0, lag = 200)

# Hamming window
period(MYE1F, window = 2)
}

\keyword{ts}

\name{marfit}
\alias{marfit}

\title{Yule-Walker Method of Fitting Multivariate AR Model}
\description{
  Fit a multivariate AR model by the Yule-Walker method.
}
\usage{
marfit(y, lag = NULL)
}
\arguments{
  \item{y}{a multivariate time series.}
  \item{lag}{highest order of fitted AR models. Default is \eqn{2 \sqrt{n}}{%
    2*sqrt(n)}, where \eqn{n} is the length of the time series \code{y}.}
}
\value{
  An object of class \code{"maryule"}, which is a list with the following
  components:
  \item{maice.order}{order of minimum AIC.}
  \item{aic}{AIC's of the AR models with order \eqn{0,\dots,}\code{lag}.}
  \item{v}{innovation covariance matrix of the AIC best model.}
  \item{arcoef}{AR coefficients of the AIC best model.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# Yaw rate, rolling, pitching and rudder angle of a ship
data(HAKUSAN)
yy <- as.matrix(HAKUSAN[, c(1,2,4)])   # Yaw rate, Pitching, Rudder angle
nc <- dim(yy)[1]
n <- seq(1, nc, by = 2) 
y <- yy[n, ]
marfit(y, 20)
}

\keyword{ts}

\name{polreg}
\alias{polreg}

\title{Polynomial Regression Model}
\description{
  Estimate the trend using the AIC best polynomial regression model.
}
\usage{
polreg(y, order, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{order}{maximum order of polynomial regression.}
  \item{plot}{logical. If \code{TRUE} (default), original data and trend
    component are plotted.}
  \item{\dots}{graphical arguments passed to \code{\link{plot.polreg}}.}
}
\value{
  An object of class \code{"polreg"}, which is a list with the following
   components:
  \item{order.maice}{MAICE (minimum AIC estimate) order.}
  \item{sigma2}{residual variance of the model with order \eqn{M}.
    (\eqn{0 \leq M \leq}{0 <= M <=} \code{order})}
  \item{aic}{AIC of the model with order \eqn{M}.
    (\eqn{0 \leq M \leq}{0 <= M <=} \code{order})}
  \item{daic}{AIC - minimum AIC.}
  \item{coef}{regression coefficients \eqn{A(I,M)} with order \eqn{M}.

    (\eqn{1 \leq M \leq}{1 <= M <=} \code{order},
    \eqn{1 \leq I \leq M}{1 <= I <= M})}
  \item{trend}{trend component.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# The daily maximum temperatures for Tokyo
data(Temperature)
polreg(Temperature, order = 7)

# Wholesale hardware data
data(WHARD)
y <- log10(WHARD)
polreg(y, order = 15)
}

\keyword{ts}

\name{fftper}
\alias{fftper}

\title{Compute a Periodogram via FFT}
\description{
  Compute a periodogram of the univariate time series via FFT.}
\usage{
fftper(y, window = 1, plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{window}{smoothing window type. (0: box-car, 1: Hanning, 2: Hamming)}
  \item{plot}{logical. If \code{TRUE} (default), smoothed (log-)periodogram is
              plotted.}
  \item{\dots}{graphical arguments passed to \code{\link{plot.spg}}.}
}
\details{
  \tabular{rll}{
    Hanning Window : \tab \eqn{W_0} = 0.5 \tab \eqn{W_1} = 0.25 \cr
    Hamming Window : \tab \eqn{W_0} = 0.54 \tab \eqn{W_1} = 0.23}
}
\value{
  An object of class \code{"spg"}, which is a list with the following components:
  \item{period}{periodogram.}
  \item{smoothed.period}{smoothed periodogram. If there is not a negative
    number, logarithm of smoothed periodogram.}
  \item{log.scale}{logical. If \code{TRUE} \code{smoothed.period} is logarithm
    of smoothed periodogram.}
  \item{tsname}{the name of the univariate time series \code{y}.}
}

\note{
  We assume that the length \eqn{N} of the input time series \code{y} is a power
  of 2. If \eqn{N} is not a power of 2, calculate using the FFT by appending 0's
  behind the data \code{y}. 
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# Yaw rate, rolling, pitching and rudder angle of a ship
data(HAKUSAN)
YawRate <- HAKUSAN[, 1]
fftper(YawRate, window = 0)
}

\keyword{ts}

\name{plot.polreg}
\alias{plot.polreg}
\title{Plot Fitted Polynomial Trend}
\description{
  Plot trend component of fitted polynomial returned by \code{\link{polreg}}.
}
\usage{
\method{plot}{polreg}(x, rdata = NULL, \dots)
}
\arguments{
  \item{x}{an object of class \code{"polreg"}.}
  \item{rdata}{original data, if necessary.}
  \item{\dots}{further graphical parameters may also be supplied as arguments.}
}
\keyword{ts}

\name{unicor}
\alias{unicor}

\title{Autocovariance and Autocorrelation}
\description{
  Compute autocovariance and autocorrelation function of the univariate time
 series.}
\usage{
unicor(y, lag = NULL, minmax = c(-1.0e+30, 1.0e+30), plot = TRUE, \dots)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{lag}{maximum lag. Default is \eqn{2 \sqrt{n}}{2*sqrt(n)}, where \eqn{n}
    is the length of the time series \code{y}.}
  \item{minmax}{thresholds for outliers in low side and high side.}
  \item{plot}{logical. If \code{TRUE} (default), autocorrelations are plotted.}
  \item{\dots}{graphical arguments passed to the \code{plot} method.}
}
\value{
  An object of class \code{"unicor"} which has a \code{plot} method. This is a
  list with the following components:
  \item{acov}{autocovariances.}
  \item{acor}{autocorrelations.}
  \item{acov.err}{error bound for autocovariances.}
  \item{acor.err}{error bound for autocorrelations.}
  \item{mean}{mean of \code{y}.}
  \item{tsname}{the name of the univariate time series \code{y}.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.
}
\examples{
# Yaw rate, rolling, pitching and rudder angle of a ship
data(HAKUSAN)
Yawrate <- HAKUSAN[, 1]
unicor(Yawrate, lag = 50)

# seismic data
data(MYE1F)
unicor(MYE1F, lag = 50)
}

\keyword{ts}
